C
C     Benchmark program for calculating time 
C                         with the random mattix of order 5
C
      use ddcommunication_h, only : eigen_init
     &                            , eigen_free
     &                            , ddcyc1d_cyc2d
     &                            , ddcyc2d_cyc1d

      implicit double precision (a-h,o-z)

      real(8), allocatable :: a(:,:),ai(:,:)
      real(8), pointer :: b(:),bi(:),w(:)
      real(8), allocatable :: ah(:,:),aih(:,:)
      real(8), allocatable :: al(:,:),ail(:,:)
      real(8), pointer :: bh(:),bih(:),wh(:)
      real(8), pointer :: bl(:),bil(:),wl(:)
      logical :: iexist

      include 'mpif.h'
      include 'trd.h'

C     Initialize MPI
      call mpi_init(ierr)
      call mpi_comm_rank(mpi_comm_world,i$inod,ierr)
      call mpi_comm_size(mpi_comm_world,i$nnod,ierr)

C     Memoris of matrix
      n=500
      m=32
      nm=(n/2)*2+1
      allocate( a(n,n), ai(n,n) )
      allocate(
     &         b((nm)*((n-1)/i$nnod+1+nm+1)),
     &         bi((nm)*((n-1)/i$nnod+1+nm+1)),
     &         w(nm),
     &         stat=istat )
      allocate( ah(n,n), aih(n,n), al(n,n), ail(n,n) )
      allocate(
     &         bh((nm)*((n-1)/i$nnod+1+nm+1)),
     &         bih((nm)*((n-1)/i$nnod+1+nm+1)),
     &         bl((nm)*((n-1)/i$nnod+1+nm+1)),
     &         bil((nm)*((n-1)/i$nnod+1+nm+1)),
     &         wh(nm), wl(nm),
     &         stat=istat )
      if(istat.ne.0) then
         print*,"Memory exhausted"
         call flush(6)
         stop
      endif

C     Set Random matrix for Double percision
      call matrix_set(n,a,ai)
      call matrix_adjust_h(a,n,b,nm)
      call matrix_adjust_hi(ai,n,bi,nm)

C     Calculating eigen values and vectors
      time_eigen_s_start = mpi_wtime()
        call eigen_h(n,b,bi,w,nm,m,0)
      time_eigen_s_finish = mpi_wtime()

C     Set Random matrix for Double-Double percision
      call ddmatrix_set(n,ah,al,aih,ail)
      call ddmatrix_adjust_h(ah,al,n,bh,bl,nm)
      call ddmatrix_adjust_hi(aih,ail,n,bih,bil,nm)

C     Calculating eigen values and vectors
      time_ddeigen_s_start = mpi_wtime()
        call ddeigen_h(n,bh,bl,bih,bil,wh,wl,nm,m,0)
      time_ddeigen_s_finish = mpi_wtime()

C     Show results
      if(myrank==1)then
        write(*,'(a15 I5 a3 I5)') 
     &  "Elapse time   ",n,"  x",n
        write(*,'(A21 F20.16 A9)') '  Eigen_s : Elapse = ',
     &              time_eigen_s_finish - time_eigen_s_start,
     &              "    (sec)"
        write(*,'(A21 F20.16 A9)') 'PQEigen_s : Elapse = ',
     &              time_ddeigen_s_finish - time_ddeigen_s_start,
     &              "    (sec)"
      endif

C     Free memory
      deallocate(a,ai,b,bi,w)
      deallocate(ah,aih,bh,bih,wh)
      deallocate(al,ail,bl,bil,wl)

C     Finalizing MPI
      call MPI_Finalize(ierr)
      end
