      module bailey
        implicit none
        double precision, parameter :: splitter = 134217729.d0
        complex(kind(0.d0)), parameter :: imag_unit = (0.0d0, 1.0d0)
        interface bmul
          module procedure mul_dd, mul_zz, mul_dz, mul_zd, mul_ddz
        end interface
        interface bsum
          module procedure sum_dd, sum_zz
        end interface
        contains

        subroutine fast_two_sum(a,b,ch,cl)
          implicit none
          double precision, intent(in) :: a, b
          double precision, intent(out) :: ch, cl
          double precision :: temp
          ch = a + b
          temp = ch - a
          cl = b - temp
        end subroutine fast_two_sum

        subroutine fast_two_sum_z(a,b,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: a, b
          complex(kind(0.d0)), intent(out) :: ch, cl
          complex(kind(0.d0)) :: temp
          ch = a + b
          temp = ch - a
          cl = b - temp
        end subroutine fast_two_sum_z

        subroutine two_sum(a,b,ch,cl)
          implicit none
          double precision, intent(in) :: a, b
          double precision, intent(out) :: ch, cl
          double precision :: a0, b0
          ch = a + b
          b0 = ch - a
          a0 = ch - b0
          cl = (a-a0) + (b-b0)
        end subroutine two_sum

        subroutine two_sum_z(a,b,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: a, b
          complex(kind(0.d0)), intent(out) :: ch, cl
          complex(kind(0.d0)) :: a0, b0
          ch = a + b
          b0 = ch - a
          a0 = ch - b0
          cl = (a-a0) + (b-b0)
        end subroutine two_sum_z

        subroutine split(a,ah,al)
          implicit none
          double precision, intent(in) :: a
          double precision, intent(out) :: ah, al
          double precision :: t0, t1
          t0 = splitter * a
          t1 = t0 - a
          ah = t0 - t1
          al = a - ah
        end subroutine split

        subroutine split_z(a,ah,al)
          implicit none
          complex(kind(0.d0)), intent(in) :: a
          complex(kind(0.d0)), intent(out) :: ah, al
          complex(kind(0.d0)) :: t0, t1
          t0 = splitter * a
          t1 = t0 - a
          ah = t0 - t1
          al = a - ah
        end subroutine split_z

        subroutine two_prod(a,b,ch,cl)
          implicit none
          double precision, intent(in) :: a, b
          double precision, intent(out) :: ch, cl
          double precision :: ah, al, bh, bl
          ch = a * b
          call split(a,ah,al)
          call split(b,bh,bl)
          cl = ((ah*bh-ch)+ah*bl+al*bh)+al*bl
        end subroutine two_prod

        subroutine two_prod_z(a,b,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: a, b
          complex(kind(0.d0)), intent(out) :: ch, cl
          complex(kind(0.d0)) :: ah, al, bh, bl
          double precision :: br, bhr, blr, bi, bhi, bli, cr, ci
          complex(kind(0.d0)) :: chr, clr, chi, cli
          call split_z(a,ah,al)
          call split_z(b,bh,bl)
          br  = dble(b)
          bhr = dble(bh)
          blr = dble(bl)
          bi  = dimag(b)
          bhi = dimag(bh)
          bli = dimag(bl)
          chr = a * br
          clr = ((ah*bhr-chr)+ah*blr+al*bhr)+al*blr
          chi = a * bi
          cli = ((ah*bhi-chi)+ah*bli+al*bhi)+al*bli
!         chi = chi * imag_unit
          cr = dble(chi)
          ci = dimag(chi)
          chi = dcmplx(-ci,cr)
          cr = dble(cli)
          ci = dimag(cli)
          cli = dcmplx(-ci,cr)
          call sum_zz(chr,clr,chi,cli,ch,cl)
        end subroutine two_prod_z

        subroutine sum_dd(ah,al,bh,bl,ch,cl)
          implicit none
          double precision, intent(in) :: ah, al, bh, bl
          double precision, intent(out) :: ch, cl
          double precision :: sumh, errorh
          call two_sum(ah,bh,sumh,errorh)
          errorh = errorh + al + bl
          call fast_two_sum( sumh, errorh, ch, cl )
        end subroutine sum_dd

        subroutine sum_zz(ah,al,bh,bl,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: ah, al, bh, bl
          complex(kind(0.d0)), intent(out) :: ch, cl
          double precision :: ahr, ahi, alr, ali, bhr, bhi, blr, bli
          double precision :: chr, chi, clr, cli
          ahr = dble(ah)
          alr = dble(al)
          bhr = dble(bh)
          blr = dble(bl)
          ahi = dimag(ah)
          ali = dimag(al)
          bhi = dimag(bh)
          bli = dimag(bl)
          call sum_dd(ahr,alr,bhr,blr,chr,clr)
          call sum_dd(ahi,ali,bhi,bli,chi,cli)
          ch = dcmplx(chr,chi)
          cl = dcmplx(clr,cli)
        end subroutine sum_zz

        subroutine mul_dd(ah,al,bh,bl,ch,cl)
          implicit none
          double precision, intent(in) :: ah, al, bh, bl
          double precision, intent(out) :: ch, cl
          double precision :: ph, pl
          call two_prod(ah,bh,ph,pl)
          pl = pl + ah*bl
          pl = pl + al*bh
          call fast_two_sum(ph,pl,ch,cl)
        end subroutine mul_dd

        subroutine mul_zz_org(ah,al,bh,bl,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: ah, al, bh, bl
          complex(kind(0.d0)), intent(out) :: ch, cl
          double precision :: ahr, ahi, alr, ali, bhr, bhi, blr, bli
          double precision :: chr, chi, clr, cli
          double precision :: chrr, chri, clrr, clri
          double precision :: chir, chii, clir, clii
          ahr = dble(ah)
          alr = dble(al)
          bhr = dble(bh)
          blr = dble(bl)
          ahi = dimag(ah)
          ali = dimag(al)
          bhi = dimag(bh)
          bli = dimag(bl)
          call mul_dd(ahr,alr,bhr,blr,chrr,clrr)
          call mul_dd(ahi,ali,bhi,bli,chii,clii)
          call mul_dd(ahr,alr,bhi,bli,chri,clri)
          call mul_dd(ahi,ali,bhr,blr,chir,clir)
          call sum_dd(chrr,clrr,-chii,-clii,chr,clr)
          call sum_dd(chri,clri,chir,clir,chi,cli)
          ch = dcmplx(chr,chi)
          cl = dcmplx(clr,cli)
        end subroutine mul_zz_org

        subroutine mul_zz(ah,al,bh,bl,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: ah, al, bh, bl
          complex(kind(0.d0)), intent(out) :: ch, cl
          complex(kind(0.d0)) :: ph, pl
          call two_prod_z(ah,bh,ph,pl)
          pl = pl + ah*bl
          pl = pl + al*bh
          call fast_two_sum_z(ph,pl,ch,cl)
        end subroutine mul_zz

        subroutine mul_dz(ah,al,bh,bl,ch,cl)
          implicit none
          double precision, intent(in) :: ah, al
          complex(kind(0.d0)), intent(in) :: bh, bl
          complex(kind(0.d0)), intent(out) :: ch, cl
          double precision :: bhr, bhi, blr, bli
          double precision :: chr, chi, clr, cli
          bhr = dble(bh)
          blr = dble(bl)
          bhi = dimag(bh)
          bli = dimag(bl)
          call mul_dd(ah,al,bhr,blr,chr,clr)
          call mul_dd(ah,al,bhi,bli,chi,cli)
          ch = dcmplx(chr,chi)
          cl = dcmplx(clr,cli)
        end subroutine mul_dz

        subroutine mul_zd(ah,al,bh,bl,ch,cl)
          implicit none
          complex(kind(0.d0)), intent(in) :: ah, al
          double precision, intent(in) :: bh, bl
          complex(kind(0.d0)), intent(out) :: ch, cl
          call mul_dz(bh, bl, ah, al, ch, cl)
        end subroutine mul_zd

        subroutine mul_ddz(ah, al, bh, bl, ch, cl)
          double precision, intent(in) :: ah, al, bh, bl
          double complex, intent(out) :: ch, cl
          double precision :: temph, templ
          call bmul(ah, al, bh, bl, temph, templ)
          ch = temph
          cl = templ
        end subroutine mul_ddz

      end module
