      SUBROUTINE DDDOT(N,DXH,DXL,INCX,DYH,DYL,INCY,DDH,DDL)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      INTEGER INCX,INCY,N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION DXH(*),DXL(*),DYH(*),DYL(*),DDH,DDL
*     ..
*
*  Purpose
*  =======
*
*     DDDOT forms the dot product of two vectors.
*     uses unrolled loops for increments equal to one.
*
*  Advancesoft corp.
*  =================
*
*     jack dongarra, linpack, 3/11/78.
*     modified 12/3/93, array(1) declarations changed to array(*)
*     modified by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Local Scalars ..
!     DOUBLE PRECISION DTEMP
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      INTEGER I,IX,IY,M,MP1
      integer IXG,IYG
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MOD
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDDOT"
#endif
      DDH = 0.0d0
      DDL = 0.0d0
!     DTEMP = 0.0d0
      IF (N.LE.0) RETURN
      IF (INCX.EQ.1 .AND. INCY.EQ.1) GO TO 20
*
*        code for unequal increments or equal increments
*          not equal to 1
*
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      IXG = IX
      IYG = IY
!$omp parallel
!$omp do
!$omp& private(I,IX,IY,s1,t1,t2,u1,p1,p2,q1,q2,r1,r2)
      DO 10 I = 1,N
          IX = IXG + INCX*(I-1)
          IY = IYG + INCY*(I-1)
!         DTEMP = DTEMP + DX(IX)*DY(IY)
!         CALL dd_mul(t1,t2,DXH(IX),DXL(IX),DYH(IY),DYL(IY))
          t1 = DXH(IX) * DYH(IY)
          u1 = 134217729.0*DXH(IX)
          p1 = u1 - (u1-DXH(IX))
          p2 = DXH(IX) - p1
          u1 = 134217729.0*DYH(IY)
          r1 = u1 - (u1-DYH(IY))
          r2 = DYH(IY) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(IX) * DYL(IY) + DXL(IX) * DYH(IY))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!          IX = IX + INCX
!          IY = IY + INCY
   10 CONTINUE
!$omp end parallel
!     DDOT = DTEMP
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDDOT"
#include "fout.h"
#endif
      RETURN
*
*        code for both increments equal to 1
*
*
*        clean-up loop
*
   20 M = MOD(N,5)
      IF (M.EQ.0) GO TO 40
      DO 30 I = 1,M
!         DTEMP = DTEMP + DX(I)*DY(I)
!         CALL dd_mul(t1,t2,DXH(I),DXL(I),DYH(I),DYL(I))
          t1 = DXH(I) * DYH(I)
          u1 = 134217729.0*DXH(I)
          p1 = u1 - (u1-DXH(I))
          p2 = DXH(I) - p1
          u1 = 134217729.0*DYH(I)
          r1 = u1 - (u1-DYH(I))
          r2 = DYH(I) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(I) * DYL(I) + DXL(I) * DYH(I))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   30 CONTINUE
      IF (N.LT.5) GO TO 60
   40 MP1 = M + 1
!$omp parallel
!$omp do
!$omp& private(I,s1,t1,t2,u1,p1,p2,q1,q2,r1,r2)
      DO 50 I = MP1,N,5
!         DTEMP = DTEMP + DX(I)*DY(I) + DX(I+1)*DY(I+1) +
!    +            DX(I+2)*DY(I+2) + DX(I+3)*DY(I+3) + DX(I+4)*DY(I+4)
!         CALL dd_mul(t1,t2,DXH(I),DXL(I),DYH(I),DYL(I))
          t1 = DXH(I) * DYH(I)
          u1 = 134217729.0*DXH(I)
          p1 = u1 - (u1-DXH(I))
          p2 = DXH(I) - p1
          u1 = 134217729.0*DYH(I)
          r1 = u1 - (u1-DYH(I))
          r2 = DYH(I) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(I) * DYL(I) + DXL(I) * DYH(I))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         CALL dd_mul(t1,t2,DXH(I+1),DXL(I+1),DYH(I+1),DYL(I+1))
          t1 = DXH(I+1) * DYH(I+1)
          u1 = 134217729.0*DXH(I+1)
          p1 = u1 - (u1-DXH(I+1))
          p2 = DXH(I+1) - p1
          u1 = 134217729.0*DYH(I+1)
          r1 = u1 - (u1-DYH(I+1))
          r2 = DYH(I+1) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(I+1) * DYL(I+1) + DXL(I+1) * DYH(I+1))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         CALL dd_mul(t1,t2,DXH(I+2),DXL(I+2),DYH(I+2),DYL(I+2))
          t1 = DXH(I+2) * DYH(I+2)
          u1 = 134217729.0*DXH(I+2)
          p1 = u1 - (u1-DXH(I+2))
          p2 = DXH(I+2) - p1
          u1 = 134217729.0*DYH(I+2)
          r1 = u1 - (u1-DYH(I+2))
          r2 = DYH(I+2) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(I+2) * DYL(I+2) + DXL(I+2) * DYH(I+2))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         CALL dd_mul(t1,t2,DXH(I+3),DXL(I+3),DYH(I+3),DYL(I+3))
          t1 = DXH(I+3) * DYH(I+3)
          u1 = 134217729.0*DXH(I+3)
          p1 = u1 - (u1-DXH(I+3))
          p2 = DXH(I+3) - p1
          u1 = 134217729.0*DYH(I+3)
          r1 = u1 - (u1-DYH(I+3))
          r2 = DYH(I+3) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(I+3) * DYL(I+3) + DXL(I+3) * DYH(I+3))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         CALL dd_mul(t1,t2,DXH(I+4),DXL(I+4),DYH(I+4),DYL(I+4))
          t1 = DXH(I+4) * DYH(I+4)
          u1 = 134217729.0*DXH(I+4)
          p1 = u1 - (u1-DXH(I+4))
          p2 = DXH(I+4) - p1
          u1 = 134217729.0*DYH(I+4)
          r1 = u1 - (u1-DYH(I+4))
          r2 = DYH(I+4) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DXH(I+4) * DYL(I+4) + DXL(I+4) * DYH(I+4))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         CALL dd_selfadd(DDH,DDL,t1,t2)
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50 CONTINUE
!$omp end parallel
   60 CONTINUE
!  60 DDOT = DTEMP
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDDOT"
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
      END
