!     SUBROUTINE DGBMV(TRANS,M,N,KL,KU,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
      subroutine ddgbmv(trans,m,n,kl,ku,
     +                  alphah, alphal,
     +                  ah, al, lda,
     +                  xh, xl, incx,
     +                  betah, betal,
     +                  yh, yl, incy
     +)
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA,BETA
      double precision alphah, alphal
      double precision betah, betal
      INTEGER INCX,INCY,KL,KU,LDA,M,N
      CHARACTER TRANS
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION A(LDA,*),X(*),Y(*)
      double precision Ah(lda,*), Al(lda,*)
      double precision xh(*), xl(*)
      double precision yh(*), yl(*)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     ..
*
*  Purpose
*  =======
*
*  DGBMV  performs one of the matrix-vector operations
*
*     y := alpha*A*x + beta*y,   or   y := alpha*A'*x + beta*y,
*
*  where alpha and beta are scalars, x and y are vectors and A is an
*  m by n band matrix, with kl sub-diagonals and ku super-diagonals.
*
*  Arguments
*  ==========
*
*  TRANS  - CHARACTER*1.
*           On entry, TRANS specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'   y := alpha*A*x + beta*y.
*
*              TRANS = 'T' or 't'   y := alpha*A'*x + beta*y.
*
*              TRANS = 'C' or 'c'   y := alpha*A'*x + beta*y.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of the matrix A.
*           M must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  KL     - INTEGER.
*           On entry, KL specifies the number of sub-diagonals of the
*           matrix A. KL must satisfy  0 .le. KL.
*           Unchanged on exit.
*
*  KU     - INTEGER.
*           On entry, KU specifies the number of super-diagonals of the
*           matrix A. KU must satisfy  0 .le. KU.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION.
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry, the leading ( kl + ku + 1 ) by n part of the
*           array A must contain the matrix of coefficients, supplied
*           column by column, with the leading diagonal of the matrix in
*           row ( ku + 1 ) of the array, the first super-diagonal
*           starting at position 2 in row ku, the first sub-diagonal
*           starting at position 1 in row ( ku + 2 ), and so on.
*           Elements in the array A that do not correspond to elements
*           in the band matrix (such as the top left ku by ku triangle)
*           are not referenced.
*           The following program segment will transfer a band matrix
*           from conventional full matrix storage to band storage:
*
*                 DO 20, J = 1, N
*                    K = KU + 1 - J
*                    DO 10, I = MAX( 1, J - KU ), MIN( M, J + KL )
*                       A( K + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           ( kl + ku + 1 ).
*           Unchanged on exit.
*
*  X      - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCX ) ) when TRANS = 'N' or 'n'
*           and at least
*           ( 1 + ( m - 1 )*abs( INCX ) ) otherwise.
*           Before entry, the incremented array X must contain the
*           vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  BETA   - DOUBLE PRECISION.
*           On entry, BETA specifies the scalar beta. When BETA is
*           supplied as zero then Y need not be set on input.
*           Unchanged on exit.
*
*  Y      - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n'
*           and at least
*           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise.
*           Before entry, the incremented array Y must contain the
*           vector y. On exit, Y is overwritten by the updated vector y.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*     .. Parameters ..
!     DOUBLE PRECISION ONE,ZERO
      double precision oneh, onel
      double precision zeroh, zerol
!     PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      double precision temph, templ
      INTEGER I,INFO,IX,IY,J,JX,JY,K,KUP1,KX,KY,LENX,LENY
      double precision ddmh, ddml
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX,MIN
*     ..
*
*     Test the input parameters.
*
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddgbmv"
#endif
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
      INFO = 0
      IF (.NOT.LSAME(TRANS,'N') .AND. .NOT.LSAME(TRANS,'T') .AND.
     +    .NOT.LSAME(TRANS,'C')) THEN
          INFO = 1
      ELSE IF (M.LT.0) THEN
          INFO = 2
      ELSE IF (N.LT.0) THEN
          INFO = 3
      ELSE IF (KL.LT.0) THEN
          INFO = 4
      ELSE IF (KU.LT.0) THEN
          INFO = 5
      ELSE IF (LDA.LT. (KL+KU+1)) THEN
          INFO = 8
      ELSE IF (INCX.EQ.0) THEN
          INFO = 10
      ELSE IF (INCY.EQ.0) THEN
          INFO = 13
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DGBMV ',INFO)
!         RETURN
          goto 9999
      END IF
*
*     Quick return if possible.
*
!     IF ((M.EQ.0) .OR. (N.EQ.0) .OR.
!    +    ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
      if((m .eq. 0) .or. (n .eq. 0) .or.
     +   (
!           (alpha .eq. zero)
     +     ((alphah .eq. zeroh) .and.
     +      (alphal .eq. zerol))
     +     .and.
!           (beta .eq. one)
     +     ((betah .eq. oneh) .and.
     +      (betal .eq. onel))
     +    )
     +) goto 9999

*
*     Set  LENX  and  LENY, the lengths of the vectors x and y, and set
*     up the start points in  X  and  Y.
*
      IF (LSAME(TRANS,'N')) THEN
          LENX = N
          LENY = M
      ELSE
          LENX = M
          LENY = N
      END IF
      IF (INCX.GT.0) THEN
          KX = 1
      ELSE
          KX = 1 - (LENX-1)*INCX
      END IF
      IF (INCY.GT.0) THEN
          KY = 1
      ELSE
          KY = 1 - (LENY-1)*INCY
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the band part of A.
*
*     First form  y := beta*y.
*
!     IF (BETA.NE.ONE) THEN
      if(
!         (beta .ne. one)
     +   ((betah .ne. oneh) .or.
     +    (betal .ne. onel))
     +) then
          IF (INCY.EQ.1) THEN
!             IF (BETA.EQ.ZERO) THEN
              if(
!           (beta .eq. zero)
     +     ((betah .eq. zeroh) .and.
     +      (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                  DO 10 I = 1,LENY
!                     Y(I) = ZERO
                      yh(i) = zeroh
                      yl(i) = zerol
   10             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                  DO 20 I = 1,LENY
!                     Y(I) = BETA*Y(I)
!<<<                  y(i) = beta * y(i)
                      t_ch = betah * yh(i)
                      t_u1 = 134217729.0 * betah
                      t_p1 = t_u1
     +                         - (t_u1 - betah)
                      t_p2 = betah - t_p1
                      t_u1 = 134217729.0 * yh(i)
                      t_r1 = t_u1
     +                         - (t_u1 - yh(i))
                      t_r2 = yh(i) - t_r1
                      t_cl = ((t_p1*t_r1
     +                         - t_ch)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      t_cl = t_cl
     +                         + (betah*yl(i)
     +                         +  betal*yh(i))
                      t_s1 = t_ch + t_cl
                      yl(i) = t_cl - (t_s1 - t_ch)
                      yh(i) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                  y(i) = beta * y(i)
   20             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          ELSE
              IY = KY
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I, iy)
                  DO 30 I = 1,LENY
                      iy = (i - 1)*incy + ky
!                     Y(IY) = ZERO
                      yh(iy) = zeroh
                      yl(iy) = zerol
!!                    IY = IY + INCY
   30             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I, iy)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                  DO 40 I = 1,LENY
                      iy = (i - 1)*incy + ky
!                     Y(IY) = BETA*Y(IY)
!<<<                  y(iy) = beta * y(iy)
                      t_ch = betah * yh(iy)
                      t_u1 = 134217729.0 * betah
                      t_p1 = t_u1
     +                         - (t_u1 - betah)
                      t_p2 = betah - t_p1
                      t_u1 = 134217729.0 * yh(iy)
                      t_r1 = t_u1
     +                         - (t_u1 - yh(iy))
                      t_r2 = yh(iy) - t_r1
                      t_cl = ((t_p1*t_r1
     +                         - t_ch)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      t_cl = t_cl
     +                         + (betah*yl(iy)
     +                         +  betal*yh(iy))
                      t_s1 = t_ch + t_cl
                      yl(iy) = t_cl - (t_s1 - t_ch)
                      yh(iy) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                  y(iy) = beta * y(iy)
!!                    IY = IY + INCY
   40             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          END IF
      END IF
!     IF (ALPHA.EQ.ZERO) RETURN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) goto 9999
      KUP1 = KU + 1
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  y := alpha*A*x + y.
*
          JX = KX
          IF (INCY.EQ.1) THEN
              DO 60 J = 1,N
!                 IF (X(JX).NE.ZERO) THEN
                  if(
!                   (x(jx) .ne. zero)
     +             ((xh(jx) .ne. zeroh) .or.
     +              (xl(jx) .ne. zerol))
     +            ) then
!                     TEMP = ALPHA*X(JX)
!<<<                  temp = alpha * x(jx)
                      temph = alphah * xh(jx)
                      t_u1 = 134217729.0 * alphah
                      t_p1 = t_u1
     +                         - (t_u1 - alphah)
                      t_p2 = alphah - t_p1
                      t_u1 = 134217729.0 * xh(jx)
                      t_r1 = t_u1
     +                         - (t_u1 - xh(jx))
                      t_r2 = xh(jx) - t_r1
                      templ = ((t_p1*t_r1
     +                        - temph)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      templ = templ
     +                      + (alphah*xl(jx)
     +                      + alphal*xh(jx))
                      t_s1 = temph + templ
                      templ = templ - (t_s1 - temph)
                      temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp = alpha * x(jx)
                      K = KUP1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 50 I = MAX(1,J-KU),MIN(M,J+KL)
!                         Y(I) = Y(I) + TEMP*A(K+I,J)
!<<<                      ddm = temp * a(k+i,j)
                          ddmh = temph * ah(k+i,j)
                          t_u1 = 134217729.0 * temph
                          t_p1 = t_u1
     +                             - (t_u1 - temph)
                          t_p2 = temph - t_p1
                          t_u1 = 134217729.0 * ah(k+i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ah(k+i,j))
                          t_r2 = ah(k+i,j) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (temph*al(k+i,j)
     +                          + templ*ah(k+i,j))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = temp * a(k+i,j)
!<<<                      y(i) = y(i) + ddm
                          t_p1 = yh(i) + ddmh
                          t_p2 = t_p1 - yh(i)
                          t_q1 = t_p1
                          t_q2 = (yh(i) - (t_p1 - t_p2))
     +                             + (ddmh - t_p2)
                          t_q2 = t_q2 + yl(i) + ddml
                          yh(i) = t_q1 + t_q2
                          yl(i) = t_q2 - (yh(i) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      y(i) = y(i) + ddm
   50                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  JX = JX + INCX
   60         CONTINUE
          ELSE
              DO 80 J = 1,N
!                 IF (X(JX).NE.ZERO) THEN
                  if(
!                   (x(jx) .ne. zero)
     +             ((xh(jx) .ne. zeroh) .or.
     +              (xl(jx) .ne. zerol))
     +            ) then
!                     TEMP = ALPHA*X(JX)
!<<<                  temp = alpha * x(jx)
                      temph = alphah * xh(jx)
                      t_u1 = 134217729.0 * alphah
                      t_p1 = t_u1
     +                         - (t_u1 - alphah)
                      t_p2 = alphah - t_p1
                      t_u1 = 134217729.0 * xh(jx)
                      t_r1 = t_u1
     +                         - (t_u1 - xh(jx))
                      t_r2 = xh(jx) - t_r1
                      templ = ((t_p1*t_r1
     +                        - temph)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      templ = templ
     +                      + (alphah*xl(jx)
     +                      + alphal*xh(jx))
                      t_s1 = temph + templ
                      templ = templ - (t_s1 - temph)
                      temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp = alpha * x(jx)
                      IY = KY
                      K = KUP1 - J
!$omp parallel
!$omp do
!$omp+ private(I, iy)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 70 I = MAX(1,J-KU),MIN(M,J+KL)
                          iy = ky + (i - max(1,j-ku))*incy
!                         Y(IY) = Y(IY) + TEMP*A(K+I,J)
!<<<                  ddm = temp * a(k+i,j)
                      ddmh = temph * ah(k+i,j)
                      t_u1 = 134217729.0 * temph
                      t_p1 = t_u1
     +                         - (t_u1 - temph)
                      t_p2 = temph - t_p1
                      t_u1 = 134217729.0 * ah(k+i,j)
                      t_r1 = t_u1
     +                         - (t_u1 - ah(k+i,j))
                      t_r2 = ah(k+i,j) - t_r1
                      ddml = ((t_p1*t_r1
     +                        - ddmh)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddml = ddml
     +                      + (temph*al(k+i,j)
     +                      + templ*ah(k+i,j))
                      t_s1 = ddmh + ddml
                      ddml = ddml - (t_s1 - ddmh)
                      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = temp * a(k+i,j)
!<<<                  y(iy) = y(iy) + ddm
                      t_p1 = yh(iy) + ddmh
                      t_p2 = t_p1 - yh(iy)
                      t_q1 = t_p1
                      t_q2 = (yh(iy) - (t_p1 - t_p2))
     +                         + (ddmh - t_p2)
                      t_q2 = t_q2 + yl(iy) + ddml
                      yh(iy) = t_q1 + t_q2
                      yl(iy) = t_q2 - (yh(iy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  y(iy) = y(iy) + ddm
!!                        IY = IY + INCY
   70                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  JX = JX + INCX
                  IF (J.GT.KU) KY = KY + INCY
   80         CONTINUE
          END IF
      ELSE
*
*        Form  y := alpha*A'*x + y.
*
          JY = KY
          IF (INCX.EQ.1) THEN
              DO 100 J = 1,N
!                 TEMP = ZERO
                  temph = zeroh
                  templ = zerol
                  K = KUP1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 90 I = MAX(1,J-KU),MIN(M,J+KL)
!                     TEMP = TEMP + A(K+I,J)*X(I)
!<<<                  ddm = a(k+i,j) * x(i)
                      ddmh = ah(k+i,j) * xh(i)
                      t_u1 = 134217729.0 * ah(k+i,j)
                      t_p1 = t_u1
     +                         - (t_u1 - ah(k+i,j))
                      t_p2 = ah(k+i,j) - t_p1
                      t_u1 = 134217729.0 * xh(i)
                      t_r1 = t_u1
     +                         - (t_u1 - xh(i))
                      t_r2 = xh(i) - t_r1
                      ddml = ((t_p1*t_r1
     +                        - ddmh)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddml = ddml
     +                      + (ah(k+i,j)*xl(i)
     +                      + al(k+i,j)*xh(i))
                      t_s1 = ddmh + ddml
                      ddml = ddml - (t_s1 - ddmh)
                      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = a(k+i,j) * x(i)
!$omp critical
!<<<                  temp = temp + ddm
                      t_p1 = temph + ddmh
                      t_p2 = t_p1 - temph
                      t_q1 = t_p1
                      t_q2 = (temph - (t_p1 - t_p2))
     +                         + (ddmh - t_p2)
                      t_q2 = t_q2 + templ + ddml
                      temph = t_q1 + t_q2
                      templ = t_q2 - (temph - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  temp = temp + ddm
!$omp end critical
   90             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP
!<<<              ddm = alpha * temp
                  ddmh = alphah * temph
                  t_u1 = 134217729.0 * alphah
                  t_p1 = t_u1
     +                     - (t_u1 - alphah)
                  t_p2 = alphah - t_p1
                  t_u1 = 134217729.0 * temph
                  t_r1 = t_u1
     +                     - (t_u1 - temph)
                  t_r2 = temph - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (alphah*templ
     +                  + alphal*temph)
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = alpha * temp
!<<<              y(jy) = y(jy) + ddm
                  t_p1 = yh(jy) + ddmh
                  t_p2 = t_p1 - yh(jy)
                  t_q1 = t_p1
                  t_q2 = (yh(jy) - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + yl(jy) + ddml
                  yh(jy) = t_q1 + t_q2
                  yl(jy) = t_q2 - (yh(jy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(jy) = y(jy) + ddm
                  JY = JY + INCY
  100         CONTINUE
          ELSE
              DO 120 J = 1,N
!                 TEMP = ZERO
                  temph = zeroh
                  templ = zerol
                  IX = KX
                  K = KUP1 - J
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 110 I = MAX(1,J-KU),MIN(M,J+KL)
                      ix = kx + (i - max(1,j-ku))*incx
!                     TEMP = TEMP + A(K+I,J)*X(IX)
!<<<                  ddm = a(k+i,j) * x(ix)
                      ddmh = ah(k+i,j) * xh(ix)
                      t_u1 = 134217729.0 * ah(k+i,j)
                      t_p1 = t_u1
     +                         - (t_u1 - ah(k+i,j))
                      t_p2 = ah(k+i,j) - t_p1
                      t_u1 = 134217729.0 * xh(ix)
                      t_r1 = t_u1
     +                         - (t_u1 - xh(ix))
                      t_r2 = xh(ix) - t_r1
                      ddml = ((t_p1*t_r1
     +                        - ddmh)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddml = ddml
     +                      + (ah(k+i,j)*xl(ix)
     +                      + al(k+i,j)*xh(ix))
                      t_s1 = ddmh + ddml
                      ddml = ddml - (t_s1 - ddmh)
                      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = a(k+i,j) * x(ix)
!$omp critical
!<<<                  temp = temp + ddm
                      t_p1 = temph + ddmh
                      t_p2 = t_p1 - temph
                      t_q1 = t_p1
                      t_q2 = (temph - (t_p1 - t_p2))
     +                         + (ddmh - t_p2)
                      t_q2 = t_q2 + templ + ddml
                      temph = t_q1 + t_q2
                      templ = t_q2 - (temph - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  temp = temp + ddm
!$omp end critical
!!                    IX = IX + INCX
  110             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP
!<<<            ddm = alpha * temp
                ddmh = alphah * temph
                t_u1 = 134217729.0 * alphah
                t_p1 = t_u1
     +                   - (t_u1 - alphah)
                t_p2 = alphah - t_p1
                t_u1 = 134217729.0 * temph
                t_r1 = t_u1
     +                   - (t_u1 - temph)
                t_r2 = temph - t_r1
                ddml = ((t_p1*t_r1
     +                  - ddmh)
     +                  + t_p1 * t_r2
     +                  + t_p2 * t_r1)
     +                  + t_p2 * t_r2
                ddml = ddml
     +                + (alphah*templ
     +                + alphal*temph)
                t_s1 = ddmh + ddml
                ddml = ddml - (t_s1 - ddmh)
                ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>            ddm = alpha * temp
!<<<            y(jy) = y(jy) + ddm
                t_p1 = yh(jy) + ddmh
                t_p2 = t_p1 - yh(jy)
                t_q1 = t_p1
                t_q2 = (yh(jy) - (t_p1 - t_p2))
     +                   + (ddmh - t_p2)
                t_q2 = t_q2 + yl(jy) + ddml
                yh(jy) = t_q1 + t_q2
                yl(jy) = t_q2 - (yh(jy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>            y(jy) = y(jy) + ddm
                  JY = JY + INCY
                  IF (J.GT.KU) KX = KX + INCX
  120         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddgbmv"
      WRITE(6,'(2A)') "trans    = ",trans
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DGBMV .
*
      END
