      SUBROUTINE DDGEMM(TRANSA,TRANSB,M,N,K,ALPHAH,ALPHAL,AH,AL,LDA,
     *                   BH,BL,LDB,BETAH,BETAL,CH,CL,LDC)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,BETAH
      DOUBLE PRECISION ALPHAL,BETAL
      INTEGER K,LDA,LDB,LDC,M,N
      CHARACTER TRANSA,TRANSB
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),BH(LDB,*),CH(LDC,*)
      DOUBLE PRECISION AL(LDA,*),BL(LDB,*),CL(LDC,*)
*     ..
*
*  Purpose
*  =======
*
*  DDGEMM  performs one of the matrix-matrix operations
*
*     C := alpha*op( A )*op( B ) + beta*C,
*
*  where  op( X ) is one of
*
*     op( X ) = X   or   op( X ) = X^T,
*
*  alpha and beta are scalars, and A, B and C are matrices, with op( A )
*  an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
*
*  Arguments
*  ==========
*
*  TRANSA - CHARACTER*1.
*           On entry, TRANSA specifies the form of op( A ) to be used in
*           the matrix multiplication as follows:
*
*              TRANSA = 'N' or 'n',  op( A ) = A.
*
*              TRANSA = 'T' or 't',  op( A ) = A^T.
*
*              TRANSA = 'C' or 'c',  op( A ) = A^T.
*
*           Unchanged on exit.
*
*  TRANSB - CHARACTER*1.
*           On entry, TRANSB specifies the form of op( B ) to be used in
*           the matrix multiplication as follows:
*
*              TRANSB = 'N' or 'n',  op( B ) = B.
*
*              TRANSB = 'T' or 't',  op( B ) = B^T.
*
*              TRANSB = 'C' or 'c',  op( B ) = B^T.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry,  M  specifies  the number  of rows  of the  matrix
*           op( A )  and of the  matrix  C.  M  must  be at least  zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry,  N  specifies the number  of columns of the matrix
*           op( B ) and the number of columns of the matrix C. N must be
*           at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry,  K  specifies  the number of columns of the matrix
*           op( A ) and the number of rows of the matrix op( B ). K must
*           be at least  zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHAH specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHAL specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANSA = 'N' or 'n',  and is m otherwise.
*           Before entry with TRANSA = 'N' or 'n', the leading m by k
*           part of the array AH must contain the high word of matrix A, 
*           otherwise the leading k by m part of the array AH must contain 
*           the high word of matrix A.
*           Unchanged on exit.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANSA = 'N' or 'n',  and is m otherwise.
*           Before entry with TRANSA = 'N' or 'n', the leading m by k
*           part of the array AL must contain the low word of matrix A,
*           otherwise the leading k by m part of the array AL must contain 
*           the low word of matrix A.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. When  TRANSA = 'N' or 'n' then
*           LDA must be at least  max( 1, m ), otherwise  LDA must be at
*           least  max( 1, k ).
*           Unchanged on exit.
*
*  BH     - DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where kb is
*           n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
*           Before entry with TRANSB = 'N' or 'n',  he leading k by n
*           part of the array BH must contain the high word matrix B,
*           otherwise the leading n by k part ofthe array BH must contain
*           the high word of matrix B.
*           Unchanged on exit.
*  BL     - DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where kb is
*           n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
*           Before entry with TRANSB = 'N' or 'n',  he leading k by n
*           part of the array BL must contain the low word matrix B,
*           otherwise the leading n by k part ofthe array BL must contain
*           the low word of matrix B.
*           Unchanged on exit.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in the calling (sub) program. When  TRANSB = 'N' or 'n' then
*           LDB must be at least  max( 1, k ), otherwise  LDB must be at
*           least  max( 1, n ).
*           Unchanged on exit.
*
*  BETAH  - DOUBLE PRECISION.
*           On entry, BETAH specifies the high word of scalar beta.
*           When BETAH and BETAL are supplied as zero then CH and CL need
*           not be set on input. Unchanged on exit.
*  BETAL  - DOUBLE PRECISION.
*           On entry, BETAL specifies the low word of scalar beta.
*           When BETAH and BETAL are supplied as zero then CH and CL need
*           not be set on input. Unchanged on exit.
*
*  CH     - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry, the leading m by n part of the array CH must
*           contain the high word of matrix C, except when beta is zero,
*           in which case CH and CL need not be set on entry.
*           On exit, the array CH is overwritten by the high word of m by n
*           matrix ( alpha*op( A )*op( B ) + beta*C ).
*  CL     - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry, the leading m by n part of the array CL must
*           contain the low word of matrix C, except when beta is zero,
*           in which case CH and CL need not be set on entry.
*           On exit, the array CL is overwritten by the low word of m by n
*           matrix ( alpha*op( A )*op( B ) + beta*C ).
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*  Further Details
*  ===============
*
*  Level 3 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      DOUBLE PRECISION T1H,T2H
      DOUBLE PRECISION T1L,T2L
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      LOGICAL LALPHA,LBETA,LBETAZ
      INTEGER I,INFO,J,L,NCOLA,NROWA,NROWB
      LOGICAL NOTA,NOTB
*     ..
*     .. Parameters ..
      DOUBLE PRECISION ONE,ZERO
      PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDGEMM"
#endif
*
*     Set  NOTA  and  NOTB  as  true if  A  and  B  respectively are not
*     transposed and set  NROWA, NCOLA and  NROWB  as the number of rows
*     and  columns of  A  and the  number of  rows  of  B  respectively.
*
      NOTA = LSAME(TRANSA,'N')
      NOTB = LSAME(TRANSB,'N')
      IF (NOTA) THEN
          NROWA = M
          NCOLA = K
      ELSE
          NROWA = K
          NCOLA = M
      END IF
      IF (NOTB) THEN
          NROWB = K
      ELSE
          NROWB = N
      END IF
*
*     Test the input parameters.
*
      INFO = 0
      IF ((.NOT.NOTA) .AND. (.NOT.LSAME(TRANSA,'C')) .AND.
     +    (.NOT.LSAME(TRANSA,'T'))) THEN
          INFO = 1
      ELSE IF ((.NOT.NOTB) .AND. (.NOT.LSAME(TRANSB,'C')) .AND.
     +         (.NOT.LSAME(TRANSB,'T'))) THEN
          INFO = 2
      ELSE IF (M.LT.0) THEN
          INFO = 3
      ELSE IF (N.LT.0) THEN
          INFO = 4
      ELSE IF (K.LT.0) THEN
          INFO = 5
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 8
      ELSE IF (LDB.LT.MAX(1,NROWB)) THEN
          INFO = 10
      ELSE IF (LDC.LT.MAX(1,M)) THEN
          INFO = 13
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDGEMM',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
      LALPHA = (ALPHAH.EQ.ZERO) .AND. (ALPHAL.EQ.ZERO)
      LBETA = (BETAH.EQ.ONE) .AND. (BETAL.EQ.ZERO)
      IF ((M.EQ.0) .OR. (N.EQ.0) .OR.
!    +    (((ALPHA.EQ.ZERO).OR. (K.EQ.0)).AND. (BETA.EQ.ONE))) RETURN
     +    ((LALPHA .OR. (K.EQ.0)).AND. LBETA)) RETURN
      LBETAZ = (BETAH.EQ.ZERO) .AND. (BETAL.EQ.ZERO)
*
*     And if  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      IF (LALPHA) THEN
!         IF (BETA.EQ.ZERO) THEN
          IF (LBETAZ) THEN
              DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I)
                  DO 10 I = 1,M
!                     C(I,J) = ZERO
                      CH(I,J) = ZERO
                      CL(I,J) = ZERO
   10             CONTINUE
!$omp end do
!$omp end parallel
   20         CONTINUE
          ELSE
              DO 40 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                  DO 30 I = 1,M
!                     C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   30             CONTINUE
!$omp end do
!$omp end parallel
   40         CONTINUE
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (NOTB) THEN
          IF (NOTA) THEN
*
*           Form  C := alpha*A*B + beta*C.
*
              DO 90 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I)
                      DO 50 I = 1,M
!                         C(I,J) = ZERO
                          CH(I,J) = ZERO
                          CL(I,J) = ZERO
   50                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  ELSE IF (.NOT.LBETA) THEN
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                      DO 60 I = 1,M
!                         C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   60                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  DO 80 L = 1,K
!                     IF (B(L,J).NE.ZERO) THEN
                      IF (BH(L,J).NE.ZERO.OR.BL(L,J).NE.ZERO) THEN
!                         TEMP = ALPHA*B(L,J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,BH(L,J),BL(L,J))
          T1H = ALPHAH * BH(L,J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*BH(L,J)
          r1 = u1 - (u1-BH(L,J))
          r2 = BH(L,J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * BL(L,J) + ALPHAL * BH(L,J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                          DO 70 I = 1,M
!                             C(I,J) = C(I,J) + TEMP*A(I,L)
!         call dd_mul(T2H,T2L,T1H,T1L,AH(I,L),AL(I,L))
          T2H = T1H * AH(I,L)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,L)
          r1 = u1 - (u1-AH(I,L))
          r2 = AH(I,L) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,L) + T1L * AH(I,L))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T2H,T2L)
          p1 = CH(I,J) + T2H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T2L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   70                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
   80             CONTINUE
   90         CONTINUE
          ELSE
*
*           Form  C := alpha*A^T*B + beta*C
*
              DO 120 J = 1,N
                  DO 110 I = 1,M
!                     TEMP = ZERO
                      T1H = ZERO
                      T1L = ZERO
!$omp parallel
!$omp do
!$omp& private(L,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                      DO 100 L = 1,K
!                         TEMP = TEMP + A(L,I)*B(L,J)
!         call dd_mul(T2H,T2L,AH(L,I),AL(L,I),BH(L,J),BL(L,J))
          T2H = AH(L,I) * BH(L,J)
          u1 = 134217729.0*AH(L,I)
          p1 = u1 - (u1-AH(L,I))
          p2 = AH(L,I) - p1
          u1 = 134217729.0*BH(L,J)
          r1 = u1 - (u1-BH(L,J))
          r2 = BH(L,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (AH(L,I) * BL(L,J) + AL(L,I) * BH(L,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T1H,T1L,T2H,T2L)
!$omp critical (name0)
          p1 = T1H + T2H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T2L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  100                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      IF (LBETAZ) THEN
!                         C(I,J) = ALPHA*TEMP
!         call dd_mul(CH(I,J),CL(I,J),ALPHAH,ALPHAL,T1H,T1L)
          CH(I,J) = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          CL(I,J) = ((p1*r1-CH(I,J)) + p1*r2 + p2*r1) + p2*r2
          CL(I,J) = CL(I,J) + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      ELSE
!                         C(I,J) = ALPHA*TEMP + BETA*C(I,J)
!         call dd_mul(T2H,T2L,ALPHAH,ALPHAL,T1H,T1L)
          T2H = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T2H,T2L)
          p1 = CH(I,J) + T2H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T2L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
                      END IF
  110             CONTINUE
  120         CONTINUE
          END IF
      ELSE
          IF (NOTA) THEN
*
*           Form  C := alpha*A*B^T + beta*C
*
              DO 170 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I)
                      DO 130 I = 1,M
!                         C(I,J) = ZERO
                          CH(I,J) = ZERO
                          CL(I,J) = ZERO
  130                 CONTINUE
!$omp end do
!$omp end parallel
 !                ELSE IF (BETA.NE.ONE) THEN
                  ELSE IF (.NOT.LBETA) THEN
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                      DO 140 I = 1,M
!                         C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
  140                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  DO 160 L = 1,K
!                     IF (B(J,L).NE.ZERO) THEN
                      IF (BH(J,L).NE.ZERO.OR.BL(J,L).NE.ZERO) THEN
!                         TEMP = ALPHA*B(J,L)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,BH(J,L),BL(J,L))
          T1H = ALPHAH * BH(J,L)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*BH(J,L)
          r1 = u1 - (u1-BH(J,L))
          r2 = BH(J,L) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * BL(J,L) + ALPHAL * BH(J,L))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                          DO 150 I = 1,M
!                             C(I,J) = C(I,J) + TEMP*A(I,L)
!         call dd_mul(T2H,T2L,T1H,T1L,AH(I,L),AL(I,L))
          T2H = T1H * AH(I,L)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,L)
          r1 = u1 - (u1-AH(I,L))
          r2 = AH(I,L) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,L) + T1L * AH(I,L))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T2H,T2L)
          p1 = CH(I,J) + T2H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T2L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  150                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  160             CONTINUE
  170         CONTINUE
          ELSE
*
*           Form  C := alpha*A^T*B^T + beta*C
*
              DO 200 J = 1,N
                  DO 190 I = 1,M
!                     TEMP = ZERO
                      T1H = ZERO
                      T1L = ZERO
!$omp parallel
!$omp do
!$omp& private(L,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                      DO 180 L = 1,K
!                         TEMP = TEMP + A(L,I)*B(J,L)
!         call dd_mul(T2H,T2L,AH(L,I),AL(L,I),BH(J,L),BL(J,L))
          T2H = AH(L,I) * BH(J,L)
          u1 = 134217729.0*AH(L,I)
          p1 = u1 - (u1-AH(L,I))
          p2 = AH(L,I) - p1
          u1 = 134217729.0*BH(J,L)
          r1 = u1 - (u1-BH(J,L))
          r2 = BH(J,L) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (AH(L,I) * BL(J,L) + AL(L,I) * BH(J,L))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T1H,T1L,T2H,T2L)
!$omp critical (name0)
          p1 = T1H + T2H
          p2 = p1 - T1H
          q2 = (T1H - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + T1L
          q2 = q2 + T2L
          T1H = q1 + q2
          T1L = q2 - (T1H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  180                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      IF (LBETAZ) THEN
!                         C(I,J) = ALPHA*TEMP
!         call dd_mul(CH(I,J),CL(I,J),ALPHAH,ALPHAL,T1H,T1L)
          CH(I,J) = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          CL(I,J) = ((p1*r1-CH(I,J)) + p1*r2 + p2*r1) + p2*r2
          CL(I,J) = CL(I,J) + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      ELSE
!                         C(I,J) = ALPHA*TEMP + BETA*C(I,J)
!         call dd_mul(T2H,T2L,ALPHAH,ALPHAL,T1H,T1L)
          T2H = ALPHAH * T1H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T1H
          r1 = u1 - (u1-T1H)
          r2 = T1H - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (ALPHAH * T1L + ALPHAL * T1H)
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T2H,T2L)
          p1 = CH(I,J) + T2H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T2L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
                      END IF
  190             CONTINUE
  200         CONTINUE
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDGEMM"
      WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
      WRITE(6,'(2A)') "TRANSA    = ",TRANSA
      WRITE(6,'(2A)') "TRANSB    = ",TRANSB
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDGEMM .
*
      END
