      SUBROUTINE DDGEMV(TRANS,M,N,ALPHAH,ALPHAL,AH,AL,LDA,XH,XL,
     *            INCX,BETAH,BETAL,YH,YL,INCY)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,BETAH
      DOUBLE PRECISION ALPHAL,BETAL
      INTEGER INCX,INCY,LDA,M,N
      CHARACTER TRANS
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),XH(*),YH(*)
      DOUBLE PRECISION AL(LDA,*),XL(*),YL(*)
*     ..
*
*  Purpose
*  =======
*
*  DDGEMV  performs one of the matrix-vector operations
*
*     y := alpha*A*x + beta*y,   or   y := alpha*A^T*x + beta*y,
*
*  where alpha and beta are scalars, x and y are vectors and A is an
*  m by n matrix.
*
*  Arguments
*  ==========
*
*  TRANS  - CHARACTER*1.
*           On entry, TRANS specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'   y := alpha*A*x + beta*y.
*
*              TRANS = 'T' or 't'   y := alpha*A^T*x + beta*y.
*
*              TRANS = 'C' or 'c'   y := alpha*A^T*x + beta*y.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of the matrix A.
*           M must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHAH specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHAL specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry, the leading m by n part of the array AH must
*           contain the high word of matrix of coefficients.
*           Unchanged on exit.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry, the leading m by n part of the array AL must
*           contain the low word of matrix of coefficients.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, m ).
*           Unchanged on exit.
*
*  XH     - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCX ) ) when TRANS = 'N' or 'n'
*           and at least
*           ( 1 + ( m - 1 )*abs( INCX ) ) otherwise.
*           Before entry, the incremented array XH must contain the
*           high word of vector x.
*           Unchanged on exit.
*  XL     - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCX ) ) when TRANS = 'N' or 'n'
*           and at least
*           ( 1 + ( m - 1 )*abs( INCX ) ) otherwise.
*           Before entry, the incremented array XL must contain the
*           low word of vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  BETAH  - DOUBLE PRECISION.
*           On entry, BETAH specifies the high word of scalar beta.
*           When BETAH and BETAL are supplied as zero then YH and YL need  
*           not be set on input. Unchanged on exit.
*  BETAL  - DOUBLE PRECISION.
*           On entry, BETAL specifies the low word of scalar beta.
*           When BETAH and BETAL are supplied as zero then YH and YL need  
*           not be set on input. Unchanged on exit.
*
*  YH     - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n'
*           and at least
*           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise.
*           Before entry with BETAH and BETAL non-zero, the incremented 
*           array YH must contain the high word of vector y. On exit, YH 
*           is overwritten by the updated vector y.
*  YL     - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n'
*           and at least
*           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise.
*           Before entry with BETAH and BETAL non-zero, the incremented 
*           array YL must contain the low word of vector y. On exit, YL 
*           is overwritten by the updated vector y.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  Level 2 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION ONE,ZERO
      PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      DOUBLE PRECISION T1H,T1L,T2H,T2L
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      LOGICAL LALPHA,LBETA,LBETAZ
      INTEGER I,INFO,IX,IY,J,JX,JY,KX,KY,LENX,LENY
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDGEMV"
#endif
*
*     Test the input parameters.
*
      INFO = 0
      IF (.NOT.LSAME(TRANS,'N') .AND. .NOT.LSAME(TRANS,'T') .AND.
     +    .NOT.LSAME(TRANS,'C')) THEN
          INFO = 1
      ELSE IF (M.LT.0) THEN
          INFO = 2
      ELSE IF (N.LT.0) THEN
          INFO = 3
      ELSE IF (LDA.LT.MAX(1,M)) THEN
          INFO = 6
      ELSE IF (INCX.EQ.0) THEN
          INFO = 8
      ELSE IF (INCY.EQ.0) THEN
          INFO = 11
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDGEMV',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
      LALPHA = (ALPHAH.EQ.ZERO) .AND. (ALPHAL.EQ.ZERO)
      LBETA = (BETAH.EQ.ONE) .AND. (BETAL.EQ.ZERO)
      IF ((M.EQ.0) .OR. (N.EQ.0) .OR.
!    +    ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
     +    (LALPHA .AND. LBETA)) RETURN
      LBETAZ = (BETAH.EQ.ZERO) .AND. (BETAL.EQ.ZERO)
*
*     Set  LENX  and  LENY, the lengths of the vectors x and y, and set
*     up the start points in  X  and  Y.
*
      IF (LSAME(TRANS,'N')) THEN
          LENX = N
          LENY = M
      ELSE
          LENX = M
          LENY = N
      END IF
      IF (INCX.GT.0) THEN
          KX = 1
      ELSE
          KX = 1 - (LENX-1)*INCX
      END IF
      IF (INCY.GT.0) THEN
          KY = 1
      ELSE
          KY = 1 - (LENY-1)*INCY
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
*     First form  y := beta*y.
*
!     IF (BETA.NE.ONE) THEN
      IF (.NOT.LBETA) THEN
          IF (INCY.EQ.1) THEN
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I)
                  DO 10 I = 1,LENY
!                     Y(I) = ZERO
                      YH(I) = ZERO
                      YL(I) = ZERO
   10             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                  DO 20 I = 1,LENY
!                     Y(I) = BETA*Y(I)
!         call dd_selfmul(YH(I),YL(I),BETAH,BETAL)
          q1 = YH(I)*BETAH
          u1 = 134217729.0*YH(I)
          p1 = u1 - (u1-YH(I))
          p2 = YH(I) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * YH(I)
          q2 = q2 + BETAH * YL(I)
          YH(I) = q1 + q2
          YL(I) = q2 - (YH(I) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   20             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          ELSE
!              IY = KY
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I,IY)
                  DO 30 I = 1,LENY
                  IY = KY + INCY*(I-1)
!                     Y(IY) = ZERO
                      YH(IY) = ZERO
                      YL(IY) = ZERO
!                      IY = IY + INCY
   30             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(IY)
                  DO 40 I = 1,LENY
                  IY = KY + INCY*(I-1)
!                     Y(IY) = BETA*Y(IY)
!         call dd_selfmul(YH(IY),YL(IY),BETAH,BETAL)
          q1 = YH(IY)*BETAH
          u1 = 134217729.0*YH(IY)
          p1 = u1 - (u1-YH(IY))
          p2 = YH(IY) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * YH(IY)
          q2 = q2 + BETAH * YL(IY)
          YH(IY) = q1 + q2
          YL(IY) = q2 - (YH(IY) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!                      IY = IY + INCY
   40             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          END IF
      END IF
!     IF (ALPHA.EQ.ZERO) RETURN
      IF (LALPHA) RETURN
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  y := alpha*A*x + y.
*
!          JX = KX
          IF (INCY.EQ.1) THEN
              DO 60 J = 1,N
                  JX = KX + INCX*(J-1)
!                 IF (X(JX).NE.ZERO) THEN
                  IF (XH(JX).NE.ZERO.OR.XL(JX).NE.ZERO) THEN
!                     TEMP = ALPHA*X(JX)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,XH(JX),XL(JX))
          T1H = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
                      DO 50 I = 1,M
!                         Y(I) = Y(I) + TEMP*A(I,J)
!         call dd_mul(T2H,T2L,T1H,T1L,AH(I,J),AL(I,J))
          T2H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(I),YL(I),T2H,T2L)
          p1 = YH(I) + T2H
          p2 = p1 - YH(I)
          q2 = (YH(I) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + YL(I)
          q2 = q2 + T2L
          YH(I) = q1 + q2
          YL(I) = q2 - (YH(I) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
!                  JX = JX + INCX
   60         CONTINUE
          ELSE
              DO 80 J = 1,N
                  JX = KX + INCX*(J-1)
!                 IF (X(JX).NE.ZERO) THEN
                  IF (XH(JX).NE.ZERO.OR.XL(JX).NE.ZERO) THEN
!                     TEMP = ALPHA*X(JX)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,XH(JX),XL(JX))
          T1H = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                      IY = KY
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T2H,T2L)
!$omp& private(IY)
                      DO 70 I = 1,M
                          IY = KY + INCY*(I-1)
!                         Y(IY) = Y(IY) + TEMP*A(I,J)
!         call dd_mul(T2H,T2L,T1H,T1L,AH(I,J),AL(I,J))
          T2H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T2L = ((p1*r1-T2H) + p1*r2 + p2*r1) + p2*r2
          T2L = T2L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T2H + T2L
          T2L = T2L - (s1 - T2H)
          T2H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(IY),YL(IY),T2H,T2L)
          p1 = YH(IY) + T2H
          p2 = p1 - YH(IY)
          q2 = (YH(IY) - (p1 - p2)) + (T2H - p2)
          q1 = p1
          q2 = q2 + YL(IY)
          q2 = q2 + T2L
          YH(IY) = q1 + q2
          YL(IY) = q2 - (YH(IY) - q1)

#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                          IY = IY + INCY
   70                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
!                  JX = JX + INCX
   80         CONTINUE
          END IF
      ELSE
*
*        Form  y := alpha*A^T*x + y.
*
!          JY = KY
          IF (INCX.EQ.1) THEN
              DO 100 J = 1,N
                  JY = KY + INCY*(J-1)
!                 TEMP = ZERO
                  T2H = ZERO
                  T2L = ZERO
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T1H,T1L)
!$omp& private(IY)
                  DO 90 I = 1,M
!                     TEMP = TEMP + A(I,J)*X(I)
!         call dd_mul(T1H,T1L,AH(I,J),AL(I,J),XH(I),XL(I))
          T1H = AH(I,J) * XH(I)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(I)
          r1 = u1 - (u1-XH(I))
          r2 = XH(I) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (AH(I,J) * XL(I) + AL(I,J) * XH(I))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T1H,T1L)
!$omp critical (name0)
          p1 = T2H + T1H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T1H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T1L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif 
   90             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,T2H,T2L)
          T1H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(JY),YL(JY),T1H,T1L)
          p1 = YH(JY) + T1H
          p2 = p1 - YH(JY)
          q2 = (YH(JY) - (p1 - p2)) + (T1H - p2)
          q1 = p1
          q2 = q2 + YL(JY)
          q2 = q2 + T1L
          YH(JY) = q1 + q2
          YL(JY) = q2 - (YH(JY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif 
!                  JY = JY + INCY
  100         CONTINUE
          ELSE
              DO 120 J = 1,N
                  JY = KY + INCY*(J-1)
!                 TEMP = ZERO
                  T2H = ZERO
                  T2L = ZERO
                  IX = KX
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T1H,T1L)
!$omp& private(IX)
                  DO 110 I = 1,M
                      IX = KX + INCX*(I-1)
!                     TEMP = TEMP + A(I,J)*X(IX)
!         call dd_mul(T1H,T1L,AH(I,J),AL(I,J),XH(IX),XL(IX))
          T1H = AH(I,J) * XH(IX)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(IX)
          r1 = u1 - (u1-XH(IX))
          r2 = XH(IX) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (AH(I,J) * XL(IX) + AL(I,J) * XH(IX))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T1H,T1L)
!$omp critical (name0)
          p1 = T2H + T1H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T1H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T1L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                      IX = IX + INCX
  110             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,T2H,T2L)
          T1H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(JY),YL(JY),T1H,T1L)
          p1 = YH(JY) + T1H
          p2 = p1 - YH(JY)
          q2 = (YH(JY) - (p1 - p2)) + (T1H - p2)
          q1 = p1
          q2 = q2 + YL(JY)
          q2 = q2 + T1L
          YH(JY) = q1 + q2
          YL(JY) = q2 - (YH(JY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                  JY = JY + INCY
  120         CONTINUE
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDGEMV"
      WRITE(6,'(2A)') "TRANS     = ",TRANS
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDGEMV .
*
      END
