      SUBROUTINE DDSYMM(SIDE,UPLO,M,N,ALPHAH,ALPHAL,AH,AL,LDA,
     *                   BH,BL,LDB,BETAH,BETAL,CH,CL,LDC)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,BETAH
      DOUBLE PRECISION ALPHAL,BETAL
      INTEGER LDA,LDB,LDC,M,N
      CHARACTER SIDE,UPLO
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),BH(LDB,*),CH(LDC,*)
      DOUBLE PRECISION AL(LDA,*),BL(LDB,*),CL(LDC,*)
*     ..
*
*  Purpose
*  =======
*
*  DDSYMM  performs one of the matrix-matrix operations
*
*     C := alpha*A*B + beta*C,
*
*  or
*
*     C := alpha*B*A + beta*C,
*
*  where alpha and beta are scalars,  A is a symmetric matrix and  B and
*  C are  m by n matrices.
*
*  Arguments
*  ==========
*
*  SIDE   - CHARACTER*1.
*           On entry,  SIDE  specifies whether  the  symmetric matrix  A
*           appears on the  left or right  in the  operation as follows:
*
*              SIDE = 'L' or 'l'   C := alpha*A*B + beta*C,
*
*              SIDE = 'R' or 'r'   C := alpha*B*A + beta*C,
*
*           Unchanged on exit.
*
*  UPLO   - CHARACTER*1.
*           On  entry,   UPLO  specifies  whether  the  upper  or  lower
*           triangular  part  of  the  symmetric  matrix   A  is  to  be
*           referenced as follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of the
*                                  symmetric matrix is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of the
*                                  symmetric matrix is to be referenced.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry,  M  specifies the number of rows of the matrix  C.
*           M  must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix C.
*           N  must be at least zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHAH specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHAL specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           m  when  SIDE = 'L' or 'l'  and is  n otherwise.
*           Before entry  with  SIDE = 'L' or 'l',  the  m by m  part of
*           the array AH must contain the high word of symmetric matrix,
*           such that when  UPLO = 'U' or 'u', the leading m by m upper 
*           triangular part of the array AH must contain the high word of 
*           upper triangular part of the  symmetric matrix and the  strictly
*           lower triangular part of AH is not referenced, and 
*           when  UPLO = 'L' or 'l',the leading  m by m  lower triangular 
*           part of the array AH must  contain  the low word of lower
*           triangular part of the  symmetric matrix and the  strictly 
*           upper triangular  part of AH is not referenced.
*           Before entry  with  SIDE = 'R' or 'r',  the  n by n  part of
*           the array AH must contain the high word of  symmetric matrix,
*           such that when  UPLO = 'U' or 'u', the leading n by n upper
*           triangular part of the array AH must contain the high word of
*           upper triangular part of the symmetric matrix and the strictly
*           lower triangular part of AH is not referenced, and when
*           UPLO = 'L' or 'l',the leading n by n lower triangular part of the
*           array AH must contain the low word of lower triangular part of
*           the  symmetric matrix and the strictly upper triangular part of AH
*           is not referenced.
*           Unchanged on exit.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           m  when  SIDE = 'L' or 'l'  and is  n otherwise.
*           Before entry  with  SIDE = 'L' or 'l',  the  m by m  part of
*           the array AL must contain the low word of symmetric matrix,
*           such that when  UPLO = 'U' or 'u', the leading m by m upper 
*           triangular part of the array AL must contain the low word of 
*           upper triangular part of the  symmetric matrix and the  strictly
*           lower triangular part of AL is not referenced, and 
*           when  UPLO = 'L' or 'l',the leading  m by m  lower triangular 
*           part of the array AL must  contain  the low word of lower
*           triangular part of the  symmetric matrix and the  strictly 
*           upper triangular  part of AL is not referenced.
*           Before entry  with  SIDE = 'R' or 'r',  the  n by n  part of
*           the array AL must contain the low word of  symmetric matrix,
*           such that when  UPLO = 'U' or 'u', the leading n by n upper
*           triangular part of the array AL must contain the low word of
*           upper triangular part of the symmetric matrix and the strictly
*           lower triangular part of AL is not referenced, and when
*           UPLO = 'L' or 'l',the leading n by n lower triangular part of the
*           array AL must contain the low word of lower triangular part of
*           the  symmetric matrix and the strictly upper triangular part of AL
*           is not referenced.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
*           LDA must be at least  max( 1, m ), otherwise  LDA must be at
*           least  max( 1, n ).
*           Unchanged on exit.
*
*  BH     - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
*           Before entry, the leading m by n part of the array BH must
*           contain the high word of matrix B.
*           Unchanged on exit.
*  BL     - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
*           Before entry, the leading m by n part of the array BL must
*           contain the low word of matrix B.
*           Unchanged on exit.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in  the  calling  (sub)  program.   LDB  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*  BETAH  - DOUBLE PRECISION.
*           On entry, BETAH specifies the high word scalar beta. When BETAH
*           and BETAL are supplied as zeros, then CH and CL need not be set 
*           on input.
*           Unchanged on exit.
*  BETAL  - DOUBLE PRECISION.
*           On entry, BETAL specifies the low word scalar beta. When BETAH
*           and BETAL are supplied as zeros, then CH and CL need not be set 
*           on input.
*           Unchanged on exit.
*
*  CH     - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry, the leading m by n part of the array CH must
*           contain the high word of matrix C, except when beta is zero,
*           in which case CH and CL need not be set on entry.
*           On exit, the array CH is overwritten by the high word of 
*           m by n updated matrix.
*  CL     - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry, the leading m by n part of the array CL must
*           contain the low word of matrix C, except when beta is zero,
*           in which case CH and CL need not be set on entry.
*           On exit, the array CL is overwritten by the low word of 
*           m by n updated matrix.
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*  Further Details
*  ===============
*
*  Level 3 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP1,TEMP2
      DOUBLE PRECISION T1H,T2H,T3H,T4H
      DOUBLE PRECISION T1L,T2L,T3L,T4L
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      LOGICAL LALPHA,LBETA,LBETAZ
      INTEGER I,INFO,J,K,NROWA
      LOGICAL UPPER
*     ..
*     .. Parameters ..
      DOUBLE PRECISION ONE,ZERO
      PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDSYMM"
#endif
*
*     Set NROWA as the number of rows of A.
*
      IF (LSAME(SIDE,'L')) THEN
          NROWA = M
      ELSE
          NROWA = N
      END IF
      UPPER = LSAME(UPLO,'U')
*
*     Test the input parameters.
*
      INFO = 0
      IF ((.NOT.LSAME(SIDE,'L')) .AND. (.NOT.LSAME(SIDE,'R'))) THEN
          INFO = 1
      ELSE IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 2
      ELSE IF (M.LT.0) THEN
          INFO = 3
      ELSE IF (N.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 7
      ELSE IF (LDB.LT.MAX(1,M)) THEN
          INFO = 9
      ELSE IF (LDC.LT.MAX(1,M)) THEN
          INFO = 12
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDSYMM',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
      LALPHA = (ALPHAH.EQ.ZERO) .AND. (ALPHAL.EQ.ZERO)
      LBETA = (BETAH.EQ.ONE) .AND. (BETAL.EQ.ZERO)
      IF ((M.EQ.0) .OR. (N.EQ.0) .OR.
!    +    ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
     +    (LALPHA.AND.LBETA)) RETURN
      LBETAZ = (BETAH.EQ.ZERO) .AND. (BETAL.EQ.ZERO)
*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      IF (LALPHA) THEN
!         IF (BETA.EQ.ZERO) THEN
          IF (LBETAZ) THEN
              DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I)
                  DO 10 I = 1,M
!                     C(I,J) = ZERO
                      CH(I,J) = ZERO
                      CL(I,J) = ZERO
   10             CONTINUE
!$omp end do
!$omp end parallel
   20         CONTINUE
          ELSE
              DO 40 J = 1,N
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                  DO 30 I = 1,M
!                     C(I,J) = BETA*C(I,J)
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   30             CONTINUE
!$omp end do
!$omp end parallel
   40         CONTINUE
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (LSAME(SIDE,'L')) THEN
*
*        Form  C := alpha*A*B + beta*C.
*
          IF (UPPER) THEN
              DO 70 J = 1,N
                  DO 60 I = 1,M
!                     TEMP1 = ALPHA*B(I,J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,BH(I,J),BL(I,J))
          T1H = ALPHAH * BH(I,J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*BH(I,J)
          r1 = u1 - (u1-BH(I,J))
          r2 = BH(I,J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * BL(I,J) + ALPHAL * BH(I,J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     TEMP2 = ZERO
                      T2H = ZERO
                      T2L = ZERO
!$omp parallel
!$omp do
!$omp& private(K,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                      DO 50 K = 1,I - 1
!                         C(K,J) = C(K,J) + TEMP1*A(K,I)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(K,I),AL(K,I))
          T3H = T1H * AH(K,I)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(K,I)
          r1 = u1 - (u1-AH(K,I))
          r2 = AH(K,I) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(K,I) + T1L * AH(K,I))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(CH(K,J),CL(K,J),T3H,T3L)
          p1 = CH(K,J) + T3H
          p2 = p1 - CH(K,J)
          q2 = (CH(K,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(K,J)
          q2 = q2 + T3L
          CH(K,J) = q1 + q2
          CL(K,J) = q2 - (CH(K,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         TEMP2 = TEMP2 + B(K,J)*A(K,I)
!         call dd_mul(T4H,T4L,BH(K,J),BL(K,J),AH(K,I),AL(K,I))
          T4H = BH(K,J) * AH(K,I)
          u1 = 134217729.0*BH(K,J)
          p1 = u1 - (u1-BH(K,J))
          p2 = BH(K,J) - p1
          u1 = 134217729.0*AH(K,I)
          r1 = u1 - (u1-AH(K,I))
          r2 = AH(K,I) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (BH(K,J) * AL(K,I) + BL(K,J) * AH(K,I))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      IF (LBETAZ) THEN
!                         C(I,J) = TEMP1*A(I,I) + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,I),AL(I,I))
          T3H = T1H * AH(I,I)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,I)
          r1 = u1 - (u1-AH(I,I))
          r2 = AH(I,I) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,I) + T1L * AH(I,I))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_add(CH(I,J),CL(I,J),T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          CL(I,J) = (T3H - (p1 - p2)) + (T4H - p2)
          CH(I,J) = p1
          CL(I,J) = CL(I,J) + T3L + T4L
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
                      ELSE
!                         C(I,J) = BETA*C(I,J) + TEMP1*A(I,I) +
!    +                             ALPHA*TEMP2
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,I),AL(I,I))
          T3H = T1H * AH(I,I)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,I)
          r1 = u1 - (u1-AH(I,I))
          r2 = AH(I,I) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,I) + T1L * AH(I,I))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
                      END IF
   60             CONTINUE
   70         CONTINUE
          ELSE
              DO 100 J = 1,N
                  DO 90 I = M,1,-1
!                     TEMP1 = ALPHA*B(I,J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,BH(I,J),BL(I,J))
          T1H = ALPHAH * BH(I,J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*BH(I,J)
          r1 = u1 - (u1-BH(I,J))
          r2 = BH(I,J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * BL(I,J) + ALPHAL * BH(I,J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                     TEMP2 = ZERO
                      T2H = ZERO
                      T2L = ZERO
!$omp parallel
!$omp do
!$omp& private(K,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                      DO 80 K = I + 1,M
!                         C(K,J) = C(K,J) + TEMP1*A(K,I)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(K,I),AL(K,I))
          T3H = T1H * AH(K,I)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(K,I)
          r1 = u1 - (u1-AH(K,I))
          r2 = AH(K,I) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(K,I) + T1L * AH(K,I))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(CH(K,J),CL(K,J),T3H,T3L)
          p1 = CH(K,J) + T3H
          p2 = p1 - CH(K,J)
          q2 = (CH(K,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(K,J)
          q2 = q2 + T3L
          CH(K,J) = q1 + q2
          CL(K,J) = q2 - (CH(K,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                         TEMP2 = TEMP2 + B(K,J)*A(K,I)
!         call dd_mul(T4H,T4L,BH(K,J),BL(K,J),AH(K,I),AL(K,I))
          T4H = BH(K,J) * AH(K,I)
          u1 = 134217729.0*BH(K,J)
          p1 = u1 - (u1-BH(K,J))
          p2 = BH(K,J) - p1
          u1 = 134217729.0*AH(K,I)
          r1 = u1 - (u1-AH(K,I))
          r2 = AH(K,I) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (BH(K,J) * AL(K,I) + BL(K,J) * AH(K,I))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   80                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      IF (LBETAZ) THEN
!                         C(I,J) = TEMP1*A(I,I) + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,I),AL(I,I))
          T3H = T1H * AH(I,I)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,I)
          r1 = u1 - (u1-AH(I,I))
          r2 = AH(I,I) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,I) + T1L * AH(I,I))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_add(CH(I,J),CL(I,J),T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          CL(I,J) = (T3H - (p1 - p2)) + (T4H - p2)
          CH(I,J) = p1
          CL(I,J) = CL(I,J) + T3L + T4L
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
                      ELSE
!                         C(I,J) = BETA*C(I,J) + TEMP1*A(I,I) +
!    +                             ALPHA*TEMP2
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,I),AL(I,I))
          T3H = T1H * AH(I,I)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,I)
          r1 = u1 - (u1-AH(I,I))
          r2 = AH(I,I) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,I) + T1L * AH(I,I))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
                      END IF
   90             CONTINUE
  100         CONTINUE
          END IF
      ELSE
*
*        Form  C := alpha*B*A + beta*C.
*
          DO 170 J = 1,N
!             TEMP1 = ALPHA*A(J,J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,AH(J,J),AL(J,J))
          T1H = ALPHAH * AH(J,J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * AL(J,J) + ALPHAL * AH(J,J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                  DO 110 I = 1,M
!                     C(I,J) = TEMP1*B(I,J)
!         call dd_mul(CH(I,J),CL(I,J),T1H,T1L,BH(I,J),BL(I,J))
          CH(I,J) = T1H * BH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*BH(I,J)
          r1 = u1 - (u1-BH(I,J))
          r2 = BH(I,J) - r1
          CL(I,J) = ((p1*r1-CH(I,J)) + p1*r2 + p2*r1) + p2*r2
          CL(I,J) = CL(I,J) + (T1H * BL(I,J) + T1L * BH(I,J))
          s1 = CH(I,J) + CL(I,J)
          CL(I,J) = CL(I,J) - (s1 - CH(I,J))
          CH(I,J) = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
  110             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3L,T3H)
                  DO 120 I = 1,M
!                     C(I,J) = BETA*C(I,J) + TEMP1*B(I,J)
!         call dd_mul(T3H,T3L,T1H,T1L,BH(I,J),BL(I,J))
          T3H = T1H * BH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*BH(I,J)
          r1 = u1 - (u1-BH(I,J))
          r2 = BH(I,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * BL(I,J) + T1L * BH(I,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfmul(CH(I,J),CL(I,J),BETAH,BETAL)
          q1 = CH(I,J)*BETAH
          u1 = 134217729.0*CH(I,J)
          p1 = u1 - (u1-CH(I,J))
          p2 = CH(I,J) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * CH(I,J)
          q2 = q2 + BETAH * CL(I,J)
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  120             CONTINUE
!$omp end do
!$omp end parallel
              END IF
              DO 140 K = 1,J - 1
                  IF (UPPER) THEN
!                     TEMP1 = ALPHA*A(K,J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,AH(K,J),AL(K,J))
          T1H = ALPHAH * AH(K,J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(K,J)
          r1 = u1 - (u1-AH(K,J))
          r2 = AH(K,J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * AL(K,J) + ALPHAL * AH(K,J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                  ELSE
!                     TEMP1 = ALPHA*A(J,K)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,AH(J,K),AL(J,K))
          T1H = ALPHAH * AH(J,K)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(J,K)
          r1 = u1 - (u1-AH(J,K))
          r2 = AH(J,K) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * AL(J,K) + ALPHAL * AH(J,K))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                  END IF
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3L,T3H)
                  DO 130 I = 1,M
!                     C(I,J) = C(I,J) + TEMP1*B(I,K)
!         call dd_mul(T3H,T3L,T1H,T1L,BH(I,K),BL(I,K))
          T3H = T1H * BH(I,K)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*BH(I,K)
          r1 = u1 - (u1-BH(I,K))
          r2 = BH(I,K) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * BL(I,K) + T1L * BH(I,K))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  130             CONTINUE
!$omp end do
!$omp end parallel
  140         CONTINUE
              DO 160 K = J + 1,N
                  IF (UPPER) THEN
!                     TEMP1 = ALPHA*A(J,K)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,AH(J,K),AL(J,K))
          T1H = ALPHAH * AH(J,K)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(J,K)
          r1 = u1 - (u1-AH(J,K))
          r2 = AH(J,K) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * AL(J,K) + ALPHAL * AH(J,K))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                  ELSE
!                     TEMP1 = ALPHA*A(K,J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,AH(K,J),AL(K,J))
          T1H = ALPHAH * AH(K,J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*AH(K,J)
          r1 = u1 - (u1-AH(K,J))
          r2 = AH(K,J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * AL(K,J) + ALPHAL * AH(K,J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                  END IF
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3L,T3H)
                  DO 150 I = 1,M
!                     C(I,J) = C(I,J) + TEMP1*B(I,K)
!         call dd_mul(T3H,T3L,T1H,T1L,BH(I,K),BL(I,K))
          T3H = T1H * BH(I,K)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*BH(I,K)
          r1 = u1 - (u1-BH(I,K))
          r2 = BH(I,K) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * BL(I,K) + T1L * BH(I,K))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(CH(I,J),CL(I,J),T3H,T3L)
          p1 = CH(I,J) + T3H
          p2 = p1 - CH(I,J)
          q2 = (CH(I,J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + CL(I,J)
          q2 = q2 + T3L
          CH(I,J) = q1 + q2
          CL(I,J) = q2 - (CH(I,J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  150             CONTINUE
!$omp end do
!$omp end parallel
  160         CONTINUE
  170     CONTINUE
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDSYMM"
      WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,'(2A)') "SIDE      = ",SIDE
      WRITE(6,'(2A)') "UPLO      = ",UPLO
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDSYMM .
*
      END
