      SUBROUTINE DDSYMV(UPLO,N,ALPHAH,ALPHAL,AH,AL,LDA,XH,XL,INCX,
     *            BETAH,BETAL,YH,YL,INCY)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,ALPHAL,BETAH,BETAL
      INTEGER INCX,INCY,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),XH(*),YH(*)
      DOUBLE PRECISION AL(LDA,*),XL(*),YL(*)
*     ..
*
*  Purpose
*  =======
*
*  DDSYMV  performs the matrix-vector  operation
*
*     y := alpha*A*x + beta*y,
*
*  where alpha and beta are scalars, x and y are n element vectors and
*  A is an n by n symmetric matrix.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the array A is to be referenced as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of A
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of A
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHAH specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHAL specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AH must contain the upper
*           triangular part of the high word of symmetric matrix and 
*           the strictly lower triangular part of AH is not referenced.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AH must contain the lower
*           triangular part of the high word of symmetric matrix and
*           the strictly upper triangular part of AH is not referenced.
*           Unchanged on exit.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AL must contain the upper
*           triangular part of the low word of symmetric matrix and 
*           the strictly lower triangular part of AL is not referenced.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AL must contain the lower
*           triangular part of the low word of symmetric matrix and
*           the strictly upper triangular part of AL is not referenced.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, n ).
*           Unchanged on exit.
*
*  XH     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XH must contain the n
*           high word of element vector y. On exit, XH 
*           is overwritten by the high word of updated vector x.
*  XL     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XL must contain the n
*           low word of element vector y. On exit, XL 
*           is overwritten by the low word of updated vector x.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  BETAH  - DOUBLE PRECISION.
*           On entry, BETAH specifies the high word of scalar beta. 
*           Unchanged on exit.
*  BETAL  - DOUBLE PRECISION.
*           On entry, BETAL specifies the low word of scalar beta.
*           Unchanged on exit.
*           When BETAH and BETAL are supplied as zero then YH and YL
*           need not be set on input.
*
*  YH     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array YH must contain the n
*           high word of element vector y. On exit, YH 
*           is overwritten by the high word of updated vector y.
*  YL     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array YL must contain the n
*           low word of element vector y. On exit, YL 
*           is overwritten by the low word of updated vector y.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  Level 2 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION ONE,ZERO
      PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP1,TEMP2
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      DOUBLE PRECISION T1H,T2H,T3H,T4H
      DOUBLE PRECISION T1L,T2L,T3L,T4L
      LOGICAL LALPHA,LBETA,LBETAZ
      INTEGER I,INFO,IX,IY,J,JX,JY,KX,KY
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDSYMV"
#endif
*
*     Test the input parameters.
*
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (LDA.LT.MAX(1,N)) THEN
          INFO = 5
      ELSE IF (INCX.EQ.0) THEN
          INFO = 7
      ELSE IF (INCY.EQ.0) THEN
          INFO = 10
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDSYMV',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
      LALPHA = (ALPHAH.EQ.ZERO) .AND. (ALPHAL.EQ.ZERO)
      LBETA = (BETAH.EQ.ONE) .AND. (BETAL.EQ.ZERO)
      IF ((N.EQ.0) .OR. (LALPHA .AND. LBETA)) RETURN
      LBETAZ = (BETAH.EQ.ZERO) .AND. (BETAL.EQ.ZERO)
*
*     Set up the start points in  X  and  Y.
*
      IF (INCX.GT.0) THEN
          KX = 1
      ELSE
          KX = 1 - (N-1)*INCX
      END IF
      IF (INCY.GT.0) THEN
          KY = 1
      ELSE
          KY = 1 - (N-1)*INCY
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the triangular part
*     of A.
*
*     First form  y := beta*y.
*
!     IF (BETA.NE.ONE) THEN
      IF (.NOT.LBETA) THEN
          IF (INCY.EQ.1) THEN
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I)
                  DO 10 I = 1,N
!                     Y(I) = ZERO
                      YH(I) = ZERO
                      YL(I) = ZERO
   10             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
                  DO 20 I = 1,N
!                     Y(I) = BETA*Y(I)
!         call dd_selfmul(YH(I),YL(I),BETAH,BETAL)
          q1 = YH(I)*BETAH
          u1 = 134217729*YH(I)
          p1 = u1 - (u1-YH(I))
          p2 = YH(I) - p1
          u1 = 134217729*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * YH(I)
          q2 = q2 + BETAH * YL(I)
          YH(I) = q1 + q2
          YL(I) = q2 - (YH(I) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   20             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          ELSE
!              IY = KY
!             IF (BETA.EQ.ZERO) THEN
              IF (LBETAZ) THEN
!$omp parallel
!$omp do
!$omp& private(I,IY)
                  DO 30 I = 1,N
                  IY = KY + INCY*(I-1)
!                     Y(IY) = ZERO
                      YH(IY) = ZERO
                      YL(IY) = ZERO
!                      IY = IY + INCY
   30             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(IY)
                  DO 40 I = 1,N
                  IY = KY + INCY*(I-1)
!                     Y(IY) = BETA*Y(IY)
!         call dd_selfmul(YH(IY),YL(IY),BETAH,BETAL)
          q1 = YH(IY)*BETAH
          u1 = 134217729.0*YH(IY)
          p1 = u1 - (u1-YH(IY))
          p2 = YH(IY) - p1
          u1 = 134217729.0*BETAH
          r1 = u1 - (u1-BETAH)
          r2 = BETAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + BETAL * YH(IY)
          q2 = q2 + BETAH * YL(IY)
          YH(IY) = q1 + q2
          YL(IY) = q2 - (YH(IY) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!                      IY = IY + INCY
   40             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          END IF
      END IF
!     IF (ALPHA.EQ.ZERO) RETURN
      IF (LALPHA) RETURN
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  y  when A is stored in upper triangle.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 60 J = 1,N
!                 TEMP1 = ALPHA*X(J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,XH(J),XL(J))
          T1H = ALPHAH * XH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(J)
          r1 = u1 - (u1-XH(J))
          r2 = XH(J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * XL(J) + ALPHAL * XH(J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                 TEMP2 = ZERO
                  T2H = ZERO
                  T2L = ZERO
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                  DO 50 I = 1,J - 1
!                     Y(I) = Y(I) + TEMP1*A(I,J)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,J),AL(I,J))
          T3H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(I),YL(I),T3H,T3L)
          p1 = YH(I) + T3H
          p2 = p1 - YH(I)
          q2 = (YH(I) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(I)
          q2 = q2 + T3L
          YH(I) = q1 + q2
          YL(I) = q2 - (YH(I) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                     TEMP2 = TEMP2 + A(I,J)*X(I)
!         call dd_mul(T4H,T4L,AH(I,J),AL(I,J),XH(I),XL(I))
          T4H = AH(I,J) * XH(I)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(I)
          r1 = u1 - (u1-XH(I))
          r2 = XH(I) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (AH(I,J) * XL(I) + AL(I,J) * XH(I))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(J) = Y(J) + TEMP1*A(J,J) + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,T1H,T1L,AH(J,J),AL(J,J))
          T3H = T1H * AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(J,J) + T1L * AH(J,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(YH(J),YL(J),T3H,T3L)
          p1 = YH(J) + T3H
          p2 = p1 - YH(J)
          q2 = (YH(J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(J)
          q2 = q2 + T3L
          YH(J) = q1 + q2
          YL(J) = q2 - (YH(J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   60         CONTINUE
          ELSE
!              JX = KX
!              JY = KY
              DO 80 J = 1,N
                  JX = KX + INCX*(J-1)
                  JY = KY + INCY*(J-1)
!                 TEMP1 = ALPHA*X(JX)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,XH(JX),XL(JX))
          T1H = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                 TEMP2 = ZERO
                  T2H = ZERO
                  T2L = ZERO
!                  IX = KX
!                  IY = KY
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
!$omp& private(IX,IY)
                  DO 70 I = 1,J - 1
                  IX = KX + INCX*(I-1)
                  IY = KY + INCY*(I-1)
!                     Y(IY) = Y(IY) + TEMP1*A(I,J)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,J),AL(I,J))
          T3H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(IY),YL(IY),T3H,T3L)
          p1 = YH(IY) + T3H
          p2 = p1 - YH(IY)
          q2 = (YH(IY) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(IY)
          q2 = q2 + T3L
          YH(IY) = q1 + q2
          YL(IY) = q2 - (YH(IY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                     TEMP2 = TEMP2 + A(I,J)*X(IX)
!         call dd_mul(T4H,T4L,AH(I,J),AL(I,J),XH(IX),XL(IX))
          T4H = AH(I,J) * XH(IX)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(IX)
          r1 = u1 - (u1-XH(IX))
          r2 = XH(IX) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (AH(I,J) * XL(IX) + AL(I,J) * XH(IX))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                      IX = IX + INCX
!                      IY = IY + INCY
   70             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + TEMP1*A(J,J) + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,T1H,T1L,AH(J,J),AL(J,J))
          T3H = T1H * AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(J,J) + T1L * AH(J,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T4H,T4L,ALPHAH,ALPHAL,T2H,T2L)
          T4H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T3H,T3L,T4H,T4L)
          p1 = T3H + T4H
          p2 = p1 - T3H
          q2 = (T3H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T3L
          q2 = q2 + T4L
          T3H = q1 + q2
          T3L = q2 - (T3H - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         call dd_selfadd(YH(JY),YL(JY),T3H,T3L)
          p1 = YH(JY) + T3H
          p2 = p1 - YH(JY)
          q2 = (YH(JY) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(JY)
          q2 = q2 + T3L
          YH(JY) = q1 + q2
          YL(JY) = q2 - (YH(JY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                  JX = JX + INCX
!                  JY = JY + INCY
   80         CONTINUE
          END IF
      ELSE
*
*        Form  y  when A is stored in lower triangle.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 100 J = 1,N
!                 TEMP1 = ALPHA*X(J)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,XH(J),XL(J))
          T1H = ALPHAH * XH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(J)
          r1 = u1 - (u1-XH(J))
          r2 = XH(J) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * XL(J) + ALPHAL * XH(J))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                 TEMP2 = ZERO
                  T2H = ZERO
                  T2L = ZERO
!                 Y(J) = Y(J) + TEMP1*A(J,J)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(J,J),AL(J,J))
          T3H = T1H * AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(J,J) + T1L * AH(J,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(J),YL(J),T3H,T3L)
          p1 = YH(J) + T3H
          p2 = p1 - YH(J)
          q2 = (YH(J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(J)
          q2 = q2 + T3L
          YH(J) = q1 + q2
          YL(J) = q2 - (YH(J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
                  DO 90 I = J + 1,N
!                     Y(I) = Y(I) + TEMP1*A(I,J)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,J),AL(I,J))
          T3H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(I),YL(I),T3H,T3L)
          p1 = YH(I) + T3H
          p2 = p1 - YH(I)
          q2 = (YH(I) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(I)
          q2 = q2 + T3L
          YH(I) = q1 + q2
          YL(I) = q2 - (YH(I) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                     TEMP2 = TEMP2 + A(I,J)*X(I)
!         call dd_mul(T4H,T4L,AH(I,J),AL(I,J),XH(I),XL(I))
          T4H = AH(I,J) * XH(I)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(I)
          r1 = u1 - (u1-XH(I))
          r2 = XH(I) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (AH(I,J) * XL(I) + AL(I,J) * XH(I))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   90             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(J) = Y(J) + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,ALPHAH,ALPHAL,T2H,T2L)
          T3H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(J),YL(J),T3H,T3L)
          p1 = YH(J) + T3H
          p2 = p1 - YH(J)
          q2 = (YH(J) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(J)
          q2 = q2 + T3L
          YH(J) = q1 + q2
          YL(J) = q2 - (YH(J) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  100         CONTINUE
          ELSE
!              JX = KX
!              JY = KY
              DO 120 J = 1,N
                  JX = KX + INCX*(J-1)
                  JY = KY + INCY*(J-1)
!                 TEMP1 = ALPHA*X(JX)
!         call dd_mul(T1H,T1L,ALPHAH,ALPHAL,XH(JX),XL(JX))
          T1H = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          T1L = ((p1*r1-T1H) + p1*r2 + p2*r1) + p2*r2
          T1L = T1L + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = T1H + T1L
          T1L = T1L - (s1 - T1H)
          T1H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                 TEMP2 = ZERO
                  T2H = ZERO
                  T2L = ZERO
!                 Y(JY) = Y(JY) + TEMP1*A(J,J)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(J,J),AL(J,J))
          T3H = T1H * AH(J,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(J,J)
          r1 = u1 - (u1-AH(J,J))
          r2 = AH(J,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(J,J) + T1L * AH(J,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(JY),YL(JY),T3H,T3L)
          p1 = YH(JY) + T3H
          p2 = p1 - YH(JY)
          q2 = (YH(JY) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(JY)
          q2 = q2 + T3L
          YH(JY) = q1 + q2
          YL(JY) = q2 - (YH(JY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                  IX = JX
!                  IY = JY
!$omp parallel
!$omp do
!$omp& private(I,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(T3H,T3L,T4H,T4L)
!$omp& private(IX,IY)
                  DO 110 I = J + 1,N
!                      IX = IX + INCX
!                      IY = IY + INCY
                      IX = JX + INCX*(I-J)
                      IY = JY + INCY*(I-J)
!                     Y(IY) = Y(IY) + TEMP1*A(I,J)
!         call dd_mul(T3H,T3L,T1H,T1L,AH(I,J),AL(I,J))
          T3H = T1H * AH(I,J)
          u1 = 134217729.0*T1H
          p1 = u1 - (u1-T1H)
          p2 = T1H - p1
          u1 = 134217729.0*AH(I,J)
          r1 = u1 - (u1-AH(I,J))
          r2 = AH(I,J) - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (T1H * AL(I,J) + T1L * AH(I,J))
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(IY),YL(IY),T3H,T3L)
          p1 = YH(IY) + T3H
          p2 = p1 - YH(IY)
          q2 = (YH(IY) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(IY)
          q2 = q2 + T3L
          YH(IY) = q1 + q2
          YL(IY) = q2 - (YH(IY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                     TEMP2 = TEMP2 + A(I,J)*X(IX)
!         call dd_mul(T4H,T4L,AH(I,J),AL(I,J),XH(IX),XL(IX))
          T4H = AH(I,J) * XH(IX)
          u1 = 134217729.0*AH(I,J)
          p1 = u1 - (u1-AH(I,J))
          p2 = AH(I,J) - p1
          u1 = 134217729.0*XH(IX)
          r1 = u1 - (u1-XH(IX))
          r2 = XH(IX) - r1
          T4L = ((p1*r1-T4H) + p1*r2 + p2*r1) + p2*r2
          T4L = T4L + (AH(I,J) * XL(IX) + AL(I,J) * XH(IX))
          s1 = T4H + T4L
          T4L = T4L - (s1 - T4H)
          T4H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T2H,T2L,T4H,T4L)
!$omp critical (name0)
          p1 = T2H + T4H
          p2 = p1 - T2H
          q2 = (T2H - (p1 - p2)) + (T4H - p2)
          q1 = p1
          q2 = q2 + T2L
          q2 = q2 + T4L
          T2H = q1 + q2
          T2L = q2 - (T2H - q1)
!$omp end critical (name0)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
  110             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP2
!         call dd_mul(T3H,T3L,ALPHAH,ALPHAL,T2H,T2L)
          T3H = ALPHAH * T2H
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*T2H
          r1 = u1 - (u1-T2H)
          r2 = T2H - r1
          T3L = ((p1*r1-T3H) + p1*r2 + p2*r1) + p2*r2
          T3L = T3L + (ALPHAH * T2L + ALPHAL * T2H)
          s1 = T3H + T3L
          T3L = T3L - (s1 - T3H)
          T3H = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(YH(JY),YL(JY),T3H,T3L)
          p1 = YH(JY) + T3H
          p2 = p1 - YH(JY)
          q2 = (YH(JY) - (p1 - p2)) + (T3H - p2)
          q1 = p1
          q2 = q2 + YL(JY)
          q2 = q2 + T3L
          YH(JY) = q1 + q2
          YL(JY) = q2 - (YH(JY) - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                  JX = JX + INCX
!                  JY = JY + INCY
  120         CONTINUE
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000)   "MODULE    = DDSYMV"
      WRITE(6,'(2A)') "UPLO      = ",UPLO
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDSYMV .
*
      END
