      SUBROUTINE DDSYR(UPLO,N,ALPHAH,ALPHAL,XH,XL,INCX,AH,AL,LDA)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION ALPHAH,ALPHAL
      INTEGER INCX,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION AH(LDA,*),AL(LDA,*),XH(*),XL(*)
*     ..
*
*  Purpose
*  =======
*
*  DDSYR  performs the symmetric rank 1 operation
*
*     A := alpha*x*x^T+ A,
*
*  where alpha is a real scalar, x is an n element vector and A is an
*  n by n symmetric matrix.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the array A is to be referenced as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of A
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of A
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHAH - DOUBLE PRECISION.
*           On entry, ALPHA specifies the high word of scalar alpha.
*           Unchanged on exit.
*  ALPHAL - DOUBLE PRECISION.
*           On entry, ALPHA specifies the low word of scalar alpha.
*           Unchanged on exit.
*
*  XH     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XH must contain the 
*           high word of the n element vector x.
*           Unchanged on exit.
*  XL     - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array XL must contain the 
*           low word of the n element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  AH     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AH must contain the upper
*           triangular part of the symmetric matrix and the strictly
*           lower triangular part of AH is not referenced. On exit, the
*           upper triangular part of the array AH is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AH must contain the lower
*           triangular part of the symmetric matrix and the strictly
*           upper triangular part of AH is not referenced. On exit, the
*           lower triangular part of the array AH is overwritten by the
*           lower triangular part of the updated matrix.
*           AH contains the high words of the matrix A.
*  AL     - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array AL must contain the upper
*           triangular part of the symmetric matrix and the strictly
*           lower triangular part of AL is not referenced. On exit, the
*           upper triangular part of the array AL is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array AL must contain the lower
*           triangular part of the symmetric matrix and the strictly
*           upper triangular part of AL is not referenced. On exit, the
*           lower triangular part of the array AL is overwritten by the
*           lower triangular part of the updated matrix.
*           AL contains the low words of the matrix A.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, n ).
*           Unchanged on exit.
*
*  Further Details
*  ===============
*
*  Level 2 DDBlas routine.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      INTEGER I,INFO,IX,J,JX,KX
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDSYR"
#endif
*
*     Test the input parameters.
*
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (INCX.EQ.0) THEN
          INFO = 5
      ELSE IF (LDA.LT.MAX(1,N)) THEN
          INFO = 7
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DDSYR ',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (ALPHA.EQ.ZERO)) RETURN
      IF ((N.EQ.0) .OR. (ALPHAH.EQ.ZERO.AND.ALPHAL.EQ.ZERO)) RETURN
*
*     Set the start point in X if the increment is not unity.
*
      IF (INCX.LE.0) THEN
          KX = 1 - (N-1)*INCX
      ELSE IF (INCX.NE.1) THEN
          KX = 1
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the triangular part
*     of A.
*
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  A  when A is stored in upper triangle.
*
          IF (INCX.EQ.1) THEN
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
              DO 20 J = 1,N
!                 IF (X(J).NE.ZERO) THEN
                  IF (XH(J).NE.ZERO.OR.XL(J).NE.ZERO) THEN
!                     TEMP = ALPHA*X(J)
!                     call dd_mul(t1,t2,ALPHAH,ALPHAL,XH(J),XL(J))
          t1 = ALPHAH * XH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(J)
          r1 = u1 - (u1-XH(J))
          r2 = XH(J) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (ALPHAH * XL(J) + ALPHAL * XH(J))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      DO 10 I = 1,J
!                         A(I,J) = A(I,J) + X(I)*TEMP
!                         call dd_mul(q1,q2,XH(I),XL(I),t1,t2)
          q1 = XH(I) * t1
          u1 = 134217729.0*XH(I)
          p1 = u1 - (u1-XH(I))
          p2 = XH(I) - p1
          u1 = 134217729.0*t1
          r1 = u1 - (u1-t1)
          r2 = t1 - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + (XH(I) * t2 + XL(I) * t1)
          s1 = q1 + q2
          u2 = q2 - (s1 - q1)
          u1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                         call dd_selfadd(AH(I,J),AL(I,J),u1,u2)
          p1 = AH(I,J) + u1
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (u1 - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + u2
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   10                 CONTINUE
                  END IF
   20         CONTINUE
!$omp end parallel
          ELSE
              JX = KX
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(IX,JX)
              DO 40 J = 1,N
                  JX = KX + INCX*(J-1)
!                 IF (X(JX).NE.ZERO) THEN
                  IF (XH(JX).NE.ZERO.OR.XL(JX).NE.ZERO) THEN
!                     TEMP = ALPHA*X(JX)
!                     call dd_mul(t1,t2,ALPHAH,ALPHAL,XH(JX),XL(JX))
          t1 = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                      IX = KX
                      DO 30 I = 1,J
                          IX = KX + INCX*(I-1)
!                         A(I,J) = A(I,J) + X(IX)*TEMP
!                         call dd_mul(q1,q2,XH(IX),XL(IX),t1,t2)
          q1 = XH(IX) * t1
          u1 = 134217729.0*XH(IX)
          p1 = u1 - (u1-XH(IX))
          p2 = XH(IX) - p1
          u1 = 134217729.0*t1
          r1 = u1 - (u1-t1)
          r2 = t1 - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + (XH(IX) * t2 + XL(IX) * t1)
          s1 = q1 + q2
          u2 = q2 - (s1 - q1)
          u1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                          call dd_selfadd(AH(I,J),AL(I,J),u1,u2)
          p1 = AH(I,J) + u1
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (u1 - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + u2
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                          IX = IX + INCX
   30                 CONTINUE
                  END IF
!                  JX = JX + INCX
   40         CONTINUE
!$omp end parallel
          END IF
      ELSE
*
*        Form  A  when A is stored in lower triangle.
*
          IF (INCX.EQ.1) THEN
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
              DO 60 J = 1,N
!                 IF (X(J).NE.ZERO) THEN
                  IF (XH(J).NE.ZERO.OR.XL(J).NE.ZERO) THEN
!                     TEMP = ALPHA*X(J)
!                     call dd_mul(t1,t2,ALPHAH,ALPHAL,XH(J),XL(J))
          t1 = ALPHAH * XH(J)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(J)
          r1 = u1 - (u1-XH(J))
          r2 = XH(J) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (ALPHAH * XL(J) + ALPHAL * XH(J))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
                      DO 50 I = J,N
!                         A(I,J) = A(I,J) + X(I)*TEMP
!                         call dd_mul(q1,q2,XH(I),XL(I),t1,t2)
          q1 = XH(I) * t1
          u1 = 134217729.0*XH(I)
          p1 = u1 - (u1-XH(I))
          p2 = XH(I) - p1
          u1 = 134217729.0*t1
          r1 = u1 - (u1-t1)
          r2 = t1 - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + (XH(I) * t2 + XL(I) * t1)
          s1 = q1 + q2
          u2 = q2 - (s1 - q1)
          u1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                          call dd_selfadd(AH(I,J),AL(I,J),u1,u2)
          p1 = AH(I,J) + u1
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (u1 - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + u2
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50                 CONTINUE
                  END IF
   60         CONTINUE
!$omp end parallel
          ELSE
              JX = KX
!$omp parallel
!$omp do
!$omp& private(I,J,s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2)
!$omp& private(IX,JX)
              DO 80 J = 1,N
                  JX = KX + INCX*(J-1)
!                 IF (X(JX).NE.ZERO) THEN
                  IF (XH(JX).NE.ZERO.OR.XL(JX).NE.ZERO) THEN
!                     TEMP = ALPHA*X(JX)
!                     call dd_mul(t1,t2,ALPHAH,ALPHAL,XH(JX),XL(JX))
          t1 = ALPHAH * XH(JX)
          u1 = 134217729.0*ALPHAH
          p1 = u1 - (u1-ALPHAH)
          p2 = ALPHAH - p1
          u1 = 134217729.0*XH(JX)
          r1 = u1 - (u1-XH(JX))
          r2 = XH(JX) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (ALPHAH * XL(JX) + ALPHAL * XH(JX))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                      IX = JX
                      DO 70 I = J,N
                          IX = KX + INCX*(I-1)
!                         A(I,J) = A(I,J) + X(IX)*TEMP
!                         call dd_mul(q1,q2,XH(IX),XL(IX),t1,t2)
          q1 = XH(IX) * t1
          u1 = 134217729.0*XH(IX)
          p1 = u1 - (u1-XH(IX))
          p2 = XH(IX) - p1
          u1 = 134217729.0*t1
          r1 = u1 - (u1-t1)
          r2 = t1 - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + (XH(IX) * t2 + XL(IX) * t1)
          s1 = q1 + q2
          u2 = q2 - (s1 - q1)
          u1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                          call dd_selfadd(AH(I,J),AL(I,J),u1,u2)
          p1 = AH(I,J) + u1
          p2 = p1 - AH(I,J)
          q2 = (AH(I,J) - (p1 - p2)) + (u1 - p2)
          q1 = p1
          q2 = q2 + AL(I,J)
          q2 = q2 + u2
          s1 = q1 + q2
          AL(I,J) = q2 - (s1 - q1)
          AH(I,J) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!                          IX = IX + INCX
   70                 CONTINUE
                  END IF
!                  JX = JX + INCX
   80         CONTINUE
!$omp end parallel
          END IF
      END IF
*
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDSYR"
      WRITE(6,'(2A)') "UPLO      = ",UPLO
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DDSYR  .
*
      END
