!     SUBROUTINE ZHBMV(UPLO,N,K,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
      subroutine zzhbmv(uplo, n, k, alphah, alphal,
     +                   ah, al, lda, xh, xl, incx,
     +                   betah, betal, yh, yl, incy)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE COMPLEX ALPHA,BETA
      double complex alphah, alphal
      double complex betah, betal
      INTEGER INCX,INCY,K,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),X(*),Y(*)
      double complex ah(lda,*), al(lda,*)
      double complex xh(*), xl(*)
      double complex yh(*), yl(*)
*     ..
*
*  Purpose
*  =======
*
*  ZHBMV  performs the matrix-vector  operation
*
*     y := alpha*A*x + beta*y,
*
*  where alpha and beta are scalars, x and y are n element vectors and
*  A is an n by n hermitian band matrix, with k super-diagonals.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the band matrix A is being supplied as
*           follows:
*
*              UPLO = 'U' or 'u'   The upper triangular part of A is
*                                  being supplied.
*
*              UPLO = 'L' or 'l'   The lower triangular part of A is
*                                  being supplied.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry, K specifies the number of super-diagonals of the
*           matrix A. K must satisfy  0 .le. K.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, n ).
*           Before entry with UPLO = 'U' or 'u', the leading ( k + 1 )
*           by n part of the array A must contain the upper triangular
*           band part of the hermitian matrix, supplied column by
*           column, with the leading diagonal of the matrix in row
*           ( k + 1 ) of the array, the first super-diagonal starting at
*           position 2 in row k, and so on. The top left k by k triangle
*           of the array A is not referenced.
*           The following program segment will transfer the upper
*           triangular part of a hermitian band matrix from conventional
*           full matrix storage to band storage:
*
*                 DO 20, J = 1, N
*                    M = K + 1 - J
*                    DO 10, I = MAX( 1, J - K ), J
*                       A( M + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Before entry with UPLO = 'L' or 'l', the leading ( k + 1 )
*           by n part of the array A must contain the lower triangular
*           band part of the hermitian matrix, supplied column by
*           column, with the leading diagonal of the matrix in row 1 of
*           the array, the first sub-diagonal starting at position 1 in
*           row 2, and so on. The bottom right k by k triangle of the
*           array A is not referenced.
*           The following program segment will transfer the lower
*           triangular part of a hermitian band matrix from conventional
*           full matrix storage to band storage:
*
*                 DO 20, J = 1, N
*                    M = 1 - J
*                    DO 10, I = J, MIN( N, J + K )
*                       A( M + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Note that the imaginary parts of the diagonal elements need
*           not be set and are assumed to be zero.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           ( k + 1 ).
*           Unchanged on exit.
*
*  X      - COMPLEX*16       array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the
*           vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  BETA   - COMPLEX*16      .
*           On entry, BETA specifies the scalar beta.
*           Unchanged on exit.
*
*  Y      - COMPLEX*16       array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array Y must contain the
*           vector y. On exit, Y is overwritten by the updated vector y.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
!     double double modefication (f.y.)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Parameters ..
!     DOUBLE COMPLEX ONE
!     PARAMETER (ONE= (1.0D+0,0.0D+0))
!     DOUBLE COMPLEX ZERO
!     PARAMETER (ZERO= (0.0D+0,0.0D+0))
      double complex oneh, onel
      double complex zeroh, zerol
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP1,TEMP2
      double complex temp1h, temp1l
      double complex temp2h, temp2l
      INTEGER I,INFO,IX,IY,J,JX,JY,KPLUS1,KX,KY,L
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DBLE,DCONJG,MAX,MIN
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzhbmv"
#endif
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
!
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (K.LT.0) THEN
          INFO = 3
      ELSE IF (LDA.LT. (K+1)) THEN
          INFO = 6
      ELSE IF (INCX.EQ.0) THEN
          INFO = 8
      ELSE IF (INCY.EQ.0) THEN
          INFO = 11
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZHBMV ',INFO)
          goto 9999
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
      IF ((N.EQ.0) .OR. (
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     + .AND.
!       (beta .eq. one)
     + ((betah .eq. oneh) .and.
     +  (betal .eq. onel))
     +)) goto 9999
*
*     Set up the start points in  X  and  Y.
*
      IF (INCX.GT.0) THEN
          KX = 1
      ELSE
          KX = 1 - (N-1)*INCX
      END IF
      IF (INCY.GT.0) THEN
          KY = 1
      ELSE
          KY = 1 - (N-1)*INCY
      END IF
*
*     Start the operations. In this version the elements of the array A
*     are accessed sequentially with one pass through A.
*
*     First form  y := beta*y.
*
!     IF (BETA.NE.ONE) THEN
      if(
!       (beta .ne. one)
     + ((betah .ne. oneh) .or.
     +  (betal .ne. onel))
     +) then
          IF (INCY.EQ.1) THEN
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                  DO 10 I = 1,N
!                     Y(I) = ZERO
                      yh(i) = zeroh
                      yl(i) = zerol
   10             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                  DO 20 I = 1,N
!                     Y(I) = BETA*Y(I)
!<<<                  y(i) = beta * y(i)
                      call bmul( betah, betal, yh(i), yl(i),
     +                           yh(i), yl(i) )
!                      x_ch = betah * yh(i)
!                      x_u1 = 134217729.0 * betah
!                      x_p1 = x_u1
!     +                         - (x_u1 - betah)
!                      x_p2 = betah - x_p1
!                      x_u1 = 134217729.0 * yh(i)
!                      x_r1 = x_u1
!     +                         - (x_u1 - yh(i))
!                      x_r2 = yh(i) - x_r1
!                     x_cl = ((x_p1*x_r1
!     +                         - x_ch)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      x_cl = x_cl
!     +                         + (betah*yl(i)
!     +                         +  betal*yh(i))
!                      x_s1 = x_ch + x_cl
!                      yl(i) = x_cl - (x_s1 - x_ch)
!                      yh(i) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                  y(i) = beta * y(i)
   20             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          ELSE
              IY = KY
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I,iy)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmh, ddml)
                  DO 30 I = 1,N
                      iy = ky + (i - 1)*incy
!                     Y(IY) = ZERO
                      yh(iy) = zeroh
                      yl(iy) = zerol
!!                    IY = IY + INCY
   30             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I,iy)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                  DO 40 I = 1,N
                      iy = ky + (i - 1)*incy
!                     Y(IY) = BETA*Y(IY)
!<<<                  y(iy) = beta * y(iy)
                      call bmul( betah, betal, yh(iy), yl(iy),
     +                           yh(iy), yl(iy) )
!                      x_ch = betah * yh(iy)
!                      x_u1 = 134217729.0 * betah
!                      x_p1 = x_u1
!     +                         - (x_u1 - betah)
!                      x_p2 = betah - x_p1
!                      x_u1 = 134217729.0 * yh(iy)
!                      x_r1 = x_u1
!     +                         - (x_u1 - yh(iy))
!                      x_r2 = yh(iy) - x_r1
!                      x_cl = ((x_p1*x_r1
!     +                         - x_ch)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      x_cl = x_cl
!     +                         + (betah*yl(iy)
!     +                         +  betal*yh(iy))
!                      x_s1 = x_ch + x_cl
!                      yl(iy) = x_cl - (x_s1 - x_ch)
!                      yh(iy) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                  y(iy) = beta * y(iy)
!!                    IY = IY + INCY
   40             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          END IF
      END IF
!     IF (ALPHA.EQ.ZERO) RETURN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) goto 9999
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  y  when upper triangle of A is stored.
*
          KPLUS1 = K + 1
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 60 J = 1,N
!                 TEMP1 = ALPHA*X(J)
!<<<              temp1 = alpha * x(j)
                  call bmul( alphah, alphal, xh(j), xl(j),
     +                       temp1h, temp1l )
!                  temp1h = alphah * xh(j)
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * xh(j)
!                  x_r1 = x_u1
!     +                     - (x_u1 - xh(j))
!                  x_r2 = xh(j) - x_r1
!                  temp1l = ((x_p1*x_r1
!     +                    - temp1h)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  temp1l = temp1l
!     +                  + (alphah*xl(j)
!     +                  + alphal*xh(j))
!                  x_s1 = temp1h + temp1l
!                  temp1l = temp1l - (x_s1 - temp1h)
!                  temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp1 = alpha * x(j)
!                 TEMP2 = ZERO
                  temp2h = zeroh
                  temp2l = zerol
                  L = KPLUS1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 50 I = MAX(1,J-K),J - 1
!                     Y(I) = Y(I) + TEMP1*A(L+I,J)
!<<<                  ddmx = temp1 * a(l+i,j)
                      call bmul( temp1h, temp1l, ah(l+i,j), al(l+i,j),
     +                       ddmxh, ddmxl )
!                      ddmxh = temp1h * ah(l+i,j)
!                      x_u1 = 134217729.0 * temp1h
!                      x_p1 = x_u1
!     +                         - (x_u1 - temp1h)
!                      x_p2 = temp1h - x_p1
!                      x_u1 = 134217729.0 * ah(l+i,j)
!                      x_r1 = x_u1
!     +                         - (x_u1 - ah(l+i,j))
!                      x_r2 = ah(l+i,j) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (temp1h*al(l+i,j)
!     +                      + temp1l*ah(l+i,j))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp1 * a(l+i,j)
!<<<                  y(i) = y(i) + ddmx
                      call bsum( yh(i), yl(i), ddmxh, ddmxl,
     +                           yh(i), yl(i) )
!                      x_p1 = yh(i) + ddmxh
!                      x_p2 = x_p1 - yh(i)
!                      x_q1 = x_p1
!                      x_q2 = (yh(i) - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + yl(i) + ddmxl
!                      yh(i) = x_q1 + x_q2
!                      yl(i) = x_q2 - (yh(i) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  y(i) = y(i) + ddmx
!                     TEMP2 = TEMP2 + DCONJG(A(L+I,J))*X(I)
                      ddaxh = dconjg(ah(l+i,j))
                      ddaxl = dconjg(al(l+i,j))
!<<<                  ddmx = ddax * x(i)
                      call bmul( ddaxh, ddaxl, xh(i), xl(i),
     +                           ddmxh, ddmxl )
!                      ddmxh = ddaxh * xh(i)
!                      x_u1 = 134217729.0 * ddaxh
!                      x_p1 = x_u1
!     +                         - (x_u1 - ddaxh)
!                      x_p2 = ddaxh - x_p1
!                      x_u1 = 134217729.0 * xh(i)
!                      x_r1 = x_u1
!     +                         - (x_u1 - xh(i))
!                      x_r2 = xh(i) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (ddaxh*xl(i)
!     +                      + ddaxl*xh(i))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = ddax * x(i)
!$omp critical
!<<<                  temp2 = temp2 + ddmx
                      call bsum( temp2h, temp2l, ddmxh, ddmxl,
     +                           temp2h, temp2l )
!                      x_p1 = temp2h + ddmxh
!                      x_p2 = x_p1 - temp2h
!                      x_q1 = x_p1
!                      x_q2 = (temp2h - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + temp2l + ddmxl
!                      temp2h = x_q1 + x_q2
!                      temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  temp2 = temp2 + ddmx
!$omp end critical
   50             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(J) = Y(J) + TEMP1*DBLE(A(KPLUS1,J)) + ALPHA*TEMP2
!<<<              ddmx = alpha * temp2
                  call bmul( alphah, alphal, temp2h, temp2l,
     +                       ddmxh, ddmxl )
!                  ddmxh = alphah * temp2h
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * temp2h
!                  x_r1 = x_u1
!     +                     - (x_u1 - temp2h)
!                  x_r2 = temp2h - x_r1
!                  ddmxl = ((x_p1*x_r1
!     +                    - ddmxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddmxl = ddmxl
!     +                  + (alphah*temp2l
!     +                  + alphal*temp2h)
!                  x_s1 = ddmxh + ddmxl
!                  ddmxl = ddmxl - (x_s1 - ddmxh)
!                  ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddmx = alpha * temp2
                  ddah = dble(ah(kplus1,j))
                  ddal = dble(al(kplus1,j))
!<<<              ddax = temp1 * dda
                  call bmul( temp1h, temp1l, ddah, ddal,
     +                       ddaxh, ddaxl )
!                  ddaxh = temp1h * ddah
!                  x_u1 = 134217729.0 * temp1h
!                  x_p1 = x_u1
!     +                     - (x_u1 - temp1h)
!                  x_p2 = temp1h - x_p1
!                  x_u1 = 134217729.0 * ddah
!                  x_r1 = x_u1
!     +                     - (x_u1 - ddah)
!                  x_r2 = ddah - x_r1
!                  ddaxl = ((x_p1*x_r1
!     +                    - ddaxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddaxl = ddaxl
!     +                  + (temp1h*ddal
!     +                  + temp1l*ddah)
!                  x_s1 = ddaxh + ddaxl
!                  ddaxl = ddaxl - (x_s1 - ddaxh)
!                  ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddax = temp1 * dda
!<<<              ddax = ddax + ddmx
                  call bsum( ddaxh, ddaxl, ddmxh, ddmxl, ddaxh, ddaxl )
!                  x_p1 = ddaxh + ddmxh
!                  x_p2 = x_p1 - ddaxh
!                  x_q1 = x_p1
!                  x_q2 = (ddaxh - (x_p1 - x_p2))
!     +                     + (ddmxh - x_p2)
!                  x_q2 = x_q2 + ddaxl + ddmxl
!                  ddaxh = x_q1 + x_q2
!                  ddaxl = x_q2 - (ddaxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              ddax = ddax + ddmx
!<<<              y(j) = y(j) + ddax
                  call bsum( yh(j), yl(j), ddaxh, ddaxl, yh(j), yl(j) )
!                  x_p1 = yh(j) + ddaxh
!                  x_p2 = x_p1 - yh(j)
!                  x_q1 = x_p1
!                  x_q2 = (yh(j) - (x_p1 - x_p2))
!     +                     + (ddaxh - x_p2)
!                  x_q2 = x_q2 + yl(j) + ddaxl
!                  yh(j) = x_q1 + x_q2
!                  yl(j) = x_q2 - (yh(j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(j) = y(j) + ddax
   60         CONTINUE
          ELSE
              JX = KX
              JY = KY
              DO 80 J = 1,N
!                 TEMP1 = ALPHA*X(JX)
!<<<              temp1 = alpha * x(jx)
                  call bmul( alphah, alphal, xh(jx), xl(jx),
     +                       temp1h, temp1l )
!                  temp1h = alphah * xh(jx)
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * xh(jx)
!                  x_r1 = x_u1
!     +                     - (x_u1 - xh(jx))
!                  x_r2 = xh(jx) - x_r1
!                  temp1l = ((x_p1*x_r1
!     +                    - temp1h)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  temp1l = temp1l
!     +                  + (alphah*xl(jx)
!     +                  + alphal*xh(jx))
!                  x_s1 = temp1h + temp1l
!                  temp1l = temp1l - (x_s1 - temp1h)
!                  temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp1 = alpha * x(jx)
!                 TEMP2 = ZERO
                  temp2h = zeroh
                  temp2l = zerol
                  IX = KX
                  IY = KY
                  L = KPLUS1 - J
!$omp parallel
!$omp do
!$omp+ private(I,ix,iy)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 70 I = MAX(1,J-K),J - 1
                      ix = kx + (i - MAX(1,J-K))*incx
                      iy = ky + (i - MAX(1,J-K))*incy
!                     Y(IY) = Y(IY) + TEMP1*A(L+I,J)
!<<<                  ddmx = temp1 * a(l+i,j)
                      call bmul( temp1h, temp1l, ah(l+i,j), al(l+i,j),
     +                      ddmxh, ddmxl )
!                      ddmxh = temp1h * ah(l+i,j)
!                      x_u1 = 134217729.0 * temp1h
!                      x_p1 = x_u1
!     +                         - (x_u1 - temp1h)
!                      x_p2 = temp1h - x_p1
!                      x_u1 = 134217729.0 * ah(l+i,j)
!                      x_r1 = x_u1
!     +                         - (x_u1 - ah(l+i,j))
!                      x_r2 = ah(l+i,j) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (temp1h*al(l+i,j)
!     +                      + temp1l*ah(l+i,j))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp1 * a(l+i,j)
!<<<                  y(iy) = y(iy) + ddmx
                      call bsum( yh(iy), yl(iy), ddmxh, ddmxl,
     +                           yh(iy), yl(iy) )
!                      x_p1 = yh(iy) + ddmxh
!                      x_p2 = x_p1 - yh(iy)
!                      x_q1 = x_p1
!                      x_q2 = (yh(iy) - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + yl(iy) + ddmxl
!                      yh(iy) = x_q1 + x_q2
!                      yl(iy) = x_q2 - (yh(iy) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  y(iy) = y(iy) + ddmx
!                     TEMP2 = TEMP2 + DCONJG(A(L+I,J))*X(IX)
                      ddaxh = dconjg(ah(l+i,j))
                      ddaxl = dconjg(al(l+i,j))
!<<<                  ddmx = ddax * x(ix)
                      call bmul( ddaxh, ddaxl, xh(ix), xl(ix),
     +                           ddmxh, ddmxl )
!                      ddmxh = ddaxh * xh(ix)
!                      x_u1 = 134217729.0 * ddaxh
!                      x_p1 = x_u1
!     +                         - (x_u1 - ddaxh)
!                      x_p2 = ddaxh - x_p1
!                      x_u1 = 134217729.0 * xh(ix)
!                      x_r1 = x_u1
!     +                         - (x_u1 - xh(ix))
!                      x_r2 = xh(ix) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (ddaxh*xl(ix)
!     +                      + ddaxl*xh(ix))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = ddax * x(ix)
!$omp critical
!<<<                  temp2 = temp2 + ddmx
                      call bsum( temp2h, temp2l, ddmxh, ddmxl,
     +                           temp2h, temp2l )
!                      x_p1 = temp2h + ddmxh
!                      x_p2 = x_p1 - temp2h
!                      x_q1 = x_p1
!                      x_q2 = (temp2h - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + temp2l + ddmxl
!                      temp2h = x_q1 + x_q2
!                      temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  temp2 = temp2 + ddmx
!$omp end critical
!!                    IX = IX + INCX
!!                    IY = IY + INCY
   70             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + TEMP1*DBLE(A(KPLUS1,J)) + ALPHA*TEMP2
!<<<              ddmx = alpha * temp2
                  call bmul( alphah, alphal, temp2h, temp2l,
     +                       ddmxh, ddmxl )
!                  ddmxh = alphah * temp2h
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * temp2h
!                  x_r1 = x_u1
!     +                     - (x_u1 - temp2h)
!                  x_r2 = temp2h - x_r1
!                  ddmxl = ((x_p1*x_r1
!     +                    - ddmxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddmxl = ddmxl
!     +                  + (alphah*temp2l
!     +                  + alphal*temp2h)
!                  x_s1 = ddmxh + ddmxl
!                  ddmxl = ddmxl - (x_s1 - ddmxh)
!                  ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddmx = alpha * temp2
                  ddah = dble(ah(kplus1,j))
                  ddal = dble(al(kplus1,j))
!<<<              ddax = temp1 * dda
                  call bmul( temp1h, temp1l, ddah, ddal, ddaxh, ddaxl )
!                  ddaxh = temp1h * ddah
!                  x_u1 = 134217729.0 * temp1h
!                  x_p1 = x_u1
!     +                     - (x_u1 - temp1h)
!                  x_p2 = temp1h - x_p1
!                  x_u1 = 134217729.0 * ddah
!                  x_r1 = x_u1
!     +                     - (x_u1 - ddah)
!                  x_r2 = ddah - x_r1
!                  ddaxl = ((x_p1*x_r1
!     +                    - ddaxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddaxl = ddaxl
!     +                  + (temp1h*ddal
!     +                  + temp1l*ddah)
!                  x_s1 = ddaxh + ddaxl
!                  ddaxl = ddaxl - (x_s1 - ddaxh)
!                  ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddax = temp1 * dda
!<<<              ddax = ddax + ddmx
                  call bsum( ddaxh, ddaxl, ddmxh, ddmxl, ddaxh, ddaxl )
!                  x_p1 = ddaxh + ddmxh
!                  x_p2 = x_p1 - ddaxh
!                  x_q1 = x_p1
!                  x_q2 = (ddaxh - (x_p1 - x_p2))
!     +                     + (ddmxh - x_p2)
!                  x_q2 = x_q2 + ddaxl + ddmxl
!                  ddaxh = x_q1 + x_q2
!                  ddaxl = x_q2 - (ddaxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              ddax = ddax + ddmx
!<<<              y(jy) = y(jy) + ddax
                  call bsum( yh(jy), yl(jy), ddaxh, ddaxl,
     +                       yh(jy), yl(jy) )
!                  x_p1 = yh(jy) + ddaxh
!                  x_p2 = x_p1 - yh(jy)
!                  x_q1 = x_p1
!                  x_q2 = (yh(jy) - (x_p1 - x_p2))
!     +                     + (ddaxh - x_p2)
!                  x_q2 = x_q2 + yl(jy) + ddaxl
!                  yh(jy) = x_q1 + x_q2
!                  yl(jy) = x_q2 - (yh(jy) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(jy) = y(jy) + ddax
                  JX = JX + INCX
                  JY = JY + INCY
                  IF (J.GT.K) THEN
                      KX = KX + INCX
                      KY = KY + INCY
                  END IF
   80         CONTINUE
          END IF
      ELSE
*
*        Form  y  when lower triangle of A is stored.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 100 J = 1,N
!                 TEMP1 = ALPHA*X(J)
!<<<              temp1 = alpha * x(j)
                  call bmul( alphah, alphal, xh(j), xl(j),
     +                       temp1h, temp1l )
!                  temp1h = alphah * xh(j)
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * xh(j)
!                  x_r1 = x_u1
!     +                     - (x_u1 - xh(j))
!                  x_r2 = xh(j) - x_r1
!                  temp1l = ((x_p1*x_r1
!     +                    - temp1h)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  temp1l = temp1l
!     +                  + (alphah*xl(j)
!     +                  + alphal*xh(j))
!                  x_s1 = temp1h + temp1l
!                  temp1l = temp1l - (x_s1 - temp1h)
!                  temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp1 = alpha * x(j)
!                 TEMP2 = ZERO
                  temp2h = zeroh
                  temp2l = zerol
!                 Y(J) = Y(J) + TEMP1*DBLE(A(1,J))
                  ddah = dble(ah(1,j))
                  ddal = dble(al(1,j))
!<<<              ddmx = temp1 * dda
                  call bmul( temp1h, temp1l, ddah, ddal, ddmxh, ddmxl )
!                  ddmxh = temp1h * ddah
!                  x_u1 = 134217729.0 * temp1h
!                  x_p1 = x_u1
!     +                     - (x_u1 - temp1h)
!                  x_p2 = temp1h - x_p1
!                  x_u1 = 134217729.0 * ddah
!                  x_r1 = x_u1
!     +                     - (x_u1 - ddah)
!                  x_r2 = ddah - x_r1
!                  ddmxl = ((x_p1*x_r1
!     +                    - ddmxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddmxl = ddmxl
!     +                  + (temp1h*ddal
!     +                  + temp1l*ddah)
!                  x_s1 = ddmxh + ddmxl
!                  ddmxl = ddmxl - (x_s1 - ddmxh)
!                  ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddmx = temp1 * dda
!<<<              y(j) = y(j) + ddmx
                  call bsum( yh(j), yl(j), ddmxh, ddmxl, yh(j), yl(j) )
!                  x_p1 = yh(j) + ddmxh
!                  x_p2 = x_p1 - yh(j)
!                  x_q1 = x_p1
!                  x_q2 = (yh(j) - (x_p1 - x_p2))
!     +                     + (ddmxh - x_p2)
!                  x_q2 = x_q2 + yl(j) + ddmxl
!                  yh(j) = x_q1 + x_q2
!                  yl(j) = x_q2 - (yh(j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(j) = y(j) + ddmx
                  L = 1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 90 I = J + 1,MIN(N,J+K)
!                     Y(I) = Y(I) + TEMP1*A(L+I,J)
!<<<                  ddmx = temp1 * a(l+i,j)
                      call bmul( temp1h, temp1l, ah(l+i,j), al(l+i,j),
     +                       ddmxh, ddmxl )
!                      ddmxh = temp1h * ah(l+i,j)
!                      x_u1 = 134217729.0 * temp1h
!                      x_p1 = x_u1
!     +                         - (x_u1 - temp1h)
!                      x_p2 = temp1h - x_p1
!                      x_u1 = 134217729.0 * ah(l+i,j)
!                      x_r1 = x_u1
!     +                         - (x_u1 - ah(l+i,j))
!                      x_r2 = ah(l+i,j) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (temp1h*al(l+i,j)
!     +                      + temp1l*ah(l+i,j))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp1 * a(l+i,j)
!<<<                  y(i) = y(i) + ddmx
                      call bsum( yh(i), yl(i), ddmxh, ddmxl,
     +                           yh(i), yl(i) )
!                      x_p1 = yh(i) + ddmxh
!                      x_p2 = x_p1 - yh(i)
!                      x_q1 = x_p1
!                      x_q2 = (yh(i) - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + yl(i) + ddmxl
!                      yh(i) = x_q1 + x_q2
!                      yl(i) = x_q2 - (yh(i) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  y(i) = y(i) + ddmx
!                     TEMP2 = TEMP2 + DCONJG(A(L+I,J))*X(I)
                      ddaxh = dconjg(ah(l+i,j))
                      ddaxl = dconjg(al(l+i,j))
!<<<                  ddmx = ddax * x(i)
                      call bmul( ddaxh, ddaxl, xh(i), xl(i),
     +                           ddmxh, ddmxl )
!                      ddmxh = ddaxh * xh(i)
!                      x_u1 = 134217729.0 * ddaxh
!                      x_p1 = x_u1
!     +                         - (x_u1 - ddaxh)
!                      x_p2 = ddaxh - x_p1
!                      x_u1 = 134217729.0 * xh(i)
!                      x_r1 = x_u1
!     +                         - (x_u1 - xh(i))
!                      x_r2 = xh(i) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (ddaxh*xl(i)
!     +                      + ddaxl*xh(i))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = ddax * x(i)
!$omp critical
!<<<                  temp2 = temp2 + ddmx
                      call bsum( temp2h, temp2l, ddmxh, ddmxl,
     +                           temp2h, temp2l )
!                      x_p1 = temp2h + ddmxh
!                      x_p2 = x_p1 - temp2h
!                      x_q1 = x_p1
!                      x_q2 = (temp2h - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + temp2l + ddmxl
!                      temp2h = x_q1 + x_q2
!                      temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  temp2 = temp2 + ddmx
!$omp end critical
   90             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(J) = Y(J) + ALPHA*TEMP2
!<<<              ddmx = alpha * temp2
                  call bmul( alphah, alphal, temp2h, temp2l,
     +                       ddmxh, ddmxl )
!                  ddmxh = alphah * temp2h
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * temp2h
!                  x_r1 = x_u1
!     +                     - (x_u1 - temp2h)
!                  x_r2 = temp2h - x_r1
!                  ddmxl = ((x_p1*x_r1
!     +                    - ddmxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddmxl = ddmxl
!     +                  + (alphah*temp2l
!     +                  + alphal*temp2h)
!                  x_s1 = ddmxh + ddmxl
!                  ddmxl = ddmxl - (x_s1 - ddmxh)
!                  ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddmx = alpha * temp2
!<<<              y(j) = y(j) + ddmx
                  call bsum( yh(j), yl(j), ddmxh, ddmxl, yh(j), yl(j) )
!                  x_p1 = yh(j) + ddmxh
!                  x_p2 = x_p1 - yh(j)
!                  x_q1 = x_p1
!                  x_q2 = (yh(j) - (x_p1 - x_p2))
!     +                     + (ddmxh - x_p2)
!                  x_q2 = x_q2 + yl(j) + ddmxl
!                  yh(j) = x_q1 + x_q2
!                  yl(j) = x_q2 - (yh(j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(j) = y(j) + ddmx
  100         CONTINUE
          ELSE
              JX = KX
              JY = KY
              DO 120 J = 1,N
!                 TEMP1 = ALPHA*X(JX)
!<<<              temp1 = alpha * x(jx)
                  call bmul( alphah, alphal, xh(jx), xl(jx),
     +                       temp1h, temp1l )
!                  temp1h = alphah * xh(jx)
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * xh(jx)
!                  x_r1 = x_u1
!     +                     - (x_u1 - xh(jx))
!                  x_r2 = xh(jx) - x_r1
!                  temp1l = ((x_p1*x_r1
!     +                    - temp1h)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  temp1l = temp1l
!     +                  + (alphah*xl(jx)
!     +                  + alphal*xh(jx))
!                  x_s1 = temp1h + temp1l
!                  temp1l = temp1l - (x_s1 - temp1h)
!                  temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              temp1 = alpha * x(jx)
!                 TEMP2 = ZERO
                  temp2h = zeroh
                  temp2l = zerol
!                 Y(JY) = Y(JY) + TEMP1*DBLE(A(1,J))
                  ddah = dble(ah(1,j))
                  ddal = dble(al(1,j))
!<<<              ddmx = temp1 * dda
                  call bmul( temp1h, temp1l, ddah, ddal, ddmxh, ddmxl )
!                  ddmxh = temp1h * ddah
!                  x_u1 = 134217729.0 * temp1h
!                  x_p1 = x_u1
!     +                     - (x_u1 - temp1h)
!                  x_p2 = temp1h - x_p1
!                  x_u1 = 134217729.0 * ddah
!                  x_r1 = x_u1
!     +                     - (x_u1 - ddah)
!                  x_r2 = ddah - x_r1
!                  ddmxl = ((x_p1*x_r1
!     +                    - ddmxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddmxl = ddmxl
!     +                  + (temp1h*ddal
!     +                  + temp1l*ddah)
!                  x_s1 = ddmxh + ddmxl
!                  ddmxl = ddmxl - (x_s1 - ddmxh)
!                  ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddmx = temp1 * dda
!<<<              y(jy) = y(jy) + ddmx
                  call bsum( yh(jy), yl(jy), ddmxh, ddmxl,
     +                       yh(jy), yl(jy) )
!                  x_p1 = yh(jy) + ddmxh
!                  x_p2 = x_p1 - yh(jy)
!                  x_q1 = x_p1
!                  x_q2 = (yh(jy) - (x_p1 - x_p2))
!     +                     + (ddmxh - x_p2)
!                  x_q2 = x_q2 + yl(jy) + ddmxl
!                  yh(jy) = x_q1 + x_q2
!                  yl(jy) = x_q2 - (yh(jy) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(jy) = y(jy) + ddmx
                  L = 1 - J
                  IX = JX
                  IY = JY
!$omp parallel
!$omp do
!$omp+ private(I,ix,iy)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 110 I = J + 1,MIN(N,J+K)
                      ix = jx + (i - j)*incx
                      iy = jy + (i - j)*incy
!!                    IX = IX + INCX
!!                    IY = IY + INCY
!                     Y(IY) = Y(IY) + TEMP1*A(L+I,J)
!<<<                  ddmx = temp1 * a(l+i,j)
                      call bmul( temp1h, temp1l, ah(l+i,j), al(l+i,j),
     +                      ddmxh, ddmxl )
!                      ddmxh = temp1h * ah(l+i,j)
!                      x_u1 = 134217729.0 * temp1h
!                      x_p1 = x_u1
!     +                         - (x_u1 - temp1h)
!                      x_p2 = temp1h - x_p1
!                      x_u1 = 134217729.0 * ah(l+i,j)
!                      x_r1 = x_u1
!     +                         - (x_u1 - ah(l+i,j))
!                      x_r2 = ah(l+i,j) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (temp1h*al(l+i,j)
!     +                      + temp1l*ah(l+i,j))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp1 * a(l+i,j)
!<<<                  y(iy) = y(iy) + ddmx
                      call bsum( yh(iy), yl(iy), ddmxh, ddmxl,
     +                  yh(iy), yl(iy) )
!                      x_p1 = yh(iy) + ddmxh
!                      x_p2 = x_p1 - yh(iy)
!                      x_q1 = x_p1
!                      x_q2 = (yh(iy) - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + yl(iy) + ddmxl
!                      yh(iy) = x_q1 + x_q2
!                      yl(iy) = x_q2 - (yh(iy) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  y(iy) = y(iy) + ddmx
!                     TEMP2 = TEMP2 + DCONJG(A(L+I,J))*X(IX)
                      ddaxh = dconjg(ah(l+i,j))
                      ddaxl = dconjg(al(l+i,j))
!<<<                  ddmx = ddax * x(ix)
                      call bmul( ddaxh, ddaxl, xh(ix), xl(ix),
     +                  ddmxh, ddmxl )
!                      ddmxh = ddaxh * xh(ix)
!                      x_u1 = 134217729.0 * ddaxh
!                      x_p1 = x_u1
!     +                         - (x_u1 - ddaxh)
!                      x_p2 = ddaxh - x_p1
!                      x_u1 = 134217729.0 * xh(ix)
!                      x_r1 = x_u1
!     +                         - (x_u1 - xh(ix))
!                      x_r2 = xh(ix) - x_r1
!                      ddmxl = ((x_p1*x_r1
!     +                        - ddmxh)
!     +                        + x_p1 * x_r2
!     +                        + x_p2 * x_r1)
!     +                        + x_p2 * x_r2
!                      ddmxl = ddmxl
!     +                      + (ddaxh*xl(ix)
!     +                      + ddaxl*xh(ix))
!                      x_s1 = ddmxh + ddmxl
!                      ddmxl = ddmxl - (x_s1 - ddmxh)
!                      ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = ddax * x(ix)
!$omp critical
!<<<                  temp2 = temp2 + ddmx
                      call bsum( temp2h, temp2l, ddmxh, ddmxl,
     +                  temp2h, temp2l )
!                      x_p1 = temp2h + ddmxh
!                      x_p2 = x_p1 - temp2h
!                      x_q1 = x_p1
!                      x_q2 = (temp2h - (x_p1 - x_p2))
!     +                         + (ddmxh - x_p2)
!                      x_q2 = x_q2 + temp2l + ddmxl
!                      temp2h = x_q1 + x_q2
!                      temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  temp2 = temp2 + ddmx
!$omp end critical
  110             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP2
!<<<              ddmx = alpha * temp2
                  call bmul( alphah, alphal, temp2h, temp2l,
     +              ddmxh, ddmxl )
!                  ddmxh = alphah * temp2h
!                  x_u1 = 134217729.0 * alphah
!                  x_p1 = x_u1
!     +                     - (x_u1 - alphah)
!                  x_p2 = alphah - x_p1
!                  x_u1 = 134217729.0 * temp2h
!                  x_r1 = x_u1
!     +                     - (x_u1 - temp2h)
!                  x_r2 = temp2h - x_r1
!                  ddmxl = ((x_p1*x_r1
!     +                    - ddmxh)
!     +                    + x_p1 * x_r2
!     +                    + x_p2 * x_r1)
!     +                    + x_p2 * x_r2
!                  ddmxl = ddmxl
!     +                  + (alphah*temp2l
!     +                  + alphal*temp2h)
!                  x_s1 = ddmxh + ddmxl
!                  ddmxl = ddmxl - (x_s1 - ddmxh)
!                  ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddmx = alpha * temp2
!<<<              y(jy) = y(jy) + ddmx
                  call bsum( yh(jy), yl(jy), ddmxh, ddmxl,
     +              yh(jy), yl(jy) )
!                  x_p1 = yh(jy) + ddmxh
!                  x_p2 = x_p1 - yh(jy)
!                  x_q1 = x_p1
!                  x_q2 = (yh(jy) - (x_p1 - x_p2))
!     +                     + (ddmxh - x_p2)
!                  x_q2 = x_q2 + yl(jy) + ddmxl
!                  yh(jy) = x_q1 + x_q2
!                  yl(jy) = x_q2 - (yh(jy) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(jy) = y(jy) + ddmx
                  JX = JX + INCX
                  JY = JY + INCY
  120         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = zzhbmv"
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZHBMV .
*
      END
