!     SUBROUTINE ZHEMM(SIDE,UPLO,M,N,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
      subroutine zzhemm(side, uplo, m, n,
     +                   alphah, alphal, ah, al, lda,
     +                   bh, bl, ldb, betah, betal, ch, cl, ldc)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE COMPLEX ALPHA,BETA
      double complex alphah, alphal
      double complex betah, betal
      INTEGER LDA,LDB,LDC,M,N
      CHARACTER SIDE,UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),B(LDB,*),C(LDC,*)
      double complex ah(lda,*), al(lda,*)
      double complex bh(ldb,*), bl(ldb,*)
      double complex ch(ldc,*), cl(ldc,*)
*     ..
*
*  Purpose
*  =======
*
*  ZHEMM  performs one of the matrix-matrix operations
*
*     C := alpha*A*B + beta*C,
*
*  or
*
*     C := alpha*B*A + beta*C,
*
*  where alpha and beta are scalars, A is an hermitian matrix and  B and
*  C are m by n matrices.
*
*  Arguments
*  ==========
*
*  SIDE   - CHARACTER*1.
*           On entry,  SIDE  specifies whether  the  hermitian matrix  A
*           appears on the  left or right  in the  operation as follows:
*
*              SIDE = 'L' or 'l'   C := alpha*A*B + beta*C,
*
*              SIDE = 'R' or 'r'   C := alpha*B*A + beta*C,
*
*           Unchanged on exit.
*
*  UPLO   - CHARACTER*1.
*           On  entry,   UPLO  specifies  whether  the  upper  or  lower
*           triangular  part  of  the  hermitian  matrix   A  is  to  be
*           referenced as follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of the
*                                  hermitian matrix is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of the
*                                  hermitian matrix is to be referenced.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry,  M  specifies the number of rows of the matrix  C.
*           M  must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix C.
*           N  must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, ka ), where ka is
*           m  when  SIDE = 'L' or 'l'  and is n  otherwise.
*           Before entry  with  SIDE = 'L' or 'l',  the  m by m  part of
*           the array  A  must contain the  hermitian matrix,  such that
*           when  UPLO = 'U' or 'u', the leading m by m upper triangular
*           part of the array  A  must contain the upper triangular part
*           of the  hermitian matrix and the  strictly  lower triangular
*           part of  A  is not referenced,  and when  UPLO = 'L' or 'l',
*           the leading  m by m  lower triangular part  of the  array  A
*           must  contain  the  lower triangular part  of the  hermitian
*           matrix and the  strictly upper triangular part of  A  is not
*           referenced.
*           Before entry  with  SIDE = 'R' or 'r',  the  n by n  part of
*           the array  A  must contain the  hermitian matrix,  such that
*           when  UPLO = 'U' or 'u', the leading n by n upper triangular
*           part of the array  A  must contain the upper triangular part
*           of the  hermitian matrix and the  strictly  lower triangular
*           part of  A  is not referenced,  and when  UPLO = 'L' or 'l',
*           the leading  n by n  lower triangular part  of the  array  A
*           must  contain  the  lower triangular part  of the  hermitian
*           matrix and the  strictly upper triangular part of  A  is not
*           referenced.
*           Note that the imaginary parts  of the diagonal elements need
*           not be set, they are assumed to be zero.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the  calling (sub) program. When  SIDE = 'L' or 'l'  then
*           LDA must be at least  max( 1, m ), otherwise  LDA must be at
*           least max( 1, n ).
*           Unchanged on exit.
*
*  B      - COMPLEX*16       array of DIMENSION ( LDB, n ).
*           Before entry, the leading  m by n part of the array  B  must
*           contain the matrix B.
*           Unchanged on exit.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in  the  calling  (sub)  program.   LDB  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*  BETA   - COMPLEX*16      .
*           On entry,  BETA  specifies the scalar  beta.  When  BETA  is
*           supplied as zero then C need not be set on input.
*           Unchanged on exit.
*
*  C      - COMPLEX*16       array of DIMENSION ( LDC, n ).
*           Before entry, the leading  m by n  part of the array  C must
*           contain the matrix  C,  except when  beta  is zero, in which
*           case C need not be set on entry.
*           On exit, the array  C  is overwritten by the  m by n updated
*           matrix.
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*
!  double double modefication (f.y.)
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DBLE,DCONJG,MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP1,TEMP2
      double complex temp1h, temp1l
      double complex temp2h, temp2l
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
      double complex ddxxh, ddxxl
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
      INTEGER I,INFO,J,K,NROWA
      LOGICAL UPPER
*     ..
*     .. Parameters ..
!     DOUBLE COMPLEX ONE
!     PARAMETER (ONE= (1.0D+0,0.0D+0))
!     DOUBLE COMPLEX ZERO
!     PARAMETER (ZERO= (0.0D+0,0.0D+0))
      double complex oneh, onel
      double complex zeroh, zerol
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzhemm"
#endif
*     ..
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
*
*     Set NROWA as the number of rows of A.
*
      IF (LSAME(SIDE,'L')) THEN
          NROWA = M
      ELSE
          NROWA = N
      END IF
      UPPER = LSAME(UPLO,'U')
*
*     Test the input parameters.
*
      INFO = 0
      IF ((.NOT.LSAME(SIDE,'L')) .AND. (.NOT.LSAME(SIDE,'R'))) THEN
          INFO = 1
      ELSE IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 2
      ELSE IF (M.LT.0) THEN
          INFO = 3
      ELSE IF (N.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 7
      ELSE IF (LDB.LT.MAX(1,M)) THEN
          INFO = 9
      ELSE IF (LDC.LT.MAX(1,M)) THEN
          INFO = 12
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZHEMM ',INFO)
          goto 9999
!         RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((M.EQ.0) .OR. (N.EQ.0) .OR.
!    +    ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
      if((m.eq.0) .or. (n.eq.0) .or. (
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +    .and.
!       (beta .eq. one)
     + ((betah .eq. oneh) .and.
     +  (betal .eq. onel))
     +)) goto 9999
*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) then
!         IF (BETA.EQ.ZERO) THEN
          if(
!           (BETA .eq. ZERO)
     +     ((BETAh .eq. ZEROh) .and.
     +      (BETAl .eq. ZEROl))
     +    ) then
              DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
                  DO 10 I = 1,M
!                     C(I,J) = ZERO
                      ch(i,j) = zeroh
                      cl(i,j) = zerol
   10             CONTINUE
!$omp end do
!$omp end parallel
   20         CONTINUE
          ELSE
              DO 40 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                  DO 30 I = 1,M
!                     C(I,J) = BETA*C(I,J)
!<<<                  c(i,j) = beta * c(i,j)
c                     x_ch = betah * ch(i,j)
c                     x_u1 = 134217729.0 * betah
c                     x_p1 = x_u1
c    +                         - (x_u1 - betah)
c                     x_p2 = betah - x_p1
c                     x_u1 = 134217729.0 * ch(i,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - ch(i,j))
c                     x_r2 = ch(i,j) - x_r1
c                     x_cl = ((x_p1*x_r1
c    +                         - x_ch)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     x_cl = x_cl
c    +                         + (betah*cl(i,j)
c    +                         +  betal*ch(i,j))
c                     x_s1 = x_ch + x_cl
c                     cl(i,j) = x_cl - (x_s1 - x_ch)
c                     ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                  c(i,j) = beta * c(i,j)
                      call bmul(betah, betal, ch(i,j), cl(i,j),
     +                    ch(i,j), cl(i,j))
   30             CONTINUE
!$omp end do
!$omp end parallel
   40         CONTINUE
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (LSAME(SIDE,'L')) THEN
*
*        Form  C := alpha*A*B + beta*C.
*
          IF (UPPER) THEN
              DO 70 J = 1,N
                  DO 60 I = 1,M
!                     TEMP1 = ALPHA*B(I,J)
!<<<                  temp1 = alpha * b(i,j)
c                     temp1h = alphah * bh(i,j)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * bh(i,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - bh(i,j))
c                     x_r2 = bh(i,j) - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*bl(i,j)
c    +                      + alphal*bh(i,j))
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * b(i,j)
                      call bmul(alphah, alphal, bh(i,j), bl(i,j),
     +                    temp1h, temp1l)
!                     TEMP2 = ZERO
                      temp2h = zeroh
                      temp2l = zerol
!$omp parallel
!$omp do
!$omp+ private(K)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 50 K = 1,I - 1
!                         C(K,J) = C(K,J) + TEMP1*A(K,I)
!<<<                      ddmx = temp1 * a(k,i)
c                         ddmxh = temp1h * ah(k,i)
c                         x_u1 = 134217729.0 * temp1h
c                         x_p1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_p2 = temp1h - x_p1
c                         x_u1 = 134217729.0 * ah(k,i)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(k,i))
c                         x_r2 = ah(k,i) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temp1h*al(k,i)
c    +                          + temp1l*ah(k,i))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp1 * a(k,i)
                          call bmul(temp1h, temp1l, ah(k,i), al(k,i),
     +                        ddmxh, ddmxl)
!<<<                      c(k,j) = c(k,j) + ddmx
c                         x_p1 = ch(k,j) + ddmxh
c                         x_p2 = x_p1 - ch(k,j)
c                         x_q1 = x_p1
c                         x_q2 = (ch(k,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + cl(k,j) + ddmxl
c                         ch(k,j) = x_q1 + x_q2
c                         cl(k,j) = x_q2 - (ch(k,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      c(k,j) = c(k,j) + ddmx
                          call bsum(ch(k,j), cl(k,j), ddmxh, ddmxl,
     +                        ch(k,j), cl(k,j))
!                         TEMP2 = TEMP2 + B(K,J)*DCONJG(A(K,I))
                          ddaxh =  dconjg(ah(k,i))
                          ddaxl =  dconjg(al(k,i))
!<<<                      ddmx = b(k,j) * ddax
c                         ddmxh = bh(k,j) * ddaxh
c                         x_u1 = 134217729.0 * bh(k,j)
c                         x_p1 = x_u1
c    +                             - (x_u1 - bh(k,j))
c                         x_p2 = bh(k,j) - x_p1
c                         x_u1 = 134217729.0 * ddaxh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_r2 = ddaxh - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (bh(k,j)*ddaxl
c    +                          + bl(k,j)*ddaxh)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = b(k,j) * ddax
                          call bmul(bh(k,j), bl(k,j), ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp2 = temp2 + ddmx
c                         x_p1 = temp2h + ddmxh
c                         x_p2 = x_p1 - temp2h
c                         x_q1 = x_p1
c                         x_q2 = (temp2h - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + temp2l + ddmxl
c                         temp2h = x_q1 + x_q2
c                         temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp2 = temp2 + ddmx
                          call bsum(temp2h, temp2l, ddmxh, ddmxl,
     +                        temp2h, temp2l)
!$omp end critical
   50                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      if(
!                       (beta .eq. zero)
     +                 ((betah .eq. zeroh) .and.
     +                  (betal .eq. zerol))
     +                ) then
!                         C(I,J) = TEMP1*DBLE(A(I,I)) + ALPHA*TEMP2
!<<<                      ddax = alpha * temp2
c                         ddaxh = alphah * temp2h
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (alphah*temp2l
c    +                          + alphal*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = alpha * temp2
                          call bmul(alphah, alphal, temp2h, temp2l,
     +                        ddaxh, ddaxl)
                          ddmh = dble(ah(i,i))
                          ddml = dble(al(i,i))
!<<<                      ddmx = temp1 * ddm
c                         ddmxh = temp1h * ddmh
c                         x_u1 = 134217729.0 * temp1h
c                         x_p1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_p2 = temp1h - x_p1
c                         x_u1 = 134217729.0 * ddmh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddmh)
c                         x_r2 = ddmh - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temp1h*ddml
c    +                          + temp1l*ddmh)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp1 * ddm
                          call bmul(temp1h, temp1l, ddmh, ddml,
     +                        ddmxh, ddmxl)
!<<<                      c(i,j) = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddmxh - (x_p1 - x_p2))
c    +                          + (ddaxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddmxl + ddaxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ch(i,j), cl(i,j))
                      ELSE
!                         C(I,J) = BETA*C(I,J) + TEMP1*DBLE(A(I,I)) +
!    +                             ALPHA*TEMP2
!<<<                      ddax = alpha * temp2
c                         ddaxh = alphah * temp2h
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (alphah*temp2l
c    +                          + alphal*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = alpha * temp2
                          call bmul(alphah, alphal, temp2h, temp2l,
     +                        ddaxh, ddaxl)
                          ddmh = dble(ah(i,i))
                          ddml = dble(al(i,i))
!<<<                      ddmx = temp1 * ddm
c                         ddmxh = temp1h * ddmh
c                         x_u1 = 134217729.0 * temp1h
c                         x_p1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_p2 = temp1h - x_p1
c                         x_u1 = 134217729.0 * ddmh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddmh)
c                         x_r2 = ddmh - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temp1h*ddml
c    +                          + temp1l*ddmh)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp1 * ddm
                          call bmul(temp1h, temp1l, ddmh, ddml,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!<<<                      ddxx = beta * c(i,j)
c                         ddxxh = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         ddxxl = ((x_p1*x_r1
c    +                            - ddxxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddxxl = ddxxl
c    +                          + (betah*cl(i,j)
c    +                          + betal*ch(i,j))
c                         x_s1 = ddxxh + ddxxl
c                         ddxxl = ddxxl - (x_s1 - ddxxh)
c                         ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddxx = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ddxxh, ddxxl)
!<<<                      c(i,j) = ddxx + ddmx
c                         x_p1 = ddxxh + ddmxh
c                         x_p2 = x_p1 - ddxxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddxxh - (x_p1 - x_p2))
c    +                          + (ddmxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddxxl + ddmxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddxx + ddmx
                          call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                        ch(i,j), cl(i,j))
                      END IF
   60             CONTINUE
   70         CONTINUE
          ELSE
              DO 100 J = 1,N
                  DO 90 I = M,1,-1
!                     TEMP1 = ALPHA*B(I,J)
!<<<                  temp1 = alpha * b(i,j)
c                     temp1h = alphah * bh(i,j)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * bh(i,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - bh(i,j))
c                     x_r2 = bh(i,j) - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*bl(i,j)
c    +                      + alphal*bh(i,j))
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * b(i,j)
                      call bmul(alphah, alphal, bh(i,j), bl(i,j),
     +                    temp1h, temp1l)
!                     TEMP2 = ZERO
                      temp2h = zeroh
                      temp2l = zerol
!$omp parallel
!$omp do
!$omp+ private(K)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 80 K = I + 1,M
!                         C(K,J) = C(K,J) + TEMP1*A(K,I)
!<<<                      ddmx = temp1 * a(k,i)
c                         ddmxh = temp1h * ah(k,i)
c                         x_u1 = 134217729.0 * temp1h
c                         x_p1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_p2 = temp1h - x_p1
c                         x_u1 = 134217729.0 * ah(k,i)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(k,i))
c                         x_r2 = ah(k,i) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temp1h*al(k,i)
c    +                          + temp1l*ah(k,i))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp1 * a(k,i)
                          call bmul(temp1h, temp1l, ah(k,i), al(k,i),
     +                        ddmxh, ddmxl)
!<<<                      c(k,j) = c(k,j) + ddmx
c                         x_p1 = ch(k,j) + ddmxh
c                         x_p2 = x_p1 - ch(k,j)
c                         x_q1 = x_p1
c                         x_q2 = (ch(k,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + cl(k,j) + ddmxl
c                         ch(k,j) = x_q1 + x_q2
c                         cl(k,j) = x_q2 - (ch(k,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      c(k,j) = c(k,j) + ddmx
                          call bsum(ch(k,j), cl(k,j), ddmxh, ddmxl,
     +                        ch(k,j), cl(k,j))
!                         TEMP2 = TEMP2 + B(K,J)*DCONJG(A(K,I))
                          ddaxh = dconjg(ah(k,i))
                          ddaxl = dconjg(al(k,i))
!<<<                      ddmx = b(k,j) * ddax
c                         ddmxh = bh(k,j) * ddaxh
c                         x_u1 = 134217729.0 * bh(k,j)
c                         x_p1 = x_u1
c    +                             - (x_u1 - bh(k,j))
c                         x_p2 = bh(k,j) - x_p1
c                         x_u1 = 134217729.0 * ddaxh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_r2 = ddaxh - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (bh(k,j)*ddaxl
c    +                          + bl(k,j)*ddaxh)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = b(k,j) * ddax
                          call bmul(bh(k,j), bl(k,j), ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp2 = temp2 + ddmx
c                         x_p1 = temp2h + ddmxh
c                         x_p2 = x_p1 - temp2h
c                         x_q1 = x_p1
c                         x_q2 = (temp2h - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + temp2l + ddmxl
c                         temp2h = x_q1 + x_q2
c                         temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp2 = temp2 + ddmx
                          call bsum(temp2h, temp2l, ddmxh, ddmxl,
     +                        temp2h, temp2l)
!$omp end critical
   80                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      if(
!                       (beta .eq. zero)
     +                 ((betah .eq. zeroh) .and.
     +                  (betal .eq. zerol))
     +                ) then
!                         C(I,J) = TEMP1*DBLE(A(I,I)) + ALPHA*TEMP2
!<<<                      ddax = alpha * temp2
c                         ddaxh = alphah * temp2h
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (alphah*temp2l
c    +                          + alphal*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = alpha * temp2
                          call bmul(alphah, alphal, temp2h, temp2l,
     +                        ddaxh, ddaxl)
                          ddmh = dble(ah(i,i))
                          ddml = dble(al(i,i))
!<<<                      ddmx = temp1 * ddm
c                         ddmxh = temp1h * ddmh
c                         x_u1 = 134217729.0 * temp1h
c                         x_p1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_p2 = temp1h - x_p1
c                         x_u1 = 134217729.0 * ddmh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddmh)
c                         x_r2 = ddmh - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temp1h*ddml
c    +                          + temp1l*ddmh)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp1 * ddm
                          call bmul(temp1h, temp1l, ddmh, ddml,
     +                        ddmxh, ddmxl)
!<<<                      c(i,j) = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddmxh - (x_p1 - x_p2))
c    +                          + (ddaxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddmxl + ddaxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ch(i,j), cl(i,j))
                      ELSE
!                         C(I,J) = BETA*C(I,J) + TEMP1*DBLE(A(I,I)) +
!    +                             ALPHA*TEMP2
!<<<                      ddax = alpha * temp2
c                         ddaxh = alphah * temp2h
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (alphah*temp2l
c    +                          + alphal*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = alpha * temp2
                          call bmul(alphah, alphal, temp2h, temp2l,
     +                        ddaxh, ddaxl)
                          ddmh = dble(ah(i,i))
                          ddml = dble(al(i,i))
!<<<                      ddmx = temp1 * ddm
c                         ddmxh = temp1h * ddmh
c                         x_u1 = 134217729.0 * temp1h
c                         x_p1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_p2 = temp1h - x_p1
c                         x_u1 = 134217729.0 * ddmh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddmh)
c                         x_r2 = ddmh - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (temp1h*ddml
c    +                          + temp1l*ddmh)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = temp1 * ddm
                          call bmul(temp1h, temp1l, ddmh, ddml,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!<<<                      ddxx = beta * c(i,j)
c                         ddxxh = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         ddxxl = ((x_p1*x_r1
c    +                            - ddxxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddxxl = ddxxl
c    +                          + (betah*cl(i,j)
c    +                          + betal*ch(i,j))
c                         x_s1 = ddxxh + ddxxl
c                         ddxxl = ddxxl - (x_s1 - ddxxh)
c                         ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddxx = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ddxxh, ddxxl)
!<<<                      c(i,j) = ddxx + ddmx
c                         x_p1 = ddxxh + ddmxh
c                         x_p2 = x_p1 - ddxxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddxxh - (x_p1 - x_p2))
c    +                          + (ddmxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddxxl + ddmxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddxx + ddmx
                          call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                        ch(i,j), cl(i,j))
                      END IF
   90             CONTINUE
  100         CONTINUE
          END IF
      ELSE
*
*        Form  C := alpha*B*A + beta*C.
*
          DO 170 J = 1,N
!             TEMP1 = ALPHA*DBLE(A(J,J))
              ddmh = dble(ah(j,j))
              ddml = dble(al(j,j))
!<<<          temp1 = alpha * ddm
c             temp1h = alphah * ddmh
c             x_u1 = 134217729.0 * alphah
c             x_p1 = x_u1
c    +                 - (x_u1 - alphah)
c             x_p2 = alphah - x_p1
c             x_u1 = 134217729.0 * ddmh
c             x_r1 = x_u1
c    +                 - (x_u1 - ddmh)
c             x_r2 = ddmh - x_r1
c             temp1l = ((x_p1*x_r1
c    +                - temp1h)
c    +                + x_p1 * x_r2
c    +                + x_p2 * x_r1)
c    +                + x_p2 * x_r2
c             temp1l = temp1l
c    +              + (alphah*ddml
c    +              + alphal*ddmh)
c             x_s1 = temp1h + temp1l
c             temp1l = temp1l - (x_s1 - temp1h)
c             temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          temp1 = alpha * ddm
              call bmul(alphah, alphal, ddmh, ddml,
     +            temp1h, temp1l)
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                  DO 110 I = 1,M
!                     C(I,J) = TEMP1*B(I,J)
!<<<                  c(i,j) = temp1 * b(i,j)
c                     ch(i,j) = temp1h * bh(i,j)
c                     x_u1 = 134217729.0 * temp1h
c                     x_p1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_p2 = temp1h - x_p1
c                     x_u1 = 134217729.0 * bh(i,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - bh(i,j))
c                     x_r2 = bh(i,j) - x_r1
c                     cl(i,j) = ((x_p1*x_r1
c    +                        - ch(i,j))
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     cl(i,j) = cl(i,j)
c    +                      + (temp1h*bl(i,j)
c    +                      + temp1l*bh(i,j))
c                     x_s1 = ch(i,j) + cl(i,j)
c                     cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                     ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  c(i,j) = temp1 * b(i,j)
                      call bmul(temp1h, temp1l, bh(i,j), bl(i,j),
     +                    ch(i,j), cl(i,j))
  110             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                  DO 120 I = 1,M
!                     C(I,J) = BETA*C(I,J) + TEMP1*B(I,J)
!<<<                  ddax = temp1 * b(i,j)
c                     ddaxh = temp1h * bh(i,j)
c                     x_u1 = 134217729.0 * temp1h
c                     x_p1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_p2 = temp1h - x_p1
c                     x_u1 = 134217729.0 * bh(i,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - bh(i,j))
c                     x_r2 = bh(i,j) - x_r1
c                     ddaxl = ((x_p1*x_r1
c    +                        - ddaxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddaxl = ddaxl
c    +                      + (temp1h*bl(i,j)
c    +                      + temp1l*bh(i,j))
c                     x_s1 = ddaxh + ddaxl
c                     ddaxl = ddaxl - (x_s1 - ddaxh)
c                     ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddax = temp1 * b(i,j)
                      call bmul(temp1h, temp1l, bh(i,j), bl(i,j),
     +                    ddaxh, ddaxl)
!<<<                  ddmx = beta * c(i,j)
c                     ddmxh = betah * ch(i,j)
c                     x_u1 = 134217729.0 * betah
c                     x_p1 = x_u1
c    +                         - (x_u1 - betah)
c                     x_p2 = betah - x_p1
c                     x_u1 = 134217729.0 * ch(i,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - ch(i,j))
c                     x_r2 = ch(i,j) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (betah*cl(i,j)
c    +                      + betal*ch(i,j))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = beta * c(i,j)
                      call bmul(betah, betal, ch(i,j), cl(i,j),
     +                    ddmxh, ddmxl)
!<<<                  c(i,j) = ddmx + ddax
c                     x_p1 = ddmxh + ddaxh
c                     x_p2 = x_p1 - ddmxh
c                     ch(i,j) = x_p1
c                     cl(i,j) = (ddmxh - (x_p1 - x_p2))
c    +                      + (ddaxh - x_p2)
c                     cl(i,j) = cl(i,j) + ddmxl + ddaxl
c                     x_s1 = ch(i,j) + cl(i,j)
c                     cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                     ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                  c(i,j) = ddmx + ddax
                      call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                    ch(i,j), cl(i,j))
  120             CONTINUE
!$omp end do
!$omp end parallel
              END IF
              DO 140 K = 1,J - 1
                  IF (UPPER) THEN
!                     TEMP1 = ALPHA*A(K,J)
!<<<                  temp1 = alpha * a(k,j)
c                     temp1h = alphah * ah(k,j)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ah(k,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - ah(k,j))
c                     x_r2 = ah(k,j) - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*al(k,j)
c    +                      + alphal*ah(k,j))
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * a(k,j)
                      call bmul(alphah, alphal, ah(k,j), al(k,j),
     +                    temp1h, temp1l)
                  ELSE
!                     TEMP1 = ALPHA*DCONJG(A(J,K))
                      ddaxh = dconjg(ah(j,k))
                      ddaxl = dconjg(al(j,k))
!<<<                  temp1 = alpha * ddax
c                     temp1h = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temp1h, temp1l)
                  END IF
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                  DO 130 I = 1,M
!                     C(I,J) = C(I,J) + TEMP1*B(I,K)
!<<<                  ddmx = temp1 * b(i,k)
c                     ddmxh = temp1h * bh(i,k)
c                     x_u1 = 134217729.0 * temp1h
c                     x_p1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_p2 = temp1h - x_p1
c                     x_u1 = 134217729.0 * bh(i,k)
c                     x_r1 = x_u1
c    +                         - (x_u1 - bh(i,k))
c                     x_r2 = bh(i,k) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (temp1h*bl(i,k)
c    +                      + temp1l*bh(i,k))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp1 * b(i,k)
                      call bmul(temp1h, temp1l, bh(i,k), bl(i,k),
     +                    ddmxh, ddmxl)
!<<<                  c(i,j) = c(i,j) + ddmx
c                     x_p1 = ch(i,j) + ddmxh
c                     x_p2 = x_p1 - ch(i,j)
c                     x_q1 = x_p1
c                     x_q2 = (ch(i,j) - (x_p1 - x_p2))
c    +                         + (ddmxh - x_p2)
c                     x_q2 = x_q2 + cl(i,j) + ddmxl
c                     ch(i,j) = x_q1 + x_q2
c                     cl(i,j) = x_q2 - (ch(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  c(i,j) = c(i,j) + ddmx
                      call bsum(ch(i,j), cl(i,j), ddmxh, ddmxl,
     +                    ch(i,j), cl(i,j))
  130             CONTINUE
!$omp end do
!$omp end parallel
  140         CONTINUE
              DO 160 K = J + 1,N
                  IF (UPPER) THEN
!                     TEMP1 = ALPHA*DCONJG(A(J,K))
                      ddaxh = dconjg(ah(j,k))
                      ddaxl = dconjg(al(j,k))
!<<<                  temp1 = alpha * ddax
c                     temp1h = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temp1h, temp1l)
                  ELSE
!                     TEMP1 = ALPHA*A(K,J)
!<<<                  temp1 = alpha * a(k,j)
c                     temp1h = alphah * ah(k,j)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ah(k,j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - ah(k,j))
c                     x_r2 = ah(k,j) - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*al(k,j)
c    +                      + alphal*ah(k,j))
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * a(k,j)
                      call bmul(alphah, alphal, ah(k,j), al(k,j),
     +                    temp1h, temp1l)
                  END IF
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                  DO 150 I = 1,M
!                     C(I,J) = C(I,J) + TEMP1*B(I,K)
!<<<                  ddmx = temp1 * b(i,k)
c                     ddmxh = temp1h * bh(i,k)
c                     x_u1 = 134217729.0 * temp1h
c                     x_p1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_p2 = temp1h - x_p1
c                     x_u1 = 134217729.0 * bh(i,k)
c                     x_r1 = x_u1
c    +                         - (x_u1 - bh(i,k))
c                     x_r2 = bh(i,k) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (temp1h*bl(i,k)
c    +                      + temp1l*bh(i,k))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp1 * b(i,k)
                      call bmul(temp1h, temp1l, bh(i,k), bl(i,k),
     +                    ddmxh, ddmxl)
!<<<                  c(i,j) = c(i,j) + ddmx
c                     x_p1 = ch(i,j) + ddmxh
c                     x_p2 = x_p1 - ch(i,j)
c                     x_q1 = x_p1
c                     x_q2 = (ch(i,j) - (x_p1 - x_p2))
c    +                         + (ddmxh - x_p2)
c                     x_q2 = x_q2 + cl(i,j) + ddmxl
c                     ch(i,j) = x_q1 + x_q2
c                     cl(i,j) = x_q2 - (ch(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  c(i,j) = c(i,j) + ddmx
                      call bsum(ch(i,j), cl(i,j), ddmxh, ddmxl,
     +                    ch(i,j), cl(i,j))
  150             CONTINUE
!$omp end do
!$omp end parallel
  160         CONTINUE
  170     CONTINUE
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = zzhemm"
!     WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZHEMM .
*
      END
