!     SUBROUTINE ZHER(UPLO,N,ALPHA,X,INCX,A,LDA)
      subroutine zzher(uplo, n, alphah, alphal,
     +                  xh, xl, incx, ah, al, lda)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA
      double precision alphah, alphal
      INTEGER INCX,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),X(*)
      double complex ah(lda,*), al(lda,*)
      double complex xh(*), xl(*)
*     ..
*
*  Purpose
*  =======
*
*  ZHER   performs the hermitian rank 1 operation
*
*     A := alpha*x*conjg( x' ) + A,
*
*  where alpha is a real scalar, x is an n element vector and A is an
*  n by n hermitian matrix.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the array A is to be referenced as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of A
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of A
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION.
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  X      - COMPLEX*16       array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the n
*           element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array A must contain the upper
*           triangular part of the hermitian matrix and the strictly
*           lower triangular part of A is not referenced. On exit, the
*           upper triangular part of the array A is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array A must contain the lower
*           triangular part of the hermitian matrix and the strictly
*           upper triangular part of A is not referenced. On exit, the
*           lower triangular part of the array A is overwritten by the
*           lower triangular part of the updated matrix.
*           Note that the imaginary parts of the diagonal elements need
*           not be set, they are assumed to be zero, and on exit they
*           are set to zero.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, n ).
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
!  double double modefication 2011/3
*     .. Parameters ..
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
!     DOUBLE COMPLEX ZERO
!     PARAMETER (ZERO= (0.0D+0,0.0D+0))
      double complex zeroh, zerol
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP
      double complex temph, templ
      INTEGER I,INFO,IX,J,JX,KX
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DBLE,DCONJG,MAX
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzher"
#endif
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (INCX.EQ.0) THEN
          INFO = 5
      ELSE IF (LDA.LT.MAX(1,N)) THEN
          INFO = 7
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZHER  ',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (ALPHA.EQ.DBLE(ZERO))) RETURN
      IF ((N.EQ.0) .OR.
!       (alpha .eq. 0.0)
     + ((alphah .eq. 0.0d+0) .and.
     +  (alphal .eq. 0.0d+0))
     +) goto 9999
*
*     Set the start point in X if the increment is not unity.
*
      IF (INCX.LE.0) THEN
          KX = 1 - (N-1)*INCX
      ELSE IF (INCX.NE.1) THEN
          KX = 1
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the triangular part
*     of A.
*
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  A  when A is stored in upper triangle.
*
          IF (INCX.EQ.1) THEN
              DO 20 J = 1,N
!                 IF (X(J).NE.ZERO) THEN
                  if(
!                   (x(j) .ne. zero)
     +             ((xh(j) .ne. zeroh) .or.
     +              (xl(j) .ne. zerol))
     +            ) then
!                     TEMP = ALPHA*DCONJG(X(J))
                      ddaxh = dconjg(xh(j))
                      ddaxl = dconjg(xl(j))
!<<<                  temp = alpha * ddax
c                     temph = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     templ = ((x_p1*x_r1
c    +                        - temph)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     templ = templ
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temph + templ
c                     templ = templ - (x_s1 - temph)
c                     temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temph, templ)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                      DO 10 I = 1,J - 1
!                         A(I,J) = A(I,J) + X(I)*TEMP
!<<<                      ddmx = x(i) * temp
c                         ddmxh = xh(i) * temph
c                         x_u1 = 134217729.0 * xh(i)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(i))
c                         x_p2 = xh(i) - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(i)*templ
c    +                          + xl(i)*temph)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(i) * temp
                          call bmul(xh(i), xl(i), temph, templ,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
   10                 CONTINUE
!$omp end do
!$omp end parallel
!                     A(J,J) = DBLE(A(J,J)) + DBLE(X(J)*TEMP)
!<<<                  ddmx = x(j) * temp
c                     ddmxh = xh(j) * temph
c                     x_u1 = 134217729.0 * xh(j)
c                     x_p1 = x_u1
c    +                         - (x_u1 - xh(j))
c                     x_p2 = xh(j) - x_p1
c                     x_u1 = 134217729.0 * temph
c                     x_r1 = x_u1
c    +                         - (x_u1 - temph)
c                     x_r2 = temph - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (xh(j)*templ
c    +                      + xl(j)*temph)
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(j) * temp
                      call bmul(xh(j), xl(j), temph, templ,
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) dda ddm
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
   20         CONTINUE
          ELSE
              JX = KX
              DO 40 J = 1,N
!                 IF (X(JX).NE.ZERO) THEN
                  if(
!                   (x(jx) .ne. zero)
     +             ((xh(jx) .ne. zeroh) .or.
     +              (xl(jx) .ne. zerol))
     +            ) then
!                     TEMP = ALPHA*DCONJG(X(JX))
                      ddaxh = dconjg(xh(jx))
                      ddaxl = dconjg(xl(jx))
!<<<                  temp = alpha * ddax
c                     temph = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     templ = ((x_p1*x_r1
c    +                        - temph)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     templ = templ
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temph + templ
c                     templ = templ - (x_s1 - temph)
c                     temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temph, templ)
                      IX = KX
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                      DO 30 I = 1,J - 1
                          ix = kx + (i - 1)*incx
!                         A(I,J) = A(I,J) + X(IX)*TEMP
!<<<                      ddmx = x(ix) * temp
c                         ddmxh = xh(ix) * temph
c                         x_u1 = 134217729.0 * xh(ix)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(ix))
c                         x_p2 = xh(ix) - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(ix)*templ
c    +                          + xl(ix)*temph)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(ix) * temp
                          call bmul(xh(ix), xl(ix), temph, templ,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
!!                        IX = IX + INCX
   30                 CONTINUE
!$omp end do
!$omp end parallel
!                     A(J,J) = DBLE(A(J,J)) + DBLE(X(JX)*TEMP)
!<<<                  ddmx = x(jx) * temp
c                     ddmxh = xh(jx) * temph
c                     x_u1 = 134217729.0 * xh(jx)
c                     x_p1 = x_u1
c    +                         - (x_u1 - xh(jx))
c                     x_p2 = xh(jx) - x_p1
c                     x_u1 = 134217729.0 * temph
c                     x_r1 = x_u1
c    +                         - (x_u1 - temph)
c                     x_r2 = temph - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (xh(jx)*templ
c    +                      + xl(jx)*temph)
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(jx) * temp
                      call bmul(xh(jx), xl(jx), temph, templ,
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) dda ddm
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
                  JX = JX + INCX
   40         CONTINUE
          END IF
      ELSE
*
*        Form  A  when A is stored in lower triangle.
*
          IF (INCX.EQ.1) THEN
              DO 60 J = 1,N
!                 IF (X(J).NE.ZERO) THEN
                  if(
!                   (x(j) .ne. zero)
     +             ((xh(j) .ne. zeroh) .or.
     +              (xl(j) .ne. zerol))
     +            ) then
!                     TEMP = ALPHA*DCONJG(X(J))
                      ddaxh = dconjg(xh(j))
                      ddaxl = dconjg(xl(j))
!<<<                  TEMP = ALPHA * ddax
c                     TEMPh = ALPHAh * ddaxh
c                     x_u1 = 134217729.0 * ALPHAh
c                     x_p1 = x_u1
c    +                         - (x_u1 - ALPHAh)
c                     x_p2 = ALPHAh - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     TEMPl = ((x_p1*x_r1
c    +                        - TEMPh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     TEMPl = TEMPl
c    +                      + (ALPHAh*ddaxl
c    +                      + ALPHAl*ddaxh)
c                     x_s1 = TEMPh + TEMPl
c                     TEMPl = TEMPl - (x_s1 - TEMPh)
c                     TEMPh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  TEMP = ALPHA * ddax
                      call bmul(ALPHAh, ALPHAl, ddaxh, ddaxl,
     +                    TEMPh, TEMPl)
!                     A(J,J) = DBLE(A(J,J)) + DBLE(TEMP*X(J))
!<<<                  ddmx = temp * x(j)
c                     ddmxh = temph * xh(j)
c                     x_u1 = 134217729.0 * temph
c                     x_p1 = x_u1
c    +                         - (x_u1 - temph)
c                     x_p2 = temph - x_p1
c                     x_u1 = 134217729.0 * xh(j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - xh(j))
c                     x_r2 = xh(j) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (temph*xl(j)
c    +                      + templ*xh(j))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp * x(j)
                      call bmul(temph, templ, xh(j), xl(j),
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     x_p1 = ddah + ddmh
c                     x_p2 = x_p1 - ddah
c                     x_q1 = x_p1
c                     x_q2 = (ddah - (x_p1 - x_p2))
c    +                         + (ddmh - x_p2)
c                     x_q2 = x_q2 + ddal + ddml
c                     ddah = x_q1 + x_q2
c                     ddal = x_q2 - (ddah - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) ddm dda
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                      DO 50 I = J + 1,N
!                         A(I,J) = A(I,J) + X(I)*TEMP
!<<<                      ddmx = x(i) * temp
c                         ddmxh = xh(i) * temph
c                         x_u1 = 134217729.0 * xh(i)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(i))
c                         x_p2 = xh(i) - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(i)*templ
c    +                          + xl(i)*temph)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(i) * temp
                          call bmul(xh(i), xl(i), temph, templ,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
   50                 CONTINUE
!$omp end do
!$omp end parallel
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
   60         CONTINUE
          ELSE
              JX = KX
              DO 80 J = 1,N
!                 IF (X(JX).NE.ZERO) THEN
                  if(
!                   (x(jx) .ne. zero)
     +             ((xh(jx) .ne. zeroh) .or.
     +              (xl(jx) .ne. zerol))
     +            ) then
!                     TEMP = ALPHA*DCONJG(X(JX))
                      ddaxh = dconjg(xh(jx))
                      ddaxl = dconjg(xl(jx))
!<<<                  temp = alpha * ddax
c                     temph = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     templ = ((x_p1*x_r1
c    +                        - temph)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     templ = templ
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temph + templ
c                     templ = templ - (x_s1 - temph)
c                     temph = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temph, templ)
!                     A(J,J) = DBLE(A(J,J)) + DBLE(TEMP*X(JX))
!<<<                  ddmx = temp * x(jx)
c                     ddmxh = temph * xh(jx)
c                     x_u1 = 134217729.0 * temph
c                     x_p1 = x_u1
c    +                         - (x_u1 - temph)
c                     x_p2 = temph - x_p1
c                     x_u1 = 134217729.0 * xh(jx)
c                     x_r1 = x_u1
c    +                         - (x_u1 - xh(jx))
c                     x_r2 = xh(jx) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (temph*xl(jx)
c    +                      + templ*xh(jx))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = temp * x(jx)
                      call bmul(temph, templ, xh(jx), xl(jx),
     +                    ddmxh, ddmxl)
                      ddmh = DBLE(ddmxh)
                      ddml = DBLE(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) ddm dda
                      IX = JX
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
                      DO 70 I = J + 1,N
!!                        IX = IX + INCX
                          ix = jx + (i - j)*incx
!                         A(I,J) = A(I,J) + X(IX)*TEMP
!<<<                      ddmx = x(ix) * temp
c                         ddmxh = xh(ix) * temph
c                         x_u1 = 134217729.0 * xh(ix)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(ix))
c                         x_p2 = xh(ix) - x_p1
c                         x_u1 = 134217729.0 * temph
c                         x_r1 = x_u1
c    +                             - (x_u1 - temph)
c                         x_r2 = temph - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(ix)*templ
c    +                          + xl(ix)*temph)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(ix) * temp
                          call bmul(xh(ix), xl(ix), temph, templ,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
   70                 CONTINUE
!$omp end do
!$omp end parallel
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
                  JX = JX + INCX
   80         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = zzher"
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZHER  .
*
      END
