#ifdef ARRAY_2D
!      subroutine eigen_dch(n,d,e,z,ldz,info,zr2d,lda3)
      subroutine ddeigen_dch(n,dh,dl,eh,el,zh,zl,ldz,info,
     &                       zr2dh,zr2dl,lda3)
#else
!      subroutine eigen_dch(n,d,e,z,ldz,info)
      subroutine ddeigen_dch(n,dh,dl,eh,el,zh,zl,ldz,info)
#endif
!
!      use communication_h, only : eigen_init
!     &               , eigen_free
!     &               , bcast_dbl
!     &               , cyc2d_cyc1d
      use ddcommunication_h, only : eigen_init
     &                            , eigen_free
     &                            , ddbcast_dbl
     &                            , ddcyc2d_cyc1d
!
      integer, intent(in)           :: n, ldz
!      real(8), intent(inout)        :: d(1:n), e(1:n)
      real(8), intent(inout)        :: dh(1:n),dl(1:n),eh(1:n),el(1:n)
!      real(8), intent(out)          :: z(1:ldz,*)
!      real(8), intent(out)          :: z(*)
      real(8), intent(out)          :: zh(*),zl(*)


! parameters blacs array descritor(the position of entry tags), etc
      integer, parameter            :: block_cyclic_2d = 1
      integer, parameter            :: dlen_  = 9
      integer, parameter            :: dtype_ = 1
      integer, parameter            :: ctxt_  = 2
      integer, parameter            :: m_     = 3
      integer, parameter            :: n_     = 4
      integer, parameter            :: mb_    = 5
      integer, parameter            :: nb_    = 6
      integer, parameter            :: rsrc_  = 7
      integer, parameter            :: csrc_  = 8
      integer, parameter            :: lld_   = 9

      integer                       :: descz( dlen_ )
      integer                       :: descw( dlen_ )
 
      integer                       :: world_size, my_rank
      integer                       :: trilwmin, lwork, liwork
 
!      real(8), pointer              :: work(:)
      real(8), pointer              :: workh(:), workl(:)
      integer, pointer              :: iwork(:)

!      real(8)                       :: s0, s1, d1, d2, d3
      double precision s0h, s0l
      double precision s1h, s1l
      double precision d1h, d1l
      double precision d2h, d2l
      double precision d3h, d3l

      include 'mpif.h'
      include 'trd.h'
#ifdef ARRAY_2D
!      real(8) :: zr2d(*)
      real(8) :: zr2dh(*),zr2dl(*)
      integer :: lda3
#endif
 
      s0=mpi_wtime()
! blacs/pblas/scalapack initialization
      call blacs_pinfo( iam, nprocs )
      if ( nprocs < 1 ) then
!  mpi group setup
         call mpi_comm_size( mpi_comm_world, world_size, ierr )
         call mpi_comm_rank( mpi_comm_world, my_rank, ierr )
         nprocs = world_size
         iam = my_rank
         call blacs_setup( iam, nprocs )
      end if
      call blacs_get( -1, 0, ictxt )
      call eigen_init(2)
      nprow = size_of_col
      npcol = size_of_row
      call blacs_gridinit( ictxt, 'column-major', nprow, npcol )
      call blacs_gridinfo( ictxt, nprow, npcol, myrow, mycol )

! blacs array registration
      nb = 48
      np = numroc( n, nb, myrow, 0, nprow )
      nq = numroc( n, nb, mycol, 0, npcol )
      lddz = (n-1)/nprow+1
      lddz = ((lddz-1)/nb+1)*nb+1
      lddw = (n-1)/npcol+1
      lddw = ((lddw-1)/nb+1)*nb+1
      call descinit( descz, n, n, nb, nb, 0, 0, ictxt, lddz, info )

! preparing working arrays
      trilwmin = 3*n + max( nb*( np+1 ), 3*nb )
      lwork  = max( max( 1+6*n+2*np*nq, trilwmin ) + 2*n, lddz*lddw)
      liwork = 2+7*n+8*npcol
!      allocate(work(lwork), iwork(liwork), stat=istat)
      allocate(workh(lwork), workl(lwork), iwork(liwork), stat=istat)
      if(istat /= 0) then
         print*,"memory exhausted"
         call flush(6)
         call mpi_abort( mpi_comm_world, 1, ierr )
      end if

#ifdef ARRAY_2D
      lda3 = lddz
#endif

! somehow, z must be nullified (originally next loop is not required.)
!      z(1:lddz*lddw) = 0.0d+00
      zh(1:lddz*lddw) = 0.0d+0
      zl(1:lddz*lddw) = 0.0d+0

#ifdef ARRAY_2D
!      call pdstedc( 'i', n, d(1), e(1), z(1), 1, 1, descz,
!     &              zr2d(1), lwork, iwork(1), liwork, info )
      call pddstedc( 'i', n, dh(1), dl(1), eh(1), el(1), zh(1), zl(1),
     &              1, 1, descz,
     &              zr2dh(1), zr2dl(1), lwork, iwork(1), liwork, info )
#else
!      call pdstedc( 'i', n, d(1), e(1), z(1), 1, 1, descz,
!     &              work(1), lwork, iwork(1), liwork, info )
      call pddstedc( 'i', n, dh(1), dl(1), eh(1), el(1), zh(1), zl(1), 
     &              1, 1, descz,
     &              workh(1), workl(1), lwork, iwork(1), liwork, info )
#endif

      call descinit( descw, n, n, 1, 1, 0, 0, ictxt, lddz, info )
#ifdef ARRAY_2D
!      call pdgemr2d( n, n, z, 1, 1, descz, zr2d, 1, 1, descw, ictxt )
      call pddgemr2d( n, n, zh, zl, 1, 1, descz, zr2dh, zr2dl, 1, 1, 
     &               descw, ictxt )
#else
      call pddgemr2d( n, n, zh, zl, 1, 1, descz, workh, workl, 1, 1,
     &                descw, ictxt )
#endif
      d1=mpi_wtime()
#ifdef ARRAY_2D
!     none
#else
!      call cyc2d_cyc1d(n,z,ldz,work,lddz, nprow, npcol, ierr)
      call ddcyc2d_cyc1d(n,zh,zl,ldz,workh,workl,lddz,nprow,npcol,ierr)
#endif
      d2=mpi_wtime()
!      call bcast_dbl( d(1), n, 1, mpi_comm_world )
      call ddbcast_dbl( dh(1), dl(1), n, 1, mpi_comm_world )
      d3=mpi_wtime()
      call eigen_free(2)

! freeing working arrays
!      deallocate(work, iwork)
      deallocate(workh, workl, iwork)

 
! blacs/pblas/scalapack finalize
      call blacs_gridexit( ictxt )

#ifdef DETAIL
      if(myrank==1) then
         print*,"   ddbcast_dbl   :: ",d2-d1,"(sec)"
         print*,"   ddcyc2d_cyc1d :: ",d3-d2,"(sec)"
      endif
#endif
#ifdef TIMER
      s1=mpi_wtime()
      if(myrank==1)then
         print*,"Exectime of 'ddeigen_dch' routine =",s1-s0,"(sec)"
      endif
#endif

      return
      end subroutine

