!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  n   : dimension of matrix
!  ar  : matrix(real part)
!  ai  : matrix(imaginary part)
!  w   : array for eigen value
!  lda : size of array "a"
!  m0  : coefficient of blocking
!  ifl : switch flag ( 0:eigen-value and eigen-vector )
!                    ( 1:eigen-value only )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#ifdef ARRAY_2D
!      subroutine eigen_h(n,ar2d,ai2d,lda,zr2d,zi2d,ldz,w,m0,ifl)
      subroutine ddeigen_h(n,ar2dh,ar2dl,ai2dh,ai2dl,lda,
     &                     zr2dh,zr2dl,zi2dh,zi2dl,ldz,wh,wl,m0,ifl)
#else
!      subroutine eigen_h(n,ar,ai,w,lda,m0,ifl)
      subroutine ddeigen_h(n,arh,arl,aih,ail,wh,wl,lda,m0,ifl)
#endif
!
!      use communication_h, only : eigen_init
!     &               , eigen_free
      use ddcommunication_h, only : eigen_init
     &                            , eigen_free
#ifdef ARRAY_2D
     &                , cyc1d_cyc2d
     &                , cyc2d_cyc1d
#endif
!
      implicit none
      integer, intent(in) :: n, lda, m0, ifl
#ifdef ARRAY_2D
!      real(8) :: ar2d(*),ai2d(*)
!      real(8) :: zr2d(*),zi2d(*)
      real(8) :: ar2dh(*),ar2dl(*),ai2dh(*),ai2dl(*)
      real(8) :: zr2dh(*),zr2dl(*),zi2dh(*),zi2dl(*)
      integer :: ldz
!      real(8), intent(inout) ::                       w(*)
      real(8), intent(inout) :: wh(*), wl(*)
#else
!      real(8), intent(inout) :: ar(lda,*), ai(lda,*), w(*)
      real(8), intent(inout) :: arh(lda,*), aih(lda,*), wh(*)
      real(8), intent(inout) :: arl(lda,*), ail(lda,*), wl(*)
#endif
!      real(8), pointer :: zr(:), zi(:), d(:), e(:), e2(:), tau(:)
      real(8), pointer :: zrh(:), zih(:), dh(:), eh(:), e2h(:), tauh(:)
      real(8), pointer :: zrl(:), zil(:), dl(:), el(:), e2l(:), taul(:)
      integer :: nb, nm, nma, nmz, nmz1, nmw, nmw1, info, ierr
      integer :: nprow, npcol, i_1, istat
      real(8) :: hs0, hs1
!
      include 'mpif.h'
      include 'trd.h'
!
      call eigen_init(2)
      NPROW = size_of_col
      NPCOL = size_of_row
      call eigen_free(0)
!
      NB  = 48
c      NB  = m0
      nmz = ((n-1)/NPROW+1)
      nmz = ((nmz-1)/NB+1)*NB+1
      nmw = ((n-1)/NPCOL+1)
      nmw = ((nmw-1)/NB+1)*NB+1

      nm=(n/2)*2+1

      nmz1=nm
      nmw1=((n-1)/nprocs+1)

      allocate(
!     &               zr(MAX(nmz*nmw+nm+1,nmz1*nmw1)),
!     &               zi(MAX(nmz*nmw+nm+1,nmz1*nmw1)),
!     &               d(nm),e(nm),e2(nm),tau(2*nm),
     &               zrh(MAX(nmz*nmw+nm+1,nmz1*nmw1)),
     &               zrl(MAX(nmz*nmw+nm+1,nmz1*nmw1)),
     &               zih(MAX(nmz*nmw+nm+1,nmz1*nmw1)),
     &               zil(MAX(nmz*nmw+nm+1,nmz1*nmw1)),
     &               dh(nm),eh(nm),e2h(nm),tauh(2*nm),
     &               dl(nm),el(nm),e2l(nm),taul(2*nm),
     &               stat=istat)
      if(istat.ne.0) then
           if(myrank==1) print*,"Memory exhausted"
           call mpi_abort(mpi_comm_world, 1, ierr)
      endif
*
         hs0=MPI_Wtime()
         nma=lda
!----
#ifdef ARRAY_2D
!         call eigen_hrd(n,ar2d,ai2d,nma,d,e,e2,tau,nm,m0)
         call ddeigen_hrd(n,ar2dh,ar2dl,ai2dh,ai2dl,nma,
     &                  dh,dl,eh,el,e2h,e2l,tauh,taul,nm,m0)
#else
!         call eigen_hrd(n,ar,ai,nma,d,e,e2,tau,nm,m0,zr,zi)
         call ddeigen_hrd(n,arh,arl,aih,ail,nma,
     &                    dh,dl,eh,el,e2h,e2l,tauh,taul,nm,m0,
     &                    zrh,zrl,zih,zil)
#endif
!----
!         e2(1:n)=e(1:n)
!         w(1:n)=d(1:n)
         e2h(1:n) = eh(1:n)
         e2l(1:n) = el(1:n)
         wh(1:n) = dh(1:n)
         wl(1:n) = dl(1:n)

!----
#ifdef ARRAY_2D
!         call eigen_dch(n,w,e2(2),zr,nmz1,info,zr2d,ldz)
         call ddeigen_dch(n,wh,wl,e2h(2),e2l(2),zrh,zrl,nmz1,info,
     &                    zr2dh,zr2dl,ldz)
#else
!         call eigen_dch(n,w,e2(2),zr,nmz1,info)
         call ddeigen_dch(n,wh,wl,e2h(2),e2l(2),zrh,zrl,nmz1,info)
#endif
!----
         nma=lda
#ifdef ARRAY_2D
!         call eigen_hbk(ar2d,ai2d,lda,zr2d,zi2d,ldz
!     &                           , zr,zi,nmz1, tau,nm,  n)
         call ddeigen_hbk(ar2dh,ar2dl,ai2dh,ai2dl,lda,
     &                    zr2dh,zr2dl,zi2dh,zi2dl,ldz,
     &                    zrh,zrl,zih,zil,nmz1,tauh,taul,nm,n)
#else
!         call ddeigen_hbk(ar,ai,nma, zr,zi,nmz1, tau,nm,  n)
         call ddeigen_hbk(arh,arl,aih,ail,nma,zrh,zrl,zih,zil,nmz1,
     &                    tauh,taul,nm,n)
#endif
!----
#ifdef TIMER
         hs1=MPI_Wtime()
         if(myrank==1)then
            print*," "
            print*,"Total Execution Time of eigen_h =",hs1-hs0,"(sec)"
         endif
#endif
#ifdef ARRAY_2D
#else
*
         do i_1=1,(n-1)/nprocs+1
!           ar(1:n,i_1)=zr(1+nmz1*(i_1-1):n+nmz1*(i_1-1))
!           ai(1:n,i_1)=zi(1+nmz1*(i_1-1):n+nmz1*(i_1-1))
         arh(1:n,i_1) = zrh(1+nmz1*(i_1-1):n+nmz1*(i_1-1))
         arl(1:n,i_1) = zrl(1+nmz1*(i_1-1):n+nmz1*(i_1-1))
         aih(1:n,i_1) = zih(1+nmz1*(i_1-1):n+nmz1*(i_1-1))
         ail(1:n,i_1) = zil(1+nmz1*(i_1-1):n+nmz1*(i_1-1))
         enddo
#endif

!         deallocate(zr,zi,d,e,e2,tau)
         deallocate(zrh,zrl,zih,zil,dh,dl,eh,el,e2h,e2l,tauh,taul)
*
      return
      end

