*> \brief \b DLAED2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED2( K, N, N1, D, Q, LDQ, INDXQ, RHO, Z, DLAMDA, W,
*                          Q2, INDX, INDXC, INDXP, COLTYP, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, K, LDQ, N, N1
*       DOUBLE PRECISION   RHO
*       ..
*       .. Array Arguments ..
*       INTEGER            COLTYP( * ), INDX( * ), INDXC( * ), INDXP( * ),
*      $                   INDXQ( * )
*       DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), Q2( * ),
*      $                   W( * ), Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAED2 merges the two sets of eigenvalues together into a single
*> sorted set.  Then it tries to deflate the size of the problem.
*> There are two ways in which deflation can occur:  when two or more
*> eigenvalues are close together or if there is a tiny entry in the
*> Z vector.  For each such occurrence the order of the related secular
*> equation problem is reduced by one.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[out] K
*> \verbatim
*>          K is INTEGER
*>         The number of non-deflated eigenvalues, and the order of the
*>         related secular equation. 0 <= K <=N.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>         The dimension of the symmetric tridiagonal matrix.  N >= 0.
*> \endverbatim
*>
*> \param[in] N1
*> \verbatim
*>          N1 is INTEGER
*>         The location of the last eigenvalue in the leading sub-matrix.
*>         min(1,N) <= N1 <= N/2.
*> \endverbatim
*>
*> \param[in,out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>         On entry, D contains the eigenvalues of the two submatrices to
*>         be combined.
*>         On exit, D contains the trailing (N-K) updated eigenvalues
*>         (those which were deflated) sorted into increasing order.
*> \endverbatim
*>
*> \param[in,out] Q
*> \verbatim
*>          Q is DOUBLE PRECISION array, dimension (LDQ, N)
*>         On entry, Q contains the eigenvectors of two submatrices in
*>         the two square blocks with corners at (1,1), (N1,N1)
*>         and (N1+1, N1+1), (N,N).
*>         On exit, Q contains the trailing (N-K) updated eigenvectors
*>         (those which were deflated) in its last N-K columns.
*> \endverbatim
*>
*> \param[in] LDQ
*> \verbatim
*>          LDQ is INTEGER
*>         The leading dimension of the array Q.  LDQ >= max(1,N).
*> \endverbatim
*>
*> \param[in,out] INDXQ
*> \verbatim
*>          INDXQ is INTEGER array, dimension (N)
*>         The permutation which separately sorts the two sub-problems
*>         in D into ascending order.  Note that elements in the second
*>         half of this permutation must first have N1 added to their
*>         values. Destroyed on exit.
*> \endverbatim
*>
*> \param[in,out] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>         On entry, the off-diagonal element associated with the rank-1
*>         cut which originally split the two submatrices which are now
*>         being recombined.
*>         On exit, RHO has been modified to the value required by
*>         DLAED3.
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension (N)
*>         On entry, Z contains the updating vector (the last
*>         row of the first sub-eigenvector matrix and the first row of
*>         the second sub-eigenvector matrix).
*>         On exit, the contents of Z have been destroyed by the updating
*>         process.
*> \endverbatim
*>
*> \param[out] DLAMDA
*> \verbatim
*>          DLAMDA is DOUBLE PRECISION array, dimension (N)
*>         A copy of the first K eigenvalues which will be used by
*>         DLAED3 to form the secular equation.
*> \endverbatim
*>
*> \param[out] W
*> \verbatim
*>          W is DOUBLE PRECISION array, dimension (N)
*>         The first k values of the final deflation-altered z-vector
*>         which will be passed to DLAED3.
*> \endverbatim
*>
*> \param[out] Q2
*> \verbatim
*>          Q2 is DOUBLE PRECISION array, dimension (N1**2+(N-N1)**2)
*>         A copy of the first K eigenvectors which will be used by
*>         DLAED3 in a matrix multiply (DGEMM) to solve for the new
*>         eigenvectors.
*> \endverbatim
*>
*> \param[out] INDX
*> \verbatim
*>          INDX is INTEGER array, dimension (N)
*>         The permutation used to sort the contents of DLAMDA into
*>         ascending order.
*> \endverbatim
*>
*> \param[out] INDXC
*> \verbatim
*>          INDXC is INTEGER array, dimension (N)
*>         The permutation used to arrange the columns of the deflated
*>         Q matrix into three groups:  the first group contains non-zero
*>         elements only at and above N1, the second contains
*>         non-zero elements only below N1, and the third is dense.
*> \endverbatim
*>
*> \param[out] INDXP
*> \verbatim
*>          INDXP is INTEGER array, dimension (N)
*>         The permutation used to place deflated values of D at the end
*>         of the array.  INDXP(1:K) points to the nondeflated D-values
*>         and INDXP(K+1:N) points to the deflated eigenvalues.
*> \endverbatim
*>
*> \param[out] COLTYP
*> \verbatim
*>          COLTYP is INTEGER array, dimension (N)
*>         During execution, a label which will indicate which of the
*>         following types a column in the Q2 matrix is:
*>         1 : non-zero in the upper half only;
*>         2 : dense;
*>         3 : non-zero in the lower half only;
*>         4 : deflated.
*>         On exit, COLTYP(i) is the number of columns of type i,
*>         for i=1 to 4 only.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit.
*>          < 0:  if INFO = -i, the i-th argument had an illegal value.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Contributors:
*  ==================
*>
*> Jeff Rutter, Computer Science Division, University of California
*> at Berkeley, USA \n
*>  Modified by Francoise Tisseur, University of Tennessee
*>
*  =====================================================================
!      SUBROUTINE DLAED2( K, N, N1, D, Q, LDQ, INDXQ, RHO, Z, DLAMDA, W,
!     $                   Q2, INDX, INDXC, INDXP, COLTYP, INFO )
      SUBROUTINE DDLAED2( K, N, N1, Dh, Dl, Qh, Ql, LDQ, INDXQ, 
     $                   RHOh, RHOl, Zh, Zl, DLAMDAh, DLAMDAl, 
     $                   Wh, Wl, Q2h, Q2l, 
     $                   INDX, INDXC, INDXP, COLTYP, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, K, LDQ, N, N1
!      DOUBLE PRECISION   RHO
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
      INTEGER            COLTYP( * ), INDX( * ), INDXC( * ), INDXP( * ),
     $                   INDXQ( * )
!      DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), Q2( * ),
!     $                   W( * ), Z( * )
      double precision Dh(*), Dl(*)
      double precision DLAMDAh(*), DLAMDAl(*)
      double precision Qh(LDQ,*), Ql(LDQ,*)
      double precision Q2h(*), Q2l(*)
      double precision Wh(*), Wl(*)
      double precision Zh(*), Zl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   MONE, ZERO, ONE, TWO, EIGHT
!      PARAMETER          ( MONE = -1.0D0, ZERO = 0.0D0, ONE = 1.0D0,
!     $                   TWO = 2.0D0, EIGHT = 8.0D0 )
      double precision MONEh, MONEl
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision EIGHTh, EIGHTl

*     ..
*     .. Local Arrays ..
      INTEGER            CTOT( 4 ), PSM( 4 )
*     ..
*     .. Local Scalars ..
      INTEGER            CT, I, IMAX, IQ1, IQ2, J, JMAX, JS, K2, N1P1,
     $                   N2, NJ, PJ
!      DOUBLE PRECISION   C, EPS, S, T, TAU, TOL
      double precision Ch, Cl
      double precision EPSh, EPSl
      double precision Sh, Sl
      double precision Th, Tl
      double precision TAUh, TAUl
      double precision TOLh, TOLl

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. External Functions ..
      INTEGER            IDAMAX
!      DOUBLE PRECISION   DLAMCH, DLAPY2
      EXTERNAL           IDAMAX
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDCOPY, DDLACPY, DDLAMRG, DDROT, DDSCAL, 
     $                   XERBLA, DDLAMCH, DDLAPY2
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT

      MONEh = -1.0d0
      MONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0d0
      TWOl = 0.0d+0
      EIGHTh = 8.0d0
      EIGHTl = 0.0d+0
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDQ.LT.MAX( 1, N ) ) THEN
         INFO = -6
      ELSE IF( MIN( 1, ( N / 2 ) ).GT.N1 .OR. ( N / 2 ).LT.N1 ) THEN
         INFO = -3
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DDLAED2', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
      N2 = N - N1
      N1P1 = N1 + 1
*
      IF(
!         RHO.LT.ZERO
!       (RHO .lt. ZERO)
     + ((RHOh .lt. ZEROh) .or.
     +  ((RHOh .eq. ZEROh) .and.
     +   (RHOl .lt. ZEROl)))
     $   ) THEN
!         CALL DSCAL( N2, MONE, Z( N1P1 ), 1 )
         CALL DDSCAL( N2, MONEh, MONEl, Zh( N1P1 ), Zl( N1P1 ), 1 )
      END IF
*
*     Normalize z so that norm(z) = 1.  Since z is the concatenation of
*     two normalized vectors, norm2(z) = sqrt(2).
*
!      T = ONE / SQRT( TWO )
!<<<  tmp1 = sqrt(TWO)
      if (TWOh.eq.0.0D+0.and.TWOl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      endif
      if (TWOh.lt.0.0d+0.or.
     +   (TWOh.eq.0.0d+0.and.TWOl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(TWOh)
      T_ax = TWOh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           TWOh,TWOl,T_z1,T_z2)
          T_p1 = TWOh - T_z1
          T_p2 = T_p1 - TWOh
          T_t2l = (TWOh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + TWOl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = sqrt(TWO)
!<<<  T = ONE / tmp1
      T_q1 = ONEh / tmp1h
      Th = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Tl = ((T_p1*T_r1
     +       - Th)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Tl = Tl + (tmp1l*T_q1)
      T_s1 = Th + Tl
      Tl = Tl - (T_s1 - Th)
      Th = T_s1
      T_s1 = ONEh - Th
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh 
     +         - (T_s1 - T_r1))
     +         - (Th + T_r1)
      T_s2 = T_s2 - Tl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      Th = T_q1 + T_q2
      Tl = T_q2
     +      - (Th - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  T = ONE / tmp1
!      CALL DSCAL( N, T, Z, 1 )
      CALL DDSCAL( N, Th, Tl, Zh, Zl, 1 )
*
*     RHO = ABS( norm(z)**2 * RHO )
*
!      RHO = ABS( TWO*RHO )
!<<<  tmp1 = TWO * RHO
      tmp1h = TWOh * RHOh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RHOh
      T_r1 = T_u1
     +         - (T_u1 - RHOh)
      T_r2 = RHOh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*RHOl
     +      + TWOl*RHOh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * RHO
!<<<  RHO = abs(tmp1)
      if (tmp1h .lt. 0.0d+0) then
        RHOh = -tmp1h
        RHOl = -tmp1l
      elseif (
     +  (tmp1h .eq. 0.0d+0).and.
     +  (tmp1l .lt. 0.0d+0)) then
        RHOh = tmp1h
        RHOl = -tmp1l
      else
        RHOh = tmp1h
        RHOl = tmp1l
      endif
!>>>  RHO = abs(tmp1)
*
*     Sort the eigenvalues into increasing order
*
      DO 10 I = N1P1, N
         INDXQ( I ) = INDXQ( I ) + N1
   10 CONTINUE
*
*     re-integrate the deflated parts from the last pass
*
      DO 20 I = 1, N
!         DLAMDA( I ) = D( INDXQ( I ) )
      DLAMDAh(I) = Dh(INDXQ(I))
      DLAMDAl(I) = Dl(INDXQ(I))
   20 CONTINUE
!      CALL DLAMRG( N1, N2, DLAMDA, 1, 1, INDXC )
      CALL DDLAMRG( N1, N2, DLAMDAh, DLAMDAl, 1, 1, INDXC )
      DO 30 I = 1, N
         INDX( I ) = INDXQ( INDXC( I ) )
   30 CONTINUE
*
*     Calculate the allowable deflation tolerance
*
      IMAX = IDAMAX( N, Zh, 1 )
      JMAX = IDAMAX( N, Dh, 1 )
!      EPS = DLAMCH( 'Epsilon' )
      CALL DDLAMCH( 'Epsilon', EPSh, EPSl )
!      TOL = EIGHT*EPS*MAX( ABS( D( JMAX ) ), ABS( Z( IMAX ) ) )
!<<<  tmp1 = abs(D(JMAX))
      if (Dh(JMAX) .lt. 0.0d+0) then
        tmp1h = -Dh(JMAX)
        tmp1l = -Dl(JMAX)
      elseif (
     +  (Dh(JMAX) .eq. 0.0d+0).and.
     +  (Dl(JMAX) .lt. 0.0d+0)) then
        tmp1h = Dh(JMAX)
        tmp1l = -Dl(JMAX)
      else
        tmp1h = Dh(JMAX)
        tmp1l = Dl(JMAX)
      endif
!>>>  tmp1 = abs(D(JMAX))
!<<<  tmp2 = abs(Z(IMAX))
      if (Zh(IMAX) .lt. 0.0d+0) then
        tmp2h = -Zh(IMAX)
        tmp2l = -Zl(IMAX)
      elseif (
     +  (Zh(IMAX) .eq. 0.0d+0).and.
     +  (Zl(IMAX) .lt. 0.0d+0)) then
        tmp2h = Zh(IMAX)
        tmp2l = -Zl(IMAX)
      else
        tmp2h = Zh(IMAX)
        tmp2l = Zl(IMAX)
      endif
!>>>  tmp2 = abs(Z(IMAX))
      IF(
!       (tmp1 .gt. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .gt. tmp2l)))
     $  )THEN
      tmp3h = tmp1h
      tmp3l = tmp1l
      ELSE
      tmp3h = tmp2h
      tmp3l = tmp2l
      END IF
!<<<  tmp4 = EPS * tmp3
      tmp4h = EPSh * tmp3h
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (EPSh*tmp3l
     +      + EPSl*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = EPS * tmp3
!<<<  TOL = EIGHT * tmp4
      TOLh = EIGHTh * tmp4h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      TOLl = ((T_p1*T_r1
     +        - TOLh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TOLl = TOLl
     +      + (EIGHTh*tmp4l
     +      + EIGHTl*tmp4h)
      T_s1 = TOLh + TOLl
      TOLl = TOLl - (T_s1 - TOLh)
      TOLh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TOL = EIGHT * tmp4
*
*     If the rank-1 modifier is small enough, no more needs to be done
*     except to reorganize Q so that its columns correspond with the
*     elements in D.
*
!<<<  tmp1 = abs(Z(IMAX))
      if (Zh(IMAX) .lt. 0.0d+0) then
        tmp1h = -Zh(IMAX)
        tmp1l = -Zl(IMAX)
      elseif (
     +  (Zh(IMAX) .eq. 0.0d+0).and.
     +  (Zl(IMAX) .lt. 0.0d+0)) then
        tmp1h = Zh(IMAX)
        tmp1l = -Zl(IMAX)
      else
        tmp1h = Zh(IMAX)
        tmp1l = Zl(IMAX)
      endif
!>>>  tmp1 = abs(Z(IMAX))
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
      IF(
!         RHO*ABS( Z( IMAX ) ).LE.TOL
!       (tmp2 .le. TOL)
     + ((tmp2h .lt. TOLh) .or.
     +  ((tmp2h .eq. TOLh) .and.
     +   (tmp2l .le. TOLl)))
     $    ) THEN
         K = 0
         IQ2 = 1
         DO 40 J = 1, N
            I = INDX( J )
!            CALL DCOPY( N, Q( 1, I ), 1, Q2( IQ2 ), 1 )
            CALL DDCOPY( N, Qh( 1, I ), Ql( 1, I ), 1, 
     $                   Q2h( IQ2 ), Q2l( IQ2 ), 1 )
!            DLAMDA( J ) = D( I )
      DLAMDAh(J) = Dh(I)
      DLAMDAl(J) = Dl(I)
            IQ2 = IQ2 + N
   40    CONTINUE
!         CALL DLACPY( 'A', N, N, Q2, N, Q, LDQ )
         CALL DDLACPY( 'A', N, N, Q2h, Q2l, N, Qh, Ql, LDQ )
!         CALL DCOPY( N, DLAMDA, 1, D, 1 )
         CALL DDCOPY( N, DLAMDAh, DLAMDAl, 1, Dh, Dl, 1 )
         GO TO 190
      END IF
*
*     If there are multiple eigenvalues then the problem deflates.  Here
*     the number of equal eigenvalues are found.  As each equal
*     eigenvalue is found, an elementary reflector is computed to rotate
*     the corresponding eigensubspace so that the corresponding
*     components of Z are zero in this new basis.
*
      DO 50 I = 1, N1
         COLTYP( I ) = 1
   50 CONTINUE
      DO 60 I = N1P1, N
         COLTYP( I ) = 3
   60 CONTINUE
*
*
      K = 0
      K2 = N + 1
      DO 70 J = 1, N
         NJ = INDX( J )
!<<<  tmp1 = abs(Z(NJ))
      if (Zh(NJ) .lt. 0.0d+0) then
        tmp1h = -Zh(NJ)
        tmp1l = -Zl(NJ)
      elseif (
     +  (Zh(NJ) .eq. 0.0d+0).and.
     +  (Zl(NJ) .lt. 0.0d+0)) then
        tmp1h = Zh(NJ)
        tmp1l = -Zl(NJ)
      else
        tmp1h = Zh(NJ)
        tmp1l = Zl(NJ)
      endif
!>>>  tmp1 = abs(Z(NJ))
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
         IF(
!            RHO*ABS( Z( NJ ) ).LE.TOL
!       (tmp2 .le. TOL)
     + ((tmp2h .lt. TOLh) .or.
     +  ((tmp2h .eq. TOLh) .and.
     +   (tmp2l .le. TOLl)))
     $      ) THEN
*
*           Deflate due to small z component.
*
            K2 = K2 - 1
            COLTYP( NJ ) = 4
            INDXP( K2 ) = NJ
            IF( J.EQ.N )
     $         GO TO 100
         ELSE
            PJ = NJ
            GO TO 80
         END IF
   70 CONTINUE
   80 CONTINUE
      J = J + 1
      NJ = INDX( J )
      IF( J.GT.N )
     $   GO TO 100
!<<<  tmp1 = abs(Z(NJ))
      if (Zh(NJ) .lt. 0.0d+0) then
        tmp1h = -Zh(NJ)
        tmp1l = -Zl(NJ)
      elseif (
     +  (Zh(NJ) .eq. 0.0d+0).and.
     +  (Zl(NJ) .lt. 0.0d+0)) then
        tmp1h = Zh(NJ)
        tmp1l = -Zl(NJ)
      else
        tmp1h = Zh(NJ)
        tmp1l = Zl(NJ)
      endif
!>>>  tmp1 = abs(Z(NJ))
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
      IF(
!         RHO*ABS( Z( NJ ) ).LE.TOL
!       (tmp2 .le. TOL)
     + ((tmp2h .lt. TOLh) .or.
     +  ((tmp2h .eq. TOLh) .and.
     +   (tmp2l .le. TOLl)))
     $   ) THEN
*
*        Deflate due to small z component.
*
         K2 = K2 - 1
         COLTYP( NJ ) = 4
         INDXP( K2 ) = NJ
      ELSE
*
*        Check if eigenvalues are close enough to allow deflation.
*
!         S = Z( PJ )
      Sh = Zh(PJ)
      Sl = Zl(PJ)
!         C = Z( NJ )
      Ch = Zh(NJ)
      Cl = Zl(NJ)
*
*        Find sqrt(a**2+b**2) without overflow or
*        destructive underflow.
*
!         TAU = DLAPY2( C, S )
         CALL DDLAPY2(Ch,Cl,Sh,Sl,TAUh,TAUl)
!         T = D( NJ ) - D( PJ )
!<<<  T = D(NJ) - D(PJ)
      T_p1 = Dh(NJ) - Dh(PJ)
      T_p2 = T_p1 - Dh(NJ)
      Th = T_p1
      Tl = (Dh(NJ) - (T_p1 - T_p2))
     +         - (Dh(PJ) + T_p2)
      Tl = Tl + Dl(NJ) - Dl(PJ)
      T_s1 = Th + Tl
      Tl = Tl - (T_s1 - Th)
      Th = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  T = D(NJ) - D(PJ)
!         C = C / TAU
!<<<  C = C / TAU
      T_q1 = Ch / TAUh
      T_ch = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TAUl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Ch - T_ch
      T_r1 = T_s1 - Ch
      T_s2 = (Ch
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Cl
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = C / TAU
!         S = -S / TAU
!<<<  S = S / TAU
      T_q1 = Sh / TAUh
      T_ch = TAUh * T_q1
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TAUl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Sh - T_ch
      T_r1 = T_s1 - Sh
      T_s2 = (Sh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Sl
      T_q2 = (T_s1 + T_s2)
     +         / TAUh
      Sh = T_q1 + T_q2
      Sl = T_q2
     +      - (Sh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S = S / TAU
!<<<  S = -S
      Sh = -Sh
      Sl = -Sl
!>>>  S = -S
!<<<  tmp1 = T * C
      tmp1h = Th * Ch
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Th*Cl
     +      + Tl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = T * C
!<<<  tmp2 = tmp1 * S
      tmp2h = tmp1h * Sh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Sl
     +      + tmp1l*Sh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * S
!<<<  tmp3 = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        tmp3h = -tmp2h
        tmp3l = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        tmp3h = tmp2h
        tmp3l = -tmp2l
      else
        tmp3h = tmp2h
        tmp3l = tmp2l
      endif
!>>>  tmp3 = abs(tmp2)
         IF(
!            ABS( T*C*S ).LE.TOL
!       (tmp3 .le. TOL)
     + ((tmp3h .lt. TOLh) .or.
     +  ((tmp3h .eq. TOLh) .and.
     +   (tmp3l .le. TOLl)))
     $       ) THEN
*
*           Deflation is possible.
*
!            Z( NJ ) = TAU
      Zh(NJ) = TAUh
      Zl(NJ) = TAUl
!            Z( PJ ) = ZERO
      Zh(PJ) = ZEROh
      Zl(PJ) = ZEROl
            IF( COLTYP( NJ ).NE.COLTYP( PJ ) )
     $         COLTYP( NJ ) = 2
            COLTYP( PJ ) = 4
!            CALL DROT( N, Q( 1, PJ ), 1, Q( 1, NJ ), 1, C, S )
            CALL DDROT( N, Qh( 1, PJ ), Ql( 1, PJ ), 1, 
     $                  Qh( 1, NJ ), Ql( 1, NJ ), 1, Ch, Cl, Sh, Sl )
!            T = D( PJ )*C**2 + D( NJ )*S**2
!<<<  tmp1 = C * C
      tmp1h = Ch * Ch
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*Cl
     +      + Cl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * C
!<<<  tmp2 = D(PJ) * tmp1
      tmp2h = Dh(PJ) * tmp1h
      T_u1 = 134217729.0 * Dh(PJ)
      T_p1 = T_u1
     +         - (T_u1 - Dh(PJ))
      T_p2 = Dh(PJ) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Dh(PJ)*tmp1l
     +      + Dl(PJ)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = D(PJ) * tmp1
!<<<  tmp3 = S * S
      tmp3h = Sh * Sh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Sh*Sl
     +      + Sl*Sh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = S * S
!<<<  tmp4 = D(NJ) * tmp3
      tmp4h = Dh(NJ) * tmp3h
      T_u1 = 134217729.0 * Dh(NJ)
      T_p1 = T_u1
     +         - (T_u1 - Dh(NJ))
      T_p2 = Dh(NJ) - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (Dh(NJ)*tmp3l
     +      + Dl(NJ)*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = D(NJ) * tmp3
!<<<  T = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Th = T_p1
      Tl = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Tl = Tl + tmp2l + tmp4l
      T_s1 = Th + Tl
      Tl = Tl - (T_s1 - Th)
      Th = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  T = tmp2 + tmp4
!            D( NJ ) = D( PJ )*S**2 + D( NJ )*C**2
!<<<  tmp1 = S * S
      tmp1h = Sh * Sh
      T_u1 = 134217729.0 * Sh
      T_p1 = T_u1
     +         - (T_u1 - Sh)
      T_p2 = Sh - T_p1
      T_u1 = 134217729.0 * Sh
      T_r1 = T_u1
     +         - (T_u1 - Sh)
      T_r2 = Sh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Sh*Sl
     +      + Sl*Sh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = S * S
!<<<  tmp2 = D(PJ) * tmp1
      tmp2h = Dh(PJ) * tmp1h
      T_u1 = 134217729.0 * Dh(PJ)
      T_p1 = T_u1
     +         - (T_u1 - Dh(PJ))
      T_p2 = Dh(PJ) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Dh(PJ)*tmp1l
     +      + Dl(PJ)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = D(PJ) * tmp1
!<<<  tmp3 = C * C
      tmp3h = Ch * Ch
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Ch*Cl
     +      + Cl*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = C * C
!<<<  tmp4 = D(NJ) * tmp3
      tmp4h = Dh(NJ) * tmp3h
      T_u1 = 134217729.0 * Dh(NJ)
      T_p1 = T_u1
     +         - (T_u1 - Dh(NJ))
      T_p2 = Dh(NJ) - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (Dh(NJ)*tmp3l
     +      + Dl(NJ)*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = D(NJ) * tmp3
!<<<  D(NJ) = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Dh(NJ) = T_p1
      Dl(NJ) = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Dl(NJ) = Dl(NJ) + tmp2l + tmp4l
      T_s1 = Dh(NJ) + Dl(NJ)
      Dl(NJ) = Dl(NJ) - (T_s1 - Dh(NJ))
      Dh(NJ) = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  D(NJ) = tmp2 + tmp4

!            D( PJ ) = T
      Dh(PJ) = Th
      Dl(PJ) = Tl
            K2 = K2 - 1
            I = 1
   90       CONTINUE
            IF( K2+I.LE.N ) THEN
               IF(
!                  D( PJ ).LT.D( INDXP( K2+I ) )
!       (D(PJ) .lt. D(INDXP(K2+I)))
     + ((Dh(PJ) .lt. Dh(INDXP(K2+I))) .or.
     +  ((Dh(PJ) .eq. Dh(INDXP(K2+I))) .and.
     +   (Dl(PJ) .lt. Dl(INDXP(K2+I)))))
     $            ) THEN
                  INDXP( K2+I-1 ) = INDXP( K2+I )
                  INDXP( K2+I ) = PJ
                  I = I + 1
                  GO TO 90
               ELSE
                  INDXP( K2+I-1 ) = PJ
               END IF
            ELSE
               INDXP( K2+I-1 ) = PJ
            END IF
            PJ = NJ
         ELSE
            K = K + 1
!            DLAMDA( K ) = D( PJ )
      DLAMDAh(K) = Dh(PJ)
      DLAMDAl(K) = Dl(PJ)
!            W( K ) = Z( PJ )
      Wh(K) = Zh(PJ)
      Wl(K) = Zl(PJ)
            INDXP( K ) = PJ
            PJ = NJ
         END IF
      END IF
      GO TO 80
  100 CONTINUE
*
*     Record the last eigenvalue.
*
      K = K + 1
!      DLAMDA( K ) = D( PJ )
      DLAMDAh(K) = Dh(PJ)
      DLAMDAl(K) = Dl(PJ)
!      W( K ) = Z( PJ )
      Wh(K) = Zh(PJ)
      Wl(K) = Zl(PJ)
      INDXP( K ) = PJ
*
*     Count up the total number of the various types of columns, then
*     form a permutation which positions the four column types into
*     four uniform groups (although one or more of these groups may be
*     empty).
*
      DO 110 J = 1, 4
         CTOT( J ) = 0
  110 CONTINUE
      DO 120 J = 1, N
         CT = COLTYP( J )
         CTOT( CT ) = CTOT( CT ) + 1
  120 CONTINUE
*
*     PSM(*) = Position in SubMatrix (of types 1 through 4)
*
      PSM( 1 ) = 1
      PSM( 2 ) = 1 + CTOT( 1 )
      PSM( 3 ) = PSM( 2 ) + CTOT( 2 )
      PSM( 4 ) = PSM( 3 ) + CTOT( 3 )
      K = N - CTOT( 4 )
*
*     Fill out the INDXC array so that the permutation which it induces
*     will place all type-1 columns first, all type-2 columns next,
*     then all type-3's, and finally all type-4's.
*
      DO 130 J = 1, N
         JS = INDXP( J )
         CT = COLTYP( JS )
         INDX( PSM( CT ) ) = JS
         INDXC( PSM( CT ) ) = J
         PSM( CT ) = PSM( CT ) + 1
  130 CONTINUE
*
*     Sort the eigenvalues and corresponding eigenvectors into DLAMDA
*     and Q2 respectively.  The eigenvalues/vectors which were not
*     deflated go into the first K slots of DLAMDA and Q2 respectively,
*     while those which were deflated go into the last N - K slots.
*
      I = 1
      IQ1 = 1
      IQ2 = 1 + ( CTOT( 1 )+CTOT( 2 ) )*N1
      DO 140 J = 1, CTOT( 1 )
         JS = INDX( I )
!         CALL DCOPY( N1, Q( 1, JS ), 1, Q2( IQ1 ), 1 )
         CALL DDCOPY( N1, Qh( 1, JS ), Ql( 1, JS ), 1, 
     $                Q2h( IQ1 ), Q2l( IQ1 ), 1 )
!         Z( I ) = D( JS )
      Zh(I) = Dh(JS)
      Zl(I) = Dl(JS)
         I = I + 1
         IQ1 = IQ1 + N1
  140 CONTINUE
*
      DO 150 J = 1, CTOT( 2 )
         JS = INDX( I )
!         CALL DCOPY( N1, Q( 1, JS ), 1, Q2( IQ1 ), 1 )
         CALL DDCOPY( N1, Qh( 1, JS ), Ql( 1, JS ), 1, 
     $                Q2h( IQ1 ), Q2l( IQ1 ), 1 )
!         CALL DCOPY( N2, Q( N1+1, JS ), 1, Q2( IQ2 ), 1 )
         CALL DDCOPY( N2, Qh( N1+1, JS ), Ql( N1+1, JS ), 1, 
     $                Q2h( IQ2 ), Q2l( IQ2 ), 1 )
!         Z( I ) = D( JS )
      Zh(I) = Dh(JS)
      Zl(I) = Dl(JS)
         I = I + 1
         IQ1 = IQ1 + N1
         IQ2 = IQ2 + N2
  150 CONTINUE
*
      DO 160 J = 1, CTOT( 3 )
         JS = INDX( I )
!         CALL DCOPY( N2, Q( N1+1, JS ), 1, Q2( IQ2 ), 1 )
         CALL DDCOPY( N2, Qh( N1+1, JS ), Ql( N1+1, JS ), 1, 
     $                Q2h( IQ2 ), Q2l( IQ2 ), 1 )
!         Z( I ) = D( JS )
      Zh(I) = Dh(JS)
      Zl(I) = Dl(JS)
         I = I + 1
         IQ2 = IQ2 + N2
  160 CONTINUE
*
      IQ1 = IQ2
      DO 170 J = 1, CTOT( 4 )
         JS = INDX( I )
!         CALL DCOPY( N, Q( 1, JS ), 1, Q2( IQ2 ), 1 )
         CALL DDCOPY( N, Qh( 1, JS ), Ql( 1, JS ), 1, 
     $                Q2h( IQ2 ), Q2l( IQ2 ), 1 )
         IQ2 = IQ2 + N
!         Z( I ) = D( JS )
      Zh(I) = Dh(JS)
      Zl(I) = Dl(JS)
         I = I + 1
  170 CONTINUE
*
*     The deflated eigenvalues and their corresponding vectors go back
*     into the last N - K slots of D and Q respectively.
*
      IF( K.LT.N ) THEN
!         CALL DLACPY( 'A', N, CTOT( 4 ), Q2( IQ1 ), N, 
!     $                Q( 1, K+1 ), LDQ )
         CALL DDLACPY( 'A', N, CTOT( 4 ), Q2h( IQ1 ), Q2l( IQ1 ), N, 
     $                Qh( 1, K+1 ), Ql( 1, K+1 ), LDQ )
!         CALL DCOPY( N-K, Z( K+1 ), 1, D( K+1 ), 1 )
         CALL DDCOPY( N-K, Zh( K+1 ), Zl( K+1 ), 1, 
     $                Dh( K+1 ), Dl( K+1 ), 1 )
      END IF         
*
*     Copy CTOT into COLTYP for referencing in DLAED3.
*
      DO 180 J = 1, 4
         COLTYP( J ) = CTOT( J )
  180 CONTINUE
*
  190 CONTINUE
      RETURN
*
*     End of DLAED2
*
      END
