*> \brief \b DLAED3
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED3 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed3.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed3.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed3.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED3( K, N, N1, D, Q, LDQ, RHO, DLAMDA, Q2, INDX,
*                          CTOT, W, S, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, K, LDQ, N, N1
*       DOUBLE PRECISION   RHO
*       ..
*       .. Array Arguments ..
*       INTEGER            CTOT( * ), INDX( * )
*       DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), Q2( * ),
*      $                   S( * ), W( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAED3 finds the roots of the secular equation, as defined by the
*> values in D, W, and RHO, between 1 and K.  It makes the
*> appropriate calls to DLAED4 and then updates the eigenvectors by
*> multiplying the matrix of eigenvectors of the pair of eigensystems
*> being combined by the matrix of eigenvectors of the K-by-K system
*> which is solved here.
*>
*> This code makes very mild assumptions about floating point
*> arithmetic. It will work on machines with a guard digit in
*> add/subtract, or on those binary machines without guard digits
*> which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or Cray-2.
*> It could conceivably fail on hexadecimal or decimal machines
*> without guard digits, but we know of none.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] K
*> \verbatim
*>          K is INTEGER
*>          The number of terms in the rational function to be solved by
*>          DLAED4.  K >= 0.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of rows and columns in the Q matrix.
*>          N >= K (deflation may result in N>K).
*> \endverbatim
*>
*> \param[in] N1
*> \verbatim
*>          N1 is INTEGER
*>          The location of the last eigenvalue in the leading submatrix.
*>          min(1,N) <= N1 <= N/2.
*> \endverbatim
*>
*> \param[out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          D(I) contains the updated eigenvalues for
*>          1 <= I <= K.
*> \endverbatim
*>
*> \param[out] Q
*> \verbatim
*>          Q is DOUBLE PRECISION array, dimension (LDQ,N)
*>          Initially the first K columns are used as workspace.
*>          On output the columns 1 to K contain
*>          the updated eigenvectors.
*> \endverbatim
*>
*> \param[in] LDQ
*> \verbatim
*>          LDQ is INTEGER
*>          The leading dimension of the array Q.  LDQ >= max(1,N).
*> \endverbatim
*>
*> \param[in] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>          The value of the parameter in the rank one update equation.
*>          RHO >= 0 required.
*> \endverbatim
*>
*> \param[in,out] DLAMDA
*> \verbatim
*>          DLAMDA is DOUBLE PRECISION array, dimension (K)
*>          The first K elements of this array contain the old roots
*>          of the deflated updating problem.  These are the poles
*>          of the secular equation. May be changed on output by
*>          having lowest order bit set to zero on Cray X-MP, Cray Y-MP,
*>          Cray-2, or Cray C-90, as described above.
*> \endverbatim
*>
*> \param[in] Q2
*> \verbatim
*>          Q2 is DOUBLE PRECISION array, dimension (LDQ2, N)
*>          The first K columns of this matrix contain the non-deflated
*>          eigenvectors for the split problem.
*> \endverbatim
*>
*> \param[in] INDX
*> \verbatim
*>          INDX is INTEGER array, dimension (N)
*>          The permutation used to arrange the columns of the deflated
*>          Q matrix into three groups (see DLAED2).
*>          The rows of the eigenvectors found by DLAED4 must be likewise
*>          permuted before the matrix multiply can take place.
*> \endverbatim
*>
*> \param[in] CTOT
*> \verbatim
*>          CTOT is INTEGER array, dimension (4)
*>          A count of the total number of the various types of columns
*>          in Q, as described in INDX.  The fourth column type is any
*>          column which has been deflated.
*> \endverbatim
*>
*> \param[in,out] W
*> \verbatim
*>          W is DOUBLE PRECISION array, dimension (K)
*>          The first K elements of this array contain the components
*>          of the deflation-adjusted updating vector. Destroyed on
*>          output.
*> \endverbatim
*>
*> \param[out] S
*> \verbatim
*>          S is DOUBLE PRECISION array, dimension (N1 + 1)*K
*>          Will contain the eigenvectors of the repaired matrix which
*>          will be multiplied by the previously accumulated eigenvectors
*>          to update the system.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit.
*>          < 0:  if INFO = -i, the i-th argument had an illegal value.
*>          > 0:  if INFO = 1, an eigenvalue did not converge
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Contributors:
*  ==================
*>
*> Jeff Rutter, Computer Science Division, University of California
*> at Berkeley, USA \n
*>  Modified by Francoise Tisseur, University of Tennessee
*>
*  =====================================================================
!      SUBROUTINE DLAED3( K, N, N1, D, Q, LDQ, RHO, DLAMDA, Q2, INDX,
!     $                   CTOT, W, S, INFO )
      SUBROUTINE DDLAED3( K, N, N1, Dh, Dl, Qh, Ql, LDQ, RHOh, RHOl, 
     $                   DLAMDAh, DLAMDAl, Q2h, Q2l, INDX,
     $                   CTOT, Wh, Wl, Sh, Sl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, K, LDQ, N, N1
!      DOUBLE PRECISION   RHO
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
      INTEGER            CTOT( * ), INDX( * )
!      DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), Q2( * ),
!     $                   S( * ), W( * )
      double precision Dh(*), Dl(*)
      double precision DLAMDAh(*), DLAMDAl(*)
      double precision Qh(LDQ,*), Ql(LDQ,*)
      double precision Q2h(*), Q2l(*)
      double precision Sh(*), Sl(*)
      double precision Wh(*), Wl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE, ZERO
!      PARAMETER          ( ONE = 1.0D0, ZERO = 0.0D0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            I, II, IQ2, J, N12, N2, N23
!      DOUBLE PRECISION   TEMP
      double precision TEMPh, TEMPl
*     ..
*     .. External Functions ..
!      DOUBLE PRECISION   DLAMC3, DNRM2
!      EXTERNAL           DLAMC3, DNRM2
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDCOPY, DDGEMM, DDLACPY, DDLAED4, DDLASET, 
     $                   XERBLA, DDLAMC3, DDNRM2

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, SIGN, SQRT
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
*
      IF( K.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.K ) THEN
         INFO = -2
      ELSE IF( LDQ.LT.MAX( 1, N ) ) THEN
         INFO = -6
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DDLAED3', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( K.EQ.0 )
     $   RETURN
*
*     Modify values DLAMDA(i) to make sure all DLAMDA(i)-DLAMDA(j) can
*     be computed with high relative accuracy (barring over/underflow).
*     This is a problem on machines without a guard digit in
*     add/subtract (Cray XMP, Cray YMP, Cray C 90 and Cray 2).
*     The following code replaces DLAMDA(I) by 2*DLAMDA(I)-DLAMDA(I),
*     which on any of these machines zeros out the bottommost
*     bit of DLAMDA(I) if it is 1; this makes the subsequent
*     subtractions DLAMDA(I)-DLAMDA(J) unproblematic when cancellation
*     occurs. On binary machines with a guard digit (almost all
*     machines) it does not change DLAMDA(I) at all. On hexadecimal
*     and decimal machines with a guard digit, it slightly
*     changes the bottommost bits of DLAMDA(I). It does not account
*     for hexadecimal or decimal machines without guard digits
*     (we know of none). We use a subroutine call to compute
*     2*DLAMBDA(I) to prevent optimizing compilers from eliminating
*     this code.
*
      DO 10 I = 1, K
!         DLAMDA( I ) = DLAMC3( DLAMDA( I ), DLAMDA( I ) ) - DLAMDA( I )
          CALL DDLAMC3(DLAMDAh(I),DLAMDAl(I),DLAMDAh(I),DLAMDAl(I),
     $                 tmp1h,tmp1l)
!<<<  DLAMDA(I) = tmp1 - DLAMDA(I)
      T_p1 = tmp1h - DLAMDAh(I)
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (DLAMDAh(I) + T_p2)
      T_cl = T_cl + tmp1l - DLAMDAl(I)
      T_s1 = T_ch + T_cl
      DLAMDAl(I) = T_cl - (T_s1 - T_ch)
      DLAMDAh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DLAMDA(I) = tmp1 - DLAMDA(I)
   10 CONTINUE
*
      DO 20 J = 1, K
!         CALL DLAED4( K, J, DLAMDA, W, Q( 1, J ), RHO, D( J ), INFO )
         CALL DDLAED4( K, J, DLAMDAh, DLAMDAl, Wh, Wl, Qh( 1, J ), 
     $                 Ql( 1, J ), RHOh, RHOl, Dh( J ), Dl( J ), INFO)
*
*        If the zero finder fails, the computation is terminated.
*
         IF( INFO.NE.0 )
     $      GO TO 120
   20 CONTINUE
*
      IF( K.EQ.1 )
     $   GO TO 110
      IF( K.EQ.2 ) THEN
         DO 30 J = 1, K
!            W( 1 ) = Q( 1, J )
      Wh(1) = Qh(1,J)
      Wl(1) = Ql(1,J)
!            W( 2 ) = Q( 2, J )
      Wh(2) = Qh(2,J)
      Wl(2) = Ql(2,J)
            II = INDX( 1 )
!            Q( 1, J ) = W( II )
      Qh(1,J) = Wh(II)
      Ql(1,J) = Wl(II)
            II = INDX( 2 )
!            Q( 2, J ) = W( II )
      Qh(2,J) = Wh(II)
      Ql(2,J) = Wl(II)
   30    CONTINUE
         GO TO 110
      END IF
*
*     Compute updated W.
*
!      CALL DCOPY( K, W, 1, S, 1 )
      CALL DDCOPY( K, Wh, Wl, 1, Sh, Sl, 1 )
*
*     Initialize W(I) = Q(I,I)
*
!      CALL DCOPY( K, Q, LDQ+1, W, 1 )
      CALL DDCOPY( K, Qh, Ql, LDQ+1, Wh, Wl, 1 )
      DO 60 J = 1, K
         DO 40 I = 1, J - 1
!            W( I ) = W( I )*( Q( I, J ) / ( DLAMDA( I )-DLAMDA( J ) ) )
!<<<  tmp1 = DLAMDA(I) - DLAMDA(J)
      T_p1 = DLAMDAh(I) - DLAMDAh(J)
      T_p2 = T_p1 - DLAMDAh(I)
      tmp1h = T_p1
      tmp1l = (DLAMDAh(I) - (T_p1 - T_p2))
     +         - (DLAMDAh(J) + T_p2)
      tmp1l = tmp1l + DLAMDAl(I) - DLAMDAl(J)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLAMDA(I) - DLAMDA(J)
!<<<  tmp2 = Q(I,J) / tmp1
      T_q1 = Qh(I,J) / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = Qh(I,J) - tmp2h
      T_r1 = T_s1 - Qh(I,J)
      T_s2 = (Qh(I,J) 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + Ql(I,J)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = Q(I,J) / tmp1
!<<<  W(I) = W(I) * tmp2
      T_ch = Wh(I) * tmp2h
      T_u1 = 134217729.0 * Wh(I)
      T_p1 = T_u1
     +         - (T_u1 - Wh(I))
      T_p2 = Wh(I) - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Wh(I)*tmp2l
     +         +  Wl(I)*tmp2h)
      T_s1 = T_ch + T_cl
      Wl(I) = T_cl - (T_s1 - T_ch)
      Wh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  W(I) = W(I) * tmp2
   40    CONTINUE
         DO 50 I = J + 1, K
!            W( I ) = W( I )*( Q( I, J ) / ( DLAMDA( I )-DLAMDA( J ) ) )
!<<<  tmp1 = DLAMDA(I) - DLAMDA(J)
      T_p1 = DLAMDAh(I) - DLAMDAh(J)
      T_p2 = T_p1 - DLAMDAh(I)
      tmp1h = T_p1
      tmp1l = (DLAMDAh(I) - (T_p1 - T_p2))
     +         - (DLAMDAh(J) + T_p2)
      tmp1l = tmp1l + DLAMDAl(I) - DLAMDAl(J)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLAMDA(I) - DLAMDA(J)
!<<<  tmp2 = Q(I,J) / tmp1
      T_q1 = Qh(I,J) / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = Qh(I,J) - tmp2h
      T_r1 = T_s1 - Qh(I,J)
      T_s2 = (Qh(I,J) 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + Ql(I,J)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = Q(I,J) / tmp1
!<<<  W(I) = W(I) * tmp2
      T_ch = Wh(I) * tmp2h
      T_u1 = 134217729.0 * Wh(I)
      T_p1 = T_u1
     +         - (T_u1 - Wh(I))
      T_p2 = Wh(I) - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Wh(I)*tmp2l
     +         +  Wl(I)*tmp2h)
      T_s1 = T_ch + T_cl
      Wl(I) = T_cl - (T_s1 - T_ch)
      Wh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  W(I) = W(I) * tmp2
   50    CONTINUE
   60 CONTINUE
      DO 70 I = 1, K
!         W( I ) = SIGN( SQRT( -W( I ) ), S( I ) )
      tmp1h = -Wh(I)
      tmp1l = -Wl(I)
          IF(
!       (S(I) .gt. 0.0)
     + ((Sh(I) .gt. 0.0d+0) .or.
     +  ((Sh(I) .eq. 0.0d+0) .and.
     +   (Sl(I) .gt. 0.0d+0)))
     $     )THEN
!<<<  W(I) = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        Wh(I) = 0.0d+0
        Wl(I) = 0.0d+0
      endif
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Wh(I) = 0.0d+0
        Wl(I) = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Wh(I) = T_t1h
          Wl(I) = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W(I) = sqrt(tmp1)
          ELSE
!<<<  W(I) = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        Wh(I) = 0.0d+0
        Wl(I) = 0.0d+0
      endif
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Wh(I) = 0.0d+0
        Wl(I) = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Wh(I) = T_t1h
          Wl(I) = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W(I) = sqrt(tmp1)
!<<<  W(I) = -W(I)
      Wh(I) = -Wh(I)
      Wl(I) = -Wl(I)
!>>>  W(I) = -W(I)
          END IF
   70 CONTINUE
*
*     Compute eigenvectors of the modified rank-1 modification.
*
      DO 100 J = 1, K
         DO 80 I = 1, K
!            Sh( I ) = Wh( I ) / Qh( I, J )
!<<<  S(I) = W(I) / Q(I,J)
      T_q1 = Wh(I) / Qh(I,J)
      Sh(I) = Qh(I,J) * T_q1
      T_u1 = 134217729.0 * Qh(I,J)
      T_p1 = T_u1
     +         - (T_u1 - Qh(I,J))
      T_p2 = Qh(I,J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Sl(I) = ((T_p1*T_r1
     +       - Sh(I))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Sl(I) = Sl(I) + (Ql(I,J)*T_q1)
      T_s1 = Sh(I) + Sl(I)
      Sl(I) = Sl(I) - (T_s1 - Sh(I))
      Sh(I) = T_s1
      T_s1 = Wh(I) - Sh(I)
      T_r1 = T_s1 - Wh(I)
      T_s2 = (Wh(I) 
     +         - (T_s1 - T_r1))
     +         - (Sh(I) + T_r1)
      T_s2 = T_s2 - Sl(I)
      T_s2 = T_s2 + Wl(I)
      T_q2 = (T_s1 + T_s2)
     +         / Qh(I,J)
      Sh(I) = T_q1 + T_q2
      Sl(I) = T_q2
     +      - (Sh(I) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S(I) = W(I) / Q(I,J)
   80    CONTINUE
         CALL DDNRM2(K,Sh,Sl,1,TEMPh,TEMPl)
         DO 90 I = 1, K
            II = INDX( I )
!            Q( I, J ) = S( II ) / TEMP
!<<<  Q(I,J) = S(II) / TEMP
      T_q1 = Sh(II) / TEMPh
      Qh(I,J) = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Ql(I,J) = ((T_p1*T_r1
     +       - Qh(I,J))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Ql(I,J) = Ql(I,J) + (TEMPl*T_q1)
      T_s1 = Qh(I,J) + Ql(I,J)
      Ql(I,J) = Ql(I,J) - (T_s1 - Qh(I,J))
      Qh(I,J) = T_s1
      T_s1 = Sh(II) - Qh(I,J)
      T_r1 = T_s1 - Sh(II)
      T_s2 = (Sh(II) 
     +         - (T_s1 - T_r1))
     +         - (Qh(I,J) + T_r1)
      T_s2 = T_s2 - Ql(I,J)
      T_s2 = T_s2 + Sl(II)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Qh(I,J) = T_q1 + T_q2
      Ql(I,J) = T_q2
     +      - (Qh(I,J) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  Q(I,J) = S(II) / TEMP
   90    CONTINUE
  100 CONTINUE
*
*     Compute the updated eigenvectors.
*
  110 CONTINUE
*
      N2 = N - N1
      N12 = CTOT( 1 ) + CTOT( 2 )
      N23 = CTOT( 2 ) + CTOT( 3 )
*
!      CALL DLACPY( 'A', N23, K, Q( CTOT( 1 )+1, 1 ), LDQ, S, N23 )
      CALL DDLACPY( 'A', N23, K, Qh( CTOT( 1 )+1, 1 ), 
     $             Ql( CTOT( 1 )+1, 1 ), LDQ, Sh, Sl, N23 )
      IQ2 = N1*N12 + 1
      IF( N23.NE.0 ) THEN
!         CALL DGEMM( 'N', 'N', N2, K, N23, ONE, Q2( IQ2 ), N2, S, N23,
!     $               ZERO, Q( N1+1, 1 ), LDQ )
         CALL DDGEMM( 'N', 'N', N2, K, N23, ONEh, ONEl, 
     $               Q2h( IQ2 ), Q2l( IQ2 ), N2, Sh, Sl, N23,
     $               ZEROh, ZEROl, Qh( N1+1, 1 ), Ql( N1+1, 1 ), LDQ )
      ELSE
!         CALL DLASET( 'A', N2, K, ZERO, ZERO, Q( N1+1, 1 ), LDQ )
         CALL DDLASET( 'A', N2, K, ZEROh, ZEROl, ZEROh, ZEROl, 
     $                 Qh( N1+1, 1 ), Ql( N1+1, 1 ), LDQ )
      END IF
*
!      CALL DLACPY( 'A', N12, K, Q, LDQ, S, N12 )
      CALL DDLACPY( 'A', N12, K, Qh, Ql, LDQ, Sh, Sl, N12 )
      IF( N12.NE.0 ) THEN
!         CALL DGEMM( 'N', 'N', N1, K, N12, ONE, Q2, N1, S, N12, ZERO, Q,
!     $               LDQ )
         CALL DDGEMM( 'N', 'N', N1, K, N12, ONEh, ONEl, Q2h, Q2l, N1, 
     $               Sh, Sl, N12, ZEROh, ZEROl, Qh, Ql,
     $               LDQ )
      ELSE
!         CALL DLASET( 'A', N1, K, ZERO, ZERO, Q( 1, 1 ), LDQ )
         CALL DDLASET( 'A', N1, K, ZEROh, ZEROl, ZEROh, ZEROl, 
     $                 Qh( 1, 1 ), Ql( 1, 1 ), LDQ )
      END IF
*
*
  120 CONTINUE
      RETURN
*
*     End of DLAED3
*
      END
