* -*- fortran -*-
*> \brief \b DLANGE
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLANGE + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlange.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlange.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlange.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       DOUBLE PRECISION FUNCTION DLANGE( NORM, M, N, A, LDA, WORK )
* 
*       .. Scalar Arguments ..
*       CHARACTER          NORM
*       INTEGER            LDA, M, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * ), WORK( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLANGE  returns the value of the one norm,  or the Frobenius norm, or
*> the  infinity norm,  or the  element of  largest absolute value  of a
*> real matrix A.
*> \endverbatim
*>
*> \return DLANGE
*> \verbatim
*>
*>    DLANGE = ( max(abs(A(i,j))), NORM = 'M' or 'm'
*>             (
*>             ( norm1(A),         NORM = '1', 'O' or 'o'
*>             (
*>             ( normI(A),         NORM = 'I' or 'i'
*>             (
*>             ( normF(A),         NORM = 'F', 'f', 'E' or 'e'
*>
*> where  norm1  denotes the  one norm of a matrix (maximum column sum),
*> normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
*> normF  denotes the  Frobenius norm of a matrix (square root of sum of
*> squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix norm.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] NORM
*> \verbatim
*>          NORM is CHARACTER*1
*>          Specifies the value to be returned in DLANGE as described
*>          above.
*> \endverbatim
*>
*> \param[in] M
*> \verbatim
*>          M is INTEGER
*>          The number of rows of the matrix A.  M >= 0.  When M = 0,
*>          DLANGE is set to zero.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of columns of the matrix A.  N >= 0.  When N = 0,
*>          DLANGE is set to zero.
*> \endverbatim
*>
*> \param[in] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          The m by n matrix A.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A.  LDA >= max(M,1).
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK)),
*>          where LWORK >= M when NORM = 'I'; otherwise, WORK is not
*>          referenced.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup doubleGEauxiliary
*
*  =====================================================================
!      DOUBLE PRECISION FUNCTION DLANGE( NORM, M, N, A, LDA, WORK )
      SUBROUTINE DDLANGE( NORM, M, N, Ah, Al, LDA, WORKh, WORKl,
     $                   DDLANGEh, DDLANGEl)
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          NORM
      INTEGER            LDA, M, N
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   A( LDA, * ), WORK( * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision WORKh(*), WORKl(*)
      double precision DDLANGEh, DDLANGEl
*     ..
*
* =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE, ZERO
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, J
!      DOUBLE PRECISION   SCALE, SUM, VALUE
      double precision SCALEh, SCALEl
      double precision SUMh, SUMl
      double precision VALUEh, VALUEl
      double precision tmp1h, tmp1l
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
*     ..
*     .. External Subroutines ..
!      EXTERNAL           DLASSQ
      EXTERNAL           DDLASSQ
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT
*     ..
*     .. Executable Statements ..
*
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      IF( MIN( M, N ).EQ.0 ) THEN
!         VALUE = ZERO
      VALUEh = 0.0d+0
      VALUEl = 0.0d+0
      ELSE IF( LSAME( NORM, 'M' ) ) THEN
*
*        Find max(abs(A(i,j))).
*
!         VALUE = ZERO
      VALUEh = 0.0d+0
      VALUEl = 0.0d+0
         DO 20 J = 1, N
            DO 10 I = 1, M
!               VALUE = MAX( VALUE, ABS( A( I, J ) ) )
!<<<  tmp1 = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        tmp1h = -Ah(I,J)
        tmp1l = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        tmp1h = Ah(I,J)
        tmp1l = -Al(I,J)
      else
        tmp1h = Ah(I,J)
        tmp1l = Al(I,J)
      endif
!>>>  tmp1 = abs(A(I,J))
!<<<  VALUE = max(tmp1,VALUE)
                IF(
!       (tmp1 .gt. VALUE)
     + ((tmp1h .gt. VALUEh) .or.
     +  ((tmp1h .eq. VALUEh) .and.
     +   (tmp1l .gt. VALUEl)))
     +            ) THEN
      VALUEh = tmp1h
      VALUEl = tmp1l
                END IF
!>>>  VALUE = max(tmp1,VALUE)
   10       CONTINUE
   20    CONTINUE
      ELSE IF( ( LSAME( NORM, 'O' ) ) .OR. ( NORM.EQ.'1' ) ) THEN
*
*        Find norm1(A).
*
!         VALUE = ZERO
      VALUEh = 0.0d+0
      VALUEl = 0.0d+0
         DO 40 J = 1, N
!            SUM = ZERO
      SUMh = 0.0d+0
      SUMl = 0.0d+0
            DO 30 I = 1, M
!               SUM = SUM + ABS( A( I, J ) )
!<<<  tmp1 = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        tmp1h = -Ah(I,J)
        tmp1l = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        tmp1h = Ah(I,J)
        tmp1l = -Al(I,J)
      else
        tmp1h = Ah(I,J)
        tmp1l = Al(I,J)
      endif
!>>>  tmp1 = abs(A(I,J))
!<<<  tmp2 = SUM + tmp1
      t_p1 = SUMh + tmp1h
      t_p2 = t_p1 - SUMh
      t_q1 = t_p1
      t_q2 = (SUMh - (t_p1 - t_p2))
     +         + (tmp1h - t_p2)
      t_q2 = t_q2 + SUMl + tmp1l
      tmp2h = t_q1 + t_q2
      tmp2l = t_q2 - (tmp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = SUM + tmp1
      SUMh = tmp2h
      SUMl = tmp2l
   30       CONTINUE
!            VALUE = MAX( VALUE, SUM )
!<<<  VALUE = max(SUM,VALUE)
            IF(
!       (SUM .gt. VALUE)
     + ((SUMh .gt. VALUEh) .or.
     +  ((SUMh .eq. VALUEh) .and.
     +   (SUMl .gt. VALUEl)))
     +        ) THEN
      VALUEh = SUMh
      VALUEl = SUMl
            END IF
!>>>  VALUE = max(SUM,VALUE)
   40    CONTINUE
      ELSE IF( LSAME( NORM, 'I' ) ) THEN
*
*        Find normI(A).
*
         DO 50 I = 1, M
!            WORK( I ) = ZERO
      WORKh(I) = 0.0d+0
      WORKl(I) = 0.0d+0
   50    CONTINUE
         DO 70 J = 1, N
            DO 60 I = 1, M
!               WORK( I ) = WORK( I ) + ABS( A( I, J ) )
!<<<  tmp1 = abs(A(I,J))
      if (Ah(I,J) .lt. 0.0d+0) then
        tmp1h = -Ah(I,J)
        tmp1l = -Al(I,J)
      elseif (
     +  (Ah(I,J) .eq. 0.0d+0).and.
     +  (Al(I,J) .lt. 0.0d+0)) then
        tmp1h = Ah(I,J)
        tmp1l = -Al(I,J)
      else
        tmp1h = Ah(I,J)
        tmp1l = Al(I,J)
      endif
!>>>  tmp1 = abs(A(I,J))
!<<<  tmp2 = WORK(I) + tmp1
      t_p1 = WORKh(I) + tmp1h
      t_p2 = t_p1 - WORKh(I)
      t_q1 = t_p1
      t_q2 = (WORKh(I) - (t_p1 - t_p2))
     +         + (tmp1h - t_p2)
      t_q2 = t_q2 + WORKl(I) + tmp1l
      tmp2h = t_q1 + t_q2
      tmp2l = t_q2 - (tmp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = WORK(I) + tmp1
      WORKh(I) = tmp2h
      WORKl(I) = tmp2l
   60       CONTINUE
   70    CONTINUE
!         VALUE = ZERO
      VALUEh = 0.0d+0
      VALUEl = 0.0d+0
         DO 80 I = 1, M
!            VALUE = MAX( VALUE, WORK( I ) )
!<<<  VALUE = max(WORK(I),VALUE)
            IF(
!       (WORK(I) .gt. VALUE)
     + ((WORKh(I) .gt. VALUEh) .or.
     +  ((WORKh(I) .eq. VALUEh) .and.
     +   (WORKl(I) .gt. VALUEl)))
     +        ) THEN
      VALUEh = WORKh(I)
      VALUEl = WORKl(I)
            END IF
!>>>  VALUE = max(WORK(I),VALUE)
   80    CONTINUE
      ELSE IF( ( LSAME( NORM, 'F' ) ) .OR. ( LSAME( NORM, 'E' ) ) ) THEN
*
*        Find normF(A).
*
!         SCALE = ZERO
      SCALEh = 0.0d+0
      SCALEl = 0.0d+0
!         SUM = ONE
      SUMh = 1.0d+0
      SUMl = 0.0d+0
         DO 90 J = 1, N
!            CALL DLASSQ( M, A( 1, J ), 1, SCALE, SUM )
            CALL DDLASSQ( M, Ah( 1, J ), Al( 1, J ), 1,
     $                   SCALEh, SCALEl, SUMh, SUMl )
   90    CONTINUE
!         VALUE = SCALE*SQRT( SUM )
!<<<  tmp1 = sqrt(SUM)
      if (SUMh.eq.0.0D+0.and.SUMl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (SUMh.lt.0.0d+0.or.
     +   (SUMh.eq.0.0d+0.and.SUMl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      t_x = 1.0D+0/sqrt(SUMh)
      t_ax = SUMh * t_x
!         call dd_mul(t_y1,t_y2,t_x,
!                     0.0,0.5,0.0)
          t_y1 = t_x * 0.5d+0
          t_u1 = 134217729.0 * t_x
          t_p1 = t_u1
     +             - (t_u1 - t_x)
          t_p2 = t_x - t_p1
          t_u1 = 134217729.0 * 0.5d+0
          t_r1 = t_u1
     +             - (t_u1 - 0.5d+0)
          t_r2 = 0.5d+0 - t_r1
          t_y2 = ((t_p1*t_r1
     +             - t_y1)
     +             + t_p1*t_r2
     +             + t_p2 * t_r1)
     +             + t_p2 * t_r2
          t_s1 = t_y1 + t_y2
          t_y2 = t_y2
     +             - (t_s1 - t_y1)
          t_y1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(t_z1,t_z2,t_ax,
!                     0.0,t_ax,0.0)
          t_z1 = t_ax * t_ax
          t_u1 = 134217729.0 * t_ax
          t_p1 = t_u1
     +             - (t_u1 - t_ax)
          t_p2 = t_ax - t_p1
          t_u1 = 134217729.0 * t_ax
          t_r1 = t_u1
     +             - (t_u1 - t_ax)
          t_r2 = t_ax - t_r1
          t_z2 = ((t_p1 * t_r1
     +             - t_z1)
     +             + t_p1 * t_r2
     +             + t_p2 * t_r1)
     +             + t_p2*t_r2
          t_s1 = t_z1 + t_z2
          t_z2 = t_z2
     +             - (t_s1 - t_z1)
          t_z1 = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(t_t2h,t_t2l,
!                           SUMh,SUMl,t_z1,t_z2)
          t_p1 = SUMh - t_z1
          t_p2 = t_p1 - SUMh
          t_t2l = (SUMh
     +             -(t_p1 - t_p2))
     +             -(t_z1 + t_p2)
          t_t2h = t_p1
          t_t2l = t_t2l
     +             + SUMl - t_z2
          t_s1 = t_t2h + t_t2l
          t_t2l = t_t2l
     +             - (t_s1 - t_t2h)
          t_t2h = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(t_t1h,t_t1l,
!                     t_t2h,t_t2l,
!                     t_y1,t_y2)
          t_t1h = t_t2h * t_y1
          t_u1 = 134217729.0 * t_t2h
          t_p1 = t_u1
     +             - (t_u1 - t_t2h)
          t_p2 = t_t2h - t_p1
          t_u1 = 134217729.0 * t_y1
          t_r1 = t_u1
     +             - (t_u1 - t_y1)
          t_r2 = t_y1 - t_r1
          t_t1l = ((t_p1*t_r1
     +             - t_t1h)
     +             + t_p1*t_r2
     +             + t_p2*t_r1)
     +             + t_p2*t_r2
          t_t1l = t_t1l
     +             + (t_t2h * t_y2
     +             +  t_t2l * t_y1)
          t_s1 = t_t1h + t_t1l
          t_t1l = t_t1l - (t_s1
     +              - t_t1h)
          t_t1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(t_t1h,t_t1l,
!                     t_ax,0.0)
          t_p1 = t_t1h + t_ax
          t_p2 = t_p1 - t_t1h
          t_q2 = (t_t1h
     +             - (t_p1 - t_p2))
     +             + (t_ax - t_p2)
          t_q1 = t_p1
          t_q2 = t_q2 + t_t1l
          t_t1h = t_q1 + t_q2
          t_t1l = t_q2
     +              - (t_t1h - t_q1)
          tmp1h = t_t1h
          tmp1l = t_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(SUM)
!<<<  VALUE = SCALE * tmp1
      t_ch = SCALEh * tmp1h
      t_u1 = 134217729.0 * SCALEh
      t_p1 = t_u1
     +         - (t_u1 - SCALEh)
      t_p2 = SCALEh - t_p1
      t_u1 = 134217729.0 * tmp1h
      t_r1 = t_u1
     +         - (t_u1 - tmp1h)
      t_r2 = tmp1h - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (SCALEh*tmp1l
     +         +  SCALEl*tmp1h)
      t_s1 = t_ch + t_cl
      VALUEl = t_cl - (t_s1 - t_ch)
      VALUEh = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  VALUE = SCALE * tmp1
      END IF
*
!      DLANGE = VALUE
      DDLANGEh = VALUEh
      DDLANGEl = VALUEl
      RETURN
*
*     End of DLANGE
*
      END
