*> \brief \b DLASQ3
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASQ3 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasq3.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasq3.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasq3.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASQ3( I0, N0, Z, PP, DMIN, SIGMA, DESIG, QMAX, NFAIL,
*                          ITER, NDIV, IEEE, TTYPE, DMIN1, DMIN2, DN, DN1,
*                          DN2, G, TAU )
* 
*       .. Scalar Arguments ..
*       LOGICAL            IEEE
*       INTEGER            I0, ITER, N0, NDIV, NFAIL, PP
*       DOUBLE PRECISION   DESIG, DMIN, DMIN1, DMIN2, DN, DN1, DN2, G,
*      $                   QMAX, SIGMA, TAU
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASQ3 checks for deflation, computes a shift (TAU) and calls dqds.
*> In case of failure it changes shifts, and tries again until output
*> is positive.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] I0
*> \verbatim
*>          I0 is INTEGER
*>         First index.
*> \endverbatim
*>
*> \param[in,out] N0
*> \verbatim
*>          N0 is INTEGER
*>         Last index.
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension ( 4*N )
*>         Z holds the qd array.
*> \endverbatim
*>
*> \param[in,out] PP
*> \verbatim
*>          PP is INTEGER
*>         PP=0 for ping, PP=1 for pong.
*>         PP=2 indicates that flipping was applied to the Z array   
*>         and that the initial tests for deflation should not be 
*>         performed.
*> \endverbatim
*>
*> \param[out] DMIN
*> \verbatim
*>          DMIN is DOUBLE PRECISION
*>         Minimum value of d.
*> \endverbatim
*>
*> \param[out] SIGMA
*> \verbatim
*>          SIGMA is DOUBLE PRECISION
*>         Sum of shifts used in current segment.
*> \endverbatim
*>
*> \param[in,out] DESIG
*> \verbatim
*>          DESIG is DOUBLE PRECISION
*>         Lower order part of SIGMA
*> \endverbatim
*>
*> \param[in] QMAX
*> \verbatim
*>          QMAX is DOUBLE PRECISION
*>         Maximum value of q.
*> \endverbatim
*>
*> \param[out] NFAIL
*> \verbatim
*>          NFAIL is INTEGER
*>         Number of times shift was too big.
*> \endverbatim
*>
*> \param[out] ITER
*> \verbatim
*>          ITER is INTEGER
*>         Number of iterations.
*> \endverbatim
*>
*> \param[out] NDIV
*> \verbatim
*>          NDIV is INTEGER
*>         Number of divisions.
*> \endverbatim
*>
*> \param[in] IEEE
*> \verbatim
*>          IEEE is LOGICAL
*>         Flag for IEEE or non IEEE arithmetic (passed to DLASQ5).
*> \endverbatim
*>
*> \param[in,out] TTYPE
*> \verbatim
*>          TTYPE is INTEGER
*>         Shift type.
*> \endverbatim
*>
*> \param[in,out] DMIN1
*> \verbatim
*>          DMIN1 is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in,out] DMIN2
*> \verbatim
*>          DMIN2 is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in,out] DN
*> \verbatim
*>          DN is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in,out] DN1
*> \verbatim
*>          DN1 is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in,out] DN2
*> \verbatim
*>          DN2 is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in,out] G
*> \verbatim
*>          G is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[in,out] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION
*>
*>         These are passed as arguments in order to save their values
*>         between calls to DLASQ3.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*  =====================================================================
!!      SUBROUTINE DLASQ3( I0, N0, Z, PP, DMIN, SIGMA, DESIG, QMAX, NFAIL,
!!     $                   ITER, NDIV, IEEE, TTYPE, DMIN1, DMIN2, DN, DN1,
!!     $                   DN2, G, TAU )
      SUBROUTINE dDLASQ3( I0, N0, Zh, Zl, PP, DMINh, DMINl,
     $                    SIGMAh, SIGMAl, DESIGh, DESIGl, QMAXh, QMAXl,
     $                    NFAIL,
     $                    ITER, NDIV, IEEE, TTYPE, DMIN1h, DMIN1l,
     $                    DMIN2h, DMIN2l, DNh, DNl, DN1h, DN1l,
     $                    DN2h, DN2l, Gh, Gl, TAUh, TAUl )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      LOGICAL            IEEE
      INTEGER            I0, ITER, N0, NDIV, NFAIL, PP
!!      DOUBLE PRECISION   DESIG, DMIN, DMIN1, DMIN2, DN, DN1, DN2, G,
!!     $                   QMAX, SIGMA, TAU
      double precision DESIGh, DESIGl
      double precision DMINh, DMINl
      double precision DMIN1h, DMIN1l
      double precision DMIN2h, DMIN2l
      double precision DNh, DNl
      double precision DN1h, DN1l
      double precision DN2h, DN2l
      double precision Gh, Gl
      double precision QMAXh, QMAXl
      double precision SIGMAh, SIGMAl
      double precision TAUh, TAUl
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   Z( * )
      double precision Zh(*), Zl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   CBIAS
!!      PARAMETER          ( CBIAS = 1.50D0 )
      double precision CBIASh, CBIASl
!!      DOUBLE PRECISION   ZERO, QURTR, HALF, ONE, TWO, HUNDRD
!!      PARAMETER          ( ZERO = 0.0D0, QURTR = 0.250D0, HALF = 0.5D0,
!!     $                     ONE = 1.0D0, TWO = 2.0D0, HUNDRD = 100.0D0 )
      double precision ZEROh, ZEROl
      double precision QURTRh, QURTRl
      double precision HALFh, HALFl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision HUNDRDh, HUNDRDl
*     ..
*     .. Local Scalars ..
      INTEGER            IPN4, J4, N0IN, NN, TTYPE
!!      DOUBLE PRECISION   EPS, S, T, TEMP, TOL, TOL2
      double precision EPSh, EPSl
      double precision Sh, Sl
      double precision Th, Tl
      double precision TEMPh, TEMPl
      double precision TOLh, TOLl
      double precision TOL2h, TOL2l
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DLASQ4, DLASQ5, DLASQ6
      EXTERNAL           dDLASQ4, dDLASQ5, dDLASQ6, dDLAMCH
*     ..
*     .. External Function ..
!!      DOUBLE PRECISION   DLAMCH
      LOGICAL            dDISNAN
!!      EXTERNAL           DISNAN, DLAMCH
      EXTERNAL           dDISNAN
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
*     ..
*     .. Executable Statements ..
      CBIASh = 1.50D0
      CBIASl = 0.0d+0
      ZEROh = 0.0D0
      ZEROl = 0.0d+0
      QURTRh = 0.250D0
      QURTRl = 0.0d+0
      HALFh = 0.5D0
      HALFl = 0.0d+0
      ONEh = 1.0D0
      ONEl = 0.0d+0
      TWOh = 2.0D0
      TWOl = 0.0d+0
      HUNDRDh = 100.0D0
      HUNDRDl = 0.0d+0
*
      N0IN = N0
!!      EPS = DLAMCH( 'Precision' )
      CALL dDLAMCH( 'Precision', EPSh, EPSl )
!!      TOL = EPS*HUNDRD
!<<<  TOL = EPS * HUNDRD
      T_ch = EPSh * HUNDRDh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * HUNDRDh
      T_r1 = T_u1
     +         - (T_u1 - HUNDRDh)
      T_r2 = HUNDRDh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (EPSh*HUNDRDl
     +         +  EPSl*HUNDRDh)
      T_s1 = T_ch + T_cl
      TOLl = T_cl - (T_s1 - T_ch)
      TOLh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TOL = EPS * HUNDRD
!!      TOL2 = TOL**2
!<<<  TOL2 = TOL * TOL
      T_ch = TOLh * TOLh
      T_u1 = 134217729.0 * TOLh
      T_p1 = T_u1
     +         - (T_u1 - TOLh)
      T_p2 = TOLh - T_p1
      T_u1 = 134217729.0 * TOLh
      T_r1 = T_u1
     +         - (T_u1 - TOLh)
      T_r2 = TOLh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLh*TOLl
     +         +  TOLl*TOLh)
      T_s1 = T_ch + T_cl
      TOL2l = T_cl - (T_s1 - T_ch)
      TOL2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TOL2 = TOL * TOL
*
*     Check for deflation.
*
   10 CONTINUE
*
      IF( N0.LT.I0 )
     $   RETURN
      IF( N0.EQ.I0 )
     $   GO TO 20
      NN = 4*N0 + PP
      IF( N0.EQ.( I0+1 ) )
     $   GO TO 40
*
*     Check whether E(N0-1) is negligible, 1 eigenvalue.
*
!!      IF( Z( NN-5 ).GT.TOL2*( SIGMA+Z( NN-3 ) ) .AND.
!!     $    Z( NN-2*PP-4 ).GT.TOL2*Z( NN-7 ) )
!!     $   GO TO 30
!<<<  tmp1 = SIGMA + Z(NN-3)
      T_p1 = SIGMAh + Zh(NN-3)
      T_p2 = T_p1 - SIGMAh
      T_q1 = T_p1
      T_q2 = (SIGMAh - (T_p1 - T_p2))
     +         + (Zh(NN-3) - T_p2)
      T_q2 = T_q2 + SIGMAl + Zl(NN-3)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = SIGMA + Z(NN-3)
!<<<  tmp2 = TOL2 * tmp1
      T_ch = TOL2h * tmp1h
      T_u1 = 134217729.0 * TOL2h
      T_p1 = T_u1
     +         - (T_u1 - TOL2h)
      T_p2 = TOL2h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOL2h*tmp1l
     +         +  TOL2l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL2 * tmp1
!<<<  tmp3 = TOL2 * Z(NN-7)
      T_ch = TOL2h * Zh(NN-7)
      T_u1 = 134217729.0 * TOL2h
      T_p1 = T_u1
     +         - (T_u1 - TOL2h)
      T_p2 = TOL2h - T_p1
      T_u1 = 134217729.0 * Zh(NN-7)
      T_r1 = T_u1
     +         - (T_u1 - Zh(NN-7))
      T_r2 = Zh(NN-7) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOL2h*Zl(NN-7)
     +         +  TOL2l*Zh(NN-7))
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = TOL2 * Z(NN-7)
      IF(
!       (Z(NN-5) .gt. tmp2)
     + ((Zh(NN-5) .gt. tmp2h) .or.
     +  ((Zh(NN-5) .eq. tmp2h) .and.
     +   (Zl(NN-5) .gt. tmp2l)))
     $      .AND.
!       (Z(NN-2*PP-4) .gt. tmp3)
     + ((Zh(NN-2*PP-4) .gt. tmp3h) .or.
     +  ((Zh(NN-2*PP-4) .eq. tmp3h) .and.
     +   (Zl(NN-2*PP-4) .gt. tmp3l)))
     $  ) THEN
         GO TO 30
      END IF
*
   20 CONTINUE
*
!!      Z( 4*N0-3 ) = Z( 4*N0+PP-3 ) + SIGMA
!<<<  Z(4*N0-3) = Z(4*N0+PP-3) + SIGMA
      T_p1 = Zh(4*N0+PP-3) + SIGMAh
      T_p2 = T_p1 - Zh(4*N0+PP-3)
      T_q1 = T_p1
      T_q2 = (Zh(4*N0+PP-3) - (T_p1 - T_p2))
     +         + (SIGMAh - T_p2)
      T_q2 = T_q2 + Zl(4*N0+PP-3) + SIGMAl
      Zh(4*N0-3) = T_q1 + T_q2
      Zl(4*N0-3) = T_q2 - (Zh(4*N0-3) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(4*N0-3) = Z(4*N0+PP-3) + SIGMA
      N0 = N0 - 1
      GO TO 10
*
*     Check  whether E(N0-2) is negligible, 2 eigenvalues.
*
   30 CONTINUE
*
!!      IF( Z( NN-9 ).GT.TOL2*SIGMA .AND.
!!     $    Z( NN-2*PP-8 ).GT.TOL2*Z( NN-11 ) )
!!     $   GO TO 50
!<<<  tmp1 = TOL2 * SIGMA
      T_ch = TOL2h * SIGMAh
      T_u1 = 134217729.0 * TOL2h
      T_p1 = T_u1
     +         - (T_u1 - TOL2h)
      T_p2 = TOL2h - T_p1
      T_u1 = 134217729.0 * SIGMAh
      T_r1 = T_u1
     +         - (T_u1 - SIGMAh)
      T_r2 = SIGMAh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOL2h*SIGMAl
     +         +  TOL2l*SIGMAh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TOL2 * SIGMA
!<<<  tmp2 = TOL2 * Z(NN-11)
      T_ch = TOL2h * Zh(NN-11)
      T_u1 = 134217729.0 * TOL2h
      T_p1 = T_u1
     +         - (T_u1 - TOL2h)
      T_p2 = TOL2h - T_p1
      T_u1 = 134217729.0 * Zh(NN-11)
      T_r1 = T_u1
     +         - (T_u1 - Zh(NN-11))
      T_r2 = Zh(NN-11) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOL2h*Zl(NN-11)
     +         +  TOL2l*Zh(NN-11))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL2 * Z(NN-11)
      IF(
!       (Z(NN-9) .gt. tmp1)
     + ((Zh(NN-9) .gt. tmp1h) .or.
     +  ((Zh(NN-9) .eq. tmp1h) .and.
     +   (Zl(NN-9) .gt. tmp1l)))
     $      .AND.
!       (Z(NN-2*PP-8) .gt. tmp2)
     + ((Zh(NN-2*PP-8) .gt. tmp2h) .or.
     +  ((Zh(NN-2*PP-8) .eq. tmp2h) .and.
     +   (Zl(NN-2*PP-8) .gt. tmp2l)))
     $   ) THEN
         GO TO 50
      END IF
*
   40 CONTINUE
*
!!      IF( Z( NN-3 ).GT.Z( NN-7 ) ) THEN
      IF(
!       (Z(NN-3) .gt. Z(NN-7))
     + ((Zh(NN-3) .gt. Zh(NN-7)) .or.
     +  ((Zh(NN-3) .eq. Zh(NN-7)) .and.
     +   (Zl(NN-3) .gt. Zl(NN-7))))
     $  ) THEN
!!         S = Z( NN-3 )
      Sh = Zh(NN-3)
      Sl = Zl(NN-3)
!!         Z( NN-3 ) = Z( NN-7 )
      Zh(NN-3) = Zh(NN-7)
      Zl(NN-3) = Zl(NN-7)
!!         Z( NN-7 ) = S
      Zh(NN-7) = Sh
      Zl(NN-7) = Sl
      END IF
!!      IF( Z( NN-5 ).GT.Z( NN-3 )*TOL2 ) THEN
!<<<  tmp1 = Z(NN-3) * TOL2
      T_ch = Zh(NN-3) * TOL2h
      T_u1 = 134217729.0 * Zh(NN-3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-3))
      T_p2 = Zh(NN-3) - T_p1
      T_u1 = 134217729.0 * TOL2h
      T_r1 = T_u1
     +         - (T_u1 - TOL2h)
      T_r2 = TOL2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(NN-3)*TOL2l
     +         +  Zl(NN-3)*TOL2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = Z(NN-3) * TOL2
      IF(
!       (Z(NN-5) .gt. tmp1)
     + ((Zh(NN-5) .gt. tmp1h) .or.
     +  ((Zh(NN-5) .eq. tmp1h) .and.
     +   (Zl(NN-5) .gt. tmp1l)))
     $  ) THEN
!!         T = HALF*( ( Z( NN-7 )-Z( NN-3 ) )+Z( NN-5 ) )
!<<<  tmp1 = Z(NN-7) - Z(NN-3)
      T_p1 = Zh(NN-7) - Zh(NN-3)
      T_p2 = T_p1 - Zh(NN-7)
      T_ch = T_p1
      T_cl = (Zh(NN-7) - (T_p1 - T_p2))
     +         - (Zh(NN-3) + T_p2)
      T_cl = T_cl + Zl(NN-7) - Zl(NN-3)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = Z(NN-7) - Z(NN-3)
!<<<  tmp2 = tmp1 + Z(NN-5)
      T_p1 = tmp1h + Zh(NN-5)
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (Zh(NN-5) - T_p2)
      T_q2 = T_q2 + tmp1l + Zl(NN-5)
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = tmp1 + Z(NN-5)
!<<<  T = HALF * tmp2
      T_ch = HALFh * tmp2h
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HALFh*tmp2l
     +         +  HALFl*tmp2h)
      T_s1 = T_ch + T_cl
      Tl = T_cl - (T_s1 - T_ch)
      Th = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  T = HALF * tmp2
!!         S = Z( NN-3 )*( Z( NN-5 ) / T )
!<<<  tmp1 = Z(NN-5) / T
      T_q1 = Zh(NN-5) / Th
      T_ch = Th * T_q1
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Tl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-5) - T_ch
      T_r1 = T_s1 - Zh(NN-5)
      T_s2 = (Zh(NN-5)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-5)
      T_q2 = (T_s1 + T_s2)
     +         / Th
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(NN-5) / T
!<<<  S = Z(NN-3) * tmp1
      T_ch = Zh(NN-3) * tmp1h
      T_u1 = 134217729.0 * Zh(NN-3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-3))
      T_p2 = Zh(NN-3) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(NN-3)*tmp1l
     +         +  Zl(NN-3)*tmp1h)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = Z(NN-3) * tmp1
!!         IF( S.LE.T ) THEN
         IF(
!       (S .le. T)
     + ((Sh .lt. Th) .or.
     +  ((Sh .eq. Th) .and.
     +   (Sl .le. Tl)))
     $     ) THEN
!!            S = Z( NN-3 )*( Z( NN-5 ) /
!!     $          ( T*( ONE+SQRT( ONE+S / T ) ) ) )
!<<<        tmp1 = S / T
            T_q1 = Sh / Th
            T_ch = Th * T_q1
            T_u1 = 134217729.0 * Th
            T_p1 = T_u1
     +               - (T_u1 - Th)
            T_p2 = Th - T_p1
            T_u1 = 134217729.0 * T_q1
            T_r1 = T_u1
     +               - (T_u1 - T_q1)
            T_r2 = T_q1 - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +               + T_p1*T_r2
     +               + T_p2*T_r1)
     +               + T_p2*T_r2
            T_cl = T_cl
     +               + (Tl * T_q1)
            T_s1 = T_ch + T_cl
            T_cl = T_cl
     +               - (T_s1 - T_ch)
            T_ch = T_s1
            T_s1 = Sh - T_ch
            T_r1 = T_s1 - Sh
            T_s2 = (Sh
     +               - (T_s1 - T_r1))
     +               - (T_ch + T_r1)
            T_s2 = T_s2 - T_cl
            T_s2 = T_s2 + Sl
            T_q2 = (T_s1 + T_s2)
     +               / Th
            tmp1h = T_q1 + T_q2
            tmp1l = T_q2
     +            - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>        tmp1 = S / T
!<<<        tmp1 = ONE + tmp1
            T_p1 = ONEh + tmp1h
            T_p2 = T_p1 - ONEh
            T_q1 = T_p1
            T_q2 = (ONEh - (T_p1 - T_p2))
     +               + (tmp1h - T_p2)
            T_q2 = T_q2 + ONEl + tmp1l
            tmp1h = T_q1 + T_q2
            tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        tmp1 = ONE + tmp1
!<<<        tmp1 = sqrt(tmp1)
            if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
              tmp1h = 0.0d+0
              tmp1l = 0.0d+0
            else
            if (tmp1h.lt.0.0d+0.or.
     +         (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
              write(6,'(a)')
     +           'SQRT CALLED WITH NEGATIVE ARGUMENT'
              tmp1h = 0.0d+0
              tmp1l = 0.0d+0
              return
            endif
            T_x = 1.0D+0/sqrt(tmp1h)
            T_ax = tmp1h * T_x
!               call dd_mul(T_y1,T_y2,T_x,
!                           0.0,0.5,0.0)
                T_y1 = T_x * 0.5d+0
                T_u1 = 134217729.0 * T_x
                T_p1 = T_u1
     +                   - (T_u1 - T_x)
                T_p2 = T_x - T_p1
                T_u1 = 134217729.0 * 0.5d+0
                T_r1 = T_u1
     +                   - (T_u1 - 0.5d+0)
                T_r2 = 0.5d+0 - T_r1
                T_y2 = ((T_p1*T_r1
     +                   - T_y1)
     +                   + T_p1*T_r2
     +                   + T_p2 * T_r1)
     +                   + T_p2 * T_r2
                T_s1 = T_y1 + T_y2
                T_y2 = T_y2
     +                   - (T_s1 - T_y1)
                T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_mul(T_z1,T_z2,T_ax,
!                           0.0,T_ax,0.0)
                T_z1 = T_ax * T_ax
                T_u1 = 134217729.0 * T_ax
                T_p1 = T_u1
     +                   - (T_u1 - T_ax)
                T_p2 = T_ax - T_p1
                T_u1 = 134217729.0 * T_ax
                T_r1 = T_u1
     +                   - (T_u1 - T_ax)
                T_r2 = T_ax - T_r1
                T_z2 = ((T_p1 * T_r1
     +                   - T_z1)
     +                   + T_p1 * T_r2
     +                   + T_p2 * T_r1)
     +                   + T_p2*T_r2
                T_s1 = T_z1 + T_z2
                T_z2 = T_z2
     +                   - (T_s1 - T_z1)
                T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
                T_p1 = tmp1h - T_z1
                T_p2 = T_p1 - tmp1h
                T_t2l = (tmp1h
     +                   -(T_p1 - T_p2))
     +                   -(T_z1 + T_p2)
                T_t2h = T_p1
                T_t2l = T_t2l
     +                   + tmp1l - T_z2
                T_s1 = T_t2h + T_t2l
                T_t2l = T_t2l
     +                   - (T_s1 - T_t2h)
                T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!               call dd_mul(T_t1h,T_t1l,
!                           T_t2h,T_t2l,
!                           T_y1,T_y2)
                T_t1h = T_t2h * T_y1
                T_u1 = 134217729.0 * T_t2h
                T_p1 = T_u1
     +                   - (T_u1 - T_t2h)
                T_p2 = T_t2h - T_p1
                T_u1 = 134217729.0 * T_y1
                T_r1 = T_u1
     +                   - (T_u1 - T_y1)
                T_r2 = T_y1 - T_r1
                T_t1l = ((T_p1*T_r1
     +                   - T_t1h)
     +                   + T_p1*T_r2
     +                   + T_p2*T_r1)
     +                   + T_p2*T_r2
                T_t1l = T_t1l
     +                   + (T_t2h * T_y2
     +                   +  T_t2l * T_y1)
                T_s1 = T_t1h + T_t1l
                T_t1l = T_t1l - (T_s1
     +                    - T_t1h)
                T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_selfadd(T_t1h,T_t1l,
!                           T_ax,0.0)
                T_p1 = T_t1h + T_ax
                T_p2 = T_p1 - T_t1h
                T_q2 = (T_t1h
     +                   - (T_p1 - T_p2))
     +                   + (T_ax - T_p2)
                T_q1 = T_p1
                T_q2 = T_q2 + T_t1l
                T_t1h = T_q1 + T_q2
                T_t1l = T_q2
     +                    - (T_t1h - T_q1)
                tmp1h = T_t1h
                tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
            endif
!>>>        tmp1 = sqrt(tmp1)
!<<<        tmp1 = ONE + tmp1
            T_p1 = ONEh + tmp1h
            T_p2 = T_p1 - ONEh
            T_q1 = T_p1
            T_q2 = (ONEh - (T_p1 - T_p2))
     +               + (tmp1h - T_p2)
            T_q2 = T_q2 + ONEl + tmp1l
            tmp1h = T_q1 + T_q2
            tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        tmp1 = ONE + tmp1
!<<<        tmp1 = T * tmp1
            T_ch = Th * tmp1h
            T_u1 = 134217729.0 * Th
            T_p1 = T_u1
     +               - (T_u1 - Th)
            T_p2 = Th - T_p1
            T_u1 = 134217729.0 * tmp1h
            T_r1 = T_u1
     +               - (T_u1 - tmp1h)
            T_r2 = tmp1h - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +              + T_p1 * T_r2
     +              + T_p2 * T_r1)
     +              + T_p2 * T_r2
            T_cl = T_cl
     +               + (Th*tmp1l
     +               +  Tl*tmp1h)
            T_s1 = T_ch + T_cl
            tmp1l = T_cl - (T_s1 - T_ch)
            tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        tmp1 = T * tmp1
!<<<        tmp1 = Z(NN-5) / tmp1
            T_q1 = Zh(NN-5) / tmp1h
            T_ch = tmp1h * T_q1
            T_u1 = 134217729.0 * tmp1h
            T_p1 = T_u1
     +               - (T_u1 - tmp1h)
            T_p2 = tmp1h - T_p1
            T_u1 = 134217729.0 * T_q1
            T_r1 = T_u1
     +               - (T_u1 - T_q1)
            T_r2 = T_q1 - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +               + T_p1*T_r2
     +               + T_p2*T_r1)
     +               + T_p2*T_r2
            T_cl = T_cl
     +               + (tmp1l * T_q1)
            T_s1 = T_ch + T_cl
            T_cl = T_cl
     +               - (T_s1 - T_ch)
            T_ch = T_s1
            T_s1 = Zh(NN-5) - T_ch
            T_r1 = T_s1 - Zh(NN-5)
            T_s2 = (Zh(NN-5)
     +               - (T_s1 - T_r1))
     +               - (T_ch + T_r1)
            T_s2 = T_s2 - T_cl
            T_s2 = T_s2 + Zl(NN-5)
            T_q2 = (T_s1 + T_s2)
     +               / tmp1h
            tmp1h = T_q1 + T_q2
            tmp1l = T_q2
     +            - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>        tmp1 = Z(NN-5) / tmp1
!<<<        S = Z(NN-3) * tmp1
            T_ch = Zh(NN-3) * tmp1h
            T_u1 = 134217729.0 * Zh(NN-3)
            T_p1 = T_u1
     +               - (T_u1 - Zh(NN-3))
            T_p2 = Zh(NN-3) - T_p1
            T_u1 = 134217729.0 * tmp1h
            T_r1 = T_u1
     +               - (T_u1 - tmp1h)
            T_r2 = tmp1h - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +              + T_p1 * T_r2
     +              + T_p2 * T_r1)
     +              + T_p2 * T_r2
            T_cl = T_cl
     +               + (Zh(NN-3)*tmp1l
     +               +  Zl(NN-3)*tmp1h)
            T_s1 = T_ch + T_cl
            Sl = T_cl - (T_s1 - T_ch)
            Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        S = Z(NN-3) * tmp1
         ELSE
!!            S = Z( NN-3 )*( Z( NN-5 ) / ( T+SQRT( T )*SQRT( T+S ) ) )
!<<<        tmp1 = T + S
            T_p1 = Th + Sh
            T_p2 = T_p1 - Th
            T_q1 = T_p1
            T_q2 = (Th - (T_p1 - T_p2))
     +               + (Sh - T_p2)
            T_q2 = T_q2 + Tl + Sl
            tmp1h = T_q1 + T_q2
            tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        tmp1 = T + S
!<<<        tmp2 = sqrt(T)
            if (Th.eq.0.0D+0.and.Tl.eq.0.0D+0) then
              tmp2h = 0.0d+0
              tmp2l = 0.0d+0
            else
            if (Th.lt.0.0d+0.or.
     +         (Th.eq.0.0d+0.and.Tl.lt.0.0d+0)) then
              write(6,'(a)')
     +           'SQRT CALLED WITH NEGATIVE ARGUMENT'
              tmp2h = 0.0d+0
              tmp2l = 0.0d+0
              return
            endif
            T_x = 1.0D+0/sqrt(Th)
            T_ax = Th * T_x
!               call dd_mul(T_y1,T_y2,T_x,
!                           0.0,0.5,0.0)
                T_y1 = T_x * 0.5d+0
                T_u1 = 134217729.0 * T_x
                T_p1 = T_u1
     +                   - (T_u1 - T_x)
                T_p2 = T_x - T_p1
                T_u1 = 134217729.0 * 0.5d+0
                T_r1 = T_u1
     +                   - (T_u1 - 0.5d+0)
                T_r2 = 0.5d+0 - T_r1
                T_y2 = ((T_p1*T_r1
     +                   - T_y1)
     +                   + T_p1*T_r2
     +                   + T_p2 * T_r1)
     +                   + T_p2 * T_r2
                T_s1 = T_y1 + T_y2
                T_y2 = T_y2
     +                   - (T_s1 - T_y1)
                T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_mul(T_z1,T_z2,T_ax,
!                           0.0,T_ax,0.0)
                T_z1 = T_ax * T_ax
                T_u1 = 134217729.0 * T_ax
                T_p1 = T_u1
     +                   - (T_u1 - T_ax)
                T_p2 = T_ax - T_p1
                T_u1 = 134217729.0 * T_ax
                T_r1 = T_u1
     +                   - (T_u1 - T_ax)
                T_r2 = T_ax - T_r1
                T_z2 = ((T_p1 * T_r1
     +                   - T_z1)
     +                   + T_p1 * T_r2
     +                   + T_p2 * T_r1)
     +                   + T_p2*T_r2
                T_s1 = T_z1 + T_z2
                T_z2 = T_z2
     +                   - (T_s1 - T_z1)
                T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_sub(T_t2h,T_t2l,
!                           Th,Tl,T_z1,T_z2)
                T_p1 = Th - T_z1
                T_p2 = T_p1 - Th
                T_t2l = (Th
     +                   -(T_p1 - T_p2))
     +                   -(T_z1 + T_p2)
                T_t2h = T_p1
                T_t2l = T_t2l
     +                   + Tl - T_z2
                T_s1 = T_t2h + T_t2l
                T_t2l = T_t2l
     +                   - (T_s1 - T_t2h)
                T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!               call dd_mul(T_t1h,T_t1l,
!                           T_t2h,T_t2l,
!                           T_y1,T_y2)
                T_t1h = T_t2h * T_y1
                T_u1 = 134217729.0 * T_t2h
                T_p1 = T_u1
     +                   - (T_u1 - T_t2h)
                T_p2 = T_t2h - T_p1
                T_u1 = 134217729.0 * T_y1
                T_r1 = T_u1
     +                   - (T_u1 - T_y1)
                T_r2 = T_y1 - T_r1
                T_t1l = ((T_p1*T_r1
     +                   - T_t1h)
     +                   + T_p1*T_r2
     +                   + T_p2*T_r1)
     +                   + T_p2*T_r2
                T_t1l = T_t1l
     +                   + (T_t2h * T_y2
     +                   +  T_t2l * T_y1)
                T_s1 = T_t1h + T_t1l
                T_t1l = T_t1l - (T_s1
     +                    - T_t1h)
                T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_selfadd(T_t1h,T_t1l,
!                           T_ax,0.0)
                T_p1 = T_t1h + T_ax
                T_p2 = T_p1 - T_t1h
                T_q2 = (T_t1h
     +                   - (T_p1 - T_p2))
     +                   + (T_ax - T_p2)
                T_q1 = T_p1
                T_q2 = T_q2 + T_t1l
                T_t1h = T_q1 + T_q2
                T_t1l = T_q2
     +                    - (T_t1h - T_q1)
                tmp2h = T_t1h
                tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
            endif
!>>>        tmp2 = sqrt(T)
!<<<        tmp3 = sqrt(tmp1)
            if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
              tmp3h = 0.0d+0
              tmp3l = 0.0d+0
            else
            if (tmp1h.lt.0.0d+0.or.
     +         (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
              write(6,'(a)')
     +           'SQRT CALLED WITH NEGATIVE ARGUMENT'
              tmp3h = 0.0d+0
              tmp3l = 0.0d+0
              return
            endif
            T_x = 1.0D+0/sqrt(tmp1h)
            T_ax = tmp1h * T_x
!               call dd_mul(T_y1,T_y2,T_x,
!                           0.0,0.5,0.0)
                T_y1 = T_x * 0.5d+0
                T_u1 = 134217729.0 * T_x
                T_p1 = T_u1
     +                   - (T_u1 - T_x)
                T_p2 = T_x - T_p1
                T_u1 = 134217729.0 * 0.5d+0
                T_r1 = T_u1
     +                   - (T_u1 - 0.5d+0)
                T_r2 = 0.5d+0 - T_r1
                T_y2 = ((T_p1*T_r1
     +                   - T_y1)
     +                   + T_p1*T_r2
     +                   + T_p2 * T_r1)
     +                   + T_p2 * T_r2
                T_s1 = T_y1 + T_y2
                T_y2 = T_y2
     +                   - (T_s1 - T_y1)
                T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_mul(T_z1,T_z2,T_ax,
!                           0.0,T_ax,0.0)
                T_z1 = T_ax * T_ax
                T_u1 = 134217729.0 * T_ax
                T_p1 = T_u1
     +                   - (T_u1 - T_ax)
                T_p2 = T_ax - T_p1
                T_u1 = 134217729.0 * T_ax
                T_r1 = T_u1
     +                   - (T_u1 - T_ax)
                T_r2 = T_ax - T_r1
                T_z2 = ((T_p1 * T_r1
     +                   - T_z1)
     +                   + T_p1 * T_r2
     +                   + T_p2 * T_r1)
     +                   + T_p2*T_r2
                T_s1 = T_z1 + T_z2
                T_z2 = T_z2
     +                   - (T_s1 - T_z1)
                T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
                T_p1 = tmp1h - T_z1
                T_p2 = T_p1 - tmp1h
                T_t2l = (tmp1h
     +                   -(T_p1 - T_p2))
     +                   -(T_z1 + T_p2)
                T_t2h = T_p1
                T_t2l = T_t2l
     +                   + tmp1l - T_z2
                T_s1 = T_t2h + T_t2l
                T_t2l = T_t2l
     +                   - (T_s1 - T_t2h)
                T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!               call dd_mul(T_t1h,T_t1l,
!                           T_t2h,T_t2l,
!                           T_y1,T_y2)
                T_t1h = T_t2h * T_y1
                T_u1 = 134217729.0 * T_t2h
                T_p1 = T_u1
     +                   - (T_u1 - T_t2h)
                T_p2 = T_t2h - T_p1
                T_u1 = 134217729.0 * T_y1
                T_r1 = T_u1
     +                   - (T_u1 - T_y1)
                T_r2 = T_y1 - T_r1
                T_t1l = ((T_p1*T_r1
     +                   - T_t1h)
     +                   + T_p1*T_r2
     +                   + T_p2*T_r1)
     +                   + T_p2*T_r2
                T_t1l = T_t1l
     +                   + (T_t2h * T_y2
     +                   +  T_t2l * T_y1)
                T_s1 = T_t1h + T_t1l
                T_t1l = T_t1l - (T_s1
     +                    - T_t1h)
                T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!               call dd_selfadd(T_t1h,T_t1l,
!                           T_ax,0.0)
                T_p1 = T_t1h + T_ax
                T_p2 = T_p1 - T_t1h
                T_q2 = (T_t1h
     +                   - (T_p1 - T_p2))
     +                   + (T_ax - T_p2)
                T_q1 = T_p1
                T_q2 = T_q2 + T_t1l
                T_t1h = T_q1 + T_q2
                T_t1l = T_q2
     +                    - (T_t1h - T_q1)
                tmp3h = T_t1h
                tmp3l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
            endif
!>>>        tmp3 = sqrt(tmp1)
!<<<        tmp4 = tmp2 * tmp3
            T_ch = tmp2h * tmp3h
            T_u1 = 134217729.0 * tmp2h
            T_p1 = T_u1
     +               - (T_u1 - tmp2h)
            T_p2 = tmp2h - T_p1
            T_u1 = 134217729.0 * tmp3h
            T_r1 = T_u1
     +               - (T_u1 - tmp3h)
            T_r2 = tmp3h - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +              + T_p1 * T_r2
     +              + T_p2 * T_r1)
     +              + T_p2 * T_r2
            T_cl = T_cl
     +               + (tmp2h*tmp3l
     +               +  tmp2l*tmp3h)
            T_s1 = T_ch + T_cl
            tmp4l = T_cl - (T_s1 - T_ch)
            tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        tmp4 = tmp2 * tmp3
!<<<        tmp5 = T + tmp4
            T_p1 = Th + tmp4h
            T_p2 = T_p1 - Th
            T_q1 = T_p1
            T_q2 = (Th - (T_p1 - T_p2))
     +               + (tmp4h - T_p2)
            T_q2 = T_q2 + Tl + tmp4l
            tmp5h = T_q1 + T_q2
            tmp5l = T_q2 - (tmp5h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>        tmp5 = T + tmp4
!<<<        tmp6 = Z(NN-5) / tmp5
            T_q1 = Zh(NN-5) / tmp5h
            T_ch = tmp5h * T_q1
            T_u1 = 134217729.0 * tmp5h
            T_p1 = T_u1
     +               - (T_u1 - tmp5h)
            T_p2 = tmp5h - T_p1
            T_u1 = 134217729.0 * T_q1
            T_r1 = T_u1
     +               - (T_u1 - T_q1)
            T_r2 = T_q1 - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +               + T_p1*T_r2
     +               + T_p2*T_r1)
     +               + T_p2*T_r2
            T_cl = T_cl
     +               + (tmp5l * T_q1)
            T_s1 = T_ch + T_cl
            T_cl = T_cl
     +               - (T_s1 - T_ch)
            T_ch = T_s1
            T_s1 = Zh(NN-5) - T_ch
            T_r1 = T_s1 - Zh(NN-5)
            T_s2 = (Zh(NN-5)
     +               - (T_s1 - T_r1))
     +               - (T_ch + T_r1)
            T_s2 = T_s2 - T_cl
            T_s2 = T_s2 + Zl(NN-5)
            T_q2 = (T_s1 + T_s2)
     +               / tmp5h
            tmp6h = T_q1 + T_q2
            tmp6l = T_q2
     +            - (tmp6h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>        tmp6 = Z(NN-5) / tmp5
!<<<        S = Z(NN-3) * tmp6
            T_ch = Zh(NN-3) * tmp6h
            T_u1 = 134217729.0 * Zh(NN-3)
            T_p1 = T_u1
     +               - (T_u1 - Zh(NN-3))
            T_p2 = Zh(NN-3) - T_p1
            T_u1 = 134217729.0 * tmp6h
            T_r1 = T_u1
     +               - (T_u1 - tmp6h)
            T_r2 = tmp6h - T_r1
            T_cl = ((T_p1*T_r1
     +               - T_ch)
     +              + T_p1 * T_r2
     +              + T_p2 * T_r1)
     +              + T_p2 * T_r2
            T_cl = T_cl
     +               + (Zh(NN-3)*tmp6l
     +               +  Zl(NN-3)*tmp6h)
            T_s1 = T_ch + T_cl
            Sl = T_cl - (T_s1 - T_ch)
            Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>        S = Z(NN-3) * tmp6
         END IF
!!         T = Z( NN-7 ) + ( S+Z( NN-5 ) )
!<<<  tmp1 = S + Z(NN-5)
      T_p1 = Sh + Zh(NN-5)
      T_p2 = T_p1 - Sh
      T_q1 = T_p1
      T_q2 = (Sh - (T_p1 - T_p2))
     +         + (Zh(NN-5) - T_p2)
      T_q2 = T_q2 + Sl + Zl(NN-5)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = S + Z(NN-5)
!<<<  T = Z(NN-7) + tmp1
      T_p1 = Zh(NN-7) + tmp1h
      T_p2 = T_p1 - Zh(NN-7)
      T_q1 = T_p1
      T_q2 = (Zh(NN-7) - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + Zl(NN-7) + tmp1l
      Th = T_q1 + T_q2
      Tl = T_q2 - (Th - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  T = Z(NN-7) + tmp1
!!         Z( NN-3 ) = Z( NN-3 )*( Z( NN-7 ) / T )
!<<<  tmp1 = Z(NN-7) / T
      T_q1 = Zh(NN-7) / Th
      T_ch = Th * T_q1
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Tl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-7) - T_ch
      T_r1 = T_s1 - Zh(NN-7)
      T_s2 = (Zh(NN-7)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-7)
      T_q2 = (T_s1 + T_s2)
     +         / Th
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(NN-7) / T
!<<<  Z(NN-3) = Z(NN-3) * tmp1
      T_ch = Zh(NN-3) * tmp1h
      T_u1 = 134217729.0 * Zh(NN-3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-3))
      T_p2 = Zh(NN-3) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(NN-3)*tmp1l
     +         +  Zl(NN-3)*tmp1h)
      T_s1 = T_ch + T_cl
      Zl(NN-3) = T_cl - (T_s1 - T_ch)
      Zh(NN-3) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(NN-3) = Z(NN-3) * tmp1
!!         Z( NN-7 ) = T
      Zh(NN-7) = Th
      Zl(NN-7) = Tl
      END IF
!!      Z( 4*N0-7 ) = Z( NN-7 ) + SIGMA
!<<<  Z(4*N0-7) = Z(NN-7) + SIGMA
      T_p1 = Zh(NN-7) + SIGMAh
      T_p2 = T_p1 - Zh(NN-7)
      T_q1 = T_p1
      T_q2 = (Zh(NN-7) - (T_p1 - T_p2))
     +         + (SIGMAh - T_p2)
      T_q2 = T_q2 + Zl(NN-7) + SIGMAl
      Zh(4*N0-7) = T_q1 + T_q2
      Zl(4*N0-7) = T_q2 - (Zh(4*N0-7) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(4*N0-7) = Z(NN-7) + SIGMA
!!      Z( 4*N0-3 ) = Z( NN-3 ) + SIGMA
!<<<  Z(4*N0-3) = Z(NN-3) + SIGMA
      T_p1 = Zh(NN-3) + SIGMAh
      T_p2 = T_p1 - Zh(NN-3)
      T_q1 = T_p1
      T_q2 = (Zh(NN-3) - (T_p1 - T_p2))
     +         + (SIGMAh - T_p2)
      T_q2 = T_q2 + Zl(NN-3) + SIGMAl
      Zh(4*N0-3) = T_q1 + T_q2
      Zl(4*N0-3) = T_q2 - (Zh(4*N0-3) - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  Z(4*N0-3) = Z(NN-3) + SIGMA
      N0 = N0 - 2
      GO TO 10
*
   50 CONTINUE
      IF( PP.EQ.2 ) 
     $   PP = 0
*
*     Reverse the qd-array, if warranted.
*
!!      IF( DMIN.LE.ZERO .OR. N0.LT.N0IN ) THEN
      IF(
!       (DMIN .le. ZERO)
     + ((DMINh .lt. ZEROh) .or.
     +  ((DMINh .eq. ZEROh) .and.
     +   (DMINl .le. ZEROl)))
     $      .OR. N0.LT.N0IN ) THEN
!!         IF( CBIAS*Z( 4*I0+PP-3 ).LT.Z( 4*N0+PP-3 ) ) THEN
!<<<  tmp1 = CBIAS * Z(4*I0+PP-3)
      T_ch = CBIASh * Zh(4*I0+PP-3)
      T_u1 = 134217729.0 * CBIASh
      T_p1 = T_u1
     +         - (T_u1 - CBIASh)
      T_p2 = CBIASh - T_p1
      T_u1 = 134217729.0 * Zh(4*I0+PP-3)
      T_r1 = T_u1
     +         - (T_u1 - Zh(4*I0+PP-3))
      T_r2 = Zh(4*I0+PP-3) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CBIASh*Zl(4*I0+PP-3)
     +         +  CBIASl*Zh(4*I0+PP-3))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CBIAS * Z(4*I0+PP-3)
         IF(
!       (tmp1 .lt. Z(4*N0+PP-3))
     + ((tmp1h .lt. Zh(4*N0+PP-3)) .or.
     +  ((tmp1h .eq. Zh(4*N0+PP-3)) .and.
     +   (tmp1l .lt. Zl(4*N0+PP-3))))
     $     ) THEN
            IPN4 = 4*( I0+N0 )
            DO 60 J4 = 4*I0, 2*( I0+N0-1 ), 4
!!               TEMP = Z( J4-3 )
      TEMPh = Zh(J4-3)
      TEMPl = Zl(J4-3)
!!               Z( J4-3 ) = Z( IPN4-J4-3 )
      Zh(J4-3) = Zh(IPN4-J4-3)
      Zl(J4-3) = Zl(IPN4-J4-3)
!!               Z( IPN4-J4-3 ) = TEMP
      Zh(IPN4-J4-3) = TEMPh
      Zl(IPN4-J4-3) = TEMPl
!!               TEMP = Z( J4-2 )
      TEMPh = Zh(J4-2)
      TEMPl = Zl(J4-2)
!!               Z( J4-2 ) = Z( IPN4-J4-2 )
      Zh(J4-2) = Zh(IPN4-J4-2)
      Zl(J4-2) = Zl(IPN4-J4-2)
!!               Z( IPN4-J4-2 ) = TEMP
      Zh(IPN4-J4-2) = TEMPh
      Zl(IPN4-J4-2) = TEMPl
!!               TEMP = Z( J4-1 )
      TEMPh = Zh(J4-1)
      TEMPl = Zl(J4-1)
!!               Z( J4-1 ) = Z( IPN4-J4-5 )
      Zh(J4-1) = Zh(IPN4-J4-5)
      Zl(J4-1) = Zl(IPN4-J4-5)
!!               Z( IPN4-J4-5 ) = TEMP
      Zh(IPN4-J4-5) = TEMPh
      Zl(IPN4-J4-5) = TEMPl
!!               TEMP = Z( J4 )
      TEMPh = Zh(J4)
      TEMPl = Zl(J4)
!!               Z( J4 ) = Z( IPN4-J4-4 )
      Zh(J4) = Zh(IPN4-J4-4)
      Zl(J4) = Zl(IPN4-J4-4)
!!               Z( IPN4-J4-4 ) = TEMP
      Zh(IPN4-J4-4) = TEMPh
      Zl(IPN4-J4-4) = TEMPl
   60       CONTINUE
            IF( N0-I0.LE.4 ) THEN
!!               Z( 4*N0+PP-1 ) = Z( 4*I0+PP-1 )
      Zh(4*N0+PP-1) = Zh(4*I0+PP-1)
      Zl(4*N0+PP-1) = Zl(4*I0+PP-1)
!!               Z( 4*N0-PP ) = Z( 4*I0-PP )
      Zh(4*N0-PP) = Zh(4*I0-PP)
      Zl(4*N0-PP) = Zl(4*I0-PP)
            END IF
!!            DMIN2 = MIN( DMIN2, Z( 4*N0+PP-1 ) )
            IF(
!       (DMIN2 .gt. Z(4*N0+PP-1))
     + ((DMIN2h .gt. Zh(4*N0+PP-1)) .or.
     +  ((DMIN2h .eq. Zh(4*N0+PP-1)) .and.
     +   (DMIN2l .gt. Zl(4*N0+PP-1))))
     $        ) THEN
      DMIN2h = Zh(4*N0+PP-1)
      DMIN2l = Zl(4*N0+PP-1)
            ENDIF
!!            Z( 4*N0+PP-1 ) = MIN( Z( 4*N0+PP-1 ), Z( 4*I0+PP-1 ),
!!     $                            Z( 4*I0+PP+3 ) )
            IF(
!       (Z(4*N0+PP-1) .gt. Z(4*I0+PP-1))
     + ((Zh(4*N0+PP-1) .gt. Zh(4*I0+PP-1)) .or.
     +  ((Zh(4*N0+PP-1) .eq. Zh(4*I0+PP-1)) .and.
     +   (Zl(4*N0+PP-1) .gt. Zl(4*I0+PP-1))))
     $            .AND.
!       (Z(4*I0+PP+3) .gt. Z(4*I0+PP-1))
     + ((Zh(4*I0+PP+3) .gt. Zh(4*I0+PP-1)) .or.
     +  ((Zh(4*I0+PP+3) .eq. Zh(4*I0+PP-1)) .and.
     +   (Zl(4*I0+PP+3) .gt. Zl(4*I0+PP-1))))
     $        ) THEN
      Zh(4*N0+PP-1) = Zh(4*I0+PP-1)
      Zl(4*N0+PP-1) = Zl(4*I0+PP-1)
            ELSE IF(
!       (Z(4*N0+PP-1) .gt. Z(4*I0+PP+3))
     + ((Zh(4*N0+PP-1) .gt. Zh(4*I0+PP+3)) .or.
     +  ((Zh(4*N0+PP-1) .eq. Zh(4*I0+PP+3)) .and.
     +   (Zl(4*N0+PP-1) .gt. Zl(4*I0+PP+3))))
     $            .AND.
!       (Z(4*I0+PP-1) .gt. Z(4*I0+PP+3))
     + ((Zh(4*I0+PP-1) .gt. Zh(4*I0+PP+3)) .or.
     +  ((Zh(4*I0+PP-1) .eq. Zh(4*I0+PP+3)) .and.
     +   (Zl(4*I0+PP-1) .gt. Zl(4*I0+PP+3))))
     $        ) THEN
      Zh(4*N0+PP-1) = Zh(4*I0+PP+3)
      Zl(4*N0+PP-1) = Zl(4*I0+PP+3)
            END IF
!!            Z( 4*N0-PP ) = MIN( Z( 4*N0-PP ), Z( 4*I0-PP ),
!!     $                          Z( 4*I0-PP+4 ) )
            IF(
!       (Z(4*N0-PP) .gt. Z(4*I0-PP))
     + ((Zh(4*N0-PP) .gt. Zh(4*I0-PP)) .or.
     +  ((Zh(4*N0-PP) .eq. Zh(4*I0-PP)) .and.
     +   (Zl(4*N0-PP) .gt. Zl(4*I0-PP))))
     $            .AND.
!       (Z(4*I0-PP+4) .gt. Z(4*I0-PP))
     + ((Zh(4*I0-PP+4) .gt. Zh(4*I0-PP)) .or.
     +  ((Zh(4*I0-PP+4) .eq. Zh(4*I0-PP)) .and.
     +   (Zl(4*I0-PP+4) .gt. Zl(4*I0-PP))))
     $        ) THEN
      Zh(4*N0-PP) = Zh(4*I0-PP)
      Zl(4*N0-PP) = Zl(4*I0-PP)
            ELSE IF(
!       (Z(4*N0-PP) .gt. Z(4*I0-PP+4))
     + ((Zh(4*N0-PP) .gt. Zh(4*I0-PP+4)) .or.
     +  ((Zh(4*N0-PP) .eq. Zh(4*I0-PP+4)) .and.
     +   (Zl(4*N0-PP) .gt. Zl(4*I0-PP+4))))
     $            .AND.
!       (Z(4*I0-PP) .gt. Z(4*I0-PP+4))
     + ((Zh(4*I0-PP) .gt. Zh(4*I0-PP+4)) .or.
     +  ((Zh(4*I0-PP) .eq. Zh(4*I0-PP+4)) .and.
     +   (Zl(4*I0-PP) .gt. Zl(4*I0-PP+4))))
     $        ) THEN
      Zh(4*N0-PP) = Zh(4*I0-PP+4)
      Zl(4*N0-PP) = Zl(4*I0-PP+4)
            END IF
!!            QMAX = MAX( QMAX, Z( 4*I0+PP-3 ), Z( 4*I0+PP+1 ) )
            IF(
!       (QMAX .gt. Z(4*I0+PP-3))
     + ((QMAXh .gt. Zh(4*I0+PP-3)) .or.
     +  ((QMAXh .eq. Zh(4*I0+PP-3)) .and.
     +   (QMAXl .gt. Zl(4*I0+PP-3))))
     $            .AND.
!       (Z(4*I0+PP+1) .gt. Z(4*I0+PP-3))
     + ((Zh(4*I0+PP+1) .gt. Zh(4*I0+PP-3)) .or.
     +  ((Zh(4*I0+PP+1) .eq. Zh(4*I0+PP-3)) .and.
     +   (Zl(4*I0+PP+1) .gt. Zl(4*I0+PP-3))))
     $        ) THEN
      QMAXh = Zh(4*I0+PP-3)
      QMAXl = Zl(4*I0+PP-3)
            ELSE IF(
!       (QMAX .gt. Z(4*I0+PP+1))
     + ((QMAXh .gt. Zh(4*I0+PP+1)) .or.
     +  ((QMAXh .eq. Zh(4*I0+PP+1)) .and.
     +   (QMAXl .gt. Zl(4*I0+PP+1))))
     $            .AND.
!       (Z(4*I0+PP-3) .gt. Z(4*I0+PP+1))
     + ((Zh(4*I0+PP-3) .gt. Zh(4*I0+PP+1)) .or.
     +  ((Zh(4*I0+PP-3) .eq. Zh(4*I0+PP+1)) .and.
     +   (Zl(4*I0+PP-3) .gt. Zl(4*I0+PP+1))))
     $        ) THEN
      QMAXh = Zh(4*I0+PP+1)
      QMAXl = Zl(4*I0+PP+1)
            END IF
!!            DMIN = -ZERO
!<<<  DMIN = -ZERO
      DMINh = -ZEROh
      DMINl = -ZEROl
!>>>  DMIN = -ZERO
         END IF
      END IF
*
*     Choose a shift.
*
!!      CALL DLASQ4( I0, N0, Z, PP, N0IN, DMIN, DMIN1, DMIN2, DN, DN1,
!!     $             DN2, TAU, TTYPE, G )
      CALL dDLASQ4( I0, N0, Zh, Zl, PP, N0IN, DMINh, DMINl,
     $              DMIN1h, DMIN1l, DMIN2h, DMIN2l, DNh, DNl,
     $              DN1h, DN1l,
     $              DN2h, DN2l, TAUh, TAUl, TTYPE, Gh, Gl )
*
*     Call dqds until DMIN > 0.
*
   70 CONTINUE
*
!!      CALL DLASQ5( I0, N0, Z, PP, TAU, DMIN, DMIN1, DMIN2, DN,
!!     $             DN1, DN2, IEEE )
      CALL dDLASQ5( I0, N0, Zh, Zl, PP, TAUh, TAUl, DMINh, DMINl,
     $              DMIN1h, DMIN1l, DMIN2h, DMIN2l, DNh, DNl,
     $              DN1h, DN1l, DN2h, DN2l, IEEE )
*
      NDIV = NDIV + ( N0-I0+2 )
      ITER = ITER + 1
*
*     Check status.
*
!!      IF( DMIN.GE.ZERO .AND. DMIN1.GT.ZERO ) THEN
      IF(
!       (DMIN .ge. ZERO)
     + ((DMINh .gt. ZEROh) .or.
     +  ((DMINh .eq. ZEROh) .and.
     +   (DMINl .ge. ZEROl)))
     $      .AND.
!       (DMIN1 .gt. ZERO)
     + ((DMIN1h .gt. ZEROh) .or.
     +  ((DMIN1h .eq. ZEROh) .and.
     +   (DMIN1l .gt. ZEROl)))
     $  ) THEN
*
*        Success.
*
         GO TO 90
      END IF
*
!!      ELSE IF( DMIN.LT.ZERO .AND. DMIN1.GT.ZERO .AND. 
!!     $         Z( 4*( N0-1 )-PP ).LT.TOL*( SIGMA+DN1 ) .AND.
!!     $         ABS( DN ).LT.TOL*SIGMA ) THEN
!<<<  tmp1 = SIGMA + DN1
      T_p1 = SIGMAh + DN1h
      T_p2 = T_p1 - SIGMAh
      T_q1 = T_p1
      T_q2 = (SIGMAh - (T_p1 - T_p2))
     +         + (DN1h - T_p2)
      T_q2 = T_q2 + SIGMAl + DN1l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = SIGMA + DN1
!<<<  tmp2 = TOL * tmp1
      T_ch = TOLh * tmp1h
      T_u1 = 134217729.0 * TOLh
      T_p1 = T_u1
     +         - (T_u1 - TOLh)
      T_p2 = TOLh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLh*tmp1l
     +         +  TOLl*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL * tmp1
!<<<  tmp3 = abs(DN)
      if (DNh .lt. 0.0d+0) then
        tmp3h = -DNh
        tmp3l = -DNl
      elseif (
     +  (DNh .eq. 0.0d+0).and.
     +  (DNl .lt. 0.0d+0)) then
        tmp3h = DNh
        tmp3l = -DNl
      else
        tmp3h = DNh
        tmp3l = DNl
      endif
!>>>  tmp3 = abs(DN)
!<<<  tmp4 = TOL * SIGMA
      T_ch = TOLh * SIGMAh
      T_u1 = 134217729.0 * TOLh
      T_p1 = T_u1
     +         - (T_u1 - TOLh)
      T_p2 = TOLh - T_p1
      T_u1 = 134217729.0 * SIGMAh
      T_r1 = T_u1
     +         - (T_u1 - SIGMAh)
      T_r2 = SIGMAh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLh*SIGMAl
     +         +  TOLl*SIGMAh)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = TOL * SIGMA
      IF (
!       (DMIN .lt. ZERO)
     + ((DMINh .lt. ZEROh) .or.
     +  ((DMINh .eq. ZEROh) .and.
     +   (DMINl .lt. ZEROl)))
     $      .AND.
!       (DMIN1 .gt. ZERO)
     + ((DMIN1h .gt. ZEROh) .or.
     +  ((DMIN1h .eq. ZEROh) .and.
     +   (DMIN1l .gt. ZEROl)))
     $      .AND.
!       (Z(4*(N0-1)-PP) .lt. tmp2)
     + ((Zh(4*(N0-1)-PP) .lt. tmp2h) .or.
     +  ((Zh(4*(N0-1)-PP) .eq. tmp2h) .and.
     +   (Zl(4*(N0-1)-PP) .lt. tmp2l)))
     $      .AND.
!       (tmp3 .lt. tmp4)
     + ((tmp3h .lt. tmp4h) .or.
     +  ((tmp3h .eq. tmp4h) .and.
     +   (tmp3l .lt. tmp4l)))
     $  ) THEN
*
*        Convergence hidden by negative DN.
*
!!         Z( 4*( N0-1 )-PP+2 ) = ZERO
      Zh(4*(N0-1)-PP+2) = ZEROh
      Zl(4*(N0-1)-PP+2) = ZEROl
!!         DMIN = ZERO
      DMINh = ZEROh
      DMINl = ZEROl
         GO TO 90
!!      ELSE IF( DMIN.LT.ZERO ) THEN
      ELSE IF(
!       (DMIN .lt. ZERO)
     + ((DMINh .lt. ZEROh) .or.
     +  ((DMINh .eq. ZEROh) .and.
     +   (DMINl .lt. ZEROl)))
     $  ) THEN
*
*        TAU too big. Select new TAU and try again.
*
         NFAIL = NFAIL + 1
         IF( TTYPE.LT.-22 ) THEN
*
*           Failed twice. Play it safe.
*
!!            TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
!!         ELSE IF( DMIN1.GT.ZERO ) THEN
         ELSE IF(
!       (DMIN1 .gt. ZERO)
     + ((DMIN1h .gt. ZEROh) .or.
     +  ((DMIN1h .eq. ZEROh) .and.
     +   (DMIN1l .gt. ZEROl)))
     $     ) THEN
*
*           Late failure. Gives excellent shift.
*
!!            TAU = ( TAU+DMIN )*( ONE-TWO*EPS )
!<<<  tmp1 = TAU + DMIN
      T_p1 = TAUh + DMINh
      T_p2 = T_p1 - TAUh
      T_q1 = T_p1
      T_q2 = (TAUh - (T_p1 - T_p2))
     +         + (DMINh - T_p2)
      T_q2 = T_q2 + TAUl + DMINl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = TAU + DMIN
!<<<  tmp2 = TWO * EPS
      T_ch = TWOh * EPSh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TWOh*EPSl
     +         +  TWOl*EPSh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TWO * EPS
!<<<  tmp3 = ONE - tmp2
      T_p1 = ONEh - tmp2h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + ONEl - tmp2l
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp3 = ONE - tmp2
!<<<  TAU = tmp1 * tmp3
      T_ch = tmp1h * tmp3h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp3l
     +         +  tmp1l*tmp3h)
      T_s1 = T_ch + T_cl
      TAUl = T_cl - (T_s1 - T_ch)
      TAUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TAU = tmp1 * tmp3
            TTYPE = TTYPE - 11
         ELSE
*
*           Early failure. Divide by 4.
*
!!            TAU = QURTR*TAU
!<<<  TAU = QURTR * TAU
      T_ch = QURTRh * TAUh
      T_u1 = 134217729.0 * QURTRh
      T_p1 = T_u1
     +         - (T_u1 - QURTRh)
      T_p2 = QURTRh - T_p1
      T_u1 = 134217729.0 * TAUh
      T_r1 = T_u1
     +         - (T_u1 - TAUh)
      T_r2 = TAUh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (QURTRh*TAUl
     +         +  QURTRl*TAUh)
      T_s1 = T_ch + T_cl
      TAUl = T_cl - (T_s1 - T_ch)
      TAUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TAU = QURTR * TAU
            TTYPE = TTYPE - 12
         END IF
         GO TO 70
!!      ELSE IF( DISNAN( DMIN ) ) THEN
      ELSE IF( dDISNAN( DMINh, DMINh ) ) THEN
*
*        NaN.
*
!!         IF( TAU.EQ.ZERO ) THEN
         IF(
!       (TAU .eq. ZERO)
     + ((TAUh .eq. ZEROh) .and.
     +  (TAUl .eq. ZEROl))
     $     ) THEN
            GO TO 80
         ELSE
!!            TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
            GO TO 70
         END IF
      ELSE
*            
*        Possible underflow. Play it safe.
*
         GO TO 80
      END IF
*
*     Risk of underflow.
*
   80 CONTINUE
!!      CALL DLASQ6( I0, N0, Z, PP, DMIN, DMIN1, DMIN2, DN, DN1, DN2 )
      CALL dDLASQ6( I0, N0, Zh, Zl, PP, DMINh, DMINl, DMIN1h, DMIN1l,
     $              DMIN2h, DMIN2l, DNh, DNl, DN1h, DN1l, DN2h, DN2l )
      NDIV = NDIV + ( N0-I0+2 )
      ITER = ITER + 1
!!      TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
*
   90 CONTINUE
!!      IF( TAU.LT.SIGMA ) THEN
      IF(
!       (TAU .lt. SIGMA)
     + ((TAUh .lt. SIGMAh) .or.
     +  ((TAUh .eq. SIGMAh) .and.
     +   (TAUl .lt. SIGMAl)))
     $  ) THEN
!!         DESIG = DESIG + TAU
!<<<  DESIG = DESIG + TAU
      T_p1 = DESIGh + TAUh
      T_p2 = T_p1 - DESIGh
      T_q1 = T_p1
      T_q2 = (DESIGh - (T_p1 - T_p2))
     +         + (TAUh - T_p2)
      T_q2 = T_q2 + DESIGl + TAUl
      DESIGh = T_q1 + T_q2
      DESIGl = T_q2 - (DESIGh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DESIG = DESIG + TAU
!!         T = SIGMA + DESIG
!<<<  T = SIGMA + DESIG
      T_p1 = SIGMAh + DESIGh
      T_p2 = T_p1 - SIGMAh
      T_q1 = T_p1
      T_q2 = (SIGMAh - (T_p1 - T_p2))
     +         + (DESIGh - T_p2)
      T_q2 = T_q2 + SIGMAl + DESIGl
      Th = T_q1 + T_q2
      Tl = T_q2 - (Th - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  T = SIGMA + DESIG
!!         DESIG = DESIG - ( T-SIGMA )
!<<<  tmp1 = T - SIGMA
      T_p1 = Th - SIGMAh
      T_p2 = T_p1 - Th
      T_ch = T_p1
      T_cl = (Th - (T_p1 - T_p2))
     +         - (SIGMAh + T_p2)
      T_cl = T_cl + Tl - SIGMAl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = T - SIGMA
!<<<  DESIG = DESIG - tmp1
      T_p1 = DESIGh - tmp1h
      T_p2 = T_p1 - DESIGh
      T_ch = T_p1
      T_cl = (DESIGh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + DESIGl - tmp1l
      T_s1 = T_ch + T_cl
      DESIGl = T_cl - (T_s1 - T_ch)
      DESIGh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DESIG = DESIG - tmp1
      ELSE
!!         T = SIGMA + TAU
!<<<  T = SIGMA + TAU
      T_p1 = SIGMAh + TAUh
      T_p2 = T_p1 - SIGMAh
      T_q1 = T_p1
      T_q2 = (SIGMAh - (T_p1 - T_p2))
     +         + (TAUh - T_p2)
      T_q2 = T_q2 + SIGMAl + TAUl
      Th = T_q1 + T_q2
      Tl = T_q2 - (Th - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  T = SIGMA + TAU
!!         DESIG = SIGMA - ( T-TAU ) + DESIG
!<<<  tmp1 = T - TAU
      T_p1 = Th - TAUh
      T_p2 = T_p1 - Th
      T_ch = T_p1
      T_cl = (Th - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      T_cl = T_cl + Tl - TAUl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = T - TAU
!<<<  tmp2 = SIGMA - tmp1
      T_p1 = SIGMAh - tmp1h
      T_p2 = T_p1 - SIGMAh
      T_ch = T_p1
      T_cl = (SIGMAh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + SIGMAl - tmp1l
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = SIGMA - tmp1
!<<<  DESIG = tmp2 + DESIG
      T_p1 = tmp2h + DESIGh
      T_p2 = T_p1 - tmp2h
      T_q1 = T_p1
      T_q2 = (tmp2h - (T_p1 - T_p2))
     +         + (DESIGh - T_p2)
      T_q2 = T_q2 + tmp2l + DESIGl
      DESIGh = T_q1 + T_q2
      DESIGl = T_q2 - (DESIGh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DESIG = tmp2 + DESIG
      END IF
!!      SIGMA = T
      SIGMAh = Th
      SIGMAl = Tl
*
      RETURN
*
*     End of DLASQ3
*
      END
