*> \brief \b DLASSQ
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASSQ + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlassq.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlassq.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlassq.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASSQ( N, X, INCX, SCALE, SUMSQ )
* 
*       .. Scalar Arguments ..
*       INTEGER            INCX, N
*       DOUBLE PRECISION   SCALE, SUMSQ
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   X( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASSQ  returns the values  scl  and  smsq  such that
*>
*>    ( scl**2 )*smsq = x( 1 )**2 +...+ x( n )**2 + ( scale**2 )*sumsq,
*>
*> where  x( i ) = X( 1 + ( i - 1 )*INCX ). The value of  sumsq  is
*> assumed to be non-negative and  scl  returns the value
*>
*>    scl = max( scale, abs( x( i ) ) ).
*>
*> scale and sumsq must be supplied in SCALE and SUMSQ and
*> scl and smsq are overwritten on SCALE and SUMSQ respectively.
*>
*> The routine makes only one pass through the vector x.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of elements to be used from the vector X.
*> \endverbatim
*>
*> \param[in] X
*> \verbatim
*>          X is DOUBLE PRECISION array, dimension (N)
*>          The vector for which a scaled sum of squares is computed.
*>             x( i )  = X( 1 + ( i - 1 )*INCX ), 1 <= i <= n.
*> \endverbatim
*>
*> \param[in] INCX
*> \verbatim
*>          INCX is INTEGER
*>          The increment between successive values of the vector X.
*>          INCX > 0.
*> \endverbatim
*>
*> \param[in,out] SCALE
*> \verbatim
*>          SCALE is DOUBLE PRECISION
*>          On entry, the value  scale  in the equation above.
*>          On exit, SCALE is overwritten with  scl , the scaling factor
*>          for the sum of squares.
*> \endverbatim
*>
*> \param[in,out] SUMSQ
*> \verbatim
*>          SUMSQ is DOUBLE PRECISION
*>          On entry, the value  sumsq  in the equation above.
*>          On exit, SUMSQ is overwritten with  smsq , the basic sum of
*>          squares from which  scl  has been factored out.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!      SUBROUTINE DLASSQ( N, X, INCX, SCALE, SUMSQ )
      SUBROUTINE DDLASSQ( N,Xh,Xl,INCX,SCALEh,SCALEl,SUMSQh,SUMSQl )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INCX, N
!      DOUBLE PRECISION   SCALE, SUMSQ
      double precision SCALEh, SCALEl
      double precision SUMSQh, SUMSQl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   X( * )
      double precision Xh(*), Xl(*)
*     ..
*
* =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ZERO
!      PARAMETER          ( ZERO = 0.0D+0 )
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            IX
!      DOUBLE PRECISION   ABSXI
      double precision ABSXIh, ABSXIl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

*     .. Executable Statements ..
*
      ZEROh =0.0d0
      ZEROl =0.0d0

      IF( N.GT.0 ) THEN
         DO 10 IX = 1, 1 + ( N-1 )*INCX, INCX
            IF( 
!              X( IX ).NE.ZERO
!       (X(IX) .ne. ZERO)
     + ((Xh(IX) .ne. ZEROh) .or.
     +  (Xl(IX) .ne. ZEROl))
     &         ) THEN
!               ABSXI = ABS( X( IX ) )
!<<<  ABSXI = abs(X(IX))
      if (Xh(IX) .lt. 0.0d+0) then
        ABSXIh = -Xh(IX)
        ABSXIl = -Xl(IX)
      elseif (
     +  (Xh(IX) .eq. 0.0d+0).and.
     +  (Xl(IX) .lt. 0.0d+0)) then
        ABSXIh = Xh(IX)
        ABSXIl = -Xl(IX)
      else
        ABSXIh = Xh(IX)
        ABSXIl = Xl(IX)
      endif
!>>>  ABSXI = abs(X(IX))
               IF( 
!                  SCALE.LT.ABSXI
!       (SCALE .lt. ABSXI)
     + ((SCALEh .lt. ABSXIh) .or.
     +  ((SCALEh .eq. ABSXIh) .and.
     +   (SCALEl .lt. ABSXIl)))
     &         ) THEN
!                  SUMSQ = 1 + SUMSQ*( SCALE / ABSXI )**2
!<<<  tmp1 = SCALE / ABSXI
      T_q1 = SCALEh / ABSXIh
      tmp1h = ABSXIh * T_q1
      T_u1 = 134217729.0 * ABSXIh
      T_p1 = T_u1
     +         - (T_u1 - ABSXIh)
      T_p2 = ABSXIh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (ABSXIl*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = SCALEh - tmp1h
      T_r1 = T_s1 - SCALEh
      T_s2 = (SCALEh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + SCALEl
      T_q2 = (T_s1 + T_s2)
     +         / ABSXIh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = SCALE / ABSXI
!<<<  tmp2 = tmp1 * tmp1
      tmp2h = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
!<<<  tmp3 = SUMSQ * tmp2
      tmp3h = SUMSQh * tmp2h
      T_u1 = 134217729.0 * SUMSQh
      T_p1 = T_u1
     +         - (T_u1 - SUMSQh)
      T_p2 = SUMSQh - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (SUMSQh*tmp2l
     +      + SUMSQl*tmp2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = SUMSQ * tmp2
      tmp4h = 1.0d+0
      tmp4l = 0.0d+0
!<<<  SUMSQ = tmp4 + tmp3
      T_p1 = tmp4h + tmp3h
      T_p2 = T_p1 - tmp4h
      SUMSQh = T_p1
      SUMSQl = (tmp4h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      SUMSQl = SUMSQl + tmp4l + tmp3l
      T_s1 = SUMSQh + SUMSQl
      SUMSQl = SUMSQl - (T_s1 - SUMSQh)
      SUMSQh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  SUMSQ = tmp4 + tmp3
!                  SCALE = ABSXI
      SCALEh = ABSXIh
      SCALEl = ABSXIl
               ELSE
!                  SUMSQ = SUMSQ + ( ABSXI / SCALE )**2
!<<<  tmp1 = ABSXI / SCALE
      T_q1 = ABSXIh / SCALEh
      tmp1h = SCALEh * T_q1
      T_u1 = 134217729.0 * SCALEh
      T_p1 = T_u1
     +         - (T_u1 - SCALEh)
      T_p2 = SCALEh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (SCALEl*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = ABSXIh - tmp1h
      T_r1 = T_s1 - ABSXIh
      T_s2 = (ABSXIh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + ABSXIl
      T_q2 = (T_s1 + T_s2)
     +         / SCALEh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = ABSXI / SCALE
!<<<  tmp2 = tmp1 * tmp1
      tmp2h = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
!<<<  SUMSQ = SUMSQ + tmp2
      T_p1 = SUMSQh + tmp2h
      T_p2 = T_p1 - SUMSQh
      T_q1 = T_p1
      T_q2 = (SUMSQh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + SUMSQl + tmp2l
      SUMSQh = T_q1 + T_q2
      SUMSQl = T_q2 - (SUMSQh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  SUMSQ = SUMSQ + tmp2
               END IF
            END IF
   10    CONTINUE
      END IF
      RETURN
*
*     End of DLASSQ
*
      END
