*> \brief \b DORGL2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DORGL2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dorgl2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dorgl2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dorgl2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DORGL2( M, N, K, A, LDA, TAU, WORK, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            INFO, K, LDA, M, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DORGL2 generates an m by n real matrix Q with orthonormal rows,
*> which is defined as the first m rows of a product of k elementary
*> reflectors of order n
*>
*>       Q  =  H(k) . . . H(2) H(1)
*>
*> as returned by DGELQF.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] M
*> \verbatim
*>          M is INTEGER
*>          The number of rows of the matrix Q. M >= 0.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of columns of the matrix Q. N >= M.
*> \endverbatim
*>
*> \param[in] K
*> \verbatim
*>          K is INTEGER
*>          The number of elementary reflectors whose product defines the
*>          matrix Q. M >= K >= 0.
*> \endverbatim
*>
*> \param[in,out] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          On entry, the i-th row must contain the vector which defines
*>          the elementary reflector H(i), for i = 1,2,...,k, as returned
*>          by DGELQF in the first k rows of its array argument A.
*>          On exit, the m-by-n matrix Q.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The first dimension of the array A. LDA >= max(1,M).
*> \endverbatim
*>
*> \param[in] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION array, dimension (K)
*>          TAU(i) must contain the scalar factor of the elementary
*>          reflector H(i), as returned by DGELQF.
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array, dimension (M)
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0: successful exit
*>          < 0: if INFO = -i, the i-th argument has an illegal value
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup doubleOTHERcomputational
*
*  =====================================================================
!!      SUBROUTINE DORGL2( M, N, K, A, LDA, TAU, WORK, INFO )
      SUBROUTINE dDORGL2( M, N, K, Ah, Al, LDA, TAUh, TAUl,
     $                   WORKh, WORKl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            INFO, K, LDA, M, N
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision TAUh(*), TAUl(*)
      double precision WORKh(*), WORKl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ONE, ZERO
!!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            I, J, L
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DLARF, DSCAL, XERBLA
      EXTERNAL           dDLARF, dDSCAL, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
      double precision tmp1h, tmp1l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Executable Statements ..
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
*
*     Test the input arguments
*
      INFO = 0
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.M ) THEN
         INFO = -2
      ELSE IF( K.LT.0 .OR. K.GT.M ) THEN
         INFO = -3
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -5
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DORGL2', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( M.LE.0 )
     $   RETURN
*
      IF( K.LT.M ) THEN
*
*        Initialise rows k+1:m to rows of the unit matrix
*
         DO 20 J = 1, N
            DO 10 L = K + 1, M
!!               A( L, J ) = ZERO
      Ah(L,J) = ZEROh
      Al(L,J) = ZEROl
   10       CONTINUE
!!            IF( J.GT.K .AND. J.LE.M )
!!     $         A( J, J ) = ONE
            IF( J.GT.K .AND. J.LE.M ) THEN
      Ah(J,J) = ONEh
      Al(J,J) = ONEl
            END IF
   20    CONTINUE
      END IF
*
      DO 40 I = K, 1, -1
*
*        Apply H(i) to A(i:m,i:n) from the right
*
         IF( I.LT.N ) THEN
            IF( I.LT.M ) THEN
!!               A( I, I ) = ONE
      Ah(I,I) = ONEh
      Al(I,I) = ONEl
!!               CALL DLARF( 'Right', M-I, N-I+1, A( I, I ), LDA,
!!     $                     TAU( I ), A( I+1, I ), LDA, WORK )
               CALL dDLARF( 'Right', M-I, N-I+1,
     $                     Ah( I, I ), Al( I, I ), LDA,
     $                     TAUh( I ), TAUl( I ),
     $                     Ah( I+1, I ), Al( I+1, I ), LDA,
     $                     WORKh, WORKl )
            END IF
!<<<  tmp1 = -TAU(I)
      tmp1h = -TAUh(I)
      tmp1l = -TAUl(I)
!>>>  tmp1 = -TAU(I)
!!            CALL DSCAL( N-I, -TAU( I ), A( I, I+1 ), LDA )
            CALL dDSCAL( N-I, tmp1h, tmp1l, Ah( I, I+1 ), Al( I, I+1 ),
     $                  LDA )
         END IF
!!         A( I, I ) = ONE - TAU( I )
!<<<  tmp1 = ONE - TAU(I)
      T_p1 = ONEh - TAUh(I)
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (TAUh(I) + T_p2)
      T_cl = T_cl + ONEl - TAUl(I)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = ONE - TAU(I)
      Ah(I,I) = tmp1h
      Al(I,I) = tmp1l
*
*        Set A(i,1:i-1) to zero
*
         DO 30 L = 1, I - 1
!!            A( I, L ) = ZERO
      Ah(I,L) = ZEROh
      Al(I,L) = ZEROl
   30    CONTINUE
   40 CONTINUE
      RETURN
*
*     End of DORGL2
*
      END
