*> \brief \b DSYTD2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DSYTD2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dsytd2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dsytd2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dsytd2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DSYTD2( UPLO, N, A, LDA, D, E, TAU, INFO )
* 
*       .. Scalar Arguments ..
*       CHARACTER          UPLO
*       INTEGER            INFO, LDA, N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * ), D( * ), E( * ), TAU( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DSYTD2 reduces a real symmetric matrix A to symmetric tridiagonal
*> form T by an orthogonal similarity transformation: Q**T * A * Q = T.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] UPLO
*> \verbatim
*>          UPLO is CHARACTER*1
*>          Specifies whether the upper or lower triangular part of the
*>          symmetric matrix A is stored:
*>          = 'U':  Upper triangular
*>          = 'L':  Lower triangular
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix A.  N >= 0.
*> \endverbatim
*>
*> \param[in,out] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          On entry, the symmetric matrix A.  If UPLO = 'U', the leading
*>          n-by-n upper triangular part of A contains the upper
*>          triangular part of the matrix A, and the strictly lower
*>          triangular part of A is not referenced.  If UPLO = 'L', the
*>          leading n-by-n lower triangular part of A contains the lower
*>          triangular part of the matrix A, and the strictly upper
*>          triangular part of A is not referenced.
*>          On exit, if UPLO = 'U', the diagonal and first superdiagonal
*>          of A are overwritten by the corresponding elements of the
*>          tridiagonal matrix T, and the elements above the first
*>          superdiagonal, with the array TAU, represent the orthogonal
*>          matrix Q as a product of elementary reflectors; if UPLO
*>          = 'L', the diagonal and first subdiagonal of A are over-
*>          written by the corresponding elements of the tridiagonal
*>          matrix T, and the elements below the first subdiagonal, with
*>          the array TAU, represent the orthogonal matrix Q as a product
*>          of elementary reflectors. See Further Details.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A.  LDA >= max(1,N).
*> \endverbatim
*>
*> \param[out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          The diagonal elements of the tridiagonal matrix T:
*>          D(i) = A(i,i).
*> \endverbatim
*>
*> \param[out] E
*> \verbatim
*>          E is DOUBLE PRECISION array, dimension (N-1)
*>          The off-diagonal elements of the tridiagonal matrix T:
*>          E(i) = A(i,i+1) if UPLO = 'U', E(i) = A(i+1,i) if UPLO = 'L'.
*> \endverbatim
*>
*> \param[out] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION array, dimension (N-1)
*>          The scalar factors of the elementary reflectors (see Further
*>          Details).
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit
*>          < 0:  if INFO = -i, the i-th argument had an illegal value.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup doubleSYcomputational
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  If UPLO = 'U', the matrix Q is represented as a product of elementary
*>  reflectors
*>
*>     Q = H(n-1) . . . H(2) H(1).
*>
*>  Each H(i) has the form
*>
*>     H(i) = I - tau * v * v**T
*>
*>  where tau is a real scalar, and v is a real vector with
*>  v(i+1:n) = 0 and v(i) = 1; v(1:i-1) is stored on exit in
*>  A(1:i-1,i+1), and tau in TAU(i).
*>
*>  If UPLO = 'L', the matrix Q is represented as a product of elementary
*>  reflectors
*>
*>     Q = H(1) H(2) . . . H(n-1).
*>
*>  Each H(i) has the form
*>
*>     H(i) = I - tau * v * v**T
*>
*>  where tau is a real scalar, and v is a real vector with
*>  v(1:i) = 0 and v(i+1) = 1; v(i+2:n) is stored on exit in A(i+2:n,i),
*>  and tau in TAU(i).
*>
*>  The contents of A on exit are illustrated by the following examples
*>  with n = 5:
*>
*>  if UPLO = 'U':                       if UPLO = 'L':
*>
*>    (  d   e   v2  v3  v4 )              (  d                  )
*>    (      d   e   v3  v4 )              (  e   d              )
*>    (          d   e   v4 )              (  v1  e   d          )
*>    (              d   e  )              (  v1  v2  e   d      )
*>    (                  d  )              (  v1  v2  v3  e   d  )
*>
*>  where d and e denote diagonal and off-diagonal elements of T, and vi
*>  denotes an element of the vector defining H(i).
*> \endverbatim
*>
*  =====================================================================
!!      SUBROUTINE DSYTD2( UPLO, N, A, LDA, D, E, TAU, INFO )
      SUBROUTINE dDSYTD2( UPLO, N, Ah, Al, LDA, Dh, Dl, Eh, El, TAUh,
     &                    TAUl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            INFO, LDA, N
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   A( LDA, * ), D( * ), E( * ), TAU( * )
      double precision Ah(LDA,*), Al(LDA,*)
      double precision Dh(*), Dl(*)
      double precision Eh(*), El(*)
      double precision TAUh(*), TAUl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ONE, ZERO, HALF
!!      PARAMETER          ( ONE = 1.0D0, ZERO = 0.0D0,
!!     $                   HALF = 1.0D0 / 2.0D0 )
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
      double precision HALFh, HALFl
*     ..
*     .. Local Scalars ..
      LOGICAL            UPPER
      INTEGER            I
!!      DOUBLE PRECISION   ALPHA, TAUI
      double precision ALPHAh, ALPHAl
      double precision TAUIh, TAUIl
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DAXPY, DLARFG, DSYMV, DSYR2, XERBLA
      EXTERNAL           dDAXPY, dDLARFG, dDSYMV, dDSYR2, XERBLA, dDDOT
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
!!      DOUBLE PRECISION   DDOT
!!      EXTERNAL           LSAME, DDOT
      EXTERNAL           LSAME
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Executable Statements ..
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      tmp1h = 2.0D0
      tmp1l = 0.0d+0
!<<<  HALF = ONE / tmp1
      T_q1 = ONEh / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = ONEh - T_ch
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      HALFh = T_q1 + T_q2
      HALFl = T_q2
     +      - (HALFh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  HALF = ONE / tmp1
*
*     Test the input parameters
*
      INFO = 0
      UPPER = LSAME( UPLO, 'U' )
      IF( .NOT.UPPER .AND. .NOT.LSAME( UPLO, 'L' ) ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -4
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DSYTD2', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.LE.0 )
     $   RETURN
*
      IF( UPPER ) THEN
*
*        Reduce the upper triangle of A
*
         DO 10 I = N - 1, 1, -1
*
*           Generate elementary reflector H(i) = I - tau * v * v**T
*           to annihilate A(1:i-1,i+1)
*
!!            CALL DLARFG( I, A( I, I+1 ), A( 1, I+1 ), 1, TAUI )
            CALL dDLARFG( I, Ah( I, I+1 ), Al( I, I+1 ), Ah( 1, I+1 ),
     &                    Al( 1, I+1 ), 1, TAUIh, TAUIl )
!!            E( I ) = A( I, I+1 )
      Eh(I) = Ah(I,I+1)
      El(I) = Al(I,I+1)
*
!!            IF( TAUI.NE.ZERO ) THEN
            IF(
!       (TAUI .ne. ZERO)
     + ((TAUIh .ne. ZEROh) .or.
     +  (TAUIl .ne. ZEROl))
     &        ) THEN
*
*              Apply H(i) from both sides to A(1:i,1:i)
*
!!               A( I, I+1 ) = ONE
      Ah(I,I+1) = ONEh
      Al(I,I+1) = ONEl
*
*              Compute  x := tau * A * v  storing x in TAU(1:i)
*
!!               CALL DSYMV( UPLO, I, TAUI, A, LDA, A( 1, I+1 ), 1, ZERO,
!!     $                     TAU, 1 )
               CALL dDSYMV( UPLO, I, TAUIh, TAUIl, Ah, Al, LDA,
     &                      Ah( 1, I+1 ), Al( 1, I+1 ), 1, 
     &                      ZEROh, ZEROl,
     &                      TAUh, TAUl, 1 )
*
*              Compute  w := x - 1/2 * tau * (x**T * v) * v
*
!!               ALPHA = -HALF*TAUI*DDOT( I, TAU, 1, A( 1, I+1 ), 1 )
!<<<  tmp1 = -HALF
      tmp1h = -HALFh
      tmp1l = -HALFl
!>>>  tmp1 = -HALF
!<<<  tmp2 = tmp1 * TAUI
      T_ch = tmp1h * TAUIh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * TAUIh
      T_r1 = T_u1
     +         - (T_u1 - TAUIh)
      T_r2 = TAUIh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*TAUIl
     +         +  tmp1l*TAUIh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * TAUI
               CALL dDDOT( I, TAUh, TAUl, 1, Ah( 1, I+1 ), 
     &                     Al( 1, I+1 ), 1, tmp1h, tmp1l )
!<<<  ALPHA = tmp2 * tmp1
      T_ch = tmp2h * tmp1h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp1l
     +         +  tmp2l*tmp1h)
      T_s1 = T_ch + T_cl
      ALPHAl = T_cl - (T_s1 - T_ch)
      ALPHAh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  ALPHA = tmp2 * tmp1
!!               CALL DAXPY( I, ALPHA, A( 1, I+1 ), 1, TAU, 1 )
               CALL dDAXPY( I, ALPHAh, ALPHAl, Ah( 1, I+1 ),
     &                      Al( 1, I+1 ), 1, TAUh, TAUl, 1 )
*
*              Apply the transformation as a rank-2 update:
*                 A := A - v * w**T - w * v**T
*
!!               CALL DSYR2( UPLO, I, -ONE, A( 1, I+1 ), 1, TAU, 1, A,
!!     $                     LDA )
!<<<  tmp1 = -ONE
      tmp1h = -ONEh
      tmp1l = -ONEl
!>>>  tmp1 = -ONE
               CALL dDSYR2( UPLO, I, tmp1h, tmp1l, Ah( 1, I+1 ),
     &                      Al( 1, I+1 ), 1, TAUh, TAUl, 1, Ah, Al,
     &                      LDA )
*
!!               A( I, I+1 ) = E( I )
      Ah(I,I+1) = Eh(I)
      Al(I,I+1) = El(I)
            END IF
!!            D( I+1 ) = A( I+1, I+1 )
      Dh(I+1) = Ah(I+1,I+1)
      Dl(I+1) = Al(I+1,I+1)
!!            TAU( I ) = TAUI
      TAUh(I) = TAUIh
      TAUl(I) = TAUIl
   10    CONTINUE
!!         D( 1 ) = A( 1, 1 )
      Dh(1) = Ah(1,1)
      Dl(1) = Al(1,1)
      ELSE
*
*        Reduce the lower triangle of A
*
         DO 20 I = 1, N - 1
*
*           Generate elementary reflector H(i) = I - tau * v * v**T
*           to annihilate A(i+2:n,i)
*
!!            CALL DLARFG( N-I, A( I+1, I ), A( MIN( I+2, N ), I ), 1,
!!     $                   TAUI )
            CALL dDLARFG( N-I, Ah( I+1, I ), Al( I+1, I ),
     &                    Ah( MIN( I+2, N ), I ),
     &                    Al( MIN( I+2, N ), I ), 1,
     &                   TAUIh, TAUIl )
!!            E( I ) = A( I+1, I )
      Eh(I) = Ah(I+1,I)
      El(I) = Al(I+1,I)
*
!!            IF( TAUI.NE.ZERO ) THEN
            IF(
!       (TAUI .ne. ZERO)
     + ((TAUIh .ne. ZEROh) .or.
     +  (TAUIl .ne. ZEROl))
     &        ) THEN
*
*              Apply H(i) from both sides to A(i+1:n,i+1:n)
*
!!               A( I+1, I ) = ONE
      Ah(I+1,I) = ONEh
      Al(I+1,I) = ONEl
*
*              Compute  x := tau * A * v  storing y in TAU(i:n-1)
*
!!               CALL DSYMV( UPLO, N-I, TAUI, A( I+1, I+1 ), LDA,
!!     $                     A( I+1, I ), 1, ZERO, TAU( I ), 1 )

               CALL dDSYMV( UPLO, N-I, TAUIh, TAUIl, Ah( I+1, I+1 ),
     &                      Al( I+1, I+1 ), LDA,
     &                      Ah( I+1, I ), Al( I+1, I ), 1,
     &                      ZEROh, ZEROl, TAUh( I ), TAUl( I ), 1 )
*
*              Compute  w := x - 1/2 * tau * (x**T * v) * v
*
!!               ALPHA = -HALF*TAUI*DDOT( N-I, TAU( I ), 1, A( I+1, I ),
!!     $                 1 )
!<<<  tmp1 = -HALF
      tmp1h = -HALFh
      tmp1l = -HALFl
!>>>  tmp1 = -HALF
!<<<  tmp2 = tmp1 * TAUI
      T_ch = tmp1h * TAUIh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * TAUIh
      T_r1 = T_u1
     +         - (T_u1 - TAUIh)
      T_r2 = TAUIh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*TAUIl
     +         +  tmp1l*TAUIh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * TAUI
               CALL dDDOT( N-I, TAUh( I ), TAUl( I ), 1,
     &                     Ah( I+1, I ), Al( I+1, I ),
     &                     1, tmp1h, tmp1l )
!<<<  ALPHA = tmp2 * tmp1
      T_ch = tmp2h * tmp1h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp1l
     +         +  tmp2l*tmp1h)
      T_s1 = T_ch + T_cl
      ALPHAl = T_cl - (T_s1 - T_ch)
      ALPHAh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  ALPHA = tmp2 * tmp1
!!               CALL DAXPY( N-I, ALPHA, A( I+1, I ), 1, TAU( I ), 1 )
               CALL dDAXPY( N-I, ALPHAh, ALPHAl, Ah( I+1, I ),
     &                      Al( I+1, I ), 1, TAUh( I ), TAUl( I ), 1 )
*
*              Apply the transformation as a rank-2 update:
*                 A := A - v * w**T - w * v**T
*
!!               CALL DSYR2( UPLO, N-I, -ONE, A( I+1, I ), 1, TAU( I ), 1,
!!     $                     A( I+1, I+1 ), LDA )
!<<<  tmp1 = -ONE
      tmp1h = -ONEh
      tmp1l = -ONEl
!>>>  tmp1 = -ONE
               CALL dDSYR2( UPLO, N-I, tmp1h, tmp1l, Ah( I+1, I ),
     &                      Al( I+1, I ), 1, TAUh( I ), TAUl( I ), 1,
     &                      Ah( I+1, I+1 ), Al( I+1, I+1 ), LDA )
*
!!               A( I+1, I ) = E( I )
      Ah(I+1,I) = Eh(I)
      Al(I+1,I) = El(I)
            END IF
!!            D( I ) = A( I, I )
      Dh(I) = Ah(I,I)
      Dl(I) = Al(I,I)
!!            TAU( I ) = TAUI
      TAUh(I) = TAUIh
      TAUl(I) = TAUIl
   20    CONTINUE
!!         D( N ) = A( N, N )
      Dh(N) = Ah(N,N)
      Dl(N) = Al(N,N)
      END IF
*
      RETURN
*
*     End of DSYTD2
*
      END
