*> \brief \b ILADLR
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download ILADLR + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/iladlr.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/iladlr.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/iladlr.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       INTEGER FUNCTION ILADLR( M, N, A, LDA )
* 
*       .. Scalar Arguments ..
*       INTEGER            M, N, LDA
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   A( LDA, * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> ILADLR scans A for its last non-zero row.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] M
*> \verbatim
*>          M is INTEGER
*>          The number of rows of the matrix A.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The number of columns of the matrix A.
*> \endverbatim
*>
*> \param[in] A
*> \verbatim
*>          A is DOUBLE PRECISION array, dimension (LDA,N)
*>          The m by n matrix A.
*> \endverbatim
*>
*> \param[in] LDA
*> \verbatim
*>          LDA is INTEGER
*>          The leading dimension of the array A. LDA >= max(1,M).
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!!      INTEGER FUNCTION ILADLR( M, N, A, LDA )
      INTEGER FUNCTION dILADLR( M, N, Ah, Al, LDA )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            M, N, LDA
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   A( LDA, * )
      double precision Ah(LDA,*), Al(LDA,*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION ZERO
!!      PARAMETER ( ZERO = 0.0D+0 )
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER I, J
*     ..
*     .. Executable Statements ..
*
*     Quick test for the common case where one corner is non-zero.

      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      IF( M.EQ.0 ) THEN
!         ILADLR = M
         dILADLR = M
      ELSE IF( 
!!     A(M, 1).NE.ZERO
!       (A(1,N) .ne. 0.0)
     + ((Ah(1,N) .ne. 0.0d+0) .or.
     +  (Al(1,N) .ne. 0.0d+0))
     $ .OR.
!! A(M, N).NE.ZERO
!       (A(M,N) .ne. 0.0)
     + ((Ah(M,N) .ne. 0.0d+0) .or.
     +  (Al(M,N) .ne. 0.0d+0))
     $     ) THEN
!         ILADLR = M
         dILADLR = M
      ELSE
*     Scan up each column tracking the last zero row seen.
!         ILADLR = 0
         dILADLR = 0
         DO J = 1, N
            I=M
            DO WHILE (
!!(A(I,J).NE.ZERO)
!       (A(I,J) .ne. 0.0)
     + ((Ah(I,J) .ne. 0.0d+0) .or.
     +  (Al(I,J) .ne. 0.0d+0))
     $           .AND.(I.GE.1))
              I=I-1
              IF (I.EQ.0) THEN
                 EXIT
              END IF
            ENDDO
!            ILADLR = MAX( ILADLR, I )
            dILADLR = MAX( dILADLR, I )
         END DO
      END IF
      RETURN
      END
