      SUBROUTINE DDMMCADD(
     &     M, N,
     &     ALPHAh, ALPHAl,
     &     Ah, Al, LDA,
     &     BETAh,  BETAl,
     &     Bh, Bl, LDB
     &     )
*
*  -- PBLAS auxiliary routine (version 2.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     April 1, 1998
*
*     .. Scalar Arguments ..
      INTEGER            LDA, LDB, M, N
!     *** Quadruple precision ***
      DOUBLE PRECISION ALPHAh, BETAh
      DOUBLE PRECISION ALPHAl, BETAl
!     ***************************
*     ..
*     .. Array Arguments ..
!     *** Quadruple precision ***
      DOUBLE PRECISION   Ah( LDA, * ), Bh( LDB, * )
      DOUBLE PRECISION   Al( LDA, * ), Bl( LDB, * )
!     ***************************
*     ..
*
*  Purpose
*  =======
*
*  DMMCADD performs the following operation:
*
*     B := alpha * A + beta * B,
*
*  where alpha, beta are scalars and A and B are m by n matrices.
*
*  Arguments
*  =========
*
*  M       (local input) INTEGER
*          On entry, M  specifies the number of rows of A and B. M  must
*          be at least zero.
*
*  N       (local input) INTEGER
*          On entry, N  specifies  the  number  of  columns  of A and B.
*          N must be at least zero.
*
*  ALPHA   (local input) DOUBLE PRECISION
*          On entry,  ALPHA  specifies the scalar alpha. When  ALPHA  is
*          supplied as zero then the local entries of the array  A  need
*          not be set on input.
*
*  A       (local input) DOUBLE PRECISION array
*          On entry, A is an array of dimension ( LDA, N ).
*
*  LDA     (local input) INTEGER
*          On entry, LDA specifies the leading dimension of the array A.
*          LDA must be at least max( 1, M ).
*
*  BETA    (local input) DOUBLE PRECISION
*          On entry,  BETA  specifies the scalar beta. When BETA is sup-
*          plied as zero then the local entries of the array B need  not
*          be set on input.
*
*  B       (local input/local output) DOUBLE PRECISION array
*          On entry, B is an array of dimension ( LDB, N ). On exit, the
*          leading m by n part of A has been added to the leading m by n
*          part of B.
*
*  LDB     (local input) INTEGER
*          On entry, LDB specifies the leading dimension of the array B.
*          LDB must be at least max( 1, M ).
*
*  -- Written on April 1, 1998 by
*     Antoine Petitet, University  of  Tennessee, Knoxville 37996, USA.
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE, ZERO
      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, J
!     *** Quadruple precision ***
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_u1
!     ***************************
*     ..
*     .. External Subroutines ..
      EXTERNAL           DAXPY, DCOPY, DSCAL
*     ..
*     .. Executable Statements ..
*
      IF( (ALPHAh.EQ.ONE).AND.(ALPHAl.EQ.ZERO) ) THEN
         IF( (BETAh.EQ.ZERO).AND.(BETAl.EQ.ZERO) ) THEN
!           *** Quadruple precision ***
            DO 20 J = 1, N
               CALL DDCOPY(
     &              M,
     &              Ah( 1, J ), Al( 1, J ), 1,
     &              Bh( 1, J ), Bl( 1, J ), 1
     &              )
*              DO 10 I = 1, M
*                 B( I, J ) = A( I, J )
*  10          CONTINUE
   20       CONTINUE
!           ***************************
         ELSE IF( (BETAh.NE.ONE).OR.(BETAl.NE.ZERO) ) THEN
!           *** Quadruple precision *** ???
            DO 40 J = 1, N
               DO 30 I = 1, M
!                 Bh( I, J ) = Ah( I, J ) + BETAh * Bh( I, J )
!                 tmp2 = BETAh  * Bh( I, J )
                  tmp2h = BETAh * Bh(I,J)
                  T_u1 = 134217729.0 * BETAh
                  T_p1 = T_u1
     +                 - (T_u1 - BETAh)
                  T_p2 = BETAh - T_p1
                  T_u1 = 134217729.0 * Bh(I,J)
                  T_r1 = T_u1
     +                 - (T_u1 - Bh(I,J))
                  T_r2 = Bh(I,J) - T_r1
                  tmp2l = ((T_p1*T_r1
     +                 - tmp1h)
     +                 + T_p1 * T_r2
     +                 + T_p2 * T_r1)
     +                 + T_p2 * T_r2
                  tmp2l = tmp2l
     +                 + (BETAh*Bl(I,J)
     +                 +  BETAl*Bh(I,J)
     +                 )
                  T_s1 = tmp2h + tmp2l
                  tmp2l = tmp2l - (T_s1 - tmp2h)
                  tmp2h = T_s1
!
!                 B( I, J ) = A( I, J ) + tmp2
                  tmp1h = Ah(I,J)
                  tmp1l = Al(I,J)
                  T_p1 = tmp1h + tmp2h
                  T_p2 = T_p1 - tmp1h
                  T_q1 = T_p1
                  T_q2 = (tmp1h - (T_p1 - T_p2))
     +                 + (tmp2h - T_p2)
                  T_q2 = T_q2 + tmp1l + tmp2l
                  Bh(I,J) = T_q1 + T_q2
                  Bl(I,J) = T_q2 - (tmp1l - T_q1)
   30          CONTINUE
   40       CONTINUE
!           ***************************
         ELSE
!           *** Quadruple precision ***
            DO 60 J = 1, N
               CALL DDAXPY(
     &              M,
     &              ONE,ZERO,
     &              Ah( 1, J ), Al( 1, J ), 1,
     &              Bh( 1, J ), Bl( 1, J ), 1
     &              )
*              DO 50 I = 1, M
*                 B( I, J ) = A( I, J ) + B( I, J )
*  50          CONTINUE
   60       CONTINUE
!           ***************************
         END IF
      ELSE IF( (ALPHAh.NE.ZERO).OR.(ALPHAl.NE.ZERO) ) THEN
         IF( (BETAh.EQ.ZERO).AND.(BETAl.EQ.ZERO) ) THEN
!           *** Quadruple precision *** ???
            DO 80 J = 1, N
               DO 70 I = 1, M
!                 Bh( I, J ) = ALPHAh * Ah( I, J )
                  tmp1h = ALPHAh * Ah(I,J)
                  T_u1 = 134217729.0 * ALPHAh
                  T_p1 = T_u1
     +                 - (T_u1 - ALPHAh)
                  T_p2 = ALPHAh - T_p1
                  T_u1 = 134217729.0 * Ah(I,J)
                  T_r1 = T_u1
     +                 - (T_u1 - Ah(I,J))
                  T_r2 = Ah(I,J) - T_r1
                  tmp1l = ((T_p1*T_r1
     +                 - tmp1h)
     +                 + T_p1 * T_r2
     +                 + T_p2 * T_r1)
     +                 + T_p2 * T_r2
                  tmp1l = tmp1l
     +                 + (ALPHAh*Al(I,J)
     +                 +  ALPHAl*Ah(I,J)
     +                 )
                  T_s1 = tmp1h + tmp1l
                  Bl(I,J) = tmp1l - (T_s1 - tmp1h)
                  Bh(I,J) = T_s1
   70          CONTINUE
   80       CONTINUE
!           ***************************
         ELSE IF( (BETAh.NE.ONE).OR.(BETAl.NE.ZERO) ) THEN
!           *** Quadruple precision *** ???
            DO 100 J = 1, N
               DO 90 I = 1, M
!                 Bh( I, J ) = ALPHAh * Ah( I, J ) + BETAh * Bh( I, J )
!                 tmp1 = ALPHAh * Ah( I, J )
                  tmp1h = ALPHAh * Ah(I,J)
                  T_u1 = 134217729.0 * ALPHAh
                  T_p1 = T_u1
     +                 - (T_u1 - ALPHAh)
                  T_p2 = ALPHAh - T_p1
                  T_u1 = 134217729.0 * Ah(I,J)
                  T_r1 = T_u1
     +                 - (T_u1 - Ah(I,J))
                  T_r2 = Ah(I,J) - T_r1
                  tmp1l = ((T_p1*T_r1
     +                 - tmp1h)
     +                 + T_p1 * T_r2
     +                 + T_p2 * T_r1)
     +                 + T_p2 * T_r2
                  tmp1l = tmp1l
     +                 + (ALPHAh*Al(I,J)
     +                 +  ALPHAl*Ah(I,J)
     +                 )
                  T_s1 = tmp1h + tmp1l
                  tmp1l = tmp1l - (T_s1 - tmp1h)
                  tmp1h = T_s1
!
!                 tmp2 = BETAh  * Bh( I, J )
                  tmp2h = BETAh * Bh(I,J)
                  T_u1 = 134217729.0 * BETAh
                  T_p1 = T_u1
     +                 - (T_u1 - BETAh)
                  T_p2 = BETAh - T_p1
                  T_u1 = 134217729.0 * Bh(I,J)
                  T_r1 = T_u1
     +                 - (T_u1 - Bh(I,J))
                  T_r2 = Bh(I,J) - T_r1
                  tmp2l = ((T_p1*T_r1
     +                 - tmp1h)
     +                 + T_p1 * T_r2
     +                 + T_p2 * T_r1)
     +                 + T_p2 * T_r2
                  tmp2l = tmp2l
     +                 + (BETAh*Bl(I,J)
     +                 +  BETAl*Bh(I,J)
     +                 )
                  T_s1 = tmp2h + tmp2l
                  tmp2l = tmp2l - (T_s1 - tmp2h)
                  tmp2h = T_s1
!
!                 B(I,J) = tmp1 + tmp2
                  T_p1 = tmp1h + tmp2h
                  T_p2 = T_p1 - tmp1h
                  T_q1 = T_p1
                  T_q2 = (tmp1h - (T_p1 - T_p2))
     +                 + (tmp2h - T_p2)
                  T_q2 = T_q2 + tmp1l + tmp2l
                  Bh(I,J) = T_q1 + T_q2
                  Bl(I,J) = T_q2 - (tmp1l - T_q1)
   90          CONTINUE
  100       CONTINUE
!           ***************************
         ELSE
!           *** Quadruple precision ***
            DO 120 J = 1, N
               CALL DDAXPY(
     &              M, ALPHAh, ALPHAl,
     &              Ah( 1, J ), Al( 1, J ), 1,
     &              Bh( 1, J ), Bl( 1, J ), 1
     &              )
*              DO 110 I = 1, M
*                 B( I, J ) = ALPHA * A( I, J ) + B( I, J )
* 110          CONTINUE
  120       CONTINUE
!           ***************************
         END IF
      ELSE
         IF( (BETAh.EQ.ZERO).AND.(BETAl.EQ.ZERO) ) THEN
!           *** Quadruple precision ***
            DO 140 J = 1, N
               DO 130 I = 1, M
                  Bh( I, J ) = ZERO
                  Bl( I, J ) = ZERO
  130          CONTINUE
  140       CONTINUE
!           ***************************
         ELSE IF( (BETAh.NE.ONE).OR.(BETAl.NE.ZERO) ) THEN
!           *** Quadruple precision ***
            DO 160 J = 1, N
               CALL DDSCAL( M, BETAh, BETAl, Bh( 1, J ), Bl( 1, J ), 1 )
*              DO 150 I = 1, M
*                 B( I, J ) = BETA * B( I, J )
* 150          CONTINUE
  160       CONTINUE
!           ***************************
         END IF
      END IF
*
      RETURN
*
*     End of DMMCADD
*
      END
