!      SUBROUTINE PDLAED3( ICTXT, K, N, NB, D, DROW, DCOL, RHO, DLAMDA,
!     $                    W, Z, U, LDU, BUF, INDX, INDCOL, INDROW,
!     $                    INDXR, INDXC, CTOT, NPCOL, INFO )
      SUBROUTINE PDDLAED3( ICTXT, K, N, NB, Dh, Dl, DROW, DCOL, 
     $                    RHOh, RHOl, DLAMDAh,DLAMDAl,
     $                    Wh, Wl, Zh, Zl, Uh, Ul, LDU, BUFh, BUFl,
     $                    INDX, INDCOL, INDROW,
     $                    INDXR, INDXC, CTOT, NPCOL, INFO )
*
*  -- ScaLAPACK auxiliary routine (version 1.7) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     December 31, 1998
*
*     .. Scalar Arguments ..
      INTEGER            DCOL, DROW, ICTXT, INFO, K, LDU, N, NB, NPCOL
!      DOUBLE PRECISION   RHO
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
      INTEGER            CTOT( 0: NPCOL-1, 4 ), INDCOL( * ),
     $                   INDROW( * ), INDX( * ), INDXC( * ), INDXR( * )
!      DOUBLE PRECISION   BUF( * ), D( * ), DLAMDA( * ), U( LDU, * ),
!     $                   W( * ), Z( * )
      double precision BUFh(*), BUFl(*)
      double precision Dh(*), Dl(*)
      double precision DLAMDAh(*), DLAMDAl(*)
      double precision Uh(LDU,*), Ul(LDU,*)
      double precision Wh(*), Wl(*)
      double precision Zh(*), Zl(*)
!... MPI ...
#ifdef COMM_MPI
      include 'mpif.h'
      integer ireq(2),ierr
      integer DEST,TAG,COMM
      integer myrank
      integer status(MPI_STATUS_SIZE)
      double precision BUFF(4*N),SBUFF(4*N)
#endif
!.............
*     ..
*
*  Purpose
*  =======
*
*  PDLAED3 finds the roots of the secular equation, as defined by the
*  values in D, W, and RHO, between 1 and K.  It makes the
*  appropriate calls to SLAED4
*
*  This code makes very mild assumptions about floating point
*  arithmetic. It will work on machines with a guard digit in
*  add/subtract, or on those binary machines without guard digits
*  which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or Cray-2.
*  It could conceivably fail on hexadecimal or decimal machines
*  without guard digits, but we know of none.
*
*  Arguments
*  =========
*
*  ICTXT  (global input) INTEGER
*         The BLACS context handle, indicating the global context of
*         the operation on the matrix. The context itself is global.
*
*  K      (output) INTEGER
*         The number of non-deflated eigenvalues, and the order of the
*         related secular equation. 0 <= K <=N.
*
*  N      (input) INTEGER
*         The dimension of the symmetric tridiagonal matrix.  N >= 0.
*
*  NB      (global input) INTEGER
*          The blocking factor used to distribute the columns of the
*          matrix. NB >= 1.
*
*  D      (input/output) DOUBLE PRECISION array, dimension (N)
*         On entry, D contains the eigenvalues of the two submatrices to
*         be combined.
*         On exit, D contains the trailing (N-K) updated eigenvalues
*         (those which were deflated) sorted into increasing order.
*
*  DROW   (global input) INTEGER
*          The process row over which the first row of the matrix D is
*          distributed. 0 <= DROW < NPROW.
*
*  DCOL   (global input) INTEGER
*          The process column over which the first column of the
*          matrix D is distributed. 0 <= DCOL < NPCOL.
*
*  Q      (input/output) DOUBLE PRECISION array, dimension (LDQ, N)
*         On entry, Q contains the eigenvectors of two submatrices in
*         the two square blocks with corners at (1,1), (N1,N1)
*         and (N1+1, N1+1), (N,N).
*         On exit, Q contains the trailing (N-K) updated eigenvectors
*         (those which were deflated) in its last N-K columns.
*
*  LDQ    (input) INTEGER
*         The leading dimension of the array Q.  LDQ >= max(1,NQ).
*
*  RHO    (global input/output) DOUBLE PRECISION
*         On entry, the off-diagonal element associated with the rank-1
*         cut which originally split the two submatrices which are now
*         being recombined.
*         On exit, RHO has been modified to the value required by
*         PDLAED3.
*
*  DLAMDA (global output) DOUBLE PRECISION array, dimension (N)
*         A copy of the first K eigenvalues which will be used by
*         SLAED3 to form the secular equation.
*
*  W      (global output) DOUBLE PRECISION array, dimension (N)
*         The first k values of the final deflation-altered z-vector
*         which will be passed to SLAED3.
*
*  Z      (global input) DOUBLE PRECISION array, dimension (N)
*         On entry, Z contains the updating vector (the last
*         row of the first sub-eigenvector matrix and the first row of
*         the second sub-eigenvector matrix).
*         On exit, the contents of Z have been destroyed by the updating
*         process.
*
*  U     (global output) DOUBLE PRECISION array
*         global dimension (N, N), local dimension (LDU, NQ).
*         Q  contains the orthonormal eigenvectors of the symmetric
*         tridiagonal matrix.
*
*  LDU    (input) INTEGER
*         The leading dimension of the array U.
*
*  QBUF   (workspace) DOUBLE PRECISION array, dimension 3*N
*
*
*  INDX   (workspace) INTEGER array, dimension (N)
*         The permutation used to sort the contents of DLAMDA into
*         ascending order.
*
*  INDCOL (workspace) INTEGER array, dimension (N)
*
*
*  INDROW (workspace) INTEGER array, dimension (N)
*
*
*  INDXR (workspace) INTEGER array, dimension (N)
*
*
*  INDXC (workspace) INTEGER array, dimension (N)
*
*  CTOT   (workspace) INTEGER array, dimension( NPCOL, 4)
*
*  NPCOL   (global input) INTEGER
*          The total number of columns over which the distributed
*           submatrix is distributed.
*
*  INFO   (output) INTEGER
*          = 0:  successful exit.
*          < 0:  if INFO = -i, the i-th argument had an illegal value.
*          > 0:  The algorithm failed to compute the ith eigenvalue.
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE
!      PARAMETER          ( ONE = 1.0D+0 )
      double precision ONEh, ONEl
*     ..
*     .. Local Scalars ..
      INTEGER            COL, GI, I, IINFO, IIU, IPD, IU, J, JJU, JU,
     $                   KK, KL, KLC, KLR, MYCOL, MYKL, MYKLR, MYROW,
     $                   NPROW, PDC, PDR, ROW
!      DOUBLE PRECISION   AUX, TEMP
      double precision AUXh, AUXl
      double precision TEMPh, TEMPl

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
#ifdef COMM_BLACS2
      double precision BUF(6*N)
#endif
*     ..
*     .. External Functions ..
      INTEGER            INDXG2L
!      DOUBLE PRECISION   DLAMC3, DNRM2
!      EXTERNAL           INDXG2L, DLAMC3, DNRM2
      EXTERNAL           INDXG2L
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DDCOPY, DGEBR2D, DGEBS2D,
     $                   DGERV2D, DGESD2D, DDLAED4, DDLAMC3, DDNRM2
!... MPI ...
#ifdef COMM_MPI
      EXTERNAL           MakeMpiInfo
#endif
!.............
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MOD, SIGN, SQRT
*     ..
*     .. Executable Statements ..
*
!... MPI ...
#ifdef COMM_MPI
      Call mpi_comm_rank(MPI_COMM_WORLD,myrank,ierr)
#endif
*     Test the input parameters.
*
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      INFO = 0
*
*     Quick return if possible
*
      IF( K.EQ.0 )
     $   RETURN
*
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
      ROW = DROW
      COL = DCOL
      DO 20 I = 1, N, NB
         DO 10 J = 0, NB - 1
            IF( I+J.LE.N ) THEN
               INDROW( I+J ) = ROW
               INDCOL( I+J ) = COL
            END IF
   10    CONTINUE
         ROW = MOD( ROW+1, NPROW )
         COL = MOD( COL+1, NPCOL )
   20 CONTINUE
*
      MYKL = CTOT( MYCOL, 1 ) + CTOT( MYCOL, 2 ) + CTOT( MYCOL, 3 )
      KLR = MYKL / NPROW
      IF( MYROW.EQ.DROW ) THEN
         MYKLR = KLR + MOD( MYKL, NPROW )
      ELSE
         MYKLR = KLR
      END IF
      PDC = 1
      COL = DCOL
   30 CONTINUE
      IF( MYCOL.NE.COL ) THEN
         PDC = PDC + CTOT( COL, 1 ) + CTOT( COL, 2 ) + CTOT( COL, 3 )
         COL = MOD( COL+1, NPCOL )
         GO TO 30
      END IF
      PDR = PDC
      KL = KLR + MOD( MYKL, NPROW )
      ROW = DROW
   40 CONTINUE
      IF( MYROW.NE.ROW ) THEN
         PDR = PDR + KL
         KL = KLR
         ROW = MOD( ROW+1, NPROW )
         GO TO 40
      END IF
*
      DO 50 I = 1, K
!         DLAMDA( I ) = DLAMC3( DLAMDA( I ), DLAMDA( I ) ) - DLAMDA( I )
         CALL DDLAMC3( DLAMDAh(I),DLAMDAl(I), 
     &                 DLAMDAh( I ), DLAMDAl( I ),tmp1h, tmp1l )
!<<<  DLAMDA(I) = tmp1 - DLAMDA(I)
      T_p1 = tmp1h - DLAMDAh(I)
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (DLAMDAh(I) + T_p2)
      T_cl = T_cl + tmp1l - DLAMDAl(I)
      T_s1 = T_ch + T_cl
      DLAMDAl(I) = T_cl - (T_s1 - T_ch)
      DLAMDAh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DLAMDA(I) = tmp1 - DLAMDA(I)
!         Z( I ) = ONE
      Zh(I) = ONEh
      Zl(I) = ONEl
   50 CONTINUE
      IF( MYKLR.GT.0 ) THEN
         KK = PDR
         DO 80 I = 1, MYKLR
!            CALL DLAED4( K, KK, DLAMDA, W, BUF, RHO, BUF( K+I ), IINFO )
            CALL DDLAED4( K, KK, DLAMDAh, DLAMDAl, Wh, Wl, BUFh, BUFl,
     &                   RHOh, RHOl, BUFh( K+I ), BUFl( K+I ), IINFO )
            IF( IINFO.NE.0 ) THEN
               INFO = KK
            END IF
*
*     ..Compute part of z
*
            DO 60 J = 1, KK - 1
!               Z( J ) = Z( J )*( BUF( J ) /
!     $                  ( DLAMDA( J )-DLAMDA( KK ) ) )
!<<<  tmp1 = DLAMDA(J) - DLAMDA(KK)
      T_p1 = DLAMDAh(J) - DLAMDAh(KK)
      T_p2 = T_p1 - DLAMDAh(J)
      tmp1h = T_p1
      tmp1l = (DLAMDAh(J) - (T_p1 - T_p2))
     +         - (DLAMDAh(KK) + T_p2)
      tmp1l = tmp1l + DLAMDAl(J) - DLAMDAl(KK)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLAMDA(J) - DLAMDA(KK)
!<<<  tmp2 = BUF(J) / tmp1
      T_q1 = BUFh(J) / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = BUFh(J) - tmp2h
      T_r1 = T_s1 - BUFh(J)
      T_s2 = (BUFh(J) 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + BUFl(J)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = BUF(J) / tmp1
!<<<  Z(J) = Z(J) * tmp2
      T_ch = Zh(J) * tmp2h
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J)*tmp2l
     +         +  Zl(J)*tmp2h)
      T_s1 = T_ch + T_cl
      Zl(J) = T_cl - (T_s1 - T_ch)
      Zh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J) = Z(J) * tmp2
   60       CONTINUE
!            Z( KK ) = Z( KK )*BUF( KK )
!<<<  Z(KK) = Z(KK) * BUF(KK)
      T_ch = Zh(KK) * BUFh(KK)
      T_u1 = 134217729.0 * Zh(KK)
      T_p1 = T_u1
     +         - (T_u1 - Zh(KK))
      T_p2 = Zh(KK) - T_p1
      T_u1 = 134217729.0 * BUFh(KK)
      T_r1 = T_u1
     +         - (T_u1 - BUFh(KK))
      T_r2 = BUFh(KK) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(KK)*BUFl(KK)
     +         +  Zl(KK)*BUFh(KK))
      T_s1 = T_ch + T_cl
      Zl(KK) = T_cl - (T_s1 - T_ch)
      Zh(KK) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(KK) = Z(KK) * BUF(KK)
            DO 70 J = KK + 1, K
!               Z( J ) = Z( J )*( BUF( J ) /
!     $                  ( DLAMDA( J )-DLAMDA( KK ) ) )
!<<<  tmp1 = DLAMDA(J) - DLAMDA(KK)
      T_p1 = DLAMDAh(J) - DLAMDAh(KK)
      T_p2 = T_p1 - DLAMDAh(J)
      tmp1h = T_p1
      tmp1l = (DLAMDAh(J) - (T_p1 - T_p2))
     +         - (DLAMDAh(KK) + T_p2)
      tmp1l = tmp1l + DLAMDAl(J) - DLAMDAl(KK)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLAMDA(J) - DLAMDA(KK)
!<<<  tmp2 = BUF(J) / tmp1
      T_q1 = BUFh(J) / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = BUFh(J) - tmp2h
      T_r1 = T_s1 - BUFh(J)
      T_s2 = (BUFh(J) 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + BUFl(J)
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = BUF(J) / tmp1
!<<<  Z(J) = Z(J) * tmp2
      T_ch = Zh(J) * tmp2h
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J)*tmp2l
     +         +  Zl(J)*tmp2h)
      T_s1 = T_ch + T_cl
      Zl(J) = T_cl - (T_s1 - T_ch)
      Zh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J) = Z(J) * tmp2
   70       CONTINUE
            KK = KK + 1
   80    CONTINUE
*
         IF( MYROW.NE.DROW ) THEN
!            CALL DCOPY( K, Z, 1, BUF, 1 )
            CALL DDCOPY( K, Zh, Zl, 1, BUFh, BUFl, 1 )
!            CALL DGESD2D( ICTXT, K+MYKLR, 1, BUF, K+MYKLR, DROW, MYCOL )
!... MPI ...
#ifdef COMM_MPI
            CALL MakeMpiInfo('A',ICTXT,DROW,MYCOL,DEST,COMM,TAG)
            CALL MPI_Isend_dd(BUFh,BUFl,SBUFF,K+MYKLR,
     $           MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
            CALL MPI_Wait_dd(SBUFF,ireq,status,ierr)
#elif defined COMM_BLACS2
            J = K+MYKLR
            DO l = 1,J
               BUF(l)   = BUFh(l)
               BUF(l+J) = BUFl(l)
            END DO
            CALL DGESD2D(ICTXT, J*2, 1, BUF, J*2, DROW, MYCOL )
#else
            CALL DGESD2D(ICTXT, K+MYKLR, 1, BUFh, K+MYKLR, DROW, MYCOL )
            CALL DGESD2D(ICTXT, K+MYKLR, 1, BUFl, K+MYKLR, DROW, MYCOL )
#endif
!...........
         ELSE
            IPD = 2*K + 1
!            CALL DCOPY( MYKLR, BUF( K+1 ), 1, BUF( IPD ), 1 )
            CALL DDCOPY( MYKLR, BUFh( K+1 ), BUFl( K+1 ), 1, 
     &                   BUFh( IPD ), BUFl( IPD ), 1 )
            IF( KLR.GT.0 ) THEN
               IPD = MYKLR + IPD
               ROW = MOD( DROW+1, NPROW )
               DO 100 I = 1, NPROW - 1
!                  CALL DGERV2D( ICTXT, K+KLR, 1, BUF, K+KLR, ROW,
!     $                          MYCOL )
!... MPI ...
#ifdef COMM_MPI
                  CALL MakeMpiInfo('A',ICTXT,ROW,MYCOL,DEST,COMM,TAG)
                  BUFF(1:K+KLR)           = BUFh(1:K+KLR)
                  BUFF(1+K+KLR:2*(K+KLR)) = BUFl(1:K+KLR)
                  CALL MPI_Irecv_dd(BUFF,K+KLR,
     $                 MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
                  CALL MPI_WaitIrecv_dd(BUFh,BUFl,BUFF,K+KLR,
     $                 ireq,status,ierr)
#elif defined COMM_BLACS2
                  J = K+KLR
                  CALL DGERV2D( ICTXT,J*2,1,BUF,J*2,ROW, MYCOL )
                  DO l = 1,J
                     BUFh(l) = BUF(l)
                     BUFl(l) = BUF(l+J)
                  END DO
#else
                  CALL DGERV2D( ICTXT, K+KLR, 1, BUFh, K+KLR, ROW,
     $                          MYCOL )
                  CALL DGERV2D( ICTXT, K+KLR, 1, BUFl, K+KLR, ROW,
     $                          MYCOL )
#endif
!...........
!                  CALL DCOPY( KLR, BUF( K+1 ), 1, BUF( IPD ), 1 )
                  CALL DDCOPY( KLR, BUFh( K+1 ), BUFl( K+1 ), 
     &                         1, BUFh( IPD ), BUFl( IPD ), 1 )
                  DO 90 J = 1, K
!                     Z( J ) = Z( J )*BUF( J )
!<<<  Z(J) = Z(J) * BUF(J)
      T_ch = Zh(J) * BUFh(J)
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * BUFh(J)
      T_r1 = T_u1
     +         - (T_u1 - BUFh(J))
      T_r2 = BUFh(J) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J)*BUFl(J)
     +         +  Zl(J)*BUFh(J))
      T_s1 = T_ch + T_cl
      Zl(J) = T_cl - (T_s1 - T_ch)
      Zh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J) = Z(J) * BUF(J)
   90             CONTINUE
                  IPD = IPD + KLR
                  ROW = MOD( ROW+1, NPROW )
  100          CONTINUE
            END IF
         END IF
      END IF
*
      IF( MYROW.EQ.DROW ) THEN
         IF( MYCOL.NE.DCOL .AND. MYKL.NE.0 ) THEN
!            CALL DCOPY( K, Z, 1, BUF, 1 )
!            CALL DCOPY( MYKL, BUF( 2*K+1 ), 1, BUF( K+1 ), 1 )
            CALL DDCOPY( K, Zh, Zl, 1, BUFh, BUFl, 1 )
            CALL DDCOPY( MYKL, BUFh( 2*K+1 ), BUFl( 2*K+1 ), 1, 
     &                   BUFh( K+1 ), BUFl( K+1 ), 1 )
!            CALL DGESD2D( ICTXT, K+MYKL, 1, BUF, K+MYKL, MYROW, DCOL )
!... MPI ...
#ifdef COMM_MPI
            CALL MakeMpiInfo('A',ICTXT,MYROW,DCOL,DEST,COMM,TAG)
            CALL MPI_Isend_dd(BUFh,BUFl,SBUFF,K+MYKL,
     $           MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
            CALL MPI_Wait_dd(SBUFF,ireq,status,ierr)
#elif defined COMM_BLACS2
            J = K+MYKL
            DO l = 1,J
               BUF(l)   = BUFh(l)
               BUF(l+J) = BUFl(l)
            END DO
            CALL DGESD2D( ICTXT, J*2, 1, BUF, J*2, MYROW, DCOL )
#else
            CALL DGESD2D( ICTXT, K+MYKL, 1, BUFh, K+MYKL, MYROW, DCOL )
            CALL DGESD2D( ICTXT, K+MYKL, 1, BUFl, K+MYKL, MYROW, DCOL )
#endif
!...........
         ELSE IF( MYCOL.EQ.DCOL ) THEN
            IPD = 2*K + 1
            COL = DCOL
            KL = MYKL
            DO 120 I = 1, NPCOL - 1
               IPD = IPD + KL
               COL = MOD( COL+1, NPCOL )
               KL = CTOT( COL, 1 ) + CTOT( COL, 2 ) + CTOT( COL, 3 )
               IF( KL.NE.0 ) THEN
!                  CALL DGERV2D( ICTXT, K+KL, 1, BUF, K+KL, MYROW, COL )
!... MPI ...
#ifdef COMM_MPI
                  CALL MakeMpiInfo('A',ICTXT,MYROW,COL,DEST,COMM,TAG)
                  BUFF(1:K+KL)          = BUFh(1:K+KL)
                  BUFF(1+K+KL:2*(K+KL)) = BUFl(1:K+KL)
                  CALL MPI_Irecv_dd(BUFF,K+KL,
     $                 MPI_DOUBLE_PRECISION,DEST,1,COMM,ireq,ierr)
                  CALL MPI_WaitIrecv_dd(BUFh,BUFl,BUFF,K+KL,
     $                 ireq,status,ierr)
#elif defined COMM_BLACS2
                  J = K+KL
                  CALL DGERV2D( ICTXT, J*2, 1, BUF, J*2, MYROW, COL )
                  DO l = 1,J
                     BUFh(l) = BUF(l)
                     BUFl(l) = BUF(l+J)
                  END DO
#else
                  CALL DGERV2D( ICTXT, K+KL, 1, BUFh, K+KL, MYROW, COL )
                  CALL DGERV2D( ICTXT, K+KL, 1, BUFl, K+KL, MYROW, COL )
#endif
!...........
!                  CALL DCOPY( KL, BUF( K+1 ), 1, BUF( IPD ), 1 )
                  CALL DDCOPY( KL, BUFh( K+1 ), BUFl( K+1 ), 1, 
     &                         BUFh( IPD ), BUFl( IPD ), 1 )
                  DO 110 J = 1, K
!                     Z( J ) = Z( J )*BUF( J )
!<<<  Z(J) = Z(J) * BUF(J)
      T_ch = Zh(J) * BUFh(J)
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * BUFh(J)
      T_r1 = T_u1
     +         - (T_u1 - BUFh(J))
      T_r2 = BUFh(J) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(J)*BUFl(J)
     +         +  Zl(J)*BUFh(J))
      T_s1 = T_ch + T_cl
      Zl(J) = T_cl - (T_s1 - T_ch)
      Zh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  Z(J) = Z(J) * BUF(J)
  110             CONTINUE
               END IF
  120       CONTINUE
            DO 130 I = 1, K
!               Z( I ) = SIGN( SQRT( -Z( I ) ), W( I ) )
!<<<  tmp1 = -Z(I)
      tmp1h = -Zh(I)
      tmp1l = -Zl(I)
!>>>  tmp1 = -Z(I)
!<<<  tmp2 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      endif
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = sqrt(tmp1)
!<<<  tmp3 = abs(tmp2)
      if (tmp2h .lt. 0.0d+0) then
        tmp3h = -tmp2h
        tmp3l = -tmp2l
      elseif (
     +  (tmp2h .eq. 0.0d+0).and.
     +  (tmp2l .lt. 0.0d+0)) then
        tmp3h = tmp2h
        tmp3l = -tmp2l
      else
        tmp3h = tmp2h
        tmp3l = tmp2l
      endif
!>>>  tmp3 = abs(tmp2)
                IF(
!       (W(I) .ge. 0.0)
     + ((Wh(I) .gt. 0.0d+0) .or.
     +  ((Wh(I) .eq. 0.0d+0) .and.
     +   (Wl(I) .ge. 0.0d+0)))
     &             )THEN
      Zh(I) = tmp3h
      Zl(I) = tmp3l
                ELSE
!<<<  Z(I) = -tmp3
      Zh(I) = -tmp3h
      Zl(I) = -tmp3l
!>>>  Z(I) = -tmp3
                END IF
  130       CONTINUE
*
         END IF
      END IF
*
*     Diffusion
*
      IF( MYROW.EQ.DROW .AND. MYCOL.EQ.DCOL ) THEN
!         CALL DCOPY( K, Z, 1, BUF, 1 )
!         CALL DCOPY( K, BUF( 2*K+1 ), 1, BUF( K+1 ), 1 )
         CALL DDCOPY( K, Zh, Zl, 1, BUFh, BUFl, 1 )
         CALL DDCOPY( K, BUFh( 2*K+1 ), BUFl( 2*K+1 ), 1, 
     &                BUFh( K+1 ), BUFl( K+1 ), 1 )
!         CALL DGEBS2D( ICTXT, 'All', ' ', 2*K, 1, BUF, 2*K )
!... MPI ...
#ifdef COMM_MPI
         CALL MakeMpiInfo('S',ICTXT,0,0,DEST,COMM,TAG )
         CALL MPI_Bcast_dd(BUFh,BUFl,2*K,
     $        MPI_DOUBLE_PRECISION,DEST,COMM,ierr)
#elif defined COMM_BLACS2
         DO l = 1,2*K
            BUF(l)     = BUFh(l)
            BUF(l+2*K) = BUFl(l)
         END DO
         CALL DGEBS2D( ICTXT, 'All', ' ', 2*2*K, 1, BUF, 2*2*K )
#else
         CALL DGEBS2D( ICTXT, 'All', ' ', 2*K, 1, BUFh, 2*K )
         CALL DGEBS2D( ICTXT, 'All', ' ', 2*K, 1, BUFl, 2*K )
#endif
!...........
      ELSE
!         CALL DGEBR2D( ICTXT, 'All', ' ', 2*K, 1, BUF, 2*K, DROW, DCOL )
!... MPI ...
#ifdef COMM_MPI
         CALL MakeMpiInfo('B',ICTXT,DROW,DCOL,DEST,COMM,TAG )
         CALL MPI_Bcast_dd(BUFh,BUFl,2*K,
     $        MPI_DOUBLE_PRECISION,DEST,COMM,ierr)
#elif defined COMM_BLACS2
         CALL DGEBR2D(ICTXT,'All', ' ',2*2*K,1,BUF,2*2*K, DROW, DCOL )
         DO l = 1,2*K
            BUFh(l) = BUF(l)
            BUFl(l) = BUF(l+2*K)
         END DO
#else
         CALL DGEBR2D( ICTXT, 'All', ' ', 2*K, 1, BUFh,2*K, DROW, DCOL )
         CALL DGEBR2D( ICTXT, 'All', ' ', 2*K, 1, BUFl,2*K, DROW, DCOL )
#endif
!...........
!         CALL DCOPY( K, BUF, 1, Z, 1 )
         CALL DDCOPY( K, BUFh, BUFl, 1, Zh, Zl, 1 )
      END IF
*
*     Copy of D at the good place
*
      KLC = 0
      KLR = 0
      DO 140 I = 1, K
         GI = INDX( I )
!         D( GI ) = BUF( K+I )
      Dh(GI) = BUFh(K+I)
      Dl(GI) = BUFl(K+I)
         COL = INDCOL( GI )
         ROW = INDROW( GI )
         IF( COL.EQ.MYCOL ) THEN
            KLC = KLC + 1
            INDXC( KLC ) = I
         END IF
         IF( ROW.EQ.MYROW ) THEN
            KLR = KLR + 1
            INDXR( KLR ) = I
         END IF
  140 CONTINUE
*
*     Compute eigenvectors of the modified rank-1 modification.
*
      IF( MYKL.NE.0 ) THEN
         DO 180 J = 1, MYKL
            KK = INDXC( J )
            JU = INDX( KK )
            JJU = INDXG2L( JU, NB, J, J, NPCOL )
!            CALL DLAED4( K, KK, DLAMDA, W, BUF, RHO, AUX, IINFO )
            CALL DDLAED4( K, KK, DLAMDAh, DLAMDAl, Wh, Wl, BUFh, BUFl,
     &                    RHOh, RHOl, AUXh, AUXl, IINFO )
            IF( IINFO.NE.0 ) THEN
               INFO = KK
            END IF
            IF( K.EQ.1 .OR. K.EQ.2 ) THEN
               DO 150 I = 1, KLR
                  KK = INDXR( I )
                  IU = INDX( KK )
                  IIU = INDXG2L( IU, NB, J, J, NPROW )
!                  U( IIU, JJU ) = BUF( KK )
      Uh(IIU,JJU) = BUFh(KK)
      Ul(IIU,JJU) = BUFl(KK)
  150          CONTINUE
               GO TO 180
            END IF
*
            DO 160 I = 1, K
!               BUF( I ) = Z( I ) / BUF( I )
!<<<  BUF(I) = Z(I) / BUF(I)
      T_q1 = Zh(I) / BUFh(I)
      T_ch = BUFh(I) * T_q1
      T_u1 = 134217729.0 * BUFh(I)
      T_p1 = T_u1
     +         - (T_u1 - BUFh(I))
      T_p2 = BUFh(I) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (BUFl(I) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(I) - T_ch
      T_r1 = T_s1 - Zh(I)
      T_s2 = (Zh(I)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(I)
      T_q2 = (T_s1 + T_s2)
     +         / BUFh(I)
      BUFh(I) = T_q1 + T_q2
      BUFl(I) = T_q2
     +      - (BUFh(I) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  BUF(I) = Z(I) / BUF(I)
  160       CONTINUE
!            TEMP = DNRM2( K, BUF, 1 )
             CALL DDNRM2(K, BUFh, BUFl, 1, TEMPh, TEMPl)
!            TEMPh = DNRM2( K, BUFh, 1 )
!            TEMPl = 0.0d0
            DO 170 I = 1, KLR
               KK = INDXR( I )
               IU = INDX( KK )
               IIU = INDXG2L( IU, NB, J, J, NPROW )
!               U( IIU, JJU ) = BUF( KK ) / TEMP
!<<<  U(IIU,JJU) = BUF(KK) / TEMP
      T_q1 = BUFh(KK) / TEMPh
      Uh(IIU,JJU) = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      Ul(IIU,JJU) = ((T_p1*T_r1
     +       - Uh(IIU,JJU))
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      Ul(IIU,JJU) = Ul(IIU,JJU) + (TEMPl*T_q1)
      T_s1 = Uh(IIU,JJU) + Ul(IIU,JJU)
      Ul(IIU,JJU) = Ul(IIU,JJU) - (T_s1 - Uh(IIU,JJU))
      Uh(IIU,JJU) = T_s1
      T_s1 = BUFh(KK) - Uh(IIU,JJU)
      T_r1 = T_s1 - BUFh(KK)
      T_s2 = (BUFh(KK) 
     +         - (T_s1 - T_r1))
     +         - (Uh(IIU,JJU) + T_r1)
      T_s2 = T_s2 - Ul(IIU,JJU)
      T_s2 = T_s2 + BUFl(KK)
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Uh(IIU,JJU) = T_q1 + T_q2
      Ul(IIU,JJU) = T_q2
     +      - (Uh(IIU,JJU) - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  U(IIU,JJU) = BUF(KK) / TEMP
  170       CONTINUE
*
  180    CONTINUE
      END IF
*
  190 CONTINUE
*
      RETURN
*
*     End of PDLAED3
*
      END
