      use communication_h, only : eigen_init
     &                , eigen_free
     &                , cyc1d_cyc2d
     &                , cyc2d_cyc1d

      implicit double precision (a-h,o-z)

      real(8), allocatable :: a(:,:),ai(:,:)
      real(8), pointer :: b(:),bi(:),c(:),ci(:),w(:)
#ifdef ARRAY_2D
      real(8), pointer  :: ar2d(:), ai2d(:)
      real(8), pointer  :: zr2d(:), zi2d(:)
#endif
!
      include 'mpif.h'
      include 'trd.h'
!
      call mpi_init(ierr)
      call mpi_comm_rank(mpi_comm_world,i$inod,ierr)
      call mpi_comm_size(mpi_comm_world,i$nnod,ierr)
*
      n=3000
#ifdef ARRAY_2D
      ldar2d=sub_ar2d(n)
      ldzr2d=sub_zr2d(n)
#endif
      m=32
      nm=(n/2)*2+1
      allocate( a(n,n), ai(n,n) )
      allocate(
     &         b((nm)*((n-1)/i$nnod+1+nm+1)),
     &         bi((nm)*((n-1)/i$nnod+1+nm+1)),
     &         c((nm)*((n-1)/i$nnod+1+nm+1)),
     &         ci((nm)*((n-1)/i$nnod+1+nm+1)),
#ifdef ARRAY_2D
     &         ar2d(ldar2d),
     &         ai2d(ldar2d),
     &         zr2d(ldzr2d),
     &         zi2d(ldzr2d),
#endif
     &         w(nm),
     &         stat=istat )
      if(istat.ne.0) then
         print*,"Memory exhausted"
         call flush(6)
         stop
      endif
*
      call matrix_set(a,n)
      call matrix_seti(ai,n)
      call matrix_adjust_h(a,n,b,nm)
      call matrix_adjust_hi(ai,n,bi,nm)
#ifdef ARRAY_2D
      lda = sub_lda1(n)

      call cyc1d_cyc2d(n,b ,nm,ar2d,lda,size_of_col,size_of_row,ierr)
      call cyc1d_cyc2d(n,bi,nm,ai2d,lda,size_of_col,size_of_row,ierr)

      call eigen_h(n,ar2d,ai2d,lda,zr2d,zi2d,ldz,w,m,0)

      call cyc2d_cyc1d(n,b ,nm,zr2d,ldz,size_of_col,size_of_row,ierr)
      call cyc2d_cyc1d(n,bi,nm,zi2d,ldz,size_of_col,size_of_row,ierr)
#else
      call eigen_h(n,b,bi,w,nm,m,0)
#endif
*
#ifdef CHECK
      call matrix_set(a,n)
      call matrix_seti(ai,n)
      call matrix_adjust_h(a,n,c,nm)
      call matrix_adjust_hi(ai,n,ci,nm)
      call ev_test2(c,ci,w,b,bi,n,nm)
#endif
*
      deallocate(a,ai,b,bi,c,ci,w)
#ifdef ARRAY_2D
      deallocate(ar2d,ai2d,zr2d,zi2d)
#endif

      call MPI_Finalize(ierr)
#ifdef ARRAY_2D
      contains

! ----------------------------------------------------------------------
!     sub_lda1
! ----------------------------------------------------------------------
      function sub_lda1(n)
      include 'trd.h'
      integer,intent(in) :: n
      integer :: m,mm,lda2,na
      integer :: sub_lda1
      call eigen_init(2)
      m = 128; mm = m*(m-1)/2/1
      lda2 = (n-1)/size_of_col+1
      call eigen_free(2)
      sub_lda1 = lda2
      end function

! ----------------------------------------------------------------------
!     sub_ar2d
! ----------------------------------------------------------------------
      function sub_ar2d(n)
      integer,intent(in) :: n
      integer :: sub_ar2d
      include 'mpif.h'
      include 'trd.h'
!
      call eigen_init(2)
      NPROW = size_of_col
      NPCOL = size_of_row
      call eigen_free(0)
!
      NB  = 48
      nmz = ((n-1)/NPROW+1)
      nmz = ((nmz-1)/NB+1)*NB+1
      nmw = ((n-1)/NPCOL+1)
      nmw = ((nmw-1)/NB+1)*NB+1

      nm=(n/2)*2+1

      nmz1=nm
      nmw1=((n-1)/nprocs+1)

      sub_ar2d = MAX(nmz*nmw+nm+1,nmz1*nmw1)
      end function

! ----------------------------------------------------------------------
!     sub_zr2d
! ----------------------------------------------------------------------
      function sub_zr2d(n)
!
      use communication_h, only : eigen_init
     &               , eigen_free
     &               , bcast_dbl
     &               , cyc2d_cyc1d
!
      integer :: sub_zr2d
      integer, intent(in)           :: n

! parameters blacs array descritor(the position of entry tags), etc
      integer, parameter            :: block_cyclic_2d = 1
      integer, parameter            :: dlen_  = 9
      integer, parameter            :: dtype_ = 1
      integer, parameter            :: ctxt_  = 2
      integer, parameter            :: m_     = 3
      integer, parameter            :: n_     = 4
      integer, parameter            :: mb_    = 5
      integer, parameter            :: nb_    = 6
      integer, parameter            :: rsrc_  = 7
      integer, parameter            :: csrc_  = 8
      integer, parameter            :: lld_   = 9

      integer                       :: descz( dlen_ )
      integer                       :: descw( dlen_ )
 
      integer                       :: world_size, my_rank
      integer                       :: trilwmin, lwork, liwork
 
      real(8), pointer              :: work(:)
      integer, pointer              :: iwork(:)

      real(8)                       :: s0, s1, d1, d2, d3
      integer :: sub_lda3

      include 'mpif.h'
      include 'trd.h'

! blacs/pblas/scalapack initialization
      call blacs_pinfo( iam, nprocs )
      if ( nprocs < 1 ) then
!  mpi group setup
         call mpi_comm_size( mpi_comm_world, world_size, ierr )
         call mpi_comm_rank( mpi_comm_world, my_rank, ierr )
         nprocs = world_size
         iam = my_rank
         call blacs_setup( iam, nprocs )
      end if
      call blacs_get( -1, 0, ictxt )
      call eigen_init(2)
      nprow = size_of_col
      npcol = size_of_row
      call blacs_gridinit( ictxt, 'column-major', nprow, npcol )
      call blacs_gridinfo( ictxt, nprow, npcol, myrow, mycol )

! blacs array registration
      nb = 48
      np = numroc( n, nb, myrow, 0, nprow )
      nq = numroc( n, nb, mycol, 0, npcol )
      lddz = (n-1)/nprow+1
      lddz = ((lddz-1)/nb+1)*nb+1
      lddw = (n-1)/npcol+1
      lddw = ((lddw-1)/nb+1)*nb+1
      call descinit( descz, n, n, nb, nb, 0, 0, ictxt, lddz, info )
 
! preparing working arrays
      trilwmin = 3*n + max( nb*( np+1 ), 3*nb )
      lwork  = max( max( 1+6*n+2*np*nq, trilwmin ) + 2*n, lddz*lddw)
      liwork = 2+7*n+8*npcol
      call eigen_free(2)
 
! blacs/pblas/scalapack finalize
      call blacs_gridexit( ictxt )

      sub_zr2d = lwork
      end function
#endif
      end

