
*  =====================================================================
*      DOUBLE PRECISION FUNCTION DZNRM2(n,xh,xl,xih,xil,incx)
*
      SUBROUTINE dd_dznrm2(n,xh,xl,xih,xil,incx,rh,rl)

*  -- Reference BLAS level1 routine (version 3.4.0) --
*  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER incx,n
*     ..
*     .. Array Arguments ..
*      COMPLEX*16 X(*)
      REAL(8) :: xh(n),xl(n),xih(n),xil(n)
      REAL(8) :: rh,rl
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
      REAL(8)  ONE,ZERO
      PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
*     ..
*     .. Local Scalars ..
      REAL(8)  NORM_h,NORM_l,SCALE_h,SCALE_l,SSQ_h,SSQ_l
C      REAL(8)  TEMP_h,TEMP_l,TEMP1_h,TEMP1_l,TEMP2_h,TEMP2_l = 0.0d+0
       double precision TEMP_h,TEMP_l,TEMP1_h,TEMP1_l,TEMP2_h,TEMP2_l
      INTEGER IX
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC ABS,SQRT
*     ..

      IF (n.LT.1 .OR. incx.LT.1) THEN
          NORM_h = ZERO
          NORM_l = ZERO

      ELSE IF (n .eq. 1 ) then
          call dd_abs(xh(1),xl(1),TEMP_h,TEMP_l)
          NORM_h = TEMP_h
          NORM_l = TEMP_l

      ELSE          
          SCALE_h = ZERO
          SCALE_l = ZERO
          SSQ_h = ONE
          SSQ_l = ZERO

*        The following loop is equivalent to this call to the LAPACK
*        auxiliary routine:
*        CALL ZLASSQ( N, X, INCX, SCALE, SSQ )
*
          DO 10 IX = 1,1 + (n-1)*incx,incx
              IF ((xh(IX).NE.ZERO).OR.(xl(IX).NE.ZERO)) THEN

                  call dd_abs(xh(IX),xl(IX),TEMP_h,TEMP_l)

                  IF((SCALE_h.LT.TEMP_h).OR.
     +              ((SCALE_h.EQ.TEMP_h).AND.(SCALE_l.LT.TEMP_l)))THEN
!                      SSQ = ONE + SSQ* (SCALE/TEMP)**2

                    call dd_divide(SCALE_h,SCALE_l,
     +                             TEMP_h,TEMP_l,
     +                             TEMP1_h,TEMP1_l)

                    call dd_multiple(TEMP1_h,TEMP1_l,
     +                               TEMP1_h,TEMP1_l,
     +                               TEMP2_h,TEMP2_l)


                    call dd_multiple(TEMP2_h,TEMP2_l,
     +                               SSQ_h,SSQ_l,
     +                               TEMP1_h,TEMP1_l)

                    call dd_add(TEMP1_h,TEMP1_l,
     +                          ONE,ZERO,
     +                          SSQ_h,SSQ_l)
                    SCALE_h = TEMP_h
                    SCALE_l = TEMP_l

                  ELSE
!                      SSQ = SSQ + (TEMP/SCALE)**2
                    call dd_divide(TEMP_h,TEMP_l,
     +                             SCALE_h,SCALE_l,
     +                             TEMP1_h,TEMP1_l)
                    call dd_multiple(TEMP1_h,TEMP1_l,
     +                               TEMP1_h,TEMP1_l,
     +                               TEMP2_h,TEMP2_l)
                    call dd_add(TEMP2_h,TEMP2_l,
     +                          SSQ_h,SSQ_l,
     +                          SSQ_h,SSQ_l)
                  END IF
              END IF
              IF ((xih(IX).NE.ZERO).OR.(xil(IX).NE.ZERO)) THEN
!                  TEMP = ABS(DIMAG(X(IX)))
                  call dd_abs(xih(IX),xil(IX),TEMP_h,TEMP_l)

                  IF ((SCALE_h.LT.TEMP_h).OR.((SCALE_h.EQ.TEMP_h) 
     &                              .AND.(SCALE_l.LT.TEMP_l))) THEN
!                      SSQ = ONE + SSQ* (SCALE/TEMP)**2
                    call dd_divide(SCALE_h,SCALE_l,
     +                             TEMP_h,TEMP_l,
     +                             TEMP1_h,TEMP1_l)
                    call dd_multiple(TEMP1_h,TEMP1_l,
     +                               TEMP1_h,TEMP1_l,
     +                               TEMP2_h,TEMP2_l)
                    call dd_multiple(TEMP2_h,TEMP2_l,
     +                               SSQ_h,SSQ_l,
     +                               TEMP1_h,TEMP1_l)
                    call dd_add(TEMP1_h,TEMP1_l,
     +                          ONE,ZERO,
     +                          SSQ_h,SSQ_l)
                    SCALE_h = TEMP_h
                    SCALE_l = TEMP_l

                  ELSE
!                      SSQ = SSQ + (TEMP/SCALE)**2
                    call dd_divide(TEMP_h,TEMP_l,
     +                             SCALE_h,SCALE_l,
     +                             TEMP1_h,TEMP1_l)
                    call dd_multiple(TEMP1_h,TEMP1_l,
     +                               TEMP1_h,TEMP1_l,
     +                               TEMP2_h,TEMP2_l)
                    call dd_add(TEMP2_h,TEMP2_l,
     +                          SSQ_h,SSQ_l,
     +                          SSQ_h,SSQ_l)

                  END IF
              END IF

   10     CONTINUE
!          NORM = SCALE*SQRT(SSQ)
!      call show_vector(n, v2_h, v2_l)

           call dd_sqrt(SSQ_h,SSQ_l,SSQ_h,SSQ_l)
           call dd_multiple(SSQ_h,SSQ_l,
     +                      SCALE_h,SCALE_l,
     +                      NORM_h,NORM_l)
      END IF
*
!      DZNRM2 = NORM
       rh = NORM_h
       rl = NORM_l

      RETURN
*
*     End of DZNRM2.
*
      END SUBROUTINE
