C
C      Subroutine for hermite matrix with Double-Double percision
C
       subroutine ddmatrix_hermite(n, ah, al, aih, ail)
       implicit double precision(a-h,o-z),integer(i-n)
       real(8)   :: ah(n,n), aih(n,n)
       real(8)   :: al(n,n), ail(n,n)
       real(16)  :: qtmp, qtmph, qtmpl

       real(8)   ::  v1_h(n),v1_l(n),v1i_h(n),v1i_l(n)
       real(8)   ::  v2_h(n),v2_l(n),v2i_h(n),v2i_l(n)
       real(8)   ::  s3_h,s3_l,s3i_h,s3i_l,s4_h,s4_l
       real(8)   ::  alphah , alphal
       real(8)   :: bh(n,n), bih(n,n)
       real(8)   :: bl(n,n), bil(n,n)

C      Ramdam matrix      
       do i=1,n
          do j=1,n
             if(i == j) then
               CALL RANDOM_NUMBER(qtmp)
               tmph = qtmp
               qtmph = tmph
               qtmpl = qtmp - qtmph
               tmpl = qtmpl
               ah(i,j)=tmph
               al(i,j)=tmpl
               aih(i,j)=0.0
               ail(i,j)=0.0

             else if(i>j) then
               CALL RANDOM_NUMBER(qtmp)
               tmph = qtmp
               qtmph = tmph
               qtmpl = qtmp - qtmph
               tmpl = qtmpl
               ah(i,j)=tmph
               al(i,j)=tmpl
               ah(j,i)=tmph
               al(j,i)=tmpl
               aih(i,j)=0.0
               ail(i,j)=0.0
               aih(j,i)=0.0
               ail(j,i)=0.0
             end if
          enddo
       enddo

C      Ortonormalization
       do i=1,n
          do j=1,n
            v1_h(j)=ah(i,j)
            v1_l(j)=al(i,j)
            v1i_h(j)=aih(i,j)
            v1i_l(j)=ail(i,j)
          end do
          if(i>1) then
            do k=1,i-1

               do j=1,n
                  v2_h(j)=ah(i-k,j)
                  v2_l(j)=al(i-k,j)
                  v2i_h(j)=aih(i-k,j)
                  v2i_l(j)=ail(i-k,j)
               end do

		alphah=-1.0
		alphal= 0.0

               call dd_idot(n,v1_h,v1_l,v1i_h,v1i_l,v2_h,v2_l,
     &                        v2i_h,v2i_l,s3_h,s3_l,s3i_h,s3i_l)

               call dd_scal(s3_h,s3_l,alphah,alphal)
               call dd_scal(s3i_h,s3i_l,alphah,alphal)
               call ddaxpy(n,s3_h,s3_l,v2_h,v2_l,1,v1_h,v1_l,1)
               call ddaxpy(n,s3i_h,s3i_l,v2i_h,v2i_l,1,v1i_h,v1i_l,1)

             end do
           endif

           call dd_dznrm2(n,v1_h,v1_l,v1i_h,v1i_l,1,s3_h,s3_l)

           call dd_reciprocal(s3_h,s3_l)
           call ddscal(n,s3_h,s3_l,v1_h,v1_l,1)
           call ddscal(n,s3_h,s3_l,v1i_h,v1i_l,1)

           do j=1,n
              ah(i,j)=v1_h(j)
              al(i,j)=v1_l(j)
              aih(i,j)=v1i_h(j)
              ail(i,j)=v1i_l(j)
           end do
         end do

       return
       end


