C
C     Soubroutine for check the result compared 
C                                   with eigen values and vectors
C
      subroutine dd_check_result(rwh,rwl,arh,arl,aih,ail,
     &                             wh,wl,zrh,zrl,zih,zil,n,nm)

      use ddcommunication_h, only : eigen_init
     &                            , eigen_free

       integer  :: n, nm
       real(8)  :: rwh(*),arh(*), aih(*), wh(*), zrh(*), zih(*)
       real(8)  :: rwl(*),arl(*), ail(*), wl(*), zrl(*), zil(*)

       include 'mpif.h'
       include 'trd.h'

C      Initalize eigen
       call eigen_init(1)
       call MPI_Barrier(MPI_COMM_WORLD,ierr)

C      Checking eigen values and vectors
       call dd_compare_v_m(rwh(1),rwl(1),arh(1),arl(1),aih(1),ail(1),
     &                      wh(1),wl(1),zrh(1),zrl(1),zih(1),zil(1),
     &                      n,nm)

C      Finarilze eigen
       call MPI_Barrier(MPI_COMM_WORLD,ierr)
       call eigen_free(0)

       return
       end

C
C     Soubroutine for comparetion with eigen values and vectors
C
       subroutine dd_compare_v_m(rwh,rwl,arh,arl,aih,ail,
     &                            wh,wl,zrh,zrl,zih,zil,n,nm)

       use ddcommunication_h, only : loop_c

       integer                :: n, nm
       real(8)                :: rwh(1:n), rwl(1:n)
       real(8)                :: arh(1:nm,*), aih(1:nm,*)
       real(8)                :: arl(1:nm,*), ail(1:nm,*)
       real(8)                :: wh(1:n), wl(1:n)
       real(8)                :: zrh(1:nm,*), zih(1:nm,*)
       real(8)                :: zrl(1:nm,*), zil(1:nm,*)

      double precision temp_h(n) , temp_l(n)
      double precision error_h(n), error_l(n)
      double precision norm_h(n),  norm_l(n)
      double precision temp1_h,    temp1_l
      double precision temp2_h(n), temp2_l(n)
      double precision max_error_ramda_h, max_error_ramda_l
      double precision ave_error_ramda_h, ave_error_ramda_l
      double precision max_norm_h, max_norm_l
      double precision dbl_n_h,dbl_n_l

      integer sgn

       include 'mpif.h'
       include 'trd.h'

       max_error_ramda_h=0.0d+0
       max_error_ramda_l=0.0d+0

C      Eigen values
       do j=1,n
         call dd_sub(wh(j), wl(j),
     &               rwh(j), rwl(j),
     &               error_h(j),error_l(j))
         call dd_abs(error_h(j),error_l(j),temp1_h,temp1_l)
         error_h(j)=temp1_h
         error_l(j)=temp1_l

         if( (max_error_ramda_h .lt. error_h(j)) .or.
     &      ((max_error_ramda_h .eq. 0.0d+0) .and.
     &       (max_error_ramda_l .lt. error_l(j)))
     &     ) then
           max_error_ramda_h = error_h(j)
           max_error_ramda_l = error_l(j)
         endif

         call dd_add(ave_error_ramda_h,ave_error_ramda_l,
     &               error_h(j),error_l(j),
     &               temp1_h,temp1_l)
         ave_error_ramda_h = temp1_h
         ave_error_ramda_l = temp1_l

       enddo

C      Eigen vectors
       dbl_n_h = n
       dbl_n_l = 0.0d+0
       call dd_divide(ave_error_ramda_h,ave_error_ramda_l,
     &                dbl_n_h,dbl_n_l,
     &                temp1_h,temp1_l)
         ave_error_ramda_h = temp1_h
         ave_error_ramda_l = temp1_l

       max_norm_h = 0.0d+0
       max_norm_l = 0.0d+0

       i_start=1
       i_end  =n
    
       do i=i_start,i_end
         sgn=arh(1,i)/abs(arh(1,i)) * zrh(1,i) /abs(zrh(1,i))
         do j=1,n
            zrh(j,i) = sgn * zrh(j,i)
            zrl(j,i) = sgn * zrl(j,i)
            call dd_sub(arh(j,i), arl(j,i),
     &                  zrh(j,i), zrl(j,i),
     &                  error_h(j),error_l(j))
         enddo

         call ddnrm2(n,error_h,error_l,1,norm_h(i),norm_l(i))

       enddo

C      Transfar results
       temp_h(1:n) = norm_h(1:n)
       temp_l(1:n) = norm_l(1:n)
       call MPI_Allreduce_dd(norm_h,norm_l,temp_h,temp_l,n,
     &      MPI_DOUBLE_PRECISION,MPI_MAX,MPI_COMM_WORLD,ierr)
       call MPI_Allreduce_dd(norm_h,norm_l,temp2_h,temp2_l,n,
     &      MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)
       norm_h(1:n) = temp_h(1:n)
       norm_l(1:n) = temp_l(1:n)

       do j=1,n
         if( (max_norm_h .lt. norm_h(j)) .or.
     &      ((max_norm_h .eq. 0.0d+0) .and.
     &       (max_norm_l .lt. norm_l(j)))
     &     ) then
           max_norm_h = norm_h(j)
           max_norm_l = norm_l(j)
         endif
         call dd_add(ave_norm_h,ave_norm_l,
     &               temp2_h(j),temp2_l(j),
     &               temp1_h   ,temp1_l)
         ave_norm_h = temp1_h
         ave_norm_l = temp1_l
       enddo

       dbl_n_h = n
       dbl_n_l = 0.0d+0

       call dd_divide(ave_norm_h,ave_norm_l,
     &                dbl_n_h,dbl_n_l,
     &                temp1_h,temp1_l)
         ave_norm_h = temp1_h
         ave_norm_l = temp1_l

C      Show results
       if(myrank==1)then
         print*,"max |r(QPEigen)-r(define)|    = ",
     &           max_error_ramda_h,max_error_ramda_l
         print*,"ave |r(QPEigen)-r(define)|    = ",
     &           ave_error_ramda_h,ave_error_ramda_l
         print*,"max||xi(QPEigen)-xi(define)|| = ",max_norm_h,max_norm_l
       endif

       return
       end
