C
C     Verifing program for calculating the eigenvalues 
C                         of hermite matrix
C
      use ddcommunication_h, only  : eigen_init
     &                             , eigen_free
     &                             , translate_l2g

      implicit double precision (a-h,o-z)

      real(8),allocatable :: ah(:,:), al(:,:)
      real(8),allocatable :: xh(:,:), xl(:,:)
      real(8),allocatable :: rh(:,:), rl(:,:)
      real(8),allocatable :: sh(:,:), sl(:,:)
      real(8),allocatable :: aih(:,:), ail(:,:)
      real(8),allocatable :: xih(:,:), xil(:,:)
      real(8),allocatable :: rih(:,:), ril(:,:)
      real(8),allocatable :: ash(:,:), asl(:,:)

      real(8), pointer ::
     &           bh(:),bih(:),zh(:),dh(:),eh(:),wh(:),
     &           bl(:),bil(:),zl(:),dl(:),el(:),wl(:)
      real(8), pointer :: ch(:),cih(:),cl(:),cil(:)
      real(8), pointer :: rwh(:),rwl(:)

      real(8) 	alphah ,alphal , betah, betal
      real(8) 	temp1_h ,temp1_l , temp2_h ,temp2_l
      real(8)   hs0, hs1

      integer, parameter :: NDIM = 2
      logical iexist

      include 'mpif.h'
      include 'trd.h'

C     Initialize MPI
      call mpi_init(ierr)
      call mpi_comm_rank(mpi_comm_world,iinod,ierr)
      call mpi_comm_size(mpi_comm_world,innod,ierr)

C     Initialize Eigen and Memoris of matrix
      n=96
      m=32
      allocate(ah(n,n),al(n,n))
      allocate(xh(n,n),xl(n,n))
      allocate(rh(n,n),rl(n,n))
      allocate(sh(n,n),sl(n,n))
      allocate(ash(n,n),asl(n,n))
      allocate(aih(n,n),ail(n,n))
      allocate(xih(n,n),xil(n,n))
      allocate(rih(n,n),ril(n,n))

      m=32
      nm=(n/2)*2+1
      allocate(
     &         bh((nm)*((n-1)/innod+1+nm+1)),
     &         bih((nm)*((n-1)/innod+1+nm+1)),
     &         ch((nm)*((n-1)/innod+1+nm+1)),
     &         cih((nm)*((n-1)/innod+1+nm+1)),
     &         bl((nm)*((n-1)/innod+1+nm+1)),
     &         bil((nm)*((n-1)/innod+1+nm+1)),
     &         cl((nm)*((n-1)/innod+1+nm+1)),
     &         cil((nm)*((n-1)/innod+1+nm+1)),
     &         wh(nm), wl(nm),
     &         rwh(nm), rwl(nm),
     &         stat=istat )
      if(istat.ne.0) then
         print*,"Memory exhausted"
         call flush(6)
         stop
      endif

C     Set matrix
      call ddmatrix_hermite(n,xh,xl,xih,xil)
      call ddmatrix_diagonal(n, rh, rl, rih, ril,rwh, rwl)

	alphah = 1.0
	alphal = 0.0
	betah  = 0.0
        betal  = 0.0


      call ddgemm("c","n",n,n,n,alphah,alphal,xh,xl,n,rh,rl,n,
     +            betah,betal,sh,sl,n)

      call ddgemm("n","n",n,n,n,alphah,alphal,sh,sl,n,xh,xl,n,
     +            betah,betal,ash,asl,n)

      call ddmatrix_adjust_h(ash,asl,n,bh,bl,nm)
      call ddmatrix_adjust_hi(aih,ail,n,bih,bil,nm)

C     Calculating eigen values and vectors
      hs0 = mpi_wtime()
      call ddeigen_h(n,bh,bl,bih,bil,wh,wl,nm,m,0)

      hs1 = mpi_wtime()
#ifdef TIMER
      if(my_rank==0)then
         print*," "
         print*,"Total Execution Time of eigen_h = ",hs1-hs0,"(sec)"
      endif
      call flush(6)
#endif

C     Check resuls
      call ddmatrix_adjust_h(ash,asl,n,ch,cl,nm)
      call ddmatrix_adjust_hi(aih,ail,n,cih,cil,nm)
      call ddev_test2(ch,cl,cih,cil,wh,wl,bh,bl,bih,bil,n,nm)

C     Sort eigen valus and vectors on variables
      call sort_matrix(n, rwh, rwl, xh, xl)

C     Verify result
      call ddmatrix_adjust_h(xh,xl,n,ch,cl,nm)
      call ddmatrix_adjust_hi(xih,xil,n,cih,cil,nm)
      call dd_check_result(rwh,rwl,ch,cl,cih,cil,
     &                      wh,wl,bh,bl,bih,bil,n,nm)

C     Free memory
      deallocate(ah,aih,bh,bih,ch,cih,wh)
      deallocate(al,ail,bl,bil,cl,cil,wl)

C     Finalizing MPI
      call MPI_Finalize(ierr)
      end



      subroutine sort_matrix(n, rwh, rwl, xh, xl)
      double precision rwh(n), rwl(n)
      double precision xh(n,n), xl(n,n)
      integer          n

      double precision temp_h,temp_l
      double precision temp1_h(n), temp1_l(n)
      double precision temp2_h(n,n), temp2_l(n,n)
      integer          buffer

      do i=1,n
          temp_h=rwh(1)
          temp_l=rwl(1)
          buffer=1
             do j=1,n
               if( (temp_h .lt. rwh(j)).or.
     &            ((temp_h .eq. rwh(j)).and.(temp_l .lt. rwl(j)))) then
                   temp_h=rwh(j)
                   temp_l=rwl(j)
                   buffer=j
               endif

             enddo
             temp1_h(n-i+1) = rwh(buffer)
             temp1_l(n-i+1) = rwl(buffer)
             do j=1,n
               temp2_h(j,n-i+1) = xh(buffer,j)
               temp2_l(j,n-i+1) = xl(buffer,j)
             enddo
             rwh(buffer)=0.0d+0
             rwl(buffer)=0.0d+0

       enddo


	rwh = temp1_h
        rwl = temp1_l
	xh = temp2_h
        Xl = temp2_l      
      end subroutine

      subroutine t_matrix(n, xh, xl)
      double precision xh(n,n), xl(n,n)
      integer          n

      double precision temp_h(n,n), temp_l(n,n)
      integer   i,j

      temp_h = xh
      temp_l = xl

	do i=1,n
	   do j=1,n
              xh(i,j)=temp_h(j,i)
              xl(i,j)=temp_l(j,i)
           end do
         end do
      end subroutine
