*> \brief \b DBDSQR
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DBDSQR + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dbdsqr.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dbdsqr.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dbdsqr.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DBDSQR( UPLO, N, NCVT, NRU, NCC, D, E, VT, LDVT, U,
*                          LDU, C, LDC, WORK, INFO )
* 
*       .. Scalar Arguments ..
*       CHARACTER          UPLO
*       INTEGER            INFO, LDC, LDU, LDVT, N, NCC, NCVT, NRU
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   C( LDC, * ), D( * ), E( * ), U( LDU, * ),
*      $                   VT( LDVT, * ), WORK( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DBDSQR computes the singular values and, optionally, the right and/or
*> left singular vectors from the singular value decomposition (SVD) of
*> a real N-by-N (upper or lower) bidiagonal matrix B using the implicit
*> zero-shift QR algorithm.  The SVD of B has the form
*> 
*>    B = Q * S * P**T
*> 
*> where S is the diagonal matrix of singular values, Q is an orthogonal
*> matrix of left singular vectors, and P is an orthogonal matrix of
*> right singular vectors.  If left singular vectors are requested, this
*> subroutine actually returns U*Q instead of Q, and, if right singular
*> vectors are requested, this subroutine returns P**T*VT instead of
*> P**T, for given real input matrices U and VT.  When U and VT are the
*> orthogonal matrices that reduce a general matrix A to bidiagonal
*> form:  A = U*B*VT, as computed by DGEBRD, then
*>
*>    A = (U*Q) * S * (P**T*VT)
*>
*> is the SVD of A.  Optionally, the subroutine may also compute Q**T*C
*> for a given real input matrix C.
*>
*> See "Computing  Small Singular Values of Bidiagonal Matrices With
*> Guaranteed High Relative Accuracy," by J. Demmel and W. Kahan,
*> LAPACK Working Note #3 (or SIAM J. Sci. Statist. Comput. vol. 11,
*> no. 5, pp. 873-912, Sept 1990) and
*> "Accurate singular values and differential qd algorithms," by
*> B. Parlett and V. Fernando, Technical Report CPAM-554, Mathematics
*> Department, University of California at Berkeley, July 1992
*> for a detailed description of the algorithm.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] UPLO
*> \verbatim
*>          UPLO is CHARACTER*1
*>          = 'U':  B is upper bidiagonal;
*>          = 'L':  B is lower bidiagonal.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix B.  N >= 0.
*> \endverbatim
*>
*> \param[in] NCVT
*> \verbatim
*>          NCVT is INTEGER
*>          The number of columns of the matrix VT. NCVT >= 0.
*> \endverbatim
*>
*> \param[in] NRU
*> \verbatim
*>          NRU is INTEGER
*>          The number of rows of the matrix U. NRU >= 0.
*> \endverbatim
*>
*> \param[in] NCC
*> \verbatim
*>          NCC is INTEGER
*>          The number of columns of the matrix C. NCC >= 0.
*> \endverbatim
*>
*> \param[in,out] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          On entry, the n diagonal elements of the bidiagonal matrix B.
*>          On exit, if INFO=0, the singular values of B in decreasing
*>          order.
*> \endverbatim
*>
*> \param[in,out] E
*> \verbatim
*>          E is DOUBLE PRECISION array, dimension (N-1)
*>          On entry, the N-1 offdiagonal elements of the bidiagonal
*>          matrix B. 
*>          On exit, if INFO = 0, E is destroyed; if INFO > 0, D and E
*>          will contain the diagonal and superdiagonal elements of a
*>          bidiagonal matrix orthogonally equivalent to the one given
*>          as input.
*> \endverbatim
*>
*> \param[in,out] VT
*> \verbatim
*>          VT is DOUBLE PRECISION array, dimension (LDVT, NCVT)
*>          On entry, an N-by-NCVT matrix VT.
*>          On exit, VT is overwritten by P**T * VT.
*>          Not referenced if NCVT = 0.
*> \endverbatim
*>
*> \param[in] LDVT
*> \verbatim
*>          LDVT is INTEGER
*>          The leading dimension of the array VT.
*>          LDVT >= max(1,N) if NCVT > 0; LDVT >= 1 if NCVT = 0.
*> \endverbatim
*>
*> \param[in,out] U
*> \verbatim
*>          U is DOUBLE PRECISION array, dimension (LDU, N)
*>          On entry, an NRU-by-N matrix U.
*>          On exit, U is overwritten by U * Q.
*>          Not referenced if NRU = 0.
*> \endverbatim
*>
*> \param[in] LDU
*> \verbatim
*>          LDU is INTEGER
*>          The leading dimension of the array U.  LDU >= max(1,NRU).
*> \endverbatim
*>
*> \param[in,out] C
*> \verbatim
*>          C is DOUBLE PRECISION array, dimension (LDC, NCC)
*>          On entry, an N-by-NCC matrix C.
*>          On exit, C is overwritten by Q**T * C.
*>          Not referenced if NCC = 0.
*> \endverbatim
*>
*> \param[in] LDC
*> \verbatim
*>          LDC is INTEGER
*>          The leading dimension of the array C.
*>          LDC >= max(1,N) if NCC > 0; LDC >=1 if NCC = 0.
*> \endverbatim
*>
*> \param[out] WORK
*> \verbatim
*>          WORK is DOUBLE PRECISION array, dimension (4*N)
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>          = 0:  successful exit
*>          < 0:  If INFO = -i, the i-th argument had an illegal value
*>          > 0:
*>             if NCVT = NRU = NCC = 0,
*>                = 1, a split was marked by a positive value in E
*>                = 2, current block of Z not diagonalized after 30*N
*>                     iterations (in inner while loop)
*>                = 3, termination criterion of outer while loop not met 
*>                     (program created more than N unreduced blocks)
*>             else NCVT = NRU = NCC = 0,
*>                   the algorithm did not converge; D and E contain the
*>                   elements of a bidiagonal matrix which is orthogonally
*>                   similar to the input matrix B;  if INFO = i, i
*>                   elements of E have not converged to zero.
*> \endverbatim
*
*> \par Internal Parameters:
*  =========================
*>
*> \verbatim
*>  TOLMUL  DOUBLE PRECISION, default = max(10,min(100,EPS**(-1/8)))
*>          TOLMUL controls the convergence criterion of the QR loop.
*>          If it is positive, TOLMUL*EPS is the desired relative
*>             precision in the computed singular values.
*>          If it is negative, abs(TOLMUL*EPS*sigma_max) is the
*>             desired absolute accuracy in the computed singular
*>             values (corresponds to relative accuracy
*>             abs(TOLMUL*EPS) in the largest singular value.
*>          abs(TOLMUL) should be between 1 and 1/EPS, and preferably
*>             between 10 (for fast convergence) and .1/EPS
*>             (for there to be some accuracy in the results).
*>          Default is to lose at either one eighth or 2 of the
*>             available decimal digits in each computed singular value
*>             (whichever is smaller).
*>
*>  MAXITR  INTEGER, default = 6
*>          MAXITR controls the maximum number of passes of the
*>          algorithm through its inner loop. The algorithms stops
*>          (and so fails to converge) if the number of passes
*>          through the inner loop exceeds MAXITR*N**2.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*  =====================================================================
!!      SUBROUTINE DBDSQR( UPLO, N, NCVT, NRU, NCC, D, E, VT, LDVT, U,
!!     $                   LDU, C, LDC, WORK, INFO )
      SUBROUTINE dDBDSQR( UPLO, N, NCVT, NRU, NCC, Dh, Dl, Eh, El,
     $                    VTh, VTl, LDVT, Uh, Ul,
     $                    LDU, Ch, Cl, LDC, WORKh, WORKl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            INFO, LDC, LDU, LDVT, N, NCC, NCVT, NRU
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   C( LDC, * ), D( * ), E( * ), U( LDU, * ),
!!     $                   VT( LDVT, * ), WORK( * )
      double precision Ch(LDC,*), Cl(LDC,*)
      double precision Dh(*), Dl(*)
      double precision Eh(*), El(*)
      double precision Uh(LDU,*), Ul(LDU,*)
      double precision VTh(LDVT,*), VTl(LDVT,*)
      double precision WORKh(*), WORKl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ZERO
!!      PARAMETER          ( ZERO = 0.0D0 )
!!      DOUBLE PRECISION   ONE
!!      PARAMETER          ( ONE = 1.0D0 )
!!      DOUBLE PRECISION   NEGONE
!!      PARAMETER          ( NEGONE = -1.0D0 )
!!      DOUBLE PRECISION   HNDRTH
!!      PARAMETER          ( HNDRTH = 0.01D0 )
!!      DOUBLE PRECISION   TEN
!!      PARAMETER          ( TEN = 10.0D0 )
!!      DOUBLE PRECISION   HNDRD
!!      PARAMETER          ( HNDRD = 100.0D0 )
!!      DOUBLE PRECISION   MEIGTH
!!      PARAMETER          ( MEIGTH = -0.125D0 )
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
      double precision tmp9h, tmp9l
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision NEGONEh, NEGONEl
      double precision HNDRTHh, HNDRTHl
      double precision TENh, TENl
      double precision HNDRDh, HNDRDl
      double precision MEIGTHh, MEIGTHl
      INTEGER            MAXITR
      PARAMETER          ( MAXITR = 6 )
*     ..
*     .. Local Scalars ..
      LOGICAL            LOWER, ROTATE
      INTEGER            I, IDIR, ISUB, ITER, J, LL, LLL, M, MAXIT, NM1,
     $                   NM12, NM13, OLDLL, OLDM
!!      DOUBLE PRECISION   ABSE, ABSS, COSL, COSR, CS, EPS, F, G, H, MU,
!!     $                   OLDCS, OLDSN, R, SHIFT, SIGMN, SIGMX, SINL,
!!     $                   SINR, SLL, SMAX, SMIN, SMINL, SMINOA,
!!     $                   SN, THRESH, TOL, TOLMUL, UNFL
      double precision ABSEh, ABSEl
      double precision ABSSh, ABSSl
      double precision COSLh, COSLl
      double precision COSRh, COSRl
      double precision CSh, CSl
      double precision EPSh, EPSl
      double precision Fh, Fl
      double precision Gh, Gl
      double precision Hh, Hl
      double precision MUh, MUl
      double precision OLDCSh, OLDCSl
      double precision OLDSNh, OLDSNl
      double precision Rh, Rl
      double precision SHIFTh, SHIFTl
      double precision SIGMNh, SIGMNl
      double precision SIGMXh, SIGMXl
      double precision SINLh, SINLl
      double precision SINRh, SINRl
      double precision SLLh, SLLl
      double precision SMAXh, SMAXl
      double precision SMINh, SMINl
      double precision SMINLh, SMINLl
      double precision SMINOAh, SMINOAl
      double precision SNh, SNl
      double precision THRESHh, THRESHl
      double precision TOLh, TOLl
      double precision TOLMULh, TOLMULl
      double precision UNFLh, UNFLl
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
!!      DOUBLE PRECISION   DLAMCH
!!      EXTERNAL           LSAME, DLAMCH
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
!!      EXTERNAL           DLARTG, DLAS2, DLASQ1, DLASR, DLASV2, DROT,
!!     $                   DSCAL, DSWAP, XERBLA
      EXTERNAL           dDLARTG, dDLAS2, dDLASQ1, dDLASR, dDLASV2,
     $                   dDROT,
     $                   dDSCAL, dDSWAP, XERBLA, dDLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, DBLE, MAX, MIN, SIGN, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0D0
      ZEROl = 0.0d+0
      ONEh = 1.0D0
      ONEl = 0.0d+0
!<<<  NEGONE = -ONE
      NEGONEh = -ONEh
      NEGONEl = -ONEl
!>>>  NEGONE = -ONE
      HNDRTHh = 0.01D0
      HNDRTHl = 0.0d+0
      TENh = 10.0D0
      TENl = 0.0d+0
      HNDRDh = 100.0D0
      HNDRDl = 0.0d+0
      tmp1h = 0.125D0
      tmp1l = 0.0d+0
!<<<  MEIGTH = -tmp1
      MEIGTHh = -tmp1h
      MEIGTHl = -tmp1l
!>>>  MEIGTH = -tmp1
*
*     Test the input parameters.
*
      INFO = 0
      LOWER = LSAME( UPLO, 'L' )
      IF( .NOT.LSAME( UPLO, 'U' ) .AND. .NOT.LOWER ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( NCVT.LT.0 ) THEN
         INFO = -3
      ELSE IF( NRU.LT.0 ) THEN
         INFO = -4
      ELSE IF( NCC.LT.0 ) THEN
         INFO = -5
      ELSE IF( ( NCVT.EQ.0 .AND. LDVT.LT.1 ) .OR.
     $         ( NCVT.GT.0 .AND. LDVT.LT.MAX( 1, N ) ) ) THEN
         INFO = -9
      ELSE IF( LDU.LT.MAX( 1, NRU ) ) THEN
         INFO = -11
      ELSE IF( ( NCC.EQ.0 .AND. LDC.LT.1 ) .OR.
     $         ( NCC.GT.0 .AND. LDC.LT.MAX( 1, N ) ) ) THEN
         INFO = -13
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DBDSQR', -INFO )
         RETURN
      END IF
      IF( N.EQ.0 )
     $   RETURN
      IF( N.EQ.1 )
     $   GO TO 160
*
*     ROTATE is true if any singular vectors desired, false otherwise
*
      ROTATE = ( NCVT.GT.0 ) .OR. ( NRU.GT.0 ) .OR. ( NCC.GT.0 )
*
*     If no singular vectors desired, use qd algorithm
*
      IF( .NOT.ROTATE ) THEN
!!         CALL DLASQ1( N, D, E, WORK, INFO )
         CALL dDLASQ1( N, Dh, Dl, Eh, El, WORKh, WORKl, INFO )
*
*     If INFO equals 2, dqds didn't finish, try to finish
*         
         IF( INFO .NE. 2 ) RETURN
         INFO = 0
      END IF
*
      NM1 = N - 1
      NM12 = NM1 + NM1
      NM13 = NM12 + NM1
      IDIR = 0
*
*     Get machine constants
*
!!      EPS = DLAMCH( 'Epsilon' )
      CALL dDLAMCH( 'Epsilon', EPSh, EPSl )
!!      UNFL = DLAMCH( 'Safe minimum' )
      CALL dDLAMCH( 'Safe minimum', UNFLh, UNFLl )
*
*     If matrix lower bidiagonal, rotate to be upper bidiagonal
*     by applying Givens rotations on the left
*
      IF( LOWER ) THEN
         DO 10 I = 1, N - 1
!!            CALL DLARTG( D( I ), E( I ), CS, SN, R )
            CALL dDLARTG( Dh( I ), Dl( I ), Eh( I ), El( I ),
     $                    CSh, CSl, SNh, SNl, Rh, Rl )
!!            D( I ) = R
      Dh(I) = Rh
      Dl(I) = Rl
!!            E( I ) = SN*D( I+1 )
!<<<  E(I) = SN * D(I+1)
      T_ch = SNh * Dh(I+1)
      T_u1 = 134217729.0 * SNh
      T_p1 = T_u1
     +         - (T_u1 - SNh)
      T_p2 = SNh - T_p1
      T_u1 = 134217729.0 * Dh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_r2 = Dh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SNh*Dl(I+1)
     +         +  SNl*Dh(I+1))
      T_s1 = T_ch + T_cl
      El(I) = T_cl - (T_s1 - T_ch)
      Eh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(I) = SN * D(I+1)
!!            D( I+1 ) = CS*D( I+1 )
!<<<  D(I+1) = CS * D(I+1)
      T_ch = CSh * Dh(I+1)
      T_u1 = 134217729.0 * CSh
      T_p1 = T_u1
     +         - (T_u1 - CSh)
      T_p2 = CSh - T_p1
      T_u1 = 134217729.0 * Dh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_r2 = Dh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CSh*Dl(I+1)
     +         +  CSl*Dh(I+1))
      T_s1 = T_ch + T_cl
      Dl(I+1) = T_cl - (T_s1 - T_ch)
      Dh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  D(I+1) = CS * D(I+1)
!!            WORK( I ) = CS
      WORKh(I) = CSh
      WORKl(I) = CSl
!!            WORK( NM1+I ) = SN
      WORKh(NM1+I) = SNh
      WORKl(NM1+I) = SNl
   10    CONTINUE
*
*        Update singular vectors if desired
*
!!         IF( NRU.GT.0 )
!!     $      CALL DLASR( 'R', 'V', 'F', NRU, N, WORK( 1 ), WORK( N ), U,
!!     $                  LDU )
         IF( NRU.GT.0 )
     $      CALL dDLASR( 'R', 'V', 'F', NRU, N, WORKh( 1 ), WORKl( 1 ),
     $                   WORKh( N ), WORKl( N ), Uh, Ul,
     $                   LDU )
!!         IF( NCC.GT.0 )
!!     $      CALL DLASR( 'L', 'V', 'F', N, NCC, WORK( 1 ), WORK( N ), C,
!!     $                  LDC )
         IF( NCC.GT.0 )
     $      CALL dDLASR( 'L', 'V', 'F', N, NCC, WORKh( 1 ), WORKl( 1 ),
     $                   WORKh( N ), WORKl( N ), Ch, Cl,
     $                   LDC )
      END IF
*
*     Compute singular values to relative accuracy TOL
*     (By setting TOL to be negative, algorithm will compute
*     singular values to absolute accuracy ABS(TOL)*norm(input matrix))
*
!      TOLMUL = MAX( TEN, MIN( HNDRD, EPS**MEIGTH ) )
!<<<  tmp1 = sqrt(eps)
      if (epsh.eq.0.0D+0.and.epsl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (epsh.lt.0.0d+0.or.
     +   (epsh.eq.0.0d+0.and.epsl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(epsh)
      T_ax = epsh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           epsh,epsl,T_z1,T_z2)
          T_p1 = epsh - T_z1
          T_p2 = T_p1 - epsh
          T_t2l = (epsh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + epsl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(eps)
!<<<  tmp1 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(tmp1)
!<<<  tmp1 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(tmp1)
!<<<  tmp1 = one / tmp1
      T_q1 = oneh / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = oneh - T_ch
      T_r1 = T_s1 - oneh
      T_s2 = (oneh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + onel
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = one / tmp1
      IF (
!       (HNDRD .lt. tmp1)
     + ((HNDRDh .lt. tmp1h) .or.
     +  ((HNDRDh .eq. tmp1h) .and.
     +   (HNDRDl .lt. tmp1l)))
     $) THEN
      tmp2h = HNDRDh
      tmp2l = HNDRDl
      ELSE
      tmp2h = tmp1h
      tmp2l = tmp1l
      END IF
      IF (
!       (TEN .gt. tmp2)
     + ((TENh .gt. tmp2h) .or.
     +  ((TENh .eq. tmp2h) .and.
     +   (TENl .gt. tmp2l)))
     &) THEN
      TOLMULh = TENh
      TOLMULl = TENl
      ELSE
      TOLMULh = tmp2h
      TOLMULl = tmp2l
      ENDIF
!!      TOL = TOLMUL*EPS
!<<<  TOL = TOLMUL * EPS
      T_ch = TOLMULh * EPSh
      T_u1 = 134217729.0 * TOLMULh
      T_p1 = T_u1
     +         - (T_u1 - TOLMULh)
      T_p2 = TOLMULh - T_p1
      T_u1 = 134217729.0 * EPSh
      T_r1 = T_u1
     +         - (T_u1 - EPSh)
      T_r2 = EPSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLMULh*EPSl
     +         +  TOLMULl*EPSh)
      T_s1 = T_ch + T_cl
      TOLl = T_cl - (T_s1 - T_ch)
      TOLh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TOL = TOLMUL * EPS
*
*     Compute approximate maximum, minimum singular values
*
!!      SMAX = ZERO
      SMAXh = ZEROh
      SMAXl = ZEROl
      DO 20 I = 1, N
!!         SMAX = MAX( SMAX, ABS( D( I ) ) )
!<<<  tmp1 = abs(D(I))
      if (Dh(I) .lt. 0.0d+0) then
        tmp1h = -Dh(I)
        tmp1l = -Dl(I)
      elseif (
     +  (Dh(I) .eq. 0.0d+0).and.
     +  (Dl(I) .lt. 0.0d+0)) then
        tmp1h = Dh(I)
        tmp1l = -Dl(I)
      else
        tmp1h = Dh(I)
        tmp1l = Dl(I)
      endif
!>>>  tmp1 = abs(D(I))
         IF(
!       (SMAX .lt. tmp1)
     + ((SMAXh .lt. tmp1h) .or.
     +  ((SMAXh .eq. tmp1h) .and.
     +   (SMAXl .lt. tmp1l)))
     $     ) THEN
      SMAXh = tmp1h
      SMAXl = tmp1l
         END IF
   20 CONTINUE
      DO 30 I = 1, N - 1
!!         SMAX = MAX( SMAX, ABS( E( I ) ) )
!<<<  tmp1 = abs(E(I))
      if (Eh(I) .lt. 0.0d+0) then
        tmp1h = -Eh(I)
        tmp1l = -El(I)
      elseif (
     +  (Eh(I) .eq. 0.0d+0).and.
     +  (El(I) .lt. 0.0d+0)) then
        tmp1h = Eh(I)
        tmp1l = -El(I)
      else
        tmp1h = Eh(I)
        tmp1l = El(I)
      endif
!>>>  tmp1 = abs(E(I))
         IF(
!       (SMAX .lt. tmp1)
     + ((SMAXh .lt. tmp1h) .or.
     +  ((SMAXh .eq. tmp1h) .and.
     +   (SMAXl .lt. tmp1l)))
     $     ) THEN
      SMAXh = tmp1h
      SMAXl = tmp1l
         END IF
   30 CONTINUE
!!      SMINL = ZERO
      SMINLh = ZEROh
      SMINLl = ZEROl
!!      IF( TOL.GE.ZERO ) THEN
      IF(
!       (TOL .ge. ZERO)
     + ((TOLh .gt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .ge. ZEROl)))
     $  ) THEN
*
*        Relative accuracy desired
*
!!         SMINOA = ABS( D( 1 ) )
!<<<  tmp1 = abs(D(1))
      if (Dh(1) .lt. 0.0d+0) then
        tmp1h = -Dh(1)
        tmp1l = -Dl(1)
      elseif (
     +  (Dh(1) .eq. 0.0d+0).and.
     +  (Dl(1) .lt. 0.0d+0)) then
        tmp1h = Dh(1)
        tmp1l = -Dl(1)
      else
        tmp1h = Dh(1)
        tmp1l = Dl(1)
      endif
!>>>  tmp1 = abs(D(1))
      SMINOAh = tmp1h
      SMINOAl = tmp1l
!!         IF( SMINOA.EQ.ZERO )
!!     $      GO TO 50
         IF(
!       (SMINOA .eq. ZERO)
     + ((SMINOAh .eq. ZEROh) .and.
     +  (SMINOAl .eq. ZEROl))
     $    ) THEN
            GO TO 50
         END IF
!!         MU = SMINOA
      MUh = SMINOAh
      MUl = SMINOAl
         DO 40 I = 2, N
!!            MU = ABS( D( I ) )*( MU / ( MU+ABS( E( I-1 ) ) ) )
!<<<  tmp1 = abs(D(I))
      if (Dh(I) .lt. 0.0d+0) then
        tmp1h = -Dh(I)
        tmp1l = -Dl(I)
      elseif (
     +  (Dh(I) .eq. 0.0d+0).and.
     +  (Dl(I) .lt. 0.0d+0)) then
        tmp1h = Dh(I)
        tmp1l = -Dl(I)
      else
        tmp1h = Dh(I)
        tmp1l = Dl(I)
      endif
!>>>  tmp1 = abs(D(I))
!<<<  tmp2 = abs(E(I-1))
      if (Eh(I-1) .lt. 0.0d+0) then
        tmp2h = -Eh(I-1)
        tmp2l = -El(I-1)
      elseif (
     +  (Eh(I-1) .eq. 0.0d+0).and.
     +  (El(I-1) .lt. 0.0d+0)) then
        tmp2h = Eh(I-1)
        tmp2l = -El(I-1)
      else
        tmp2h = Eh(I-1)
        tmp2l = El(I-1)
      endif
!>>>  tmp2 = abs(E(I-1))
!<<<  tmp3 = MU + tmp2
      T_p1 = MUh + tmp2h
      T_p2 = T_p1 - MUh
      T_q1 = T_p1
      T_q2 = (MUh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + MUl + tmp2l
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = MU + tmp2
!<<<  tmp4 = MU / tmp3
      T_q1 = MUh / tmp3h
      T_ch = tmp3h * T_q1
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp3l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = MUh - T_ch
      T_r1 = T_s1 - MUh
      T_s2 = (MUh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + MUl
      T_q2 = (T_s1 + T_s2)
     +         / tmp3h
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = MU / tmp3
!<<<  MU = tmp1 * tmp4
      T_ch = tmp1h * tmp4h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp4l
     +         +  tmp1l*tmp4h)
      T_s1 = T_ch + T_cl
      MUl = T_cl - (T_s1 - T_ch)
      MUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  MU = tmp1 * tmp4
!!            SMINOA = MIN( SMINOA, MU )
            IF(
!       (SMINOA .gt. MU)
     + ((SMINOAh .gt. MUh) .or.
     +  ((SMINOAh .eq. MUh) .and.
     +   (SMINOAl .gt. MUl)))
     $        ) THEN
      SMINOAh = MUh
      SMINOAl = MUl
            END IF
!!            IF( SMINOA.EQ.ZERO )
!!     $         GO TO 50
            IF(
!       (SMINOA .eq. ZERO)
     + ((SMINOAh .eq. ZEROh) .and.
     +  (SMINOAl .eq. ZEROl))
     $        ) THEN
               GO TO 50
            END IF
   40    CONTINUE
   50    CONTINUE
!!         SMINOA = SMINOA / SQRT( DBLE( N ) )
      tmp1h = N
      tmp1l = 0.0d+0
!<<<  tmp2 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp2 = sqrt(tmp1)
!<<<  SMINOA = SMINOA / tmp2
      T_q1 = SMINOAh / tmp2h
      T_ch = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = SMINOAh - T_ch
      T_r1 = T_s1 - SMINOAh
      T_s2 = (SMINOAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + SMINOAl
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      SMINOAh = T_q1 + T_q2
      SMINOAl = T_q2
     +      - (SMINOAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SMINOA = SMINOA / tmp2
!!         THRESH = MAX( TOL*SMINOA, MAXITR*N*N*UNFL )
!<<<  tmp1 = TOL * SMINOA
      T_ch = TOLh * SMINOAh
      T_u1 = 134217729.0 * TOLh
      T_p1 = T_u1
     +         - (T_u1 - TOLh)
      T_p2 = TOLh - T_p1
      T_u1 = 134217729.0 * SMINOAh
      T_r1 = T_u1
     +         - (T_u1 - SMINOAh)
      T_r2 = SMINOAh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLh*SMINOAl
     +         +  TOLl*SMINOAh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TOL * SMINOA
      tmp2h = MAXITR
      tmp2l = 0.0d+0
      tmp3h = N
      tmp3l = 0.0d+0
!<<<  tmp4 = tmp2 * tmp3
      T_ch = tmp2h * tmp3h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp3l
     +         +  tmp2l*tmp3h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp2 * tmp3
!<<<  tmp5 = tmp4 * tmp3
      T_ch = tmp4h * tmp3h
      T_u1 = 134217729.0 * tmp4h
      T_p1 = T_u1
     +         - (T_u1 - tmp4h)
      T_p2 = tmp4h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp4h*tmp3l
     +         +  tmp4l*tmp3h)
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp5 = tmp4 * tmp3
!<<<  tmp6 = tmp5 * UNFL
      T_ch = tmp5h * UNFLh
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * UNFLh
      T_r1 = T_u1
     +         - (T_u1 - UNFLh)
      T_r2 = UNFLh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp5h*UNFLl
     +         +  tmp5l*UNFLh)
      T_s1 = T_ch + T_cl
      tmp6l = T_cl - (T_s1 - T_ch)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp6 = tmp5 * UNFL
         IF(
!       (tmp1 .gt. tmp6)
     + ((tmp1h .gt. tmp6h) .or.
     +  ((tmp1h .eq. tmp6h) .and.
     +   (tmp1l .gt. tmp6l)))
     $     ) THEN
      THRESHh = tmp1h
      THRESHl = tmp1l
         ELSE
      THRESHh = tmp6h
      THRESHl = tmp6l
         END IF
      ELSE
*
*        Absolute accuracy desired
*
!!         THRESH = MAX( ABS( TOL )*SMAX, MAXITR*N*N*UNFL )
!<<<  tmp1 = abs(TOL)
      if (TOLh .lt. 0.0d+0) then
        tmp1h = -TOLh
        tmp1l = -TOLl
      elseif (
     +  (TOLh .eq. 0.0d+0).and.
     +  (TOLl .lt. 0.0d+0)) then
        tmp1h = TOLh
        tmp1l = -TOLl
      else
        tmp1h = TOLh
        tmp1l = TOLl
      endif
!>>>  tmp1 = abs(TOL)
!<<<  tmp2 = tmp1 * SMAX
      T_ch = tmp1h * SMAXh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * SMAXh
      T_r1 = T_u1
     +         - (T_u1 - SMAXh)
      T_r2 = SMAXh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*SMAXl
     +         +  tmp1l*SMAXh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * SMAX
      tmp3h = MAXITR
      tmp3l = 0.0d+0
      tmp4h = N
      tmp4l = 0.0d+0
!<<<  tmp5 = tmp3 * tmp4
      T_ch = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*tmp4l
     +         +  tmp3l*tmp4h)
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
!<<<  tmp6 = tmp5 * tmp4
      T_ch = tmp5h * tmp4h
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp5h*tmp4l
     +         +  tmp5l*tmp4h)
      T_s1 = T_ch + T_cl
      tmp6l = T_cl - (T_s1 - T_ch)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp6 = tmp5 * tmp4
!<<<  tmp7 = tmp6 * UNFL
      T_ch = tmp6h * UNFLh
      T_u1 = 134217729.0 * tmp6h
      T_p1 = T_u1
     +         - (T_u1 - tmp6h)
      T_p2 = tmp6h - T_p1
      T_u1 = 134217729.0 * UNFLh
      T_r1 = T_u1
     +         - (T_u1 - UNFLh)
      T_r2 = UNFLh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp6h*UNFLl
     +         +  tmp6l*UNFLh)
      T_s1 = T_ch + T_cl
      tmp7l = T_cl - (T_s1 - T_ch)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp7 = tmp6 * UNFL
         IF(
!       (tmp2 .gt. tmp7)
     + ((tmp2h .gt. tmp7h) .or.
     +  ((tmp2h .eq. tmp7h) .and.
     +   (tmp2l .gt. tmp7l)))
     $     ) THEN
      THRESHh = tmp2h
      THRESHl = tmp2l
         ELSE
      THRESHh = tmp7h
      THRESHl = tmp7l
         END IF
      END IF
*
*     Prepare for main iteration loop for the singular values
*     (MAXIT is the maximum number of passes through the inner
*     loop permitted before nonconvergence signalled.)
*
      MAXIT = MAXITR*N*N
      ITER = 0
      OLDLL = -1
      OLDM = -1
*
*     M points to last element of unconverged part of matrix
*
      M = N
*
*     Begin main iteration loop
*
   60 CONTINUE
*
*     Check for convergence or exceeding iteration count
*
      IF( M.LE.1 )
     $   GO TO 160
      IF( ITER.GT.MAXIT )
     $   GO TO 200
*
*     Find diagonal block of matrix to work on
*
!!      IF( TOL.LT.ZERO .AND. ABS( D( M ) ).LE.THRESH )
!!     $   D( M ) = ZERO
!<<<  tmp1 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp1h = -Dh(M)
        tmp1l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp1h = Dh(M)
        tmp1l = -Dl(M)
      else
        tmp1h = Dh(M)
        tmp1l = Dl(M)
      endif
!>>>  tmp1 = abs(D(M))
      IF(
!       (TOL .lt. ZERO)
     + ((TOLh .lt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .lt. ZEROl)))
     $      .AND.
!       (tmp1 .le. THRESH)
     + ((tmp1h .lt. THRESHh) .or.
     +  ((tmp1h .eq. THRESHh) .and.
     +   (tmp1l .le. THRESHl)))
     $  ) THEN
      Dh(M) = ZEROh
      Dl(M) = ZEROl
      END IF
!!      SMAX = ABS( D( M ) )
!<<<  SMAX = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        SMAXh = -Dh(M)
        SMAXl = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        SMAXh = Dh(M)
        SMAXl = -Dl(M)
      else
        SMAXh = Dh(M)
        SMAXl = Dl(M)
      endif
!>>>  SMAX = abs(D(M))
!!      SMIN = SMAX
      SMINh = SMAXh
      SMINl = SMAXl
      DO 70 LLL = 1, M - 1
         LL = M - LLL
!!         ABSS = ABS( D( LL ) )
!<<<  ABSS = abs(D(LL))
      if (Dh(LL) .lt. 0.0d+0) then
        ABSSh = -Dh(LL)
        ABSSl = -Dl(LL)
      elseif (
     +  (Dh(LL) .eq. 0.0d+0).and.
     +  (Dl(LL) .lt. 0.0d+0)) then
        ABSSh = Dh(LL)
        ABSSl = -Dl(LL)
      else
        ABSSh = Dh(LL)
        ABSSl = Dl(LL)
      endif
!>>>  ABSS = abs(D(LL))
!!         ABSE = ABS( E( LL ) )
!<<<  ABSE = abs(E(LL))
      if (Eh(LL) .lt. 0.0d+0) then
        ABSEh = -Eh(LL)
        ABSEl = -El(LL)
      elseif (
     +  (Eh(LL) .eq. 0.0d+0).and.
     +  (El(LL) .lt. 0.0d+0)) then
        ABSEh = Eh(LL)
        ABSEl = -El(LL)
      else
        ABSEh = Eh(LL)
        ABSEl = El(LL)
      endif
!>>>  ABSE = abs(E(LL))
!!         IF( TOL.LT.ZERO .AND. ABSS.LE.THRESH )
!!     $      D( LL ) = ZERO
         IF(
!       (TOL .lt. ZERO)
     + ((TOLh .lt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .lt. ZEROl)))
     $         .AND.
!       (ABSS .le. THRESH)
     + ((ABSSh .lt. THRESHh) .or.
     +  ((ABSSh .eq. THRESHh) .and.
     +   (ABSSl .le. THRESHl)))
     $     ) THEN
      Dh(LL) = ZEROh
      Dl(LL) = ZEROl
         END IF
!!         IF( ABSE.LE.THRESH )
!!     $      GO TO 80
         IF(
!       (ABSE .le. THRESH)
     + ((ABSEh .lt. THRESHh) .or.
     +  ((ABSEh .eq. THRESHh) .and.
     +   (ABSEl .le. THRESHl)))
     $     ) THEN
            GO TO 80
         END IF
!!         SMIN = MIN( SMIN, ABSS )
         IF(
!       (SMIN .gt. ABSS)
     + ((SMINh .gt. ABSSh) .or.
     +  ((SMINh .eq. ABSSh) .and.
     +   (SMINl .gt. ABSSl)))
     $     ) THEN
      SMINh = ABSSh
      SMINl = ABSSl
         END IF
!!         SMAX = MAX( SMAX, ABSS, ABSE )
         IF(
!       (SMAX .lt. ABSS)
     + ((SMAXh .lt. ABSSh) .or.
     +  ((SMAXh .eq. ABSSh) .and.
     +   (SMAXl .lt. ABSSl)))
     $         .AND.
!       (ABSE .lt. ABSS)
     + ((ABSEh .lt. ABSSh) .or.
     +  ((ABSEh .eq. ABSSh) .and.
     +   (ABSEl .lt. ABSSl)))
     $     ) THEN
      SMAXh = ABSSh
      SMAXl = ABSSl
         ELSE IF(
!       (SMAX .lt. ABSE)
     + ((SMAXh .lt. ABSEh) .or.
     +  ((SMAXh .eq. ABSEh) .and.
     +   (SMAXl .lt. ABSEl)))
     $         .AND.
!       (ABSS .lt. ABSE)
     + ((ABSSh .lt. ABSEh) .or.
     +  ((ABSSh .eq. ABSEh) .and.
     +   (ABSSl .lt. ABSEl)))
     $     ) THEN
      SMAXh = ABSEh
      SMAXl = ABSEl
         END IF
   70 CONTINUE
      LL = 0
      GO TO 90
   80 CONTINUE
!!      E( LL ) = ZERO
      Eh(LL) = ZEROh
      El(LL) = ZEROl
*
*     Matrix splits since E(LL) = 0
*
      IF( LL.EQ.M-1 ) THEN
*
*        Convergence of bottom singular value, return to top of loop
*
         M = M - 1
         GO TO 60
      END IF
   90 CONTINUE
      LL = LL + 1
*
*     E(LL) through E(M-1) are nonzero, E(LL-1) is zero
*
      IF( LL.EQ.M-1 ) THEN
*
*        2 by 2 block, handle separately
*
!!         CALL DLASV2( D( M-1 ), E( M-1 ), D( M ), SIGMN, SIGMX, SINR,
!!     $                COSR, SINL, COSL )
         CALL dDLASV2( Dh( M-1 ), Dl( M-1 ), Eh( M-1 ), El( M-1 ),
     $                 Dh( M ), Dl( M ), SIGMNh, SIGMNl,
     $                 SIGMXh, SIGMXl, SINRh, SINRl,
     $                 COSRh, COSRl, SINLh, SINLl, COSLh, COSLl )
!!         D( M-1 ) = SIGMX
      Dh(M-1) = SIGMXh
      Dl(M-1) = SIGMXl
!!         E( M-1 ) = ZERO
      Eh(M-1) = ZEROh
      El(M-1) = ZEROl
!!         D( M ) = SIGMN
      Dh(M) = SIGMNh
      Dl(M) = SIGMNl
*
*        Compute singular vectors, if desired
*
!!         IF( NCVT.GT.0 )
!!     $      CALL DROT( NCVT, VT( M-1, 1 ), LDVT, VT( M, 1 ), LDVT, COSR,
!!     $                 SINR )
         IF( NCVT.GT.0 )
     $      CALL dDROT( NCVT, VTh( M-1, 1 ), VTl( M-1, 1 ), LDVT,
     $                  VTh( M, 1 ), VTl( M, 1 ), LDVT, COSRh, COSRl,
     $                  SINRh, SINRl )
!!         IF( NRU.GT.0 )
!!     $      CALL DROT( NRU, U( 1, M-1 ), 1, U( 1, M ), 1, COSL, SINL )
         IF( NRU.GT.0 )
     $      CALL dDROT( NRU, Uh( 1, M-1 ), Ul( 1, M-1 ), 1,
     $                  Uh( 1, M ), Ul( 1, M ), 1, COSLh, COSLl,
     $                  SINLh, SINLl )
!!         IF( NCC.GT.0 )
!!     $      CALL DROT( NCC, C( M-1, 1 ), LDC, C( M, 1 ), LDC, COSL,
!!     $                 SINL )
         IF( NCC.GT.0 )
     $      CALL dDROT( NCC, Ch( M-1, 1 ), Cl( M-1, 1 ), LDC,
     $                  Ch( M, 1 ), Cl( M, 1 ), LDC, COSLh, COSLl,
     $                  SINLh, SINLl )
         M = M - 2
         GO TO 60
      END IF
*
*     If working on new submatrix, choose shift direction
*     (from larger end diagonal element towards smaller)
*
      IF( LL.GT.OLDM .OR. M.LT.OLDLL ) THEN
!!         IF( ABS( D( LL ) ).GE.ABS( D( M ) ) ) THEN
!<<<  tmp1 = abs(D(LL))
      if (Dh(LL) .lt. 0.0d+0) then
        tmp1h = -Dh(LL)
        tmp1l = -Dl(LL)
      elseif (
     +  (Dh(LL) .eq. 0.0d+0).and.
     +  (Dl(LL) .lt. 0.0d+0)) then
        tmp1h = Dh(LL)
        tmp1l = -Dl(LL)
      else
        tmp1h = Dh(LL)
        tmp1l = Dl(LL)
      endif
!>>>  tmp1 = abs(D(LL))
!<<<  tmp2 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp2h = -Dh(M)
        tmp2l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp2h = Dh(M)
        tmp2l = -Dl(M)
      else
        tmp2h = Dh(M)
        tmp2l = Dl(M)
      endif
!>>>  tmp2 = abs(D(M))
         IF(
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     $     ) THEN
*
*           Chase bulge from top (big end) to bottom (small end)
*
            IDIR = 1
         ELSE
*
*           Chase bulge from bottom (big end) to top (small end)
*
            IDIR = 2
         END IF
      END IF
*
*     Apply convergence tests
*
      IF( IDIR.EQ.1 ) THEN
*
*        Run convergence test in forward direction
*        First apply standard test to bottom of matrix
*
!!         IF( ABS( E( M-1 ) ).LE.ABS( TOL )*ABS( D( M ) ) .OR.
!!     $       ( TOL.LT.ZERO .AND. ABS( E( M-1 ) ).LE.THRESH ) ) THEN
!<<<  tmp1 = abs(E(M-1))
      if (Eh(M-1) .lt. 0.0d+0) then
        tmp1h = -Eh(M-1)
        tmp1l = -El(M-1)
      elseif (
     +  (Eh(M-1) .eq. 0.0d+0).and.
     +  (El(M-1) .lt. 0.0d+0)) then
        tmp1h = Eh(M-1)
        tmp1l = -El(M-1)
      else
        tmp1h = Eh(M-1)
        tmp1l = El(M-1)
      endif
!>>>  tmp1 = abs(E(M-1))
!<<<  tmp2 = abs(TOL)
      if (TOLh .lt. 0.0d+0) then
        tmp2h = -TOLh
        tmp2l = -TOLl
      elseif (
     +  (TOLh .eq. 0.0d+0).and.
     +  (TOLl .lt. 0.0d+0)) then
        tmp2h = TOLh
        tmp2l = -TOLl
      else
        tmp2h = TOLh
        tmp2l = TOLl
      endif
!>>>  tmp2 = abs(TOL)
!<<<  tmp3 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp3h = -Dh(M)
        tmp3l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp3h = Dh(M)
        tmp3l = -Dl(M)
      else
        tmp3h = Dh(M)
        tmp3l = Dl(M)
      endif
!>>>  tmp3 = abs(D(M))
!<<<  tmp4 = tmp2 * tmp3
      T_ch = tmp2h * tmp3h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp3l
     +         +  tmp2l*tmp3h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp2 * tmp3
!<<<  tmp5 = abs(E(M-1))
      if (Eh(M-1) .lt. 0.0d+0) then
        tmp5h = -Eh(M-1)
        tmp5l = -El(M-1)
      elseif (
     +  (Eh(M-1) .eq. 0.0d+0).and.
     +  (El(M-1) .lt. 0.0d+0)) then
        tmp5h = Eh(M-1)
        tmp5l = -El(M-1)
      else
        tmp5h = Eh(M-1)
        tmp5l = El(M-1)
      endif
!>>>  tmp5 = abs(E(M-1))
         IF(
!       (tmp1 .le. tmp4)
     + ((tmp1h .lt. tmp4h) .or.
     +  ((tmp1h .eq. tmp4h) .and.
     +   (tmp1l .le. tmp4l)))
     $         .OR. (
!       (TOL .lt. ZERO)
     + ((TOLh .lt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .lt. ZEROl)))
     $         .AND.
!       (tmp5 .le. THRESH)
     + ((tmp5h .lt. THRESHh) .or.
     +  ((tmp5h .eq. THRESHh) .and.
     +   (tmp5l .le. THRESHl)))
     $     ) ) THEN
!!            E( M-1 ) = ZERO
      Eh(M-1) = ZEROh
      El(M-1) = ZEROl
            GO TO 60
         END IF
*
!!         IF( TOL.GE.ZERO ) THEN
         IF(
!       (TOL .ge. ZERO)
     + ((TOLh .gt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .ge. ZEROl)))
     &     ) THEN
*
*           If relative accuracy desired,
*           apply convergence criterion forward
*
!!            MU = ABS( D( LL ) )
!<<<  MU = abs(D(LL))
      if (Dh(LL) .lt. 0.0d+0) then
        MUh = -Dh(LL)
        MUl = -Dl(LL)
      elseif (
     +  (Dh(LL) .eq. 0.0d+0).and.
     +  (Dl(LL) .lt. 0.0d+0)) then
        MUh = Dh(LL)
        MUl = -Dl(LL)
      else
        MUh = Dh(LL)
        MUl = Dl(LL)
      endif
!>>>  MU = abs(D(LL))
!!            SMINL = MU
      SMINLh = MUh
      SMINLl = MUl
            DO 100 LLL = LL, M - 1
!!               IF( ABS( E( LLL ) ).LE.TOL*MU ) THEN
!<<<  tmp1 = abs(E(LLL))
      if (Eh(LLL) .lt. 0.0d+0) then
        tmp1h = -Eh(LLL)
        tmp1l = -El(LLL)
      elseif (
     +  (Eh(LLL) .eq. 0.0d+0).and.
     +  (El(LLL) .lt. 0.0d+0)) then
        tmp1h = Eh(LLL)
        tmp1l = -El(LLL)
      else
        tmp1h = Eh(LLL)
        tmp1l = El(LLL)
      endif
!>>>  tmp1 = abs(E(LLL))
!<<<  tmp2 = TOL * MU
      T_ch = TOLh * MUh
      T_u1 = 134217729.0 * TOLh
      T_p1 = T_u1
     +         - (T_u1 - TOLh)
      T_p2 = TOLh - T_p1
      T_u1 = 134217729.0 * MUh
      T_r1 = T_u1
     +         - (T_u1 - MUh)
      T_r2 = MUh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLh*MUl
     +         +  TOLl*MUh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL * MU
               IF(
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $           ) THEN
!!                  E( LLL ) = ZERO
      Eh(LLL) = ZEROh
      El(LLL) = ZEROl
                  GO TO 60
               END IF
!!               MU = ABS( D( LLL+1 ) )*( MU / ( MU+ABS( E( LLL ) ) ) )
!<<<  tmp1 = abs(D(LLL+1))
      if (Dh(LLL+1) .lt. 0.0d+0) then
        tmp1h = -Dh(LLL+1)
        tmp1l = -Dl(LLL+1)
      elseif (
     +  (Dh(LLL+1) .eq. 0.0d+0).and.
     +  (Dl(LLL+1) .lt. 0.0d+0)) then
        tmp1h = Dh(LLL+1)
        tmp1l = -Dl(LLL+1)
      else
        tmp1h = Dh(LLL+1)
        tmp1l = Dl(LLL+1)
      endif
!>>>  tmp1 = abs(D(LLL+1))
!<<<  tmp2 = abs(E(LLL))
      if (Eh(LLL) .lt. 0.0d+0) then
        tmp2h = -Eh(LLL)
        tmp2l = -El(LLL)
      elseif (
     +  (Eh(LLL) .eq. 0.0d+0).and.
     +  (El(LLL) .lt. 0.0d+0)) then
        tmp2h = Eh(LLL)
        tmp2l = -El(LLL)
      else
        tmp2h = Eh(LLL)
        tmp2l = El(LLL)
      endif
!>>>  tmp2 = abs(E(LLL))
!<<<  tmp3 = MU + tmp2
      T_p1 = MUh + tmp2h
      T_p2 = T_p1 - MUh
      T_q1 = T_p1
      T_q2 = (MUh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + MUl + tmp2l
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = MU + tmp2
!<<<  tmp4 = MU / tmp3
      T_q1 = MUh / tmp3h
      T_ch = tmp3h * T_q1
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp3l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = MUh - T_ch
      T_r1 = T_s1 - MUh
      T_s2 = (MUh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + MUl
      T_q2 = (T_s1 + T_s2)
     +         / tmp3h
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = MU / tmp3
!<<<  MU = tmp1 * tmp4
      T_ch = tmp1h * tmp4h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp4l
     +         +  tmp1l*tmp4h)
      T_s1 = T_ch + T_cl
      MUl = T_cl - (T_s1 - T_ch)
      MUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  MU = tmp1 * tmp4
!!               SMINL = MIN( SMINL, MU )
               IF(
!       (SMINL .gt. MU)
     + ((SMINLh .gt. MUh) .or.
     +  ((SMINLh .eq. MUh) .and.
     +   (SMINLl .gt. MUl)))
     $           ) THEN
      SMINLh = MUh
      SMINLl = MUl
               END IF
  100       CONTINUE
         END IF
*
      ELSE
*
*        Run convergence test in backward direction
*        First apply standard test to top of matrix
*
!!         IF( ABS( E( LL ) ).LE.ABS( TOL )*ABS( D( LL ) ) .OR.
!!     $       ( TOL.LT.ZERO .AND. ABS( E( LL ) ).LE.THRESH ) ) THEN
!<<<  tmp1 = abs(E(LL))
      if (Eh(LL) .lt. 0.0d+0) then
        tmp1h = -Eh(LL)
        tmp1l = -El(LL)
      elseif (
     +  (Eh(LL) .eq. 0.0d+0).and.
     +  (El(LL) .lt. 0.0d+0)) then
        tmp1h = Eh(LL)
        tmp1l = -El(LL)
      else
        tmp1h = Eh(LL)
        tmp1l = El(LL)
      endif
!>>>  tmp1 = abs(E(LL))
!<<<  tmp2 = abs(TOL)
      if (TOLh .lt. 0.0d+0) then
        tmp2h = -TOLh
        tmp2l = -TOLl
      elseif (
     +  (TOLh .eq. 0.0d+0).and.
     +  (TOLl .lt. 0.0d+0)) then
        tmp2h = TOLh
        tmp2l = -TOLl
      else
        tmp2h = TOLh
        tmp2l = TOLl
      endif
!>>>  tmp2 = abs(TOL)
!<<<  tmp3 = abs(D(LL))
      if (Dh(LL) .lt. 0.0d+0) then
        tmp3h = -Dh(LL)
        tmp3l = -Dl(LL)
      elseif (
     +  (Dh(LL) .eq. 0.0d+0).and.
     +  (Dl(LL) .lt. 0.0d+0)) then
        tmp3h = Dh(LL)
        tmp3l = -Dl(LL)
      else
        tmp3h = Dh(LL)
        tmp3l = Dl(LL)
      endif
!>>>  tmp3 = abs(D(LL))
!<<<  tmp4 = tmp2 * tmp3
      T_ch = tmp2h * tmp3h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp3l
     +         +  tmp2l*tmp3h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp2 * tmp3
!<<<  tmp5 = abs(E(LL))
      if (Eh(LL) .lt. 0.0d+0) then
        tmp5h = -Eh(LL)
        tmp5l = -El(LL)
      elseif (
     +  (Eh(LL) .eq. 0.0d+0).and.
     +  (El(LL) .lt. 0.0d+0)) then
        tmp5h = Eh(LL)
        tmp5l = -El(LL)
      else
        tmp5h = Eh(LL)
        tmp5l = El(LL)
      endif
!>>>  tmp5 = abs(E(LL))
         IF(
!       (tmp1 .le. tmp4)
     + ((tmp1h .lt. tmp4h) .or.
     +  ((tmp1h .eq. tmp4h) .and.
     +   (tmp1l .le. tmp4l)))
     $         .OR. (
!       (TOL .lt. ZERO)
     + ((TOLh .lt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .lt. ZEROl)))
     $         .AND.
!       (tmp5 .le. THRESH)
     + ((tmp5h .lt. THRESHh) .or.
     +  ((tmp5h .eq. THRESHh) .and.
     +   (tmp5l .le. THRESHl)))
     $     ) ) THEN
!!            E( LL ) = ZERO
      Eh(LL) = ZEROh
      El(LL) = ZEROl
            GO TO 60
         END IF
*
!!         IF( TOL.GE.ZERO ) THEN
         IF(
!       (TOL .ge. ZERO)
     + ((TOLh .gt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .ge. ZEROl)))
     $     ) THEN
*
*           If relative accuracy desired,
*           apply convergence criterion backward
*
!!            MU = ABS( D( M ) )
!<<<  MU = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        MUh = -Dh(M)
        MUl = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        MUh = Dh(M)
        MUl = -Dl(M)
      else
        MUh = Dh(M)
        MUl = Dl(M)
      endif
!>>>  MU = abs(D(M))
!!            SMINL = MU
      SMINLh = MUh
      SMINLl = MUl
            DO 110 LLL = M - 1, LL, -1
!!               IF( ABS( E( LLL ) ).LE.TOL*MU ) THEN
!<<<  tmp1 = abs(E(LLL))
      if (Eh(LLL) .lt. 0.0d+0) then
        tmp1h = -Eh(LLL)
        tmp1l = -El(LLL)
      elseif (
     +  (Eh(LLL) .eq. 0.0d+0).and.
     +  (El(LLL) .lt. 0.0d+0)) then
        tmp1h = Eh(LLL)
        tmp1l = -El(LLL)
      else
        tmp1h = Eh(LLL)
        tmp1l = El(LLL)
      endif
!>>>  tmp1 = abs(E(LLL))
!<<<  tmp2 = TOL * MU
      T_ch = TOLh * MUh
      T_u1 = 134217729.0 * TOLh
      T_p1 = T_u1
     +         - (T_u1 - TOLh)
      T_p2 = TOLh - T_p1
      T_u1 = 134217729.0 * MUh
      T_r1 = T_u1
     +         - (T_u1 - MUh)
      T_r2 = MUh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TOLh*MUl
     +         +  TOLl*MUh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = TOL * MU
               IF(
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $           ) THEN
!!                  E( LLL ) = ZERO
      Eh(LLL) = ZEROh
      El(LLL) = ZEROl
                  GO TO 60
               END IF
!!               MU = ABS( D( LLL ) )*( MU / ( MU+ABS( E( LLL ) ) ) )
!<<<  tmp1 = abs(D(LLL))
      if (Dh(LLL) .lt. 0.0d+0) then
        tmp1h = -Dh(LLL)
        tmp1l = -Dl(LLL)
      elseif (
     +  (Dh(LLL) .eq. 0.0d+0).and.
     +  (Dl(LLL) .lt. 0.0d+0)) then
        tmp1h = Dh(LLL)
        tmp1l = -Dl(LLL)
      else
        tmp1h = Dh(LLL)
        tmp1l = Dl(LLL)
      endif
!>>>  tmp1 = abs(D(LLL))
!<<<  tmp2 = abs(E(LLL))
      if (Eh(LLL) .lt. 0.0d+0) then
        tmp2h = -Eh(LLL)
        tmp2l = -El(LLL)
      elseif (
     +  (Eh(LLL) .eq. 0.0d+0).and.
     +  (El(LLL) .lt. 0.0d+0)) then
        tmp2h = Eh(LLL)
        tmp2l = -El(LLL)
      else
        tmp2h = Eh(LLL)
        tmp2l = El(LLL)
      endif
!>>>  tmp2 = abs(E(LLL))
!<<<  tmp3 = MU + tmp2
      T_p1 = MUh + tmp2h
      T_p2 = T_p1 - MUh
      T_q1 = T_p1
      T_q2 = (MUh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + MUl + tmp2l
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = MU + tmp2
!<<<  tmp4 = MU / tmp3
      T_q1 = MUh / tmp3h
      T_ch = tmp3h * T_q1
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp3l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = MUh - T_ch
      T_r1 = T_s1 - MUh
      T_s2 = (MUh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + MUl
      T_q2 = (T_s1 + T_s2)
     +         / tmp3h
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = MU / tmp3
!<<<  MU = tmp1 * tmp4
      T_ch = tmp1h * tmp4h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp4l
     +         +  tmp1l*tmp4h)
      T_s1 = T_ch + T_cl
      MUl = T_cl - (T_s1 - T_ch)
      MUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  MU = tmp1 * tmp4
!!               SMINL = MIN( SMINL, MU )
               IF(
!       (SMINL .gt. MU)
     + ((SMINLh .gt. MUh) .or.
     +  ((SMINLh .eq. MUh) .and.
     +   (SMINLl .gt. MUl)))
     $           ) THEN
      SMINLh = MUh
      SMINLl = MUl
               END IF
  110       CONTINUE
         END IF
      END IF
      OLDLL = LL
      OLDM = M
*
*     Compute shift.  First, test if shifting would ruin relative
*     accuracy, and if so set the shift to zero.
*
!!      IF( TOL.GE.ZERO .AND. N*TOL*( SMINL / SMAX ).LE.
!!     $    MAX( EPS, HNDRTH*TOL ) ) THEN
      tmp1h = N
      tmp1l = 0.0d+0
!<<<  tmp2 = tmp1 * TOL
      T_ch = tmp1h * TOLh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * TOLh
      T_r1 = T_u1
     +         - (T_u1 - TOLh)
      T_r2 = TOLh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*TOLl
     +         +  tmp1l*TOLh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * TOL
!<<<  tmp3 = SMINL / SMAX
      T_q1 = SMINLh / SMAXh
      T_ch = SMAXh * T_q1
      T_u1 = 134217729.0 * SMAXh
      T_p1 = T_u1
     +         - (T_u1 - SMAXh)
      T_p2 = SMAXh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (SMAXl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = SMINLh - T_ch
      T_r1 = T_s1 - SMINLh
      T_s2 = (SMINLh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + SMINLl
      T_q2 = (T_s1 + T_s2)
     +         / SMAXh
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = SMINL / SMAX
!<<<  tmp4 = tmp2 * tmp3
      T_ch = tmp2h * tmp3h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp3l
     +         +  tmp2l*tmp3h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp2 * tmp3
!<<<  tmp5 = HNDRTH * TOL
      T_ch = HNDRTHh * TOLh
      T_u1 = 134217729.0 * HNDRTHh
      T_p1 = T_u1
     +         - (T_u1 - HNDRTHh)
      T_p2 = HNDRTHh - T_p1
      T_u1 = 134217729.0 * TOLh
      T_r1 = T_u1
     +         - (T_u1 - TOLh)
      T_r2 = TOLh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HNDRTHh*TOLl
     +         +  HNDRTHl*TOLh)
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp5 = HNDRTH * TOL
      IF(
!       (EPS .gt. tmp5)
     + ((EPSh .gt. tmp5h) .or.
     +  ((EPSh .eq. tmp5h) .and.
     +   (EPSl .gt. tmp5l)))
     $  ) THEN
      tmp6h = EPSh
      tmp6l = EPSl
      ELSE
      tmp6h = tmp5h
      tmp6l = tmp5l
      END IF
      IF(
!       (TOL .ge. ZERO)
     + ((TOLh .gt. ZEROh) .or.
     +  ((TOLh .eq. ZEROh) .and.
     +   (TOLl .ge. ZEROl)))
     $      .AND.
!       (tmp4 .le. tmp6)
     + ((tmp4h .lt. tmp6h) .or.
     +  ((tmp4h .eq. tmp6h) .and.
     +   (tmp4l .le. tmp6l)))
     $  ) THEN
*
*        Use a zero shift to avoid loss of relative accuracy
*
!!         SHIFT = ZERO
      SHIFTh = ZEROh
      SHIFTl = ZEROl
      ELSE
*
*        Compute the shift from 2-by-2 block at end of matrix
*
         IF( IDIR.EQ.1 ) THEN
!!            SLL = ABS( D( LL ) )
!<<<  SLL = abs(D(LL))
      if (Dh(LL) .lt. 0.0d+0) then
        SLLh = -Dh(LL)
        SLLl = -Dl(LL)
      elseif (
     +  (Dh(LL) .eq. 0.0d+0).and.
     +  (Dl(LL) .lt. 0.0d+0)) then
        SLLh = Dh(LL)
        SLLl = -Dl(LL)
      else
        SLLh = Dh(LL)
        SLLl = Dl(LL)
      endif
!>>>  SLL = abs(D(LL))
!!            CALL DLAS2( D( M-1 ), E( M-1 ), D( M ), SHIFT, R )
            CALL dDLAS2( Dh( M-1 ), Dl( M-1 ), Eh( M-1 ), El( M-1 ),
     $                   Dh( M ), Dl( M ), SHIFTh, SHIFTl, Rh, Rl )
         ELSE
!!            SLL = ABS( D( M ) )
!<<<  SLL = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        SLLh = -Dh(M)
        SLLl = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        SLLh = Dh(M)
        SLLl = -Dl(M)
      else
        SLLh = Dh(M)
        SLLl = Dl(M)
      endif
!>>>  SLL = abs(D(M))
!!            CALL DLAS2( D( LL ), E( LL ), D( LL+1 ), SHIFT, R )
            CALL dDLAS2( Dh( LL ), Dl( LL ), Eh( LL ), El( LL ),
     $                   Dh( LL+1 ), Dl( LL+1 ), SHIFTh, SHIFTl,
     $                   Rh, Rl )
         END IF
*
*        Test if shift negligible, and if so set to zero
*
!!         IF( SLL.GT.ZERO ) THEN
         IF(
!       (SLL .gt. ZERO)
     + ((SLLh .gt. ZEROh) .or.
     +  ((SLLh .eq. ZEROh) .and.
     +   (SLLl .gt. ZEROl)))
     $     ) THEN
!!            IF( ( SHIFT / SLL )**2.LT.EPS )
!!     $         SHIFT = ZERO
!<<<  tmp1 = SHIFT / SLL
      T_q1 = SHIFTh / SLLh
      T_ch = SLLh * T_q1
      T_u1 = 134217729.0 * SLLh
      T_p1 = T_u1
     +         - (T_u1 - SLLh)
      T_p2 = SLLh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (SLLl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = SHIFTh - T_ch
      T_r1 = T_s1 - SHIFTh
      T_s2 = (SHIFTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + SHIFTl
      T_q2 = (T_s1 + T_s2)
     +         / SLLh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = SHIFT / SLL
!<<<  tmp2 = tmp1 * tmp1
      T_ch = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp1l
     +         +  tmp1l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
            IF(
!       (tmp2 .lt. EPS)
     + ((tmp2h .lt. EPSh) .or.
     +  ((tmp2h .eq. EPSh) .and.
     +   (tmp2l .lt. EPSl)))
     $        ) THEN
      SHIFTh = ZEROh
      SHIFTl = ZEROl
            END IF
         END IF
      END IF
*
*     Increment iteration count
*
      ITER = ITER + M - LL
*
*     If SHIFT = 0, do simplified QR iteration
*
!!      IF( SHIFT.EQ.ZERO ) THEN
      IF(
!       (SHIFT .eq. ZERO)
     + ((SHIFTh .eq. ZEROh) .and.
     +  (SHIFTl .eq. ZEROl))
     $  ) THEN
         IF( IDIR.EQ.1 ) THEN
*
*           Chase bulge from top to bottom
*           Save cosines and sines for later singular vector updates
*
!!            CS = ONE
      CSh = ONEh
      CSl = ONEl
!!            OLDCS = ONE
      OLDCSh = ONEh
      OLDCSl = ONEl
            DO 120 I = LL, M - 1
!!               CALL DLARTG( D( I )*CS, E( I ), CS, SN, R )
!<<<  tmp1 = D(I) * CS
      T_ch = Dh(I) * CSh
      T_u1 = 134217729.0 * Dh(I)
      T_p1 = T_u1
     +         - (T_u1 - Dh(I))
      T_p2 = Dh(I) - T_p1
      T_u1 = 134217729.0 * CSh
      T_r1 = T_u1
     +         - (T_u1 - CSh)
      T_r2 = CSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh(I)*CSl
     +         +  Dl(I)*CSh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = D(I) * CS
               CALL dDLARTG( tmp1h, tmp1l, Eh( I ), El( I ), CSh, CSl,
     $                       SNh, SNl, Rh, Rl )
!!               IF( I.GT.LL )
!!     $            E( I-1 ) = OLDSN*R
               IF( I.GT.LL ) THEN
!<<<  tmp1 = OLDSN * R
      T_ch = OLDSNh * Rh
      T_u1 = 134217729.0 * OLDSNh
      T_p1 = T_u1
     +         - (T_u1 - OLDSNh)
      T_p2 = OLDSNh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (OLDSNh*Rl
     +         +  OLDSNl*Rh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = OLDSN * R
      Eh(I-1) = tmp1h
      El(I-1) = tmp1l
               END IF
!!               CALL DLARTG( OLDCS*R, D( I+1 )*SN, OLDCS, OLDSN, D( I ) )
!<<<  tmp1 = OLDCS * R
      T_ch = OLDCSh * Rh
      T_u1 = 134217729.0 * OLDCSh
      T_p1 = T_u1
     +         - (T_u1 - OLDCSh)
      T_p2 = OLDCSh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (OLDCSh*Rl
     +         +  OLDCSl*Rh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = OLDCS * R
!<<<  tmp2 = D(I+1) * SN
      T_ch = Dh(I+1) * SNh
      T_u1 = 134217729.0 * Dh(I+1)
      T_p1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_p2 = Dh(I+1) - T_p1
      T_u1 = 134217729.0 * SNh
      T_r1 = T_u1
     +         - (T_u1 - SNh)
      T_r2 = SNh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh(I+1)*SNl
     +         +  Dl(I+1)*SNh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = D(I+1) * SN
               CALL dDLARTG( tmp1h, tmp1l, tmp2h, tmp2l,
     $                       OLDCSh, OLDCSl, OLDSNh, OLDSNl,
     $                       Dh( I ), Dl( I ) )
!!               WORK( I-LL+1 ) = CS
      WORKh(I-LL+1) = CSh
      WORKl(I-LL+1) = CSl
!!               WORK( I-LL+1+NM1 ) = SN
      WORKh(I-LL+1+NM1) = SNh
      WORKl(I-LL+1+NM1) = SNl
!!               WORK( I-LL+1+NM12 ) = OLDCS
      WORKh(I-LL+1+NM12) = OLDCSh
      WORKl(I-LL+1+NM12) = OLDCSl
!!               WORK( I-LL+1+NM13 ) = OLDSN
      WORKh(I-LL+1+NM13) = OLDSNh
      WORKl(I-LL+1+NM13) = OLDSNl
  120       CONTINUE
!!            H = D( M )*CS
!<<<  H = D(M) * CS
      T_ch = Dh(M) * CSh
      T_u1 = 134217729.0 * Dh(M)
      T_p1 = T_u1
     +         - (T_u1 - Dh(M))
      T_p2 = Dh(M) - T_p1
      T_u1 = 134217729.0 * CSh
      T_r1 = T_u1
     +         - (T_u1 - CSh)
      T_r2 = CSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh(M)*CSl
     +         +  Dl(M)*CSh)
      T_s1 = T_ch + T_cl
      Hl = T_cl - (T_s1 - T_ch)
      Hh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  H = D(M) * CS
!!            D( M ) = H*OLDCS
!<<<  D(M) = H * OLDCS
      T_ch = Hh * OLDCSh
      T_u1 = 134217729.0 * Hh
      T_p1 = T_u1
     +         - (T_u1 - Hh)
      T_p2 = Hh - T_p1
      T_u1 = 134217729.0 * OLDCSh
      T_r1 = T_u1
     +         - (T_u1 - OLDCSh)
      T_r2 = OLDCSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Hh*OLDCSl
     +         +  Hl*OLDCSh)
      T_s1 = T_ch + T_cl
      Dl(M) = T_cl - (T_s1 - T_ch)
      Dh(M) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  D(M) = H * OLDCS
!!            E( M-1 ) = H*OLDSN
!<<<  E(M-1) = H * OLDSN
      T_ch = Hh * OLDSNh
      T_u1 = 134217729.0 * Hh
      T_p1 = T_u1
     +         - (T_u1 - Hh)
      T_p2 = Hh - T_p1
      T_u1 = 134217729.0 * OLDSNh
      T_r1 = T_u1
     +         - (T_u1 - OLDSNh)
      T_r2 = OLDSNh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Hh*OLDSNl
     +         +  Hl*OLDSNh)
      T_s1 = T_ch + T_cl
      El(M-1) = T_cl - (T_s1 - T_ch)
      Eh(M-1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(M-1) = H * OLDSN
*
*           Update singular vectors
*
!!            IF( NCVT.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'F', M-LL+1, NCVT, WORK( 1 ),
!!     $                     WORK( N ), VT( LL, 1 ), LDVT )
            IF( NCVT.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'F', M-LL+1, NCVT,
     $                      WORKh( 1 ), WORKl( 1 ),
     $                      WORKh( N ), WORKl( N ),
     $                      VTh( LL, 1 ), VTl( LL, 1 ), LDVT )
!!            IF( NRU.GT.0 )
!!     $         CALL DLASR( 'R', 'V', 'F', NRU, M-LL+1, WORK( NM12+1 ),
!!     $                     WORK( NM13+1 ), U( 1, LL ), LDU )
            IF( NRU.GT.0 )
     $         CALL dDLASR( 'R', 'V', 'F', NRU, M-LL+1,
     $                      WORKh( NM12+1 ), WORKl( NM12+1 ),
     $                      WORKh( NM13+1 ), WORKl( NM13+1 ),
     $                      Uh( 1, LL ), Ul( 1, LL ), LDU )
!!            IF( NCC.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'F', M-LL+1, NCC, WORK( NM12+1 ),
!!     $                     WORK( NM13+1 ), C( LL, 1 ), LDC )
            IF( NCC.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'F', M-LL+1, NCC,
     $                      WORKh( NM12+1 ), WORKl( NM12+1 ),
     $                      WORKh( NM13+1 ), WORKl( NM13+1 ),
     $                      Ch( LL, 1 ), Cl( LL, 1 ), LDC )
*
*           Test convergence
*
!!            IF( ABS( E( M-1 ) ).LE.THRESH )
!!     $         E( M-1 ) = ZERO
!<<<  tmp1 = abs(E(M-1))
      if (Eh(M-1) .lt. 0.0d+0) then
        tmp1h = -Eh(M-1)
        tmp1l = -El(M-1)
      elseif (
     +  (Eh(M-1) .eq. 0.0d+0).and.
     +  (El(M-1) .lt. 0.0d+0)) then
        tmp1h = Eh(M-1)
        tmp1l = -El(M-1)
      else
        tmp1h = Eh(M-1)
        tmp1l = El(M-1)
      endif
!>>>  tmp1 = abs(E(M-1))
            IF(
!       (tmp1 .le. THRESH)
     + ((tmp1h .lt. THRESHh) .or.
     +  ((tmp1h .eq. THRESHh) .and.
     +   (tmp1l .le. THRESHl)))
     &        ) THEN
      Eh(M-1) = ZEROh
      El(M-1) = ZEROl
            END IF
*
         ELSE
*
*           Chase bulge from bottom to top
*           Save cosines and sines for later singular vector updates
*
!!            CS = ONE
      CSh = ONEh
      CSl = ONEl
!!            OLDCS = ONE
      OLDCSh = ONEh
      OLDCSl = ONEl
            DO 130 I = M, LL + 1, -1
!!               CALL DLARTG( D( I )*CS, E( I-1 ), CS, SN, R )
!<<<  tmp1 = D(I) * CS
      T_ch = Dh(I) * CSh
      T_u1 = 134217729.0 * Dh(I)
      T_p1 = T_u1
     +         - (T_u1 - Dh(I))
      T_p2 = Dh(I) - T_p1
      T_u1 = 134217729.0 * CSh
      T_r1 = T_u1
     +         - (T_u1 - CSh)
      T_r2 = CSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh(I)*CSl
     +         +  Dl(I)*CSh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = D(I) * CS
               CALL dDLARTG( tmp1h, tmp1l, Eh( I-1 ), El( I-1 ),
     $                       CSh, CSl, SNh, SNl, Rh, Rl )
!!               IF( I.LT.M )
!!     $            E( I ) = OLDSN*R
               IF( I.LT.M ) THEN
!<<<  E(I) = OLDSN * R
      T_ch = OLDSNh * Rh
      T_u1 = 134217729.0 * OLDSNh
      T_p1 = T_u1
     +         - (T_u1 - OLDSNh)
      T_p2 = OLDSNh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (OLDSNh*Rl
     +         +  OLDSNl*Rh)
      T_s1 = T_ch + T_cl
      El(I) = T_cl - (T_s1 - T_ch)
      Eh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(I) = OLDSN * R
               END IF
!!               CALL DLARTG( OLDCS*R, D( I-1 )*SN, OLDCS, OLDSN, D( I ) )
!<<<  tmp1 = OLDCS * R
      T_ch = OLDCSh * Rh
      T_u1 = 134217729.0 * OLDCSh
      T_p1 = T_u1
     +         - (T_u1 - OLDCSh)
      T_p2 = OLDCSh - T_p1
      T_u1 = 134217729.0 * Rh
      T_r1 = T_u1
     +         - (T_u1 - Rh)
      T_r2 = Rh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (OLDCSh*Rl
     +         +  OLDCSl*Rh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = OLDCS * R
!<<<  tmp2 = D(I-1) * SN
      T_ch = Dh(I-1) * SNh
      T_u1 = 134217729.0 * Dh(I-1)
      T_p1 = T_u1
     +         - (T_u1 - Dh(I-1))
      T_p2 = Dh(I-1) - T_p1
      T_u1 = 134217729.0 * SNh
      T_r1 = T_u1
     +         - (T_u1 - SNh)
      T_r2 = SNh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh(I-1)*SNl
     +         +  Dl(I-1)*SNh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = D(I-1) * SN
               CALL dDLARTG( tmp1h, tmp1l, tmp2h, tmp2l,
     $                       OLDCSh, OLDCSl, OLDSNh, OLDSNl,
     $                       Dh( I ), Dl( I ) )
!!               WORK( I-LL ) = CS
      WORKh(I-LL) = CSh
      WORKl(I-LL) = CSl
!!               WORK( I-LL+NM1 ) = -SN
!<<<  WORK(I-LL+NM1) = -SN
      WORKh(I-LL+NM1) = -SNh
      WORKl(I-LL+NM1) = -SNl
!>>>  WORK(I-LL+NM1) = -SN
!!               WORK( I-LL+NM12 ) = OLDCS
      WORKh(I-LL+NM12) = OLDCSh
      WORKl(I-LL+NM12) = OLDCSl
!!               WORK( I-LL+NM13 ) = -OLDSN
!<<<  WORK(I-LL+NM13) = -OLDSN
      WORKh(I-LL+NM13) = -OLDSNh
      WORKl(I-LL+NM13) = -OLDSNl
!>>>  WORK(I-LL+NM13) = -OLDSN
  130       CONTINUE
!!            H = D( LL )*CS
!<<<  H = D(LL) * CS
      T_ch = Dh(LL) * CSh
      T_u1 = 134217729.0 * Dh(LL)
      T_p1 = T_u1
     +         - (T_u1 - Dh(LL))
      T_p2 = Dh(LL) - T_p1
      T_u1 = 134217729.0 * CSh
      T_r1 = T_u1
     +         - (T_u1 - CSh)
      T_r2 = CSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Dh(LL)*CSl
     +         +  Dl(LL)*CSh)
      T_s1 = T_ch + T_cl
      Hl = T_cl - (T_s1 - T_ch)
      Hh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  H = D(LL) * CS
!!            D( LL ) = H*OLDCS
!<<<  D(LL) = H * OLDCS
      T_ch = Hh * OLDCSh
      T_u1 = 134217729.0 * Hh
      T_p1 = T_u1
     +         - (T_u1 - Hh)
      T_p2 = Hh - T_p1
      T_u1 = 134217729.0 * OLDCSh
      T_r1 = T_u1
     +         - (T_u1 - OLDCSh)
      T_r2 = OLDCSh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Hh*OLDCSl
     +         +  Hl*OLDCSh)
      T_s1 = T_ch + T_cl
      Dl(LL) = T_cl - (T_s1 - T_ch)
      Dh(LL) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  D(LL) = H * OLDCS
!!            E( LL ) = H*OLDSN
!<<<  E(LL) = H * OLDSN
      T_ch = Hh * OLDSNh
      T_u1 = 134217729.0 * Hh
      T_p1 = T_u1
     +         - (T_u1 - Hh)
      T_p2 = Hh - T_p1
      T_u1 = 134217729.0 * OLDSNh
      T_r1 = T_u1
     +         - (T_u1 - OLDSNh)
      T_r2 = OLDSNh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Hh*OLDSNl
     +         +  Hl*OLDSNh)
      T_s1 = T_ch + T_cl
      El(LL) = T_cl - (T_s1 - T_ch)
      Eh(LL) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(LL) = H * OLDSN
*
*           Update singular vectors
*
!!            IF( NCVT.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'B', M-LL+1, NCVT, WORK( NM12+1 ),
!!     $                     WORK( NM13+1 ), VT( LL, 1 ), LDVT )
            IF( NCVT.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'B', M-LL+1, NCVT,
     $                      WORKh( NM12+1 ), WORKl( NM12+1 ),
     $                      WORKh( NM13+1 ), WORKl( NM13+1 ),
     $                      VTh( LL, 1 ), VTl( LL, 1 ), LDVT )
!!            IF( NRU.GT.0 )
!!     $         CALL DLASR( 'R', 'V', 'B', NRU, M-LL+1, WORK( 1 ),
!!     $                     WORK( N ), U( 1, LL ), LDU )
            IF( NRU.GT.0 )
     $         CALL dDLASR( 'R', 'V', 'B', NRU, M-LL+1,
     $                      WORKh( 1 ), WORKl( 1 ),
     $                      WORKh( N ), WORKl( N ),
     $                      Uh( 1, LL ), Ul( 1, LL ), LDU )
!!            IF( NCC.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'B', M-LL+1, NCC, WORK( 1 ),
!!     $                     WORK( N ), C( LL, 1 ), LDC )
            IF( NCC.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'B', M-LL+1, NCC,
     $                      WORKh( 1 ), WORKl( 1 ),
     $                      WORKh( N ), WORKl( N ),
     $                      Ch( LL, 1 ), Cl( LL, 1 ), LDC )
*
*           Test convergence
*
!!            IF( ABS( E( LL ) ).LE.THRESH )
!!     $         E( LL ) = ZERO
!<<<  tmp1 = abs(E(LL))
      if (Eh(LL) .lt. 0.0d+0) then
        tmp1h = -Eh(LL)
        tmp1l = -El(LL)
      elseif (
     +  (Eh(LL) .eq. 0.0d+0).and.
     +  (El(LL) .lt. 0.0d+0)) then
        tmp1h = Eh(LL)
        tmp1l = -El(LL)
      else
        tmp1h = Eh(LL)
        tmp1l = El(LL)
      endif
!>>>  tmp1 = abs(E(LL))
            IF(
!       (tmp1 .le. THRESH)
     + ((tmp1h .lt. THRESHh) .or.
     +  ((tmp1h .eq. THRESHh) .and.
     +   (tmp1l .le. THRESHl)))
     $        ) THEN
      Eh(LL) = ZEROh
      El(LL) = ZEROl
            END IF
         END IF
      ELSE
*
*        Use nonzero shift
*
         IF( IDIR.EQ.1 ) THEN
*
*           Chase bulge from top to bottom
*           Save cosines and sines for later singular vector updates
*
!!            F = ( ABS( D( LL ) )-SHIFT )*
!!     $          ( SIGN( ONE, D( LL ) )+SHIFT / D( LL ) )
!<<<  tmp1 = abs(D(LL))
      if (Dh(LL) .lt. 0.0d+0) then
        tmp1h = -Dh(LL)
        tmp1l = -Dl(LL)
      elseif (
     +  (Dh(LL) .eq. 0.0d+0).and.
     +  (Dl(LL) .lt. 0.0d+0)) then
        tmp1h = Dh(LL)
        tmp1l = -Dl(LL)
      else
        tmp1h = Dh(LL)
        tmp1l = Dl(LL)
      endif
!>>>  tmp1 = abs(D(LL))
!<<<  tmp2 = tmp1 - SHIFT
      T_p1 = tmp1h - SHIFTh
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (SHIFTh + T_p2)
      T_cl = T_cl + tmp1l - SHIFTl
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = tmp1 - SHIFT
            IF(
!       (D(LL) .ge. 0.0)
     + ((Dh(LL) .gt. 0.0d+0) .or.
     +  ((Dh(LL) .eq. 0.0d+0) .and.
     +   (Dl(LL) .ge. 0.0d+0)))
     $        ) THEN
      tmp3h = ONEh
      tmp3l = ONEl
            ELSE
!<<<  tmp3 = -ONE
      tmp3h = -ONEh
      tmp3l = -ONEl
!>>>  tmp3 = -ONE
            END IF
!<<<  tmp4 = SHIFT / D(LL)
      T_q1 = SHIFTh / Dh(LL)
      T_ch = Dh(LL) * T_q1
      T_u1 = 134217729.0 * Dh(LL)
      T_p1 = T_u1
     +         - (T_u1 - Dh(LL))
      T_p2 = Dh(LL) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Dl(LL) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = SHIFTh - T_ch
      T_r1 = T_s1 - SHIFTh
      T_s2 = (SHIFTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + SHIFTl
      T_q2 = (T_s1 + T_s2)
     +         / Dh(LL)
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = SHIFT / D(LL)
!<<<  tmp5 = tmp3 + tmp4
      T_p1 = tmp3h + tmp4h
      T_p2 = T_p1 - tmp3h
      T_q1 = T_p1
      T_q2 = (tmp3h - (T_p1 - T_p2))
     +         + (tmp4h - T_p2)
      T_q2 = T_q2 + tmp3l + tmp4l
      tmp5h = T_q1 + T_q2
      tmp5l = T_q2 - (tmp5h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = tmp3 + tmp4
!<<<  F = tmp2 * tmp5
      T_ch = tmp2h * tmp5h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp5l
     +         +  tmp2l*tmp5h)
      T_s1 = T_ch + T_cl
      Fl = T_cl - (T_s1 - T_ch)
      Fh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  F = tmp2 * tmp5
!!            G = E( LL )
      Gh = Eh(LL)
      Gl = El(LL)
            DO 140 I = LL, M - 1
!!               CALL DLARTG( F, G, COSR, SINR, R )
               CALL dDLARTG( Fh, Fl, Gh, Gl, COSRh, COSRl,
     $                       SINRh, SINRl, Rh, Rl )
!!               IF( I.GT.LL )
!!     $            E( I-1 ) = R
               IF( I.GT.LL ) THEN
      Eh(I-1) = Rh
      El(I-1) = Rl
               END IF
!!               F = COSR*D( I ) + SINR*E( I )
!<<<  tmp1 = COSR * D(I)
      T_ch = COSRh * Dh(I)
      T_u1 = 134217729.0 * COSRh
      T_p1 = T_u1
     +         - (T_u1 - COSRh)
      T_p2 = COSRh - T_p1
      T_u1 = 134217729.0 * Dh(I)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I))
      T_r2 = Dh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSRh*Dl(I)
     +         +  COSRl*Dh(I))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSR * D(I)
!<<<  tmp2 = SINR * E(I)
      T_ch = SINRh * Eh(I)
      T_u1 = 134217729.0 * SINRh
      T_p1 = T_u1
     +         - (T_u1 - SINRh)
      T_p2 = SINRh - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINRh*El(I)
     +         +  SINRl*Eh(I))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINR * E(I)
!<<<  F = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + tmp1l + tmp2l
      Fh = T_q1 + T_q2
      Fl = T_q2 - (Fh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  F = tmp1 + tmp2
!!               E( I ) = COSR*E( I ) - SINR*D( I )
!<<<  tmp1 = COSR * E(I)
      T_ch = COSRh * Eh(I)
      T_u1 = 134217729.0 * COSRh
      T_p1 = T_u1
     +         - (T_u1 - COSRh)
      T_p2 = COSRh - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSRh*El(I)
     +         +  COSRl*Eh(I))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSR * E(I)
!<<<  tmp2 = SINR * D(I)
      T_ch = SINRh * Dh(I)
      T_u1 = 134217729.0 * SINRh
      T_p1 = T_u1
     +         - (T_u1 - SINRh)
      T_p2 = SINRh - T_p1
      T_u1 = 134217729.0 * Dh(I)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I))
      T_r2 = Dh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINRh*Dl(I)
     +         +  SINRl*Dh(I))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINR * D(I)
!<<<  E(I) = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + tmp1l - tmp2l
      T_s1 = T_ch + T_cl
      El(I) = T_cl - (T_s1 - T_ch)
      Eh(I) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  E(I) = tmp1 - tmp2
!!               G = SINR*D( I+1 )
!<<<  G = SINR * D(I+1)
      T_ch = SINRh * Dh(I+1)
      T_u1 = 134217729.0 * SINRh
      T_p1 = T_u1
     +         - (T_u1 - SINRh)
      T_p2 = SINRh - T_p1
      T_u1 = 134217729.0 * Dh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_r2 = Dh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINRh*Dl(I+1)
     +         +  SINRl*Dh(I+1))
      T_s1 = T_ch + T_cl
      Gl = T_cl - (T_s1 - T_ch)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G = SINR * D(I+1)
!!               D( I+1 ) = COSR*D( I+1 )
!<<<  D(I+1) = COSR * D(I+1)
      T_ch = COSRh * Dh(I+1)
      T_u1 = 134217729.0 * COSRh
      T_p1 = T_u1
     +         - (T_u1 - COSRh)
      T_p2 = COSRh - T_p1
      T_u1 = 134217729.0 * Dh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_r2 = Dh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSRh*Dl(I+1)
     +         +  COSRl*Dh(I+1))
      T_s1 = T_ch + T_cl
      Dl(I+1) = T_cl - (T_s1 - T_ch)
      Dh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  D(I+1) = COSR * D(I+1)
!!               CALL DLARTG( F, G, COSL, SINL, R )
               CALL dDLARTG( Fh, Fl, Gh, Gl, COSLh, COSLl,
     $                       SINLh, SINLl, Rh, Rl )
!!               D( I ) = R
      Dh(I) = Rh
      Dl(I) = Rl
!!               F = COSL*E( I ) + SINL*D( I+1 )
!<<<  tmp1 = COSL * E(I)
      T_ch = COSLh * Eh(I)
      T_u1 = 134217729.0 * COSLh
      T_p1 = T_u1
     +         - (T_u1 - COSLh)
      T_p2 = COSLh - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSLh*El(I)
     +         +  COSLl*Eh(I))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSL * E(I)
!<<<  tmp2 = SINL * D(I+1)
      T_ch = SINLh * Dh(I+1)
      T_u1 = 134217729.0 * SINLh
      T_p1 = T_u1
     +         - (T_u1 - SINLh)
      T_p2 = SINLh - T_p1
      T_u1 = 134217729.0 * Dh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_r2 = Dh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINLh*Dl(I+1)
     +         +  SINLl*Dh(I+1))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINL * D(I+1)
!<<<  F = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + tmp1l + tmp2l
      Fh = T_q1 + T_q2
      Fl = T_q2 - (Fh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  F = tmp1 + tmp2
!!               D( I+1 ) = COSL*D( I+1 ) - SINL*E( I )
!<<<  tmp1 = COSL * D(I+1)
      T_ch = COSLh * Dh(I+1)
      T_u1 = 134217729.0 * COSLh
      T_p1 = T_u1
     +         - (T_u1 - COSLh)
      T_p2 = COSLh - T_p1
      T_u1 = 134217729.0 * Dh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I+1))
      T_r2 = Dh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSLh*Dl(I+1)
     +         +  COSLl*Dh(I+1))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSL * D(I+1)
!<<<  tmp2 = SINL * E(I)
      T_ch = SINLh * Eh(I)
      T_u1 = 134217729.0 * SINLh
      T_p1 = T_u1
     +         - (T_u1 - SINLh)
      T_p2 = SINLh - T_p1
      T_u1 = 134217729.0 * Eh(I)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I))
      T_r2 = Eh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINLh*El(I)
     +         +  SINLl*Eh(I))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINL * E(I)
!<<<  D(I+1) = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + tmp1l - tmp2l
      T_s1 = T_ch + T_cl
      Dl(I+1) = T_cl - (T_s1 - T_ch)
      Dh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D(I+1) = tmp1 - tmp2
               IF( I.LT.M-1 ) THEN
!!                  G = SINL*E( I+1 )
!<<<  G = SINL * E(I+1)
      T_ch = SINLh * Eh(I+1)
      T_u1 = 134217729.0 * SINLh
      T_p1 = T_u1
     +         - (T_u1 - SINLh)
      T_p2 = SINLh - T_p1
      T_u1 = 134217729.0 * Eh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I+1))
      T_r2 = Eh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINLh*El(I+1)
     +         +  SINLl*Eh(I+1))
      T_s1 = T_ch + T_cl
      Gl = T_cl - (T_s1 - T_ch)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G = SINL * E(I+1)
!!                  E( I+1 ) = COSL*E( I+1 )
!<<<  E(I+1) = COSL * E(I+1)
      T_ch = COSLh * Eh(I+1)
      T_u1 = 134217729.0 * COSLh
      T_p1 = T_u1
     +         - (T_u1 - COSLh)
      T_p2 = COSLh - T_p1
      T_u1 = 134217729.0 * Eh(I+1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I+1))
      T_r2 = Eh(I+1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSLh*El(I+1)
     +         +  COSLl*Eh(I+1))
      T_s1 = T_ch + T_cl
      El(I+1) = T_cl - (T_s1 - T_ch)
      Eh(I+1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(I+1) = COSL * E(I+1)
               END IF
!!               WORK( I-LL+1 ) = COSR
      WORKh(I-LL+1) = COSRh
      WORKl(I-LL+1) = COSRl
!!               WORK( I-LL+1+NM1 ) = SINR
      WORKh(I-LL+1+NM1) = SINRh
      WORKl(I-LL+1+NM1) = SINRl
!!               WORK( I-LL+1+NM12 ) = COSL
      WORKh(I-LL+1+NM12) = COSLh
      WORKl(I-LL+1+NM12) = COSLl
!!               WORK( I-LL+1+NM13 ) = SINL
      WORKh(I-LL+1+NM13) = SINLh
      WORKl(I-LL+1+NM13) = SINLl
  140       CONTINUE
!!            E( M-1 ) = F
      Eh(M-1) = Fh
      El(M-1) = Fl
*
*           Update singular vectors
*
!!            IF( NCVT.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'F', M-LL+1, NCVT, WORK( 1 ),
!!     $                     WORK( N ), VT( LL, 1 ), LDVT )
            IF( NCVT.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'F', M-LL+1, NCVT,
     $                      WORKh( 1 ), WORKl( 1 ),
     $                      WORKh( N ), WORKl( N ),
     $                      VTh( LL, 1 ), VTl( LL, 1 ), LDVT )
!!            IF( NRU.GT.0 )
!!     $         CALL DLASR( 'R', 'V', 'F', NRU, M-LL+1, WORK( NM12+1 ),
!!     $                     WORK( NM13+1 ), U( 1, LL ), LDU )
            IF( NRU.GT.0 )
     $         CALL dDLASR( 'R', 'V', 'F', NRU, M-LL+1,
     $                      WORKh( NM12+1 ), WORKl( NM12+1 ),
     $                      WORKh( NM13+1 ), WORKl( NM13+1 ),
     $                      Uh( 1, LL ), Ul( 1, LL ), LDU )
!!            IF( NCC.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'F', M-LL+1, NCC, WORK( NM12+1 ),
!!     $                     WORK( NM13+1 ), C( LL, 1 ), LDC )
            IF( NCC.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'F', M-LL+1, NCC,
     $                      WORKh( NM12+1 ), WORKl( NM12+1 ),
     $                      WORKh( NM13+1 ), WORKl( NM13+1 ),
     $                      Ch( LL, 1 ), Cl( LL, 1 ), LDC )
*
*           Test convergence
*
!!            IF( ABS( E( M-1 ) ).LE.THRESH )
!!     $         E( M-1 ) = ZERO
!<<<  tmp1 = abs(E(M-1))
      if (Eh(M-1) .lt. 0.0d+0) then
        tmp1h = -Eh(M-1)
        tmp1l = -El(M-1)
      elseif (
     +  (Eh(M-1) .eq. 0.0d+0).and.
     +  (El(M-1) .lt. 0.0d+0)) then
        tmp1h = Eh(M-1)
        tmp1l = -El(M-1)
      else
        tmp1h = Eh(M-1)
        tmp1l = El(M-1)
      endif
!>>>  tmp1 = abs(E(M-1))
            IF(
!       (tmp1 .le. THRESH)
     + ((tmp1h .lt. THRESHh) .or.
     +  ((tmp1h .eq. THRESHh) .and.
     +   (tmp1l .le. THRESHl)))
     $        ) THEN
      Eh(M-1) = ZEROh
      El(M-1) = ZEROl
            END IF
*
         ELSE
*
*           Chase bulge from bottom to top
*           Save cosines and sines for later singular vector updates
*
!!            F = ( ABS( D( M ) )-SHIFT )*( SIGN( ONE, D( M ) )+SHIFT /
!!     $          D( M ) )
!<<<  tmp1 = abs(D(M))
      if (Dh(M) .lt. 0.0d+0) then
        tmp1h = -Dh(M)
        tmp1l = -Dl(M)
      elseif (
     +  (Dh(M) .eq. 0.0d+0).and.
     +  (Dl(M) .lt. 0.0d+0)) then
        tmp1h = Dh(M)
        tmp1l = -Dl(M)
      else
        tmp1h = Dh(M)
        tmp1l = Dl(M)
      endif
!>>>  tmp1 = abs(D(M))
!<<<  tmp2 = tmp1 - SHIFT
      T_p1 = tmp1h - SHIFTh
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (SHIFTh + T_p2)
      T_cl = T_cl + tmp1l - SHIFTl
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = tmp1 - SHIFT
            IF(
!       (D(M) .ge. 0.0)
     + ((Dh(M) .gt. 0.0d+0) .or.
     +  ((Dh(M) .eq. 0.0d+0) .and.
     +   (Dl(M) .ge. 0.0d+0)))
     &        ) THEN
      tmp3h = ONEh
      tmp3l = ONEl
            ELSE
!<<<  tmp3 = -ONE
      tmp3h = -ONEh
      tmp3l = -ONEl
!>>>  tmp3 = -ONE
            ENDIF
!<<<  tmp4 = SHIFT / D(M)
      T_q1 = SHIFTh / Dh(M)
      T_ch = Dh(M) * T_q1
      T_u1 = 134217729.0 * Dh(M)
      T_p1 = T_u1
     +         - (T_u1 - Dh(M))
      T_p2 = Dh(M) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Dl(M) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = SHIFTh - T_ch
      T_r1 = T_s1 - SHIFTh
      T_s2 = (SHIFTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + SHIFTl
      T_q2 = (T_s1 + T_s2)
     +         / Dh(M)
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = SHIFT / D(M)
!<<<  tmp5 = tmp3 + tmp4
      T_p1 = tmp3h + tmp4h
      T_p2 = T_p1 - tmp3h
      T_q1 = T_p1
      T_q2 = (tmp3h - (T_p1 - T_p2))
     +         + (tmp4h - T_p2)
      T_q2 = T_q2 + tmp3l + tmp4l
      tmp5h = T_q1 + T_q2
      tmp5l = T_q2 - (tmp5h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = tmp3 + tmp4
!<<<  F = tmp2 * tmp5
      T_ch = tmp2h * tmp5h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*tmp5l
     +         +  tmp2l*tmp5h)
      T_s1 = T_ch + T_cl
      Fl = T_cl - (T_s1 - T_ch)
      Fh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  F = tmp2 * tmp5
!!            G = E( M-1 )
      Gh = Eh(M-1)
      Gl = El(M-1)
            DO 150 I = M, LL + 1, -1
!!               CALL DLARTG( F, G, COSR, SINR, R )
               CALL dDLARTG( Fh, Fl, Gh, Gl, COSRh, COSRl,
     $                       SINRh, SINRl, Rh, Rl )
!!               IF( I.LT.M )
!!     $            E( I ) = R
               IF( I.LT.M ) THEN
      Eh(I) = Rh
      El(I) = Rl
               END IF
!!               F = COSR*D( I ) + SINR*E( I-1 )
!<<<  tmp1 = COSR * D(I)
      T_ch = COSRh * Dh(I)
      T_u1 = 134217729.0 * COSRh
      T_p1 = T_u1
     +         - (T_u1 - COSRh)
      T_p2 = COSRh - T_p1
      T_u1 = 134217729.0 * Dh(I)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I))
      T_r2 = Dh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSRh*Dl(I)
     +         +  COSRl*Dh(I))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSR * D(I)
!<<<  tmp2 = SINR * E(I-1)
      T_ch = SINRh * Eh(I-1)
      T_u1 = 134217729.0 * SINRh
      T_p1 = T_u1
     +         - (T_u1 - SINRh)
      T_p2 = SINRh - T_p1
      T_u1 = 134217729.0 * Eh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I-1))
      T_r2 = Eh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINRh*El(I-1)
     +         +  SINRl*Eh(I-1))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINR * E(I-1)
!<<<  F = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + tmp1l + tmp2l
      Fh = T_q1 + T_q2
      Fl = T_q2 - (Fh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  F = tmp1 + tmp2
!!               E( I-1 ) = COSR*E( I-1 ) - SINR*D( I )
!<<<  tmp1 = COSR * E(I-1)
      T_ch = COSRh * Eh(I-1)
      T_u1 = 134217729.0 * COSRh
      T_p1 = T_u1
     +         - (T_u1 - COSRh)
      T_p2 = COSRh - T_p1
      T_u1 = 134217729.0 * Eh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I-1))
      T_r2 = Eh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSRh*El(I-1)
     +         +  COSRl*Eh(I-1))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSR * E(I-1)
!<<<  tmp2 = SINR * D(I)
      T_ch = SINRh * Dh(I)
      T_u1 = 134217729.0 * SINRh
      T_p1 = T_u1
     +         - (T_u1 - SINRh)
      T_p2 = SINRh - T_p1
      T_u1 = 134217729.0 * Dh(I)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I))
      T_r2 = Dh(I) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINRh*Dl(I)
     +         +  SINRl*Dh(I))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINR * D(I)
!<<<  E(I-1) = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + tmp1l - tmp2l
      T_s1 = T_ch + T_cl
      El(I-1) = T_cl - (T_s1 - T_ch)
      Eh(I-1) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  E(I-1) = tmp1 - tmp2
!!               G = SINR*D( I-1 )
!<<<  G = SINR * D(I-1)
      T_ch = SINRh * Dh(I-1)
      T_u1 = 134217729.0 * SINRh
      T_p1 = T_u1
     +         - (T_u1 - SINRh)
      T_p2 = SINRh - T_p1
      T_u1 = 134217729.0 * Dh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I-1))
      T_r2 = Dh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINRh*Dl(I-1)
     +         +  SINRl*Dh(I-1))
      T_s1 = T_ch + T_cl
      Gl = T_cl - (T_s1 - T_ch)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G = SINR * D(I-1)
!!               D( I-1 ) = COSR*D( I-1 )
!<<<  D(I-1) = COSR * D(I-1)
      T_ch = COSRh * Dh(I-1)
      T_u1 = 134217729.0 * COSRh
      T_p1 = T_u1
     +         - (T_u1 - COSRh)
      T_p2 = COSRh - T_p1
      T_u1 = 134217729.0 * Dh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I-1))
      T_r2 = Dh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSRh*Dl(I-1)
     +         +  COSRl*Dh(I-1))
      T_s1 = T_ch + T_cl
      Dl(I-1) = T_cl - (T_s1 - T_ch)
      Dh(I-1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  D(I-1) = COSR * D(I-1)
!!               CALL DLARTG( F, G, COSL, SINL, R )
               CALL dDLARTG( Fh, Fl, Gh, Gl, COSLh, COSLl,
     $                       SINLh, SINLl, Rh, Rl )
!!               D( I ) = R
      Dh(I) = Rh
      Dl(I) = Rl
!!               F = COSL*E( I-1 ) + SINL*D( I-1 )
!<<<  tmp1 = COSL * E(I-1)
      T_ch = COSLh * Eh(I-1)
      T_u1 = 134217729.0 * COSLh
      T_p1 = T_u1
     +         - (T_u1 - COSLh)
      T_p2 = COSLh - T_p1
      T_u1 = 134217729.0 * Eh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I-1))
      T_r2 = Eh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSLh*El(I-1)
     +         +  COSLl*Eh(I-1))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSL * E(I-1)
!<<<  tmp2 = SINL * D(I-1)
      T_ch = SINLh * Dh(I-1)
      T_u1 = 134217729.0 * SINLh
      T_p1 = T_u1
     +         - (T_u1 - SINLh)
      T_p2 = SINLh - T_p1
      T_u1 = 134217729.0 * Dh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I-1))
      T_r2 = Dh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINLh*Dl(I-1)
     +         +  SINLl*Dh(I-1))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINL * D(I-1)
!<<<  F = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + tmp1l + tmp2l
      Fh = T_q1 + T_q2
      Fl = T_q2 - (Fh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  F = tmp1 + tmp2
!!               D( I-1 ) = COSL*D( I-1 ) - SINL*E( I-1 )
!<<<  tmp1 = COSL * D(I-1)
      T_ch = COSLh * Dh(I-1)
      T_u1 = 134217729.0 * COSLh
      T_p1 = T_u1
     +         - (T_u1 - COSLh)
      T_p2 = COSLh - T_p1
      T_u1 = 134217729.0 * Dh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(I-1))
      T_r2 = Dh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSLh*Dl(I-1)
     +         +  COSLl*Dh(I-1))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = COSL * D(I-1)
!<<<  tmp2 = SINL * E(I-1)
      T_ch = SINLh * Eh(I-1)
      T_u1 = 134217729.0 * SINLh
      T_p1 = T_u1
     +         - (T_u1 - SINLh)
      T_p2 = SINLh - T_p1
      T_u1 = 134217729.0 * Eh(I-1)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I-1))
      T_r2 = Eh(I-1) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINLh*El(I-1)
     +         +  SINLl*Eh(I-1))
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = SINL * E(I-1)
!<<<  D(I-1) = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + tmp1l - tmp2l
      T_s1 = T_ch + T_cl
      Dl(I-1) = T_cl - (T_s1 - T_ch)
      Dh(I-1) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D(I-1) = tmp1 - tmp2
               IF( I.GT.LL+1 ) THEN
!!                  G = SINL*E( I-2 )
!<<<  G = SINL * E(I-2)
      T_ch = SINLh * Eh(I-2)
      T_u1 = 134217729.0 * SINLh
      T_p1 = T_u1
     +         - (T_u1 - SINLh)
      T_p2 = SINLh - T_p1
      T_u1 = 134217729.0 * Eh(I-2)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I-2))
      T_r2 = Eh(I-2) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SINLh*El(I-2)
     +         +  SINLl*Eh(I-2))
      T_s1 = T_ch + T_cl
      Gl = T_cl - (T_s1 - T_ch)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G = SINL * E(I-2)
!!                  E( I-2 ) = COSL*E( I-2 )
!<<<  E(I-2) = COSL * E(I-2)
      T_ch = COSLh * Eh(I-2)
      T_u1 = 134217729.0 * COSLh
      T_p1 = T_u1
     +         - (T_u1 - COSLh)
      T_p2 = COSLh - T_p1
      T_u1 = 134217729.0 * Eh(I-2)
      T_r1 = T_u1
     +         - (T_u1 - Eh(I-2))
      T_r2 = Eh(I-2) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (COSLh*El(I-2)
     +         +  COSLl*Eh(I-2))
      T_s1 = T_ch + T_cl
      El(I-2) = T_cl - (T_s1 - T_ch)
      Eh(I-2) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  E(I-2) = COSL * E(I-2)
               END IF
!!               WORK( I-LL ) = COSR
      WORKh(I-LL) = COSRh
      WORKl(I-LL) = COSRl
!!               WORK( I-LL+NM1 ) = -SINR
!<<<  WORK(I-LL+NM1) = -SINR
      WORKh(I-LL+NM1) = -SINRh
      WORKl(I-LL+NM1) = -SINRl
!>>>  WORK(I-LL+NM1) = -SINR
!!               WORK( I-LL+NM12 ) = COSL
      WORKh(I-LL+NM12) = COSLh
      WORKl(I-LL+NM12) = COSLl
!!               WORK( I-LL+NM13 ) = -SINL
!<<<  WORK(I-LL+NM13) = -SINL
      WORKh(I-LL+NM13) = -SINLh
      WORKl(I-LL+NM13) = -SINLl
!>>>  WORK(I-LL+NM13) = -SINL
  150       CONTINUE
!!            E( LL ) = F
      Eh(LL) = Fh
      El(LL) = Fl
*
*           Test convergence
*
!!            IF( ABS( E( LL ) ).LE.THRESH )
!!     $         E( LL ) = ZERO
!<<<  tmp1 = abs(E(LL))
      if (Eh(LL) .lt. 0.0d+0) then
        tmp1h = -Eh(LL)
        tmp1l = -El(LL)
      elseif (
     +  (Eh(LL) .eq. 0.0d+0).and.
     +  (El(LL) .lt. 0.0d+0)) then
        tmp1h = Eh(LL)
        tmp1l = -El(LL)
      else
        tmp1h = Eh(LL)
        tmp1l = El(LL)
      endif
!>>>  tmp1 = abs(E(LL))
            IF(
!       (tmp1 .le. THRESH)
     + ((tmp1h .lt. THRESHh) .or.
     +  ((tmp1h .eq. THRESHh) .and.
     +   (tmp1l .le. THRESHl)))
     $        ) THEN
      Eh(LL) = ZEROh
      El(LL) = ZEROl
            END IF
*
*           Update singular vectors if desired
*
!!            IF( NCVT.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'B', M-LL+1, NCVT, WORK( NM12+1 ),
!!     $                     WORK( NM13+1 ), VT( LL, 1 ), LDVT )
            IF( NCVT.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'B', M-LL+1, NCVT,
     $                      WORKh( NM12+1 ), WORKl( NM12+1 ),
     $                      WORKh( NM13+1 ), WORKl( NM13+1 ),
     $                      VTh( LL, 1 ), VTl( LL, 1 ), LDVT )
!!            IF( NRU.GT.0 )
!!     $         CALL DLASR( 'R', 'V', 'B', NRU, M-LL+1, WORK( 1 ),
!!     $                     WORK( N ), U( 1, LL ), LDU )
            IF( NRU.GT.0 )
     $         CALL dDLASR( 'R', 'V', 'B', NRU, M-LL+1,
     $                      WORKh( 1 ), WORKl( 1 ),
     $                      WORKh( N ), WORKl( N ),
     $                      Uh( 1, LL ), Ul( 1, LL ), LDU )
!!            IF( NCC.GT.0 )
!!     $         CALL DLASR( 'L', 'V', 'B', M-LL+1, NCC, WORK( 1 ),
!!     $                     WORK( N ), C( LL, 1 ), LDC )
            IF( NCC.GT.0 )
     $         CALL dDLASR( 'L', 'V', 'B', M-LL+1, NCC,
     $                      WORKh( 1 ), WORKl( 1 ),
     $                      WORKh( N ), WORKl( N ),
     $                      Ch( LL, 1 ), Cl( LL, 1 ), LDC )
         END IF
      END IF
*
*     QR iteration finished, go back and check convergence
*
      GO TO 60
*
*     All singular values converged, so make them positive
*
  160 CONTINUE
      DO 170 I = 1, N
!!         IF( D( I ).LT.ZERO ) THEN
         IF(
!       (D(I) .lt. ZERO)
     + ((Dh(I) .lt. ZEROh) .or.
     +  ((Dh(I) .eq. ZEROh) .and.
     +   (Dl(I) .lt. ZEROl)))
     $     ) THEN
!!            D( I ) = -D( I )
!<<<  D(I) = -D(I)
      Dh(I) = -Dh(I)
      Dl(I) = -Dl(I)
!>>>  D(I) = -D(I)
*
*           Change sign of singular vectors, if desired
*
!!            IF( NCVT.GT.0 )
!!     $         CALL DSCAL( NCVT, NEGONE, VT( I, 1 ), LDVT )
            IF( NCVT.GT.0 )
     $         CALL dDSCAL( NCVT, NEGONEh, NEGONEl,
     $                      VTh( I, 1 ), VTl( I, 1 ),
     $                      LDVT )
         END IF
  170 CONTINUE
*
*     Sort the singular values into decreasing order (insertion sort on
*     singular values, but only one transposition per singular vector)
*
      DO 190 I = 1, N - 1
*
*        Scan for smallest D(I)
*
         ISUB = 1
!!         SMIN = D( 1 )
      SMINh = Dh(1)
      SMINl = Dl(1)
         DO 180 J = 2, N + 1 - I
!!            IF( D( J ).LE.SMIN ) THEN
            IF(
!       (D(J) .le. SMIN)
     + ((Dh(J) .lt. SMINh) .or.
     +  ((Dh(J) .eq. SMINh) .and.
     +   (Dl(J) .le. SMINl)))
     $        ) THEN
               ISUB = J
!!               SMIN = D( J )
      SMINh = Dh(J)
      SMINl = Dl(J)
            END IF
  180    CONTINUE
         IF( ISUB.NE.N+1-I ) THEN
*
*           Swap singular values and vectors
*
!!            D( ISUB ) = D( N+1-I )
      Dh(ISUB) = Dh(N+1-I)
      Dl(ISUB) = Dl(N+1-I)
!!            D( N+1-I ) = SMIN
      Dh(N+1-I) = SMINh
      Dl(N+1-I) = SMINl
!!            IF( NCVT.GT.0 )
!!     $         CALL DSWAP( NCVT, VT( ISUB, 1 ), LDVT, VT( N+1-I, 1 ),
!!     $                     LDVT )
            IF( NCVT.GT.0 )
     $         CALL dDSWAP( NCVT, VTh( ISUB, 1 ), VTl( ISUB, 1 ), LDVT,
     $                      VTh( N+1-I, 1 ), VTl( N+1-I, 1 ),
     $                     LDVT )
!!            IF( NRU.GT.0 )
!!     $         CALL DSWAP( NRU, U( 1, ISUB ), 1, U( 1, N+1-I ), 1 )
            IF( NRU.GT.0 )
     $         CALL dDSWAP( NRU, Uh( 1, ISUB ), Ul( 1, ISUB ), 1,
     $                      Uh( 1, N+1-I ), Ul( 1, N+1-I ), 1 )
!!            IF( NCC.GT.0 )
!!     $         CALL DSWAP( NCC, C( ISUB, 1 ), LDC, C( N+1-I, 1 ), LDC )
            IF( NCC.GT.0 )
     $         CALL dDSWAP( NCC, Ch( ISUB, 1 ), Cl( ISUB, 1 ), LDC,
     $                      Ch( N+1-I, 1 ), Cl( N+1-I, 1 ), LDC )
         END IF
  190 CONTINUE
      GO TO 220
*
*     Maximum number of iterations exceeded, failure to converge
*
  200 CONTINUE
      INFO = 0
      DO 210 I = 1, N - 1
!!         IF( E( I ).NE.ZERO )
!!     $      INFO = INFO + 1
         IF(
!       (E(I) .ne. ZERO)
     + ((Eh(I) .ne. ZEROh) .or.
     +  (El(I) .ne. ZEROl))
     $     ) INFO = INFO + 1
  210 CONTINUE
  220 CONTINUE
      RETURN
*
*     End of DBDSQR
*
      END
