*> \brief \b DLAED4
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED4 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed4.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed4.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed4.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED4( N, I, D, Z, DELTA, RHO, DLAM, INFO )
* 
*       .. Scalar Arguments ..
*       INTEGER            I, INFO, N
*       DOUBLE PRECISION   DLAM, RHO
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( * ), DELTA( * ), Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> This subroutine computes the I-th updated eigenvalue of a symmetric
*> rank-one modification to a diagonal matrix whose elements are
*> given in the array d, and that
*>
*>            D(i) < D(j)  for  i < j
*>
*> and that RHO > 0.  This is arranged by the calling routine, and is
*> no loss in generality.  The rank-one modified system is thus
*>
*>            diag( D )  +  RHO * Z * Z_transpose.
*>
*> where we assume the Euclidean norm of Z is 1.
*>
*> The method consists of approximating the rational functions in the
*> secular equation by simpler interpolating rational functions.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>         The length of all arrays.
*> \endverbatim
*>
*> \param[in] I
*> \verbatim
*>          I is INTEGER
*>         The index of the eigenvalue to be computed.  1 <= I <= N.
*> \endverbatim
*>
*> \param[in] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>         The original eigenvalues.  It is assumed that they are in
*>         order, D(I) < D(J)  for I < J.
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension (N)
*>         The components of the updating vector.
*> \endverbatim
*>
*> \param[out] DELTA
*> \verbatim
*>          DELTA is DOUBLE PRECISION array, dimension (N)
*>         If N .GT. 2, DELTA contains (D(j) - lambda_I) in its  j-th
*>         component.  If N = 1, then DELTA(1) = 1. If N = 2, see DLAED5
*>         for detail. The vector DELTA contains the information necessary
*>         to construct the eigenvectors by DLAED3 and DLAED9.
*> \endverbatim
*>
*> \param[in] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>         The scalar in the symmetric updating formula.
*> \endverbatim
*>
*> \param[out] DLAM
*> \verbatim
*>          DLAM is DOUBLE PRECISION
*>         The computed lambda_I, the I-th updated eigenvalue.
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>         = 0:  successful exit
*>         > 0:  if INFO = 1, the updating process failed.
*> \endverbatim
*
*> \par Internal Parameters:
*  =========================
*>
*> \verbatim
*>  Logical variable ORGATI (origin-at-i?) is used for distinguishing
*>  whether D(i) or D(i+1) is treated as the origin.
*>
*>            ORGATI = .true.    origin at i
*>            ORGATI = .false.   origin at i+1
*>
*>   Logical variable SWTCH3 (switch-for-3-poles?) is for noting
*>   if we are working with THREE poles!
*>
*>   MAXIT is the maximum number of iterations allowed for each
*>   eigenvalue.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Contributors:
*  ==================
*>
*>     Ren-Cang Li, Computer Science Division, University of California
*>     at Berkeley, USA
*>
*  =====================================================================
!      SUBROUTINE DLAED4( N, I, D, Z, DELTA, RHO, DLAM, INFO )
      SUBROUTINE DDLAED4( N, I, Dh,Dl, Zh,Zl, DELTAh,DELTAl, 
     &                    RHOh, RHOl, DLAMh, DLAMl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            I, INFO, N
!      DOUBLE PRECISION   DLAM, RHO
      double precision DLAMh, DLAMl
      double precision RHOh, RHOl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( * ), DELTA( * ), Z( * )
      double precision Dh(*), Dl(*)
      double precision DELTAh(*), DELTAl(*)
      double precision Zh(*), Zl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            MAXIT
      PARAMETER          ( MAXIT = 30 )
!      DOUBLE PRECISION   ZERO, ONE, TWO, THREE, FOUR, EIGHT, TEN
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision THREEh, THREEl
      double precision FOURh, FOURl
      double precision EIGHTh, EIGHTl
      double precision TENh, TENl
!      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0,
!     $                   THREE = 3.0D0, FOUR = 4.0D0, EIGHT = 8.0D0,
!     $                   TEN = 10.0D0 )
*     ..
*     .. Local Scalars ..
      LOGICAL            ORGATI, SWTCH, SWTCH3
      INTEGER            II, IIM1, IIP1, IP1, ITER, J, NITER
!      DOUBLE PRECISION   A, B, C, DEL, DLTLB, DLTUB, DPHI, DPSI, DW,
!     $                   EPS, ERRETM, ETA, MIDPT, PHI, PREW, PSI,
!     $                   RHOINV, TAU, TEMP, TEMP1, W
      double precision Ah, Al
      double precision Bh, Bl
      double precision Ch, Cl
      double precision DELh, DELl
      double precision DLTLBh, DLTLBl
      double precision DLTUBh, DLTUBl
      double precision DPHIh, DPHIl
      double precision DPSIh, DPSIl
      double precision DWh, DWl
      double precision EPSh, EPSl
      double precision ERRETMh, ERRETMl
      double precision ETAh, ETAl
      double precision MIDPTh, MIDPTl
      double precision PHIh, PHIl
      double precision PREWh, PREWl
      double precision PSIh, PSIl
      double precision RHOINVh, RHOINVl
      double precision TAUh, TAUl
      double precision TEMPh, TEMPl
      double precision TEMP1h, TEMP1l
      double precision Wh, Wl
*     ..
*     .. Local Arrays ..
!      DOUBLE PRECISION   ZZ( 3 )
      double precision ZZh(3), ZZl(3)
*     ..
*     .. External Functions ..
!      DOUBLE PRECISION   DLAMCH
!      EXTERNAL           DLAMCH
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDLAMCH, DDLAED5, DDLAED6
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
      double precision tmp9h, tmp9l
      double precision tmp10h, tmp10l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l


      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0D+0
      TWOl = 0.0d+0
      THREEh = 3.0D+0
      THREEl = 0.0d+0
      FOURh = 4.0D+0
      FOURl = 0.0d+0
      EIGHTh = 8.0D+0
      EIGHTl = 0.0d+0
      TENh = 10.0D+0
      TENl = 0.0d+0
*     .. Executable Statements ..
*
*     Since this routine is called in an inner loop, we do no argument
*     checking.
*
*     Quick return for N=1 and 2.
*
      INFO = 0
      IF( N.EQ.1 ) THEN
*
*         Presumably, I=1 upon entry
*
!         DLAM = D( 1 ) + RHO*Z( 1 )*Z( 1 )
!<<<  tmp1 = Z(1) * Z(1)
      tmp1h = Zh(1) * Zh(1)
      T_u1 = 134217729.0 * Zh(1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(1))
      T_p2 = Zh(1) - T_p1
      T_u1 = 134217729.0 * Zh(1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(1))
      T_r2 = Zh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(1)*Zl(1)
     +      + Zl(1)*Zh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(1) * Z(1)
!<<<  tmp2 = RHO * tmp1
      tmp2h = RHOh * tmp1h
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (RHOh*tmp1l
     +      + RHOl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = RHO * tmp1
!<<<  DLAM = D(1) + tmp2
      T_p1 = Dh(1) + tmp2h
      T_p2 = T_p1 - Dh(1)
      DLAMh = T_p1
      DLAMl = (Dh(1) - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      DLAMl = DLAMl + Dl(1) + tmp2l
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(1) + tmp2
!         DELTA( 1 ) = ONE
      DELTAh(1) = ONEh
      DELTAl(1) = ONEl
         RETURN
      END IF
      IF( N.EQ.2 ) THEN
!         CALL DLAED5( I, D, Z, DELTA, RHO, DLAM )
         CALL DDLAED5( I, Dh,Dl, Zh,Zl, DELTAh,DELTAl, 
     &                RHOh,RHOl, DLAMh, DLAMl )

         RETURN
      END IF
*
*     Compute machine epsilon
*
!      EPS = DLAMCH( 'Epsilon' )
      CALL DDLAMCH('Epsilon',EPSh,EPSl)
!      RHOINV = ONE / RHO
!<<<  RHOINV = ONE / RHO
      T_q1 = ONEh / RHOh
      RHOINVh = RHOh * T_q1
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      RHOINVl = ((T_p1*T_r1
     +       - RHOINVh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      RHOINVl = RHOINVl + (RHOl*T_q1)
      T_s1 = RHOINVh + RHOINVl
      RHOINVl = RHOINVl - (T_s1 - RHOINVh)
      RHOINVh = T_s1
      T_s1 = ONEh - RHOINVh
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh 
     +         - (T_s1 - T_r1))
     +         - (RHOINVh + T_r1)
      T_s2 = T_s2 - RHOINVl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / RHOh
      RHOINVh = T_q1 + T_q2
      RHOINVl = T_q2
     +      - (RHOINVh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  RHOINV = ONE / RHO
*
*     The case I = N
*
      IF( I.EQ.N ) THEN
*
*        Initialize some basic variables
*
         II = N - 1
         NITER = 1
*
*        Calculate initial guess
*
!         MIDPT = RHO / TWO
!<<<  MIDPT = RHO / TWO
      T_q1 = RHOh / TWOh
      MIDPTh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      MIDPTl = ((T_p1*T_r1
     +       - MIDPTh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      MIDPTl = MIDPTl + (TWOl*T_q1)
      T_s1 = MIDPTh + MIDPTl
      MIDPTl = MIDPTl - (T_s1 - MIDPTh)
      MIDPTh = T_s1
      T_s1 = RHOh - MIDPTh
      T_r1 = T_s1 - RHOh
      T_s2 = (RHOh 
     +         - (T_s1 - T_r1))
     +         - (MIDPTh + T_r1)
      T_s2 = T_s2 - MIDPTl
      T_s2 = T_s2 + RHOl
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      MIDPTh = T_q1 + T_q2
      MIDPTl = T_q2
     +      - (MIDPTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  MIDPT = RHO / TWO
*
*        If ||Z||_2 is not one, then TEMP should be set to
*        RHO * ||Z||_2^2 / TWO
*
         DO 10 J = 1, N
!            DELTA( J ) = ( D( J )-D( I ) ) - MIDPT
!<<<  tmp1 = D(J) - D(I)
      T_p1 = Dh(J) - Dh(I)
      T_p2 = T_p1 - Dh(J)
      tmp1h = T_p1
      tmp1l = (Dh(J) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      tmp1l = tmp1l + Dl(J) - Dl(I)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(J) - D(I)
!<<<  DELTA(J) = tmp1 - MIDPT
      T_p1 = tmp1h - MIDPTh
      T_p2 = T_p1 - tmp1h
      DELTAh(J) = T_p1
      DELTAl(J) = (tmp1h - (T_p1 - T_p2))
     +         - (MIDPTh + T_p2)
      DELTAl(J) = DELTAl(J) + tmp1l - MIDPTl
      T_s1 = DELTAh(J) + DELTAl(J)
      DELTAl(J) = DELTAl(J) - (T_s1 - DELTAh(J))
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DELTA(J) = tmp1 - MIDPT
   10    CONTINUE
*
!         PSI = ZERO
      PSIh = ZEROh
      PSIl = ZEROl
         DO 20 J = 1, N - 2
!            PSI = PSI + Z( J )*Z( J ) / DELTA( J )
!<<<  tmp1 = Z(J) * Z(J)
      tmp1h = Zh(J) * Zh(J)
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * Zh(J)
      T_r1 = T_u1
     +         - (T_u1 - Zh(J))
      T_r2 = Zh(J) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*Zl(J)
     +      + Zl(J)*Zh(J))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * Z(J)
!<<<  tmp2 = tmp1 / DELTA(J)
      T_q1 = tmp1h / DELTAh(J)
      tmp2h = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (DELTAl(J)*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / DELTA(J)
!<<<  PSI = PSI + tmp2
      T_p1 = PSIh + tmp2h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + PSIl + tmp2l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp2
   20    CONTINUE
*
!         C = RHOINV + PSI
!<<<  C = RHOINV + PSI
      T_p1 = RHOINVh + PSIh
      T_p2 = T_p1 - RHOINVh
      Ch = T_p1
      Cl = (RHOINVh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      Cl = Cl + RHOINVl + PSIl
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  C = RHOINV + PSI
!         W = C + Z( II )*Z( II ) / DELTA( II ) +
!     $       Z( N )*Z( N ) / DELTA( N )
!<<<  tmp1 = Z(N) * Z(N)
      tmp1h = Zh(N) * Zh(N)
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * Zh(N)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N))
      T_r2 = Zh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(N)*Zl(N)
     +      + Zl(N)*Zh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(N) * Z(N)
!<<<  tmp2 = tmp1 / DELTA(N)
      T_q1 = tmp1h / DELTAh(N)
      tmp2h = DELTAh(N) * T_q1
      T_u1 = 134217729.0 * DELTAh(N)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_p2 = DELTAh(N) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (DELTAl(N)*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(N)
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / DELTA(N)
!<<<  tmp3 = Z(II) * Z(II)
      tmp3h = Zh(II) * Zh(II)
      T_u1 = 134217729.0 * Zh(II)
      T_p1 = T_u1
     +         - (T_u1 - Zh(II))
      T_p2 = Zh(II) - T_p1
      T_u1 = 134217729.0 * Zh(II)
      T_r1 = T_u1
     +         - (T_u1 - Zh(II))
      T_r2 = Zh(II) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(II)*Zl(II)
     +      + Zl(II)*Zh(II))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(II) * Z(II)
!<<<  tmp4 = tmp3 / DELTA(II)
      T_q1 = tmp3h / DELTAh(II)
      tmp4h = DELTAh(II) * T_q1
      T_u1 = 134217729.0 * DELTAh(II)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(II))
      T_p2 = DELTAh(II) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp4l = ((T_p1*T_r1
     +       - tmp4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp4l = tmp4l + (DELTAl(II)*T_q1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
      T_s1 = tmp3h - tmp4h
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h 
     +         - (T_s1 - T_r1))
     +         - (tmp4h + T_r1)
      T_s2 = T_s2 - tmp4l
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(II)
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = tmp3 / DELTA(II)
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  W = C + tmp5
      T_p1 = Ch + tmp5h
      T_p2 = T_p1 - Ch
      Wh = T_p1
      Wl = (Ch - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      Wl = Wl + Cl + tmp5l
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = C + tmp5
*
         IF( 
!            W.LE.ZERO
!       (W .le. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .le. ZEROl)))
     $      ) THEN
!            TEMP = Z( N-1 )*Z( N-1 ) / ( D( N )-D( N-1 )+RHO ) +
!     $             Z( N )*Z( N ) / RHO
!<<<  tmp1 = Z(N) * Z(N)
      tmp1h = Zh(N) * Zh(N)
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * Zh(N)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N))
      T_r2 = Zh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(N)*Zl(N)
     +      + Zl(N)*Zh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(N) * Z(N)
!<<<  tmp2 = tmp1 / RHO
      T_q1 = tmp1h / RHOh
      tmp2h = RHOh * T_q1
      T_u1 = 134217729.0 * RHOh
      T_p1 = T_u1
     +         - (T_u1 - RHOh)
      T_p2 = RHOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (RHOl*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / RHOh
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / RHO
!<<<  tmp3 = Z(N-1) * Z(N-1)
      tmp3h = Zh(N-1) * Zh(N-1)
      T_u1 = 134217729.0 * Zh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N-1))
      T_p2 = Zh(N-1) - T_p1
      T_u1 = 134217729.0 * Zh(N-1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N-1))
      T_r2 = Zh(N-1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(N-1)*Zl(N-1)
     +      + Zl(N-1)*Zh(N-1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(N-1) * Z(N-1)
!<<<  tmp4 = D(N) - D(N-1)
      T_p1 = Dh(N) - Dh(N-1)
      T_p2 = T_p1 - Dh(N)
      tmp4h = T_p1
      tmp4l = (Dh(N) - (T_p1 - T_p2))
     +         - (Dh(N-1) + T_p2)
      tmp4l = tmp4l + Dl(N) - Dl(N-1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = D(N) - D(N-1)
!<<<  tmp5 = tmp4 + RHO
      T_p1 = tmp4h + RHOh
      T_p2 = T_p1 - tmp4h
      tmp5h = T_p1
      tmp5l = (tmp4h - (T_p1 - T_p2))
     +      + (RHOh - T_p2)
      tmp5l = tmp5l + tmp4l + RHOl
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp4 + RHO
!<<<  tmp6 = tmp3 / tmp5
      T_q1 = tmp3h / tmp5h
      tmp6h = tmp5h * T_q1
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp6l = ((T_p1*T_r1
     +       - tmp6h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp6l = tmp6l + (tmp5l*T_q1)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
      T_s1 = tmp3h - tmp6h
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h 
     +         - (T_s1 - T_r1))
     +         - (tmp6h + T_r1)
      T_s2 = T_s2 - tmp6l
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / tmp5h
      tmp6h = T_q1 + T_q2
      tmp6l = T_q2
     +      - (tmp6h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp6 = tmp3 / tmp5
!<<<  TEMP = tmp6 + tmp2
      T_p1 = tmp6h + tmp2h
      T_p2 = T_p1 - tmp6h
      TEMPh = T_p1
      TEMPl = (tmp6h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      TEMPl = TEMPl + tmp6l + tmp2l
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = tmp6 + tmp2
            IF( 
!              C.LE.TEMP
!       (C .le. TEMP)
     + ((Ch .lt. TEMPh) .or.
     +  ((Ch .eq. TEMPh) .and.
     +   (Cl .le. TEMPl)))
     $        ) THEN
!               TAU = RHO
      TAUh = RHOh
      TAUl = RHOl
             ELSE
!               DEL = D( N ) - D( N-1 )
!<<<  DEL = D(N) - D(N-1)
      T_p1 = Dh(N) - Dh(N-1)
      T_p2 = T_p1 - Dh(N)
      DELh = T_p1
      DELl = (Dh(N) - (T_p1 - T_p2))
     +         - (Dh(N-1) + T_p2)
      DELl = DELl + Dl(N) - Dl(N-1)
      T_s1 = DELh + DELl
      DELl = DELl - (T_s1 - DELh)
      DELh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DEL = D(N) - D(N-1)
!               A = -C*DEL + Z( N-1 )*Z( N-1 ) + Z( N )*Z( N )
!<<<  tmp1 = Z(N) * Z(N)
      tmp1h = Zh(N) * Zh(N)
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * Zh(N)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N))
      T_r2 = Zh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(N)*Zl(N)
     +      + Zl(N)*Zh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(N) * Z(N)
!<<<  tmp2 = Z(N-1) * Z(N-1)
      tmp2h = Zh(N-1) * Zh(N-1)
      T_u1 = 134217729.0 * Zh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N-1))
      T_p2 = Zh(N-1) - T_p1
      T_u1 = 134217729.0 * Zh(N-1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N-1))
      T_r2 = Zh(N-1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(N-1)*Zl(N-1)
     +      + Zl(N-1)*Zh(N-1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(N-1) * Z(N-1)
!<<<  tmp3 = C * DEL
      tmp3h = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Ch*DELl
     +      + Cl*DELh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = C * DEL
!<<<  tmp4 = -tmp3
      tmp4h = -tmp3h
      tmp4l = -tmp3l
!>>>  tmp4 = -tmp3
!<<<  tmp5 = tmp4 + tmp2
      T_p1 = tmp4h + tmp2h
      T_p2 = T_p1 - tmp4h
      tmp5h = T_p1
      tmp5l = (tmp4h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp5l = tmp5l + tmp4l + tmp2l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp4 + tmp2
!<<<  A = tmp5 + tmp1
      T_p1 = tmp5h + tmp1h
      T_p2 = T_p1 - tmp5h
      Ah = T_p1
      Al = (tmp5h - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Al = Al + tmp5l + tmp1l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp5 + tmp1
!               B = Z( N )*Z( N )*DEL
!<<<  tmp1 = Z(N) * Z(N)
      tmp1h = Zh(N) * Zh(N)
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * Zh(N)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N))
      T_r2 = Zh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(N)*Zl(N)
     +      + Zl(N)*Zh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(N) * Z(N)
!<<<  B = tmp1 * DEL
      Bh = tmp1h * DELh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*DELl
     +      + tmp1l*DELh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * DEL
               IF( 
!                  A.LT.ZERO
!       (A .lt. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .lt. ZEROl)))
     $            ) THEN
!                  TAU = TWO*B / ( SQRT( A*A+FOUR*B*C )-A )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = tmp6 - A
      T_p1 = tmp6h - Ah
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +         - (Ah + T_p2)
      tmp7l = tmp7l + tmp6l - Al
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = tmp6 - A
!<<<  TAU = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      TAUh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp7l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp7
               ELSE
!                  TAU = ( A+SQRT( A*A+FOUR*B*C ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = tmp6 + A
      T_p1 = tmp6h + Ah
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +      + (Ah - T_p2)
      tmp7l = tmp7l + tmp6l + Al
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = tmp6 + A
!<<<  TAU = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      TAUh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp1l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp7h - TAUh
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp7 / tmp1
               END IF
            END IF
*
*           It can be proved that
*               D(N)+RHO/2 <= LAMBDA(N) < D(N)+TAU <= D(N)+RHO
*
!            DLTLB = MIDPT
      DLTLBh = MIDPTh
      DLTLBl = MIDPTl
!            DLTUB = RHO
      DLTUBh = RHOh
      DLTUBl = RHOl
         ELSE
!            DEL = D( N ) - D( N-1 )
!<<<  DEL = D(N) - D(N-1)
      T_p1 = Dh(N) - Dh(N-1)
      T_p2 = T_p1 - Dh(N)
      DELh = T_p1
      DELl = (Dh(N) - (T_p1 - T_p2))
     +         - (Dh(N-1) + T_p2)
      DELl = DELl + Dl(N) - Dl(N-1)
      T_s1 = DELh + DELl
      DELl = DELl - (T_s1 - DELh)
      DELh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DEL = D(N) - D(N-1)
!            A = -C*DEL + Z( N-1 )*Z( N-1 ) + Z( N )*Z( N )
!<<<  tmp1 = Z(N) * Z(N)
      tmp1h = Zh(N) * Zh(N)
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * Zh(N)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N))
      T_r2 = Zh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(N)*Zl(N)
     +      + Zl(N)*Zh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(N) * Z(N)
!<<<  tmp2 = Z(N-1) * Z(N-1)
      tmp2h = Zh(N-1) * Zh(N-1)
      T_u1 = 134217729.0 * Zh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N-1))
      T_p2 = Zh(N-1) - T_p1
      T_u1 = 134217729.0 * Zh(N-1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N-1))
      T_r2 = Zh(N-1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(N-1)*Zl(N-1)
     +      + Zl(N-1)*Zh(N-1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(N-1) * Z(N-1)
!<<<  tmp3 = C * DEL
      tmp3h = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Ch*DELl
     +      + Cl*DELh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = C * DEL
!<<<  tmp4 = -tmp3
      tmp4h = -tmp3h
      tmp4l = -tmp3l
!>>>  tmp4 = -tmp3
!<<<  tmp5 = tmp4 + tmp2
      T_p1 = tmp4h + tmp2h
      T_p2 = T_p1 - tmp4h
      tmp5h = T_p1
      tmp5l = (tmp4h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp5l = tmp5l + tmp4l + tmp2l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp4 + tmp2
!<<<  A = tmp5 + tmp1
      T_p1 = tmp5h + tmp1h
      T_p2 = T_p1 - tmp5h
      Ah = T_p1
      Al = (tmp5h - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Al = Al + tmp5l + tmp1l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp5 + tmp1
!            B = Z( N )*Z( N )*DEL
!<<<  tmp1 = Z(N) * Z(N)
      tmp1h = Zh(N) * Zh(N)
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * Zh(N)
      T_r1 = T_u1
     +         - (T_u1 - Zh(N))
      T_r2 = Zh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(N)*Zl(N)
     +      + Zl(N)*Zh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(N) * Z(N)
!<<<  B = tmp1 * DEL
      Bh = tmp1h * DELh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*DELl
     +      + tmp1l*DELh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * DEL
            IF( 
!               A.LT.ZERO
!       (A .lt. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .lt. ZEROl)))
     $        ) THEN
!               TAU = TWO*B / ( SQRT( A*A+FOUR*B*C )-A )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = tmp6 - A
      T_p1 = tmp6h - Ah
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +         - (Ah + T_p2)
      tmp7l = tmp7l + tmp6l - Al
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = tmp6 - A
!<<<  TAU = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      TAUh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp7l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp7
            ELSE
!               TAU = ( A+SQRT( A*A+FOUR*B*C ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = tmp6 + A
      T_p1 = tmp6h + Ah
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +      + (Ah - T_p2)
      tmp7l = tmp7l + tmp6l + Al
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = tmp6 + A
!<<<  TAU = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      TAUh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp1l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp7h - TAUh
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp7 / tmp1
            END IF
*
*           It can be proved that
*               D(N) < D(N)+TAU < LAMBDA(N) < D(N)+RHO/2
*
!            DLTLB = ZERO
      DLTLBh = ZEROh
      DLTLBl = ZEROl
!            DLTUB = MIDPT
      DLTUBh = MIDPTh
      DLTUBl = MIDPTl
         END IF
*
         DO 30 J = 1, N
!            DELTA( J ) = ( D( J )-D( I ) ) - TAU
!<<<  tmp1 = D(J) - D(I)
      T_p1 = Dh(J) - Dh(I)
      T_p2 = T_p1 - Dh(J)
      tmp1h = T_p1
      tmp1l = (Dh(J) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      tmp1l = tmp1l + Dl(J) - Dl(I)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(J) - D(I)
!<<<  DELTA(J) = tmp1 - TAU
      T_p1 = tmp1h - TAUh
      T_p2 = T_p1 - tmp1h
      DELTAh(J) = T_p1
      DELTAl(J) = (tmp1h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      DELTAl(J) = DELTAl(J) + tmp1l - TAUl
      T_s1 = DELTAh(J) + DELTAl(J)
      DELTAl(J) = DELTAl(J) - (T_s1 - DELTAh(J))
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DELTA(J) = tmp1 - TAU
   30    CONTINUE
*
*        Evaluate PSI and the derivative DPSI
*
!         DPSI = ZERO
      DPSIh = ZEROh
      DPSIl = ZEROl
!         PSI = ZERO
      PSIh = ZEROh
      PSIl = ZEROl
!         ERRETM = ZERO
      ERRETMh = ZEROh
      ERRETMl = ZEROl
         DO 40 J = 1, II
!            TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!            PSI = PSI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PSI = PSI + tmp1
      T_p1 = PSIh + tmp1h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PSIl + tmp1l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp1
!            DPSI = DPSI + TEMP*TEMP
!<<<  tmp1 = TEMP * TEMP
      tmp1h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp1
      T_p1 = DPSIh + tmp1h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp1l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp1
!            ERRETM = ERRETM + PSI
!<<<  ERRETM = ERRETM + PSI
      T_p1 = ERRETMh + PSIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PSIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PSI
   40    CONTINUE
!         ERRETM = ABS( ERRETM )
!<<<  ERRETM = abs(ERRETM)
      if (ERRETMh .lt. 0.0d+0) then
        ERRETMh = -ERRETMh
        ERRETMl = -ERRETMl
      elseif (
     +  (ERRETMh .eq. 0.0d+0).and.
     +  (ERRETMl .lt. 0.0d+0)) then
        ERRETMh = ERRETMh
        ERRETMl = -ERRETMl
      else
        ERRETMh = ERRETMh
        ERRETMl = ERRETMl
      endif
!>>>  ERRETM = abs(ERRETM)
*
*        Evaluate PHI and the derivative DPHI
*
!         TEMP = Z( N ) / DELTA( N )
!<<<  TEMP = Z(N) / DELTA(N)
      T_q1 = Zh(N) / DELTAh(N)
      TEMPh = DELTAh(N) * T_q1
      T_u1 = 134217729.0 * DELTAh(N)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_p2 = DELTAh(N) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(N)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(N) - TEMPh
      T_r1 = T_s1 - Zh(N)
      T_s2 = (Zh(N) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(N)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(N)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(N) / DELTA(N)
!         PHI = Z( N )*TEMP
!<<<  PHI = Z(N) * TEMP
      PHIh = Zh(N) * TEMPh
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      PHIl = ((T_p1*T_r1
     +        - PHIh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      PHIl = PHIl
     +      + (Zh(N)*TEMPl
     +      + Zl(N)*TEMPh)
      T_s1 = PHIh + PHIl
      PHIl = PHIl - (T_s1 - PHIh)
      PHIh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  PHI = Z(N) * TEMP
!         DPHI = TEMP*TEMP
!<<<  DPHI = TEMP * TEMP
      DPHIh = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      DPHIl = ((T_p1*T_r1
     +        - DPHIh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      DPHIl = DPHIl
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = DPHIh + DPHIl
      DPHIl = DPHIl - (T_s1 - DPHIh)
      DPHIh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  DPHI = TEMP * TEMP
!         ERRETM = EIGHT*( -PHI-PSI ) + ERRETM - PHI + RHOINV +
!     $            ABS( TAU )*( DPSI+DPHI )
!<<<  tmp1 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp1l = tmp1l + DPSIl + DPHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DPSI + DPHI
!<<<  tmp2 = abs(TAU)
      if (TAUh .lt. 0.0d+0) then
        tmp2h = -TAUh
        tmp2l = -TAUl
      elseif (
     +  (TAUh .eq. 0.0d+0).and.
     +  (TAUl .lt. 0.0d+0)) then
        tmp2h = TAUh
        tmp2l = -TAUl
      else
        tmp2h = TAUh
        tmp2l = TAUl
      endif
!>>>  tmp2 = abs(TAU)
!<<<  tmp3 = tmp1 * tmp2
      tmp3h = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (tmp1h*tmp2l
     +      + tmp1l*tmp2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
!<<<  tmp4 = PHI + PSI
      T_p1 = PHIh + PSIh
      T_p2 = T_p1 - PHIh
      tmp4h = T_p1
      tmp4l = (PHIh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      tmp4l = tmp4l + PHIl + PSIl
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = PHI + PSI
!<<<  tmp5 = -tmp4
      tmp5h = -tmp4h
      tmp5l = -tmp4l
!>>>  tmp5 = -tmp4
!<<<  tmp6 = EIGHT * tmp5
      tmp6h = EIGHTh * tmp5h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      tmp6l = ((T_p1*T_r1
     +        - tmp6h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp6l = tmp6l
     +      + (EIGHTh*tmp5l
     +      + EIGHTl*tmp5h)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp6 = EIGHT * tmp5
!<<<  ERRETM = tmp6 + ERRETM
      T_p1 = tmp6h + ERRETMh
      T_p2 = T_p1 - tmp6h
      T_q1 = T_p1
      T_q2 = (tmp6h - (T_p1 - T_p2))
     +         + (ERRETMh - T_p2)
      T_q2 = T_q2 + tmp6l + ERRETMl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = tmp6 + ERRETM
!<<<  ERRETM = ERRETM - PHI
      T_p1 = ERRETMh - PHIh
      T_p2 = T_p1 - ERRETMh
      T_ch = T_p1
      T_cl = (ERRETMh - (T_p1 - T_p2))
     +         - (PHIh + T_p2)
      T_cl = T_cl + ERRETMl - PHIl
      T_s1 = T_ch + T_cl
      ERRETMl = T_cl - (T_s1 - T_ch)
      ERRETMh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  ERRETM = ERRETM - PHI
!<<<  ERRETM = ERRETM + RHOINV
      T_p1 = ERRETMh + RHOINVh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (RHOINVh - T_p2)
      T_q2 = T_q2 + ERRETMl + RHOINVl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + RHOINV
!<<<  ERRETM = ERRETM + tmp3
      T_p1 = ERRETMh + tmp3h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp3h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp3l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp3
*
!         W = RHOINV + PHI + PSI
!<<<  W = RHOINV + PHI
      T_p1 = RHOINVh + PHIh
      T_p2 = T_p1 - RHOINVh
      Wh = T_p1
      Wl = (RHOINVh - (T_p1 - T_p2))
     +      + (PHIh - T_p2)
      Wl = Wl + RHOINVl + PHIl
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = RHOINV + PHI
!<<<  W = W + PSI
      T_p1 = Wh + PSIh
      T_p2 = T_p1 - Wh
      T_q1 = T_p1
      T_q2 = (Wh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + Wl + PSIl
      Wh = T_q1 + T_q2
      Wl = T_q2 - (Wh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W = W + PSI
*
*        Test for convergence
*
!<<<  tmp1 = abs(W)
      if (Wh .lt. 0.0d+0) then
        tmp1h = -Wh
        tmp1l = -Wl
      elseif (
     +  (Wh .eq. 0.0d+0).and.
     +  (Wl .lt. 0.0d+0)) then
        tmp1h = Wh
        tmp1l = -Wl
      else
        tmp1h = Wh
        tmp1l = Wl
      endif
!>>>  tmp1 = abs(W)
!<<<  tmp2 = EPS * ERRETM
      tmp2h = EPSh * ERRETMh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * ERRETMh
      T_r1 = T_u1
     +         - (T_u1 - ERRETMh)
      T_r2 = ERRETMh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EPSh*ERRETMl
     +      + EPSl*ERRETMh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EPS * ERRETM
         IF( 
!           ABS( W ).LE.EPS*ERRETM
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $     ) THEN
!            DLAM = D( I ) + TAU
!<<<  DLAM = D(I) + TAU
      T_p1 = Dh(I) + TAUh
      T_p2 = T_p1 - Dh(I)
      DLAMh = T_p1
      DLAMl = (Dh(I) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(I) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(I) + TAU
            GO TO 250
         END IF
*
         IF( 
!            W.LE.ZERO
!       (W .le. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .le. ZEROl)))
     &      ) THEN
!            DLTLB = MAX( DLTLB, TAU )
             IF(
!       (DLTLB .gt. TAU)
     + ((DLTLBh .gt. TAUh) .or.
     +  ((DLTLBh .eq. TAUh) .and.
     +   (DLTLBl .gt. TAUl)))
     &        ) THEN
      DLTLBh = DLTLBh
      DLTLBl = DLTLBl
               ELSE
      DLTLBh = TAUh
      DLTLBl = TAUl
               END IF
         ELSE
!            DLTUB = MIN( DLTUB, TAU )
             IF( 
!       (DLTUB .lt. TAU)
     + ((DLTUBh .lt. TAUh) .or.
     +  ((DLTUBh .eq. TAUh) .and.
     +   (DLTUBl .lt. TAUl)))
     &      ) THEN
      DLTUBh = DLTUBh
      DLTUBl = DLTUBl
             ELSE
      DLTUBh = TAUh
      DLTUBl = TAUl
             END IF
         END IF
*
*        Calculate the new step
*
         NITER = NITER + 1
!         C = W - DELTA( N-1 )*DPSI - DELTA( N )*DPHI
!<<<  tmp1 = DELTA(N) * DPHI
      tmp1h = DELTAh(N) * DPHIh
      T_u1 = 134217729.0 * DELTAh(N)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_p2 = DELTAh(N) - T_p1
      T_u1 = 134217729.0 * DPHIh
      T_r1 = T_u1
     +         - (T_u1 - DPHIh)
      T_r2 = DPHIh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(N)*DPHIl
     +      + DELTAl(N)*DPHIh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(N) * DPHI
!<<<  tmp2 = DELTA(N-1) * DPSI
      tmp2h = DELTAh(N-1) * DPSIh
      T_u1 = 134217729.0 * DELTAh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N-1))
      T_p2 = DELTAh(N-1) - T_p1
      T_u1 = 134217729.0 * DPSIh
      T_r1 = T_u1
     +         - (T_u1 - DPSIh)
      T_r2 = DPSIh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(N-1)*DPSIl
     +      + DELTAl(N-1)*DPSIh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(N-1) * DPSI
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  C = W - tmp3
      T_p1 = Wh - tmp3h
      T_p2 = T_p1 - Wh
      Ch = T_p1
      Cl = (Wh - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      Cl = Cl + Wl - tmp3l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = W - tmp3
!         A = ( DELTA( N-1 )+DELTA( N ) )*W -
!     $       DELTA( N-1 )*DELTA( N )*( DPSI+DPHI )
!<<<  tmp1 = DELTA(N-1) + DELTA(N)
      T_p1 = DELTAh(N-1) + DELTAh(N)
      T_p2 = T_p1 - DELTAh(N-1)
      tmp1h = T_p1
      tmp1l = (DELTAh(N-1) - (T_p1 - T_p2))
     +      + (DELTAh(N) - T_p2)
      tmp1l = tmp1l + DELTAl(N-1) + DELTAl(N)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DELTA(N-1) + DELTA(N)
!<<<  tmp2 = tmp1 * W
      tmp2h = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * W
!<<<  tmp3 = DELTA(N-1) * DELTA(N)
      tmp3h = DELTAh(N-1) * DELTAh(N)
      T_u1 = 134217729.0 * DELTAh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N-1))
      T_p2 = DELTAh(N-1) - T_p1
      T_u1 = 134217729.0 * DELTAh(N)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_r2 = DELTAh(N) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(N-1)*DELTAl(N)
     +      + DELTAl(N-1)*DELTAh(N))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(N-1) * DELTA(N)
!<<<  tmp4 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp4h = T_p1
      tmp4l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp4l = tmp4l + DPSIl + DPHIl
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = DPSI + DPHI
!<<<  tmp5 = tmp3 * tmp4
      tmp5h = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp3h*tmp4l
     +      + tmp3l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
!<<<  A = tmp2 - tmp5
      T_p1 = tmp2h - tmp5h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Al = Al + tmp2l - tmp5l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  A = tmp2 - tmp5
!         B = DELTA( N-1 )*DELTA( N )*W
!<<<  tmp1 = DELTA(N-1) * DELTA(N)
      tmp1h = DELTAh(N-1) * DELTAh(N)
      T_u1 = 134217729.0 * DELTAh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N-1))
      T_p2 = DELTAh(N-1) - T_p1
      T_u1 = 134217729.0 * DELTAh(N)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_r2 = DELTAh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(N-1)*DELTAl(N)
     +      + DELTAl(N-1)*DELTAh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(N-1) * DELTA(N)
!<<<  B = tmp1 * W
      Bh = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * W
         IF(
!            C.LT.ZERO
!       (C .lt. ZERO)
     + ((Ch .lt. ZEROh) .or.
     +  ((Ch .eq. ZEROh) .and.
     +   (Cl .lt. ZEROl)))
     $      ) THEN
!     $      C = ABS( C )
!<<<  C = abs(C)
      if (Ch .lt. 0.0d+0) then
        Ch = -Ch
        Cl = -Cl
      elseif (
     +  (Ch .eq. 0.0d+0).and.
     +  (Cl .lt. 0.0d+0)) then
        Ch = Ch
        Cl = -Cl
      else
        Ch = Ch
        Cl = Cl
      endif
!>>>  C = abs(C)
         END IF
         IF(
!            C.EQ.ZERO
!       (C .eq. ZERO)
     + ((Ch .eq. ZEROh) .and.
     +  (Cl .eq. ZEROl))
     $      ) THEN
*          ETA = B/A
*           ETA = RHO - TAU
!            ETA = DLTUB - TAU
!<<<  ETA = DLTUB - TAU
      T_p1 = DLTUBh - TAUh
      T_p2 = T_p1 - DLTUBh
      ETAh = T_p1
      ETAl = (DLTUBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      ETAl = ETAl + DLTUBl - TAUl
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  ETA = DLTUB - TAU
         ELSE IF(
!                A.GE.ZERO
!       (A .ge. ZERO)
     + ((Ah .gt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .ge. ZEROl)))
     $           ) THEN
!            ETA = ( A+SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      tmp5l = tmp5l + tmp2l - tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = tmp2 - tmp4
!<<<  tmp6 = abs(tmp5)
      if (tmp5h .lt. 0.0d+0) then
        tmp6h = -tmp5h
        tmp6l = -tmp5l
      elseif (
     +  (tmp5h .eq. 0.0d+0).and.
     +  (tmp5l .lt. 0.0d+0)) then
        tmp6h = tmp5h
        tmp6l = -tmp5l
      else
        tmp6h = tmp5h
        tmp6l = tmp5l
      endif
!>>>  tmp6 = abs(tmp5)
!<<<  tmp7 = sqrt(tmp6)
      if (tmp6h.eq.0.0D+0.and.tmp6l.eq.0.0D+0) then
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
      endif
      if (tmp6h.lt.0.0d+0.or.
     +   (tmp6h.eq.0.0d+0.and.tmp6l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp6h)
      T_ax = tmp6h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp6h,tmp6l,T_z1,T_z2)
          T_p1 = tmp6h - T_z1
          T_p2 = T_p1 - tmp6h
          T_t2l = (tmp6h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp6l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp7h = T_t1h
          tmp7l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = sqrt(tmp6)
!<<<  tmp8 = A + tmp7
      T_p1 = Ah + tmp7h
      T_p2 = T_p1 - Ah
      tmp8h = T_p1
      tmp8l = (Ah - (T_p1 - T_p2))
     +      + (tmp7h - T_p2)
      tmp8l = tmp8l + Al + tmp7l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp8 = A + tmp7
!<<<  ETA = tmp8 / tmp1
      T_q1 = tmp8h / tmp1h
      ETAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp1l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp8h - ETAh
      T_r1 = T_s1 - tmp8h
      T_s2 = (tmp8h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp8l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp8 / tmp1
         ELSE
!            ETA = TWO*B / ( A-SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      tmp5l = tmp5l + tmp2l - tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = tmp2 - tmp4
!<<<  tmp6 = abs(tmp5)
      if (tmp5h .lt. 0.0d+0) then
        tmp6h = -tmp5h
        tmp6l = -tmp5l
      elseif (
     +  (tmp5h .eq. 0.0d+0).and.
     +  (tmp5l .lt. 0.0d+0)) then
        tmp6h = tmp5h
        tmp6l = -tmp5l
      else
        tmp6h = tmp5h
        tmp6l = tmp5l
      endif
!>>>  tmp6 = abs(tmp5)
!<<<  tmp7 = sqrt(tmp6)
      if (tmp6h.eq.0.0D+0.and.tmp6l.eq.0.0D+0) then
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
      endif
      if (tmp6h.lt.0.0d+0.or.
     +   (tmp6h.eq.0.0d+0.and.tmp6l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp6h)
      T_ax = tmp6h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp6h,tmp6l,T_z1,T_z2)
          T_p1 = tmp6h - T_z1
          T_p2 = T_p1 - tmp6h
          T_t2l = (tmp6h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp6l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp7h = T_t1h
          tmp7l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = sqrt(tmp6)
!<<<  tmp8 = A - tmp7
      T_p1 = Ah - tmp7h
      T_p2 = T_p1 - Ah
      tmp8h = T_p1
      tmp8l = (Ah - (T_p1 - T_p2))
     +         - (tmp7h + T_p2)
      tmp8l = tmp8l + Al - tmp7l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp8 = A - tmp7
!<<<  ETA = tmp1 / tmp8
      T_q1 = tmp1h / tmp8h
      ETAh = tmp8h * T_q1
      T_u1 = 134217729.0 * tmp8h
      T_p1 = T_u1
     +         - (T_u1 - tmp8h)
      T_p2 = tmp8h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp8l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp8h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / tmp8
         END IF
*
*        Note, eta should be positive if w is negative, and
*        eta should be negative otherwise. However,
*        if for some reason caused by roundoff, eta*w > 0,
*        we simply use one Newton step instead. This way
*        will guarantee eta*w < 0.
*
!<<<  tmp1 = W * ETA
      tmp1h = Wh * ETAh
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * ETAh
      T_r1 = T_u1
     +         - (T_u1 - ETAh)
      T_r2 = ETAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Wh*ETAl
     +      + Wl*ETAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = W * ETA
         IF(
!            W*ETA.GT.ZERO
!       (tmp1 .gt. ZERO)
     + ((tmp1h .gt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .gt. ZEROl)))
     $      ) THEN
!     $      ETA = -W / ( DPSI+DPHI )
!<<<  tmp2 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + DPSIl + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + DPHI
!<<<  tmp3 = W / tmp2
      T_q1 = Wh / tmp2h
      tmp3h = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (tmp2l*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Wh - tmp3h
      T_r1 = T_s1 - Wh
      T_s2 = (Wh 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Wl
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = W / tmp2
!<<<  ETA = -tmp3
      ETAh = -tmp3h
      ETAl = -tmp3l
!>>>  ETA = -tmp3
         END IF
!         TEMP = TAU + ETA
!<<<  TEMP = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      TEMPh = T_p1
      TEMPl = (TAUh - (T_p1 - T_p2))
     +      + (ETAh - T_p2)
      TEMPl = TEMPl + TAUl + ETAl
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = TAU + ETA
         IF(
!           TEMP.GT.DLTUB 
!       (TEMP .gt. DLTUB)
     + ((TEMPh .gt. DLTUBh) .or.
     +  ((TEMPh .eq. DLTUBh) .and.
     +   (TEMPl .gt. DLTUBl)))
     $      .OR. 
!           TEMP.LT.DLTLB 
!       (TEMP .lt. DLTLB)
     + ((TEMPh .lt. DLTLBh) .or.
     +  ((TEMPh .eq. DLTLBh) .and.
     +   (TEMPl .lt. DLTLBl)))
     $       ) THEN
            IF( 
!               W.LT.ZERO
!       (W .lt. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .lt. ZEROl)))
     $         ) THEN
!               ETA = ( DLTUB-TAU ) / TWO
!<<<  tmp1 = DLTUB - TAU
      T_p1 = DLTUBh - TAUh
      T_p2 = T_p1 - DLTUBh
      tmp1h = T_p1
      tmp1l = (DLTUBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTUBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTUB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
            ELSE
!               ETA = ( DLTLB-TAU ) / TWO
!<<<  tmp1 = DLTLB - TAU
      T_p1 = DLTLBh - TAUh
      T_p2 = T_p1 - DLTLBh
      tmp1h = T_p1
      tmp1l = (DLTLBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTLBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTLB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
            END IF
         END IF
         DO 50 J = 1, N
!            DELTA( J ) = DELTA( J ) - ETA
!<<<  DELTA(J) = DELTA(J) - ETA
      T_p1 = DELTAh(J) - ETAh
      T_p2 = T_p1 - DELTAh(J)
      T_ch = T_p1
      T_cl = (DELTAh(J) - (T_p1 - T_p2))
     +         - (ETAh + T_p2)
      T_cl = T_cl + DELTAl(J) - ETAl
      T_s1 = T_ch + T_cl
      DELTAl(J) = T_cl - (T_s1 - T_ch)
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DELTA(J) = DELTA(J) - ETA
   50    CONTINUE
*
!         TAU = TAU + ETA
!<<<  TAU = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      T_q1 = T_p1
      T_q2 = (TAUh - (T_p1 - T_p2))
     +         + (ETAh - T_p2)
      T_q2 = T_q2 + TAUl + ETAl
      TAUh = T_q1 + T_q2
      TAUl = T_q2 - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TAU = TAU + ETA
*
*        Evaluate PSI and the derivative DPSI
*
!         DPSI = ZERO
      DPSIh = ZEROh
      DPSIl = ZEROl
!         PSI = ZERO
      PSIh = ZEROh
      PSIl = ZEROl
!         ERRETM = ZERO
      ERRETMh = ZEROh
      ERRETMl = ZEROl
         DO 60 J = 1, II
!            TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!            PSI = PSI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PSI = PSI + tmp1
      T_p1 = PSIh + tmp1h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PSIl + tmp1l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp1
!            DPSI = DPSI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp2
      T_p1 = DPSIh + tmp2h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp2l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp2
!            ERRETM = ERRETM + PSI
!<<<  ERRETM = ERRETM + PSI
      T_p1 = ERRETMh + PSIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PSIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PSI
   60    CONTINUE
!         ERRETM = ABS( ERRETM )
!<<<  ERRETM = abs(ERRETM)
      if (ERRETMh .lt. 0.0d+0) then
        ERRETMh = -ERRETMh
        ERRETMl = -ERRETMl
      elseif (
     +  (ERRETMh .eq. 0.0d+0).and.
     +  (ERRETMl .lt. 0.0d+0)) then
        ERRETMh = ERRETMh
        ERRETMl = -ERRETMl
      else
        ERRETMh = ERRETMh
        ERRETMl = ERRETMl
      endif
!>>>  ERRETM = abs(ERRETM)
*
*        Evaluate PHI and the derivative DPHI
*
!         TEMP = Z( N ) / DELTA( N )
!<<<  TEMP = Z(N) / DELTA(N)
      T_q1 = Zh(N) / DELTAh(N)
      TEMPh = DELTAh(N) * T_q1
      T_u1 = 134217729.0 * DELTAh(N)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_p2 = DELTAh(N) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(N)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(N) - TEMPh
      T_r1 = T_s1 - Zh(N)
      T_s2 = (Zh(N) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(N)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(N)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(N) / DELTA(N)
!         PHI = Z( N )*TEMP
!<<<  PHI = Z(N) * TEMP
      PHIh = Zh(N) * TEMPh
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      PHIl = ((T_p1*T_r1
     +        - PHIh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      PHIl = PHIl
     +      + (Zh(N)*TEMPl
     +      + Zl(N)*TEMPh)
      T_s1 = PHIh + PHIl
      PHIl = PHIl - (T_s1 - PHIh)
      PHIh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  PHI = Z(N) * TEMP
!         DPHI = TEMP*TEMP
!<<<  DPHI = TEMP * TEMP
      DPHIh = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      DPHIl = ((T_p1*T_r1
     +        - DPHIh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      DPHIl = DPHIl
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = DPHIh + DPHIl
      DPHIl = DPHIl - (T_s1 - DPHIh)
      DPHIh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  DPHI = TEMP * TEMP
!         ERRETM = EIGHT*( -PHI-PSI ) + ERRETM - PHI + RHOINV +
!     $            ABS( TAU )*( DPSI+DPHI )
!<<<  tmp1 = PHI + PSI
      T_p1 = PHIh + PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      tmp1l = tmp1l + PHIl + PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = PHI + PSI
!<<<  tmp1 = -tmp1
      tmp1h = -tmp1h
      tmp1l = -tmp1l
!>>>  tmp1 = -tmp1
!<<<  tmp2 = EIGHT * tmp1
      tmp2h = EIGHTh * tmp1h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EIGHTh*tmp1l
     +      + EIGHTl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EIGHT * tmp1
!<<<  tmp3 = abs(TAU)
      if (TAUh .lt. 0.0d+0) then
        tmp3h = -TAUh
        tmp3l = -TAUl
      elseif (
     +  (TAUh .eq. 0.0d+0).and.
     +  (TAUl .lt. 0.0d+0)) then
        tmp3h = TAUh
        tmp3l = -TAUl
      else
        tmp3h = TAUh
        tmp3l = TAUl
      endif
!>>>  tmp3 = abs(TAU)
!<<<  tmp4 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp4h = T_p1
      tmp4l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp4l = tmp4l + DPSIl + DPHIl
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = DPSI + DPHI
!<<<  tmp5 = tmp3 * tmp4
      tmp5h = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp3h*tmp4l
     +      + tmp3l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
!<<<  tmp6 = tmp2 + ERRETM
      T_p1 = tmp2h + ERRETMh
      T_p2 = T_p1 - tmp2h
      tmp6h = T_p1
      tmp6l = (tmp2h - (T_p1 - T_p2))
     +      + (ERRETMh - T_p2)
      tmp6l = tmp6l + tmp2l + ERRETMl
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = tmp2 + ERRETM
!<<<  tmp7 = tmp6 - PHI
      T_p1 = tmp6h - PHIh
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +         - (PHIh + T_p2)
      tmp7l = tmp7l + tmp6l - PHIl
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = tmp6 - PHI
!<<<  tmp8 = tmp7 + RHOINV
      T_p1 = tmp7h + RHOINVh
      T_p2 = T_p1 - tmp7h
      tmp8h = T_p1
      tmp8l = (tmp7h - (T_p1 - T_p2))
     +      + (RHOINVh - T_p2)
      tmp8l = tmp8l + tmp7l + RHOINVl
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp8 = tmp7 + RHOINV
!<<<  ERRETM = tmp8 + tmp5
      T_p1 = tmp8h + tmp5h
      T_p2 = T_p1 - tmp8h
      ERRETMh = T_p1
      ERRETMl = (tmp8h - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      ERRETMl = ERRETMl + tmp8l + tmp5l
      T_s1 = ERRETMh + ERRETMl
      ERRETMl = ERRETMl - (T_s1 - ERRETMh)
      ERRETMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  ERRETM = tmp8 + tmp5
*
!         W = RHOINV + PHI + PSI
!<<<  tmp1 = PHI + PSI
      T_p1 = PHIh + PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      tmp1l = tmp1l + PHIl + PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = PHI + PSI
!<<<  W = RHOINV + tmp1
      T_p1 = RHOINVh + tmp1h
      T_p2 = T_p1 - RHOINVh
      Wh = T_p1
      Wl = (RHOINVh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Wl = Wl + RHOINVl + tmp1l
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = RHOINV + tmp1
*
*        Main loop to update the values of the array   DELTA
*
         ITER = NITER + 1
*
         DO 90 NITER = ITER, MAXIT
*
*           Test for convergence
*
!<<<  tmp1 = abs(W)
      if (Wh .lt. 0.0d+0) then
        tmp1h = -Wh
        tmp1l = -Wl
      elseif (
     +  (Wh .eq. 0.0d+0).and.
     +  (Wl .lt. 0.0d+0)) then
        tmp1h = Wh
        tmp1l = -Wl
      else
        tmp1h = Wh
        tmp1l = Wl
      endif
!>>>  tmp1 = abs(W)
!<<<  tmp2 = EPS * ERRETM
      tmp2h = EPSh * ERRETMh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * ERRETMh
      T_r1 = T_u1
     +         - (T_u1 - ERRETMh)
      T_r2 = ERRETMh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EPSh*ERRETMl
     +      + EPSl*ERRETMh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EPS * ERRETM
            IF( 
!              ABS( W ).LE.EPS*ERRETM
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     &         ) THEN
!               DLAM = D( I ) + TAU
!<<<  DLAM = D(I) + TAU
      T_p1 = Dh(I) + TAUh
      T_p2 = T_p1 - Dh(I)
      DLAMh = T_p1
      DLAMl = (Dh(I) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(I) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(I) + TAU
               GO TO 250
            END IF
*
            IF(
!              W.LE.ZERO
!       (W .le. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .le. ZEROl)))
     &         ) THEN
!               DLTLB = MAX( DLTLB, TAU )
                IF(
!       (DLTLB .ge. TAU)
     + ((DLTLBh .gt. TAUh) .or.
     +  ((DLTLBh .eq. TAUh) .and.
     +   (DLTLBl .ge. TAUl)))
     &         ) THEN
      DLTLBh = DLTLBh
      DLTLBl = DLTLBl
                ELSE
      DLTLBh = TAUh
      DLTLBl = TAUl
                END IF
            ELSE
!               DLTUB = MIN( DLTUB, TAU )
                IF(
!       (DLTUB .le. TAU)
     + ((DLTUBh .lt. TAUh) .or.
     +  ((DLTUBh .eq. TAUh) .and.
     +   (DLTUBl .le. TAUl)))
     &         ) THEN
      DLTUBh = DLTUBh
      DLTUBl = DLTUBl
                ELSE
      DLTUBh = TAUh
      DLTUBl = TAUl
                END IF
            END IF
*
*           Calculate the new step
*
!            C = W - DELTA( N-1 )*DPSI - DELTA( N )*DPHI
!<<<  tmp1 = DELTA(N-1) * DPSI
      tmp1h = DELTAh(N-1) * DPSIh
      T_u1 = 134217729.0 * DELTAh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N-1))
      T_p2 = DELTAh(N-1) - T_p1
      T_u1 = 134217729.0 * DPSIh
      T_r1 = T_u1
     +         - (T_u1 - DPSIh)
      T_r2 = DPSIh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(N-1)*DPSIl
     +      + DELTAl(N-1)*DPSIh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(N-1) * DPSI
!<<<  tmp2 = DELTA(N) * DPHI
      tmp2h = DELTAh(N) * DPHIh
      T_u1 = 134217729.0 * DELTAh(N)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_p2 = DELTAh(N) - T_p1
      T_u1 = 134217729.0 * DPHIh
      T_r1 = T_u1
     +         - (T_u1 - DPHIh)
      T_r2 = DPHIh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(N)*DPHIl
     +      + DELTAl(N)*DPHIh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(N) * DPHI
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  C = W - tmp3
      T_p1 = Wh - tmp3h
      T_p2 = T_p1 - Wh
      Ch = T_p1
      Cl = (Wh - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      Cl = Cl + Wl - tmp3l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = W - tmp3
!            A = ( DELTA( N-1 )+DELTA( N ) )*W -
!     $          DELTA( N-1 )*DELTA( N )*( DPSI+DPHI )
!<<<  tmp1 = DELTA(N-1) + DELTA(N)
      T_p1 = DELTAh(N-1) + DELTAh(N)
      T_p2 = T_p1 - DELTAh(N-1)
      tmp1h = T_p1
      tmp1l = (DELTAh(N-1) - (T_p1 - T_p2))
     +      + (DELTAh(N) - T_p2)
      tmp1l = tmp1l + DELTAl(N-1) + DELTAl(N)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DELTA(N-1) + DELTA(N)
!<<<  tmp2 = tmp1 * W
      tmp2h = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * W
!<<<  tmp3 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp3h = T_p1
      tmp3l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp3l = tmp3l + DPSIl + DPHIl
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = DPSI + DPHI
!<<<  tmp4 = DELTA(N-1) * DELTA(N)
      tmp4h = DELTAh(N-1) * DELTAh(N)
      T_u1 = 134217729.0 * DELTAh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N-1))
      T_p2 = DELTAh(N-1) - T_p1
      T_u1 = 134217729.0 * DELTAh(N)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_r2 = DELTAh(N) - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (DELTAh(N-1)*DELTAl(N)
     +      + DELTAl(N-1)*DELTAh(N))
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = DELTA(N-1) * DELTA(N)
!<<<  tmp5 = tmp4 * tmp3
      tmp5h = tmp4h * tmp3h
      T_u1 = 134217729.0 * tmp4h
      T_p1 = T_u1
     +         - (T_u1 - tmp4h)
      T_p2 = tmp4h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp4h*tmp3l
     +      + tmp4l*tmp3h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp4 * tmp3
!<<<  A = tmp2 - tmp5
      T_p1 = tmp2h - tmp5h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Al = Al + tmp2l - tmp5l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  A = tmp2 - tmp5
!            B = DELTA( N-1 )*DELTA( N )*W
!<<<  tmp1 = DELTA(N-1) * DELTA(N)
      tmp1h = DELTAh(N-1) * DELTAh(N)
      T_u1 = 134217729.0 * DELTAh(N-1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N-1))
      T_p2 = DELTAh(N-1) - T_p1
      T_u1 = 134217729.0 * DELTAh(N)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_r2 = DELTAh(N) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(N-1)*DELTAl(N)
     +      + DELTAl(N-1)*DELTAh(N))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(N-1) * DELTA(N)
!<<<  B = tmp1 * W
      Bh = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * W
            IF( 
!               A.GE.ZERO
!       (A .ge. ZERO)
     + ((Ah .gt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .ge. ZEROl)))
     &          ) THEN
!               ETA = ( A+SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A + tmp6
      T_p1 = Ah + tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Al + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = A + tmp6
!<<<  ETA = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      ETAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp1l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp7h - ETAh
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp7 / tmp1
            ELSE
!               ETA = TWO*B / ( A-SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A - tmp6
      T_p1 = Ah - tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      tmp7l = tmp7l + Al - tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = A - tmp6
!<<<  ETA = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      ETAh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp7l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / tmp7

            END IF
*
*           Note, eta should be positive if w is negative, and
*           eta should be negative otherwise. However,
*           if for some reason caused by roundoff, eta*w > 0,
*           we simply use one Newton step instead. This way
*           will guarantee eta*w < 0.
*
!<<<  tmp1 = W * ETA
      tmp1h = Wh * ETAh
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * ETAh
      T_r1 = T_u1
     +         - (T_u1 - ETAh)
      T_r2 = ETAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Wh*ETAl
     +      + Wl*ETAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = W * ETA
            IF(
!               W*ETA.GT.ZERO
!       (tmp1 .gt. ZERO)
     + ((tmp1h .gt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .gt. ZEROl)))
     $          ) THEN
!     $         ETA = -W / ( DPSI+DPHI )
!<<<  tmp1 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp1l = tmp1l + DPSIl + DPHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DPSI + DPHI
!<<<  tmp2 = W / tmp1
      T_q1 = Wh / tmp1h
      tmp2h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (tmp1l*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = Wh - tmp2h
      T_r1 = T_s1 - Wh
      T_s2 = (Wh 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + Wl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = W / tmp1
!<<<  ETA = -tmp2
      ETAh = -tmp2h
      ETAl = -tmp2l
!>>>  ETA = -tmp2
             END IF
!            TEMP = TAU + ETA
!<<<  TEMP = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      TEMPh = T_p1
      TEMPl = (TAUh - (T_p1 - T_p2))
     +      + (ETAh - T_p2)
      TEMPl = TEMPl + TAUl + ETAl
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = TAU + ETA
            IF(
!               TEMP.GT.DLTUB 
!       (TEMP .gt. DLTUB)
     + ((TEMPh .gt. DLTUBh) .or.
     +  ((TEMPh .eq. DLTUBh) .and.
     +   (TEMPl .gt. DLTUBl)))
     $         .OR. 
!               TEMP.LT.DLTLB
!       (TEMP .lt. DLTLB)
     + ((TEMPh .lt. DLTLBh) .or.
     +  ((TEMPh .eq. DLTLBh) .and.
     +   (TEMPl .lt. DLTLBl)))
     $          ) THEN
               IF(
!                 W.LT.ZERO
!       (W .lt. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .lt. ZEROl)))
     $          ) THEN
!                  ETA = ( DLTUB-TAU ) / TWO
!<<<  tmp1 = DLTUB - TAU
      T_p1 = DLTUBh - TAUh
      T_p2 = T_p1 - DLTUBh
      tmp1h = T_p1
      tmp1l = (DLTUBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTUBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTUB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
               ELSE
!                  ETA = ( DLTLB-TAU ) / TWO
!<<<  tmp1 = DLTLB - TAU
      T_p1 = DLTLBh - TAUh
      T_p2 = T_p1 - DLTLBh
      tmp1h = T_p1
      tmp1l = (DLTLBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTLBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTLB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
               END IF
            END IF
            DO 70 J = 1, N
!               DELTA( J ) = DELTA( J ) - ETA
!<<<  DELTA(J) = DELTA(J) - ETA
      T_p1 = DELTAh(J) - ETAh
      T_p2 = T_p1 - DELTAh(J)
      T_ch = T_p1
      T_cl = (DELTAh(J) - (T_p1 - T_p2))
     +         - (ETAh + T_p2)
      T_cl = T_cl + DELTAl(J) - ETAl
      T_s1 = T_ch + T_cl
      DELTAl(J) = T_cl - (T_s1 - T_ch)
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DELTA(J) = DELTA(J) - ETA
   70       CONTINUE
*
!            TAU = TAU + ETA
!<<<  TAU = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      T_q1 = T_p1
      T_q2 = (TAUh - (T_p1 - T_p2))
     +         + (ETAh - T_p2)
      T_q2 = T_q2 + TAUl + ETAl
      TAUh = T_q1 + T_q2
      TAUl = T_q2 - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TAU = TAU + ETA
*
*           Evaluate PSI and the derivative DPSI
*
!            DPSI = ZERO
      DPSIh = ZEROh
      DPSIl = ZEROl
!            PSI = ZERO
      PSIh = ZEROh
      PSIl = ZEROl
!            ERRETM = ZERO
      ERRETMh = ZEROh
      ERRETMl = ZEROl
            DO 80 J = 1, II
!               TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!               PSI = PSI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PSI = PSI + tmp1
      T_p1 = PSIh + tmp1h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PSIl + tmp1l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp1
!               DPSI = DPSI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp2
      T_p1 = DPSIh + tmp2h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp2l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp2
!               ERRETM = ERRETM + PSI
!<<<  ERRETM = ERRETM + PSI
      T_p1 = ERRETMh + PSIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PSIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PSI
   80       CONTINUE
!            ERRETM = ABS( ERRETM )
!<<<  ERRETM = abs(ERRETM)
      if (ERRETMh .lt. 0.0d+0) then
        ERRETMh = -ERRETMh
        ERRETMl = -ERRETMl
      elseif (
     +  (ERRETMh .eq. 0.0d+0).and.
     +  (ERRETMl .lt. 0.0d+0)) then
        ERRETMh = ERRETMh
        ERRETMl = -ERRETMl
      else
        ERRETMh = ERRETMh
        ERRETMl = ERRETMl
      endif
!>>>  ERRETM = abs(ERRETM)
*
*           Evaluate PHI and the derivative DPHI
*
!            TEMP = Z( N ) / DELTA( N )
!<<<  TEMP = Z(N) / DELTA(N)
      T_q1 = Zh(N) / DELTAh(N)
      TEMPh = DELTAh(N) * T_q1
      T_u1 = 134217729.0 * DELTAh(N)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(N))
      T_p2 = DELTAh(N) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(N)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(N) - TEMPh
      T_r1 = T_s1 - Zh(N)
      T_s2 = (Zh(N) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(N)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(N)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(N) / DELTA(N)
!            PHI = Z( N )*TEMP
!<<<  PHI = Z(N) * TEMP
      PHIh = Zh(N) * TEMPh
      T_u1 = 134217729.0 * Zh(N)
      T_p1 = T_u1
     +         - (T_u1 - Zh(N))
      T_p2 = Zh(N) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      PHIl = ((T_p1*T_r1
     +        - PHIh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      PHIl = PHIl
     +      + (Zh(N)*TEMPl
     +      + Zl(N)*TEMPh)
      T_s1 = PHIh + PHIl
      PHIl = PHIl - (T_s1 - PHIh)
      PHIh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  PHI = Z(N) * TEMP
!            DPHI = TEMP*TEMP
!<<<  DPHI = TEMP * TEMP
      DPHIh = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      DPHIl = ((T_p1*T_r1
     +        - DPHIh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      DPHIl = DPHIl
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = DPHIh + DPHIl
      DPHIl = DPHIl - (T_s1 - DPHIh)
      DPHIh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  DPHI = TEMP * TEMP
!            ERRETM = EIGHT*( -PHI-PSI ) + ERRETM - PHI + RHOINV +
!     $               ABS( TAU )*( DPSI+DPHI )
!<<<  tmp1 = PHI + PSI
      T_p1 = PHIh + PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      tmp1l = tmp1l + PHIl + PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = PHI + PSI
!<<<  tmp1 = -tmp1
      tmp1h = -tmp1h
      tmp1l = -tmp1l
!>>>  tmp1 = -tmp1
!<<<  tmp2 = EIGHT * tmp1
      tmp2h = EIGHTh * tmp1h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EIGHTh*tmp1l
     +      + EIGHTl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EIGHT * tmp1
!<<<  tmp3 = abs(TAU)
      if (TAUh .lt. 0.0d+0) then
        tmp3h = -TAUh
        tmp3l = -TAUl
      elseif (
     +  (TAUh .eq. 0.0d+0).and.
     +  (TAUl .lt. 0.0d+0)) then
        tmp3h = TAUh
        tmp3l = -TAUl
      else
        tmp3h = TAUh
        tmp3l = TAUl
      endif
!>>>  tmp3 = abs(TAU)
!<<<  tmp4 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp4h = T_p1
      tmp4l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp4l = tmp4l + DPSIl + DPHIl
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = DPSI + DPHI
!<<<  tmp5 = tmp3 * tmp4
      tmp5h = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp3h*tmp4l
     +      + tmp3l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
!<<<  tmp6 = tmp2 + ERRETM
      T_p1 = tmp2h + ERRETMh
      T_p2 = T_p1 - tmp2h
      tmp6h = T_p1
      tmp6l = (tmp2h - (T_p1 - T_p2))
     +      + (ERRETMh - T_p2)
      tmp6l = tmp6l + tmp2l + ERRETMl
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = tmp2 + ERRETM
!<<<  tmp7 = tmp6 - PHI
      T_p1 = tmp6h - PHIh
      T_p2 = T_p1 - tmp6h
      tmp7h = T_p1
      tmp7l = (tmp6h - (T_p1 - T_p2))
     +         - (PHIh + T_p2)
      tmp7l = tmp7l + tmp6l - PHIl
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = tmp6 - PHI
!<<<  tmp8 = tmp7 + RHOINV
      T_p1 = tmp7h + RHOINVh
      T_p2 = T_p1 - tmp7h
      tmp8h = T_p1
      tmp8l = (tmp7h - (T_p1 - T_p2))
     +      + (RHOINVh - T_p2)
      tmp8l = tmp8l + tmp7l + RHOINVl
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp8 = tmp7 + RHOINV
!<<<  ERRETM = tmp8 + tmp5
      T_p1 = tmp8h + tmp5h
      T_p2 = T_p1 - tmp8h
      ERRETMh = T_p1
      ERRETMl = (tmp8h - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      ERRETMl = ERRETMl + tmp8l + tmp5l
      T_s1 = ERRETMh + ERRETMl
      ERRETMl = ERRETMl - (T_s1 - ERRETMh)
      ERRETMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  ERRETM = tmp8 + tmp5
*
!            W = RHOINV + PHI + PSI
!<<<  tmp1 = RHOINV + PHI
      T_p1 = RHOINVh + PHIh
      T_p2 = T_p1 - RHOINVh
      tmp1h = T_p1
      tmp1l = (RHOINVh - (T_p1 - T_p2))
     +      + (PHIh - T_p2)
      tmp1l = tmp1l + RHOINVl + PHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = RHOINV + PHI
!<<<  W = tmp1 + PSI
      T_p1 = tmp1h + PSIh
      T_p2 = T_p1 - tmp1h
      Wh = T_p1
      Wl = (tmp1h - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      Wl = Wl + tmp1l + PSIl
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = tmp1 + PSI
   90    CONTINUE
*
*        Return with INFO = 1, NITER = MAXIT and not converged
*
         INFO = 1
!         DLAM = D( I ) + TAU
!<<<  DLAM = D(I) + TAU
      T_p1 = Dh(I) + TAUh
      T_p2 = T_p1 - Dh(I)
      DLAMh = T_p1
      DLAMl = (Dh(I) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(I) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(I) + TAU
         GO TO 250
*
*        End for the case I = N
*
      ELSE
*
*        The case for I < N
*
         NITER = 1
         IP1 = I + 1
*
*        Calculate initial guess
*
!         DEL = D( IP1 ) - D( I )
!<<<  DEL = D(IP1) - D(I)
      T_p1 = Dh(IP1) - Dh(I)
      T_p2 = T_p1 - Dh(IP1)
      DELh = T_p1
      DELl = (Dh(IP1) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      DELl = DELl + Dl(IP1) - Dl(I)
      T_s1 = DELh + DELl
      DELl = DELl - (T_s1 - DELh)
      DELh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DEL = D(IP1) - D(I)
!         MIDPT = DEL / TWO
!<<<  MIDPT = DEL / TWO
      T_q1 = DELh / TWOh
      MIDPTh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      MIDPTl = ((T_p1*T_r1
     +       - MIDPTh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      MIDPTl = MIDPTl + (TWOl*T_q1)
      T_s1 = MIDPTh + MIDPTl
      MIDPTl = MIDPTl - (T_s1 - MIDPTh)
      MIDPTh = T_s1
      T_s1 = DELh - MIDPTh
      T_r1 = T_s1 - DELh
      T_s2 = (DELh 
     +         - (T_s1 - T_r1))
     +         - (MIDPTh + T_r1)
      T_s2 = T_s2 - MIDPTl
      T_s2 = T_s2 + DELl
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      MIDPTh = T_q1 + T_q2
      MIDPTl = T_q2
     +      - (MIDPTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  MIDPT = DEL / TWO
         DO 100 J = 1, N
!            DELTA( J ) = ( D( J )-D( I ) ) - MIDPT
!<<<  tmp1 = D(J) - D(I)
      T_p1 = Dh(J) - Dh(I)
      T_p2 = T_p1 - Dh(J)
      tmp1h = T_p1
      tmp1l = (Dh(J) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      tmp1l = tmp1l + Dl(J) - Dl(I)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(J) - D(I)
!<<<  DELTA(J) = tmp1 - MIDPT
      T_p1 = tmp1h - MIDPTh
      T_p2 = T_p1 - tmp1h
      DELTAh(J) = T_p1
      DELTAl(J) = (tmp1h - (T_p1 - T_p2))
     +         - (MIDPTh + T_p2)
      DELTAl(J) = DELTAl(J) + tmp1l - MIDPTl
      T_s1 = DELTAh(J) + DELTAl(J)
      DELTAl(J) = DELTAl(J) - (T_s1 - DELTAh(J))
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DELTA(J) = tmp1 - MIDPT
  100    CONTINUE
*
!         PSI = ZERO
      PSIh = ZEROh
      PSIl = ZEROl
         DO 110 J = 1, I - 1
!            PSI = PSI + Z( J )*Z( J ) / DELTA( J )
!<<<  tmp1 = Z(J) * Z(J)
      tmp1h = Zh(J) * Zh(J)
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * Zh(J)
      T_r1 = T_u1
     +         - (T_u1 - Zh(J))
      T_r2 = Zh(J) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*Zl(J)
     +      + Zl(J)*Zh(J))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * Z(J)
!<<<  tmp2 = tmp1 / DELTA(J)
      T_q1 = tmp1h / DELTAh(J)
      tmp2h = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (DELTAl(J)*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / DELTA(J)
!<<<  PSI = PSI + tmp2
      T_p1 = PSIh + tmp2h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + PSIl + tmp2l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp2
  110    CONTINUE
*
!         PHI = ZERO
      PHIh = ZEROh
      PHIl = ZEROl
         DO 120 J = N, I + 2, -1
!            PHI = PHI + Z( J )*Z( J ) / DELTA( J )
!<<<  tmp1 = Z(J) * Z(J)
      tmp1h = Zh(J) * Zh(J)
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * Zh(J)
      T_r1 = T_u1
     +         - (T_u1 - Zh(J))
      T_r2 = Zh(J) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*Zl(J)
     +      + Zl(J)*Zh(J))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * Z(J)
!<<<  tmp2 = tmp1 / DELTA(J)
      T_q1 = tmp1h / DELTAh(J)
      tmp2h = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (DELTAl(J)*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / DELTA(J)
!<<<  PHI = PHI + tmp2
      T_p1 = PHIh + tmp2h
      T_p2 = T_p1 - PHIh
      T_q1 = T_p1
      T_q2 = (PHIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + PHIl + tmp2l
      PHIh = T_q1 + T_q2
      PHIl = T_q2 - (PHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PHI = PHI + tmp2
  120    CONTINUE
!         C = RHOINV + PSI + PHI
!<<<  tmp1 = RHOINV + PSI
      T_p1 = RHOINVh + PSIh
      T_p2 = T_p1 - RHOINVh
      tmp1h = T_p1
      tmp1l = (RHOINVh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      tmp1l = tmp1l + RHOINVl + PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = RHOINV + PSI
!<<<  C = tmp1 + PHI
      T_p1 = tmp1h + PHIh
      T_p2 = T_p1 - tmp1h
      Ch = T_p1
      Cl = (tmp1h - (T_p1 - T_p2))
     +      + (PHIh - T_p2)
      Cl = Cl + tmp1l + PHIl
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  C = tmp1 + PHI
!         W = C + Z( I )*Z( I ) / DELTA( I ) +
!     $       Z( IP1 )*Z( IP1 ) / DELTA( IP1 )
!<<<  tmp1 = Z(I) * Z(I)
      tmp1h = Zh(I) * Zh(I)
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * Zh(I)
      T_r1 = T_u1
     +         - (T_u1 - Zh(I))
      T_r2 = Zh(I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(I)*Zl(I)
     +      + Zl(I)*Zh(I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(I) * Z(I)
!<<<  tmp2 = tmp1 / DELTA(I)
      T_q1 = tmp1h / DELTAh(I)
      tmp2h = DELTAh(I) * T_q1
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (DELTAl(I)*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(I)
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / DELTA(I)
!<<<  tmp3 = Z(IP1) * Z(IP1)
      tmp3h = Zh(IP1) * Zh(IP1)
      T_u1 = 134217729.0 * Zh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_p2 = Zh(IP1) - T_p1
      T_u1 = 134217729.0 * Zh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_r2 = Zh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(IP1)*Zl(IP1)
     +      + Zl(IP1)*Zh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(IP1) * Z(IP1)
!<<<  tmp4 = tmp3 / DELTA(IP1)
      T_q1 = tmp3h / DELTAh(IP1)
      tmp4h = DELTAh(IP1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp4l = ((T_p1*T_r1
     +       - tmp4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp4l = tmp4l + (DELTAl(IP1)*T_q1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
      T_s1 = tmp3h - tmp4h
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h 
     +         - (T_s1 - T_r1))
     +         - (tmp4h + T_r1)
      T_s2 = T_s2 - tmp4l
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IP1)
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = tmp3 / DELTA(IP1)
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  W = C + tmp5
      T_p1 = Ch + tmp5h
      T_p2 = T_p1 - Ch
      Wh = T_p1
      Wl = (Ch - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      Wl = Wl + Cl + tmp5l
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = C + tmp5
*
         IF(
!            W.GT.ZERO
!       (W .gt. ZERO)
     + ((Wh .gt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .gt. ZEROl)))
     $      ) THEN
*
*           d(i)< the ith eigenvalue < (d(i)+d(i+1))/2
*
*           We choose d(i) as origin.
*
            ORGATI = .TRUE.
!            A = C*DEL + Z( I )*Z( I ) + Z( IP1 )*Z( IP1 )
!<<<  tmp1 = C * DEL
      tmp1h = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*DELl
     +      + Cl*DELh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * DEL
!<<<  tmp2 = Z(I) * Z(I)
      tmp2h = Zh(I) * Zh(I)
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * Zh(I)
      T_r1 = T_u1
     +         - (T_u1 - Zh(I))
      T_r2 = Zh(I) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(I)*Zl(I)
     +      + Zl(I)*Zh(I))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(I) * Z(I)
!<<<  tmp3 = Z(IP1) * Z(IP1)
      tmp3h = Zh(IP1) * Zh(IP1)
      T_u1 = 134217729.0 * Zh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_p2 = Zh(IP1) - T_p1
      T_u1 = 134217729.0 * Zh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_r2 = Zh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(IP1)*Zl(IP1)
     +      + Zl(IP1)*Zh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(IP1) * Z(IP1)
!<<<  tmp4 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp4h = T_p1
      tmp4l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp4l = tmp4l + tmp1l + tmp2l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp1 + tmp2
!<<<  A = tmp4 + tmp3
      T_p1 = tmp4h + tmp3h
      T_p2 = T_p1 - tmp4h
      Ah = T_p1
      Al = (tmp4h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      Al = Al + tmp4l + tmp3l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp4 + tmp3
!            B = Z( I )*Z( I )*DEL
!<<<  tmp1 = Z(I) * Z(I)
      tmp1h = Zh(I) * Zh(I)
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * Zh(I)
      T_r1 = T_u1
     +         - (T_u1 - Zh(I))
      T_r2 = Zh(I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(I)*Zl(I)
     +      + Zl(I)*Zh(I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(I) * Z(I)
!<<<  B = tmp1 * DEL
      Bh = tmp1h * DELh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*DELl
     +      + tmp1l*DELh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * DEL
            IF(
!               A.GT.ZERO
!       (A .gt. ZERO)
     + ((Ah .gt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .gt. ZEROl)))
     $         ) THEN
!               TAU = TWO*B / ( A+SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A + tmp6
      T_p1 = Ah + tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Al + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = A + tmp6
!<<<  TAU = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      TAUh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp7l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp7

            ELSE
!               TAU = ( A-SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A - tmp6
      T_p1 = Ah - tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      tmp7l = tmp7l + Al - tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = A - tmp6
!<<<  TAU = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      TAUh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp1l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp7h - TAUh
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp7 / tmp1
            END IF
!            DLTLB = ZERO
!            DLTUB = MIDPT
      DLTLBh = ZEROh
      DLTLBl = ZEROl
      DLTUBh = MIDPTh
      DLTUBl = MIDPTl
         ELSE
*
*           (d(i)+d(i+1))/2 <= the ith eigenvalue < d(i+1)
*
*           We choose d(i+1) as origin.
*
            ORGATI = .FALSE.
!            A = C*DEL - Z( I )*Z( I ) - Z( IP1 )*Z( IP1 )
!<<<  tmp1 = C * DEL
      tmp1h = Ch * DELh
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*DELl
     +      + Cl*DELh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * DEL
!<<<  tmp2 = Z(I) * Z(I)
      tmp2h = Zh(I) * Zh(I)
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * Zh(I)
      T_r1 = T_u1
     +         - (T_u1 - Zh(I))
      T_r2 = Zh(I) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Zh(I)*Zl(I)
     +      + Zl(I)*Zh(I))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = Z(I) * Z(I)
!<<<  tmp3 = Z(IP1) * Z(IP1)
      tmp3h = Zh(IP1) * Zh(IP1)
      T_u1 = 134217729.0 * Zh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_p2 = Zh(IP1) - T_p1
      T_u1 = 134217729.0 * Zh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_r2 = Zh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(IP1)*Zl(IP1)
     +      + Zl(IP1)*Zh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(IP1) * Z(IP1)
!<<<  tmp4 = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      tmp4h = T_p1
      tmp4l = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      tmp4l = tmp4l + tmp1l - tmp2l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp1 - tmp2
!<<<  A = tmp4 - tmp3
      T_p1 = tmp4h - tmp3h
      T_p2 = T_p1 - tmp4h
      Ah = T_p1
      Al = (tmp4h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      Al = Al + tmp4l - tmp3l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  A = tmp4 - tmp3
!            B = Z( IP1 )*Z( IP1 )*DEL
!<<<  tmp1 = Z(IP1) * Z(IP1)
      tmp1h = Zh(IP1) * Zh(IP1)
      T_u1 = 134217729.0 * Zh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_p2 = Zh(IP1) - T_p1
      T_u1 = 134217729.0 * Zh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_r2 = Zh(IP1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(IP1)*Zl(IP1)
     +      + Zl(IP1)*Zh(IP1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(IP1) * Z(IP1)
!<<<  B = tmp1 * DEL
      Bh = tmp1h * DELh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DELh
      T_r1 = T_u1
     +         - (T_u1 - DELh)
      T_r2 = DELh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*DELl
     +      + tmp1l*DELh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * DEL
            IF(
!              A.LT.ZERO
!       (A .lt. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .lt. ZEROl)))
     $         ) THEN
!               TAU = TWO*B / ( A-SQRT( ABS( A*A+FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp4l = tmp4l + tmp2l + tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp2 + tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A - tmp6
      T_p1 = Ah - tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      tmp7l = tmp7l + Al - tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = A - tmp6
!<<<  TAU = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      TAUh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp7l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp7
            ELSE
!               TAU = -( A+SQRT( ABS( A*A+FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp4l = tmp4l + tmp2l + tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp4 = tmp2 + tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A + tmp6
      T_p1 = Ah + tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Al + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = A + tmp6
!<<<  tmp8 = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      tmp8h = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp8l = ((T_p1*T_r1
     +       - tmp8h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp8l = tmp8l + (tmp1l*T_q1)
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
      T_s1 = tmp7h - tmp8h
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (tmp8h + T_r1)
      T_s2 = T_s2 - tmp8l
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp8h = T_q1 + T_q2
      tmp8l = T_q2
     +      - (tmp8h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp8 = tmp7 / tmp1
!<<<  TAU = -tmp8
      TAUh = -tmp8h
      TAUl = -tmp8l
!>>>  TAU = -tmp8

            END IF
!            DLTLB = -MIDPT
!            DLTUB = ZERO
!<<<  DLTLB = -MIDPT
      DLTLBh = -MIDPTh
      DLTLBl = -MIDPTl
!>>>  DLTLB = -MIDPT
      DLTUBh = ZEROh
      DLTUBl = ZEROl
         END IF
*
         IF( ORGATI ) THEN
            DO 130 J = 1, N
!               DELTA( J ) = ( D( J )-D( I ) ) - TAU
!<<<  tmp1 = D(J) - D(I)
      T_p1 = Dh(J) - Dh(I)
      T_p2 = T_p1 - Dh(J)
      tmp1h = T_p1
      tmp1l = (Dh(J) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      tmp1l = tmp1l + Dl(J) - Dl(I)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(J) - D(I)
!<<<  DELTA(J) = tmp1 - TAU
      T_p1 = tmp1h - TAUh
      T_p2 = T_p1 - tmp1h
      DELTAh(J) = T_p1
      DELTAl(J) = (tmp1h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      DELTAl(J) = DELTAl(J) + tmp1l - TAUl
      T_s1 = DELTAh(J) + DELTAl(J)
      DELTAl(J) = DELTAl(J) - (T_s1 - DELTAh(J))
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DELTA(J) = tmp1 - TAU
  130       CONTINUE
         ELSE
            DO 140 J = 1, N
!               DELTA( J ) = ( D( J )-D( IP1 ) ) - TAU
!<<<  tmp1 = D(J) - D(IP1)
      T_p1 = Dh(J) - Dh(IP1)
      T_p2 = T_p1 - Dh(J)
      tmp1h = T_p1
      tmp1l = (Dh(J) - (T_p1 - T_p2))
     +         - (Dh(IP1) + T_p2)
      tmp1l = tmp1l + Dl(J) - Dl(IP1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(J) - D(IP1)
!<<<  DELTA(J) = tmp1 - TAU
      T_p1 = tmp1h - TAUh
      T_p2 = T_p1 - tmp1h
      DELTAh(J) = T_p1
      DELTAl(J) = (tmp1h - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      DELTAl(J) = DELTAl(J) + tmp1l - TAUl
      T_s1 = DELTAh(J) + DELTAl(J)
      DELTAl(J) = DELTAl(J) - (T_s1 - DELTAh(J))
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DELTA(J) = tmp1 - TAU
  140       CONTINUE
         END IF
         IF( ORGATI ) THEN
            II = I
         ELSE
            II = I + 1
         END IF
         IIM1 = II - 1
         IIP1 = II + 1
*
*        Evaluate PSI and the derivative DPSI
*
!         DPSI = ZERO
!         PSI = ZERO
!         ERRETM = ZERO
      DPSIh = ZEROh
      DPSIl = ZEROl
      PSIh = ZEROh
      PSIl = ZEROl
      ERRETMh = ZEROh
      ERRETMl = ZEROl
         DO 150 J = 1, IIM1
!            TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!            PSI = PSI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PSI = PSI + tmp1
      T_p1 = PSIh + tmp1h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PSIl + tmp1l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp1
!            DPSI = DPSI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp2
      T_p1 = DPSIh + tmp2h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp2l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp2
!            ERRETM = ERRETM + PSI
!<<<  ERRETM = ERRETM + PSI
      T_p1 = ERRETMh + PSIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PSIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PSI
  150    CONTINUE
!         ERRETM = ABS( ERRETM )
!<<<  ERRETM = abs(ERRETM)
      if (ERRETMh .lt. 0.0d+0) then
        ERRETMh = -ERRETMh
        ERRETMl = -ERRETMl
      elseif (
     +  (ERRETMh .eq. 0.0d+0).and.
     +  (ERRETMl .lt. 0.0d+0)) then
        ERRETMh = ERRETMh
        ERRETMl = -ERRETMl
      else
        ERRETMh = ERRETMh
        ERRETMl = ERRETMl
      endif
!>>>  ERRETM = abs(ERRETM)
*
*        Evaluate PHI and the derivative DPHI
*
!         DPHI = ZERO
!         PHI = ZERO
      DPHIh = ZEROh
      DPHIl = ZEROl
      PHIh = ZEROh
      PHIl = ZEROl
         DO 160 J = N, IIP1, -1
!            TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!            PHI = PHI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PHI = PHI + tmp1
      T_p1 = PHIh + tmp1h
      T_p2 = T_p1 - PHIh
      T_q1 = T_p1
      T_q2 = (PHIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PHIl + tmp1l
      PHIh = T_q1 + T_q2
      PHIl = T_q2 - (PHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PHI = PHI + tmp1
!            DPHI = DPHI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPHI = DPHI + tmp2
      T_p1 = DPHIh + tmp2h
      T_p2 = T_p1 - DPHIh
      T_q1 = T_p1
      T_q2 = (DPHIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPHIl + tmp2l
      DPHIh = T_q1 + T_q2
      DPHIl = T_q2 - (DPHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPHI = DPHI + tmp2
!            ERRETM = ERRETM + PHI
!<<<  ERRETM = ERRETM + PHI
      T_p1 = ERRETMh + PHIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PHIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PHIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PHI
  160    CONTINUE
*
!         W = RHOINV + PHI + PSI
!<<<  tmp1 = PHI + PSI
      T_p1 = PHIh + PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      tmp1l = tmp1l + PHIl + PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = PHI + PSI
!<<<  W = RHOINV + tmp1
      T_p1 = RHOINVh + tmp1h
      T_p2 = T_p1 - RHOINVh
      Wh = T_p1
      Wl = (RHOINVh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Wl = Wl + RHOINVl + tmp1l
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = RHOINV + tmp1
*
*        W is the value of the secular function with
*        its ii-th element removed.
*
         SWTCH3 = .FALSE.
         IF( ORGATI ) THEN
            IF(
!               W.LT.ZERO
!       (W .lt. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .lt. ZEROl)))
     $          )
     $         SWTCH3 = .TRUE.
         ELSE
            IF(
!               W.GT.ZERO
!       (W .gt. ZERO)
     + ((Wh .gt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .gt. ZEROl)))
     $           )
     $         SWTCH3 = .TRUE.
         END IF
         IF( II.EQ.1 .OR. II.EQ.N )
     $      SWTCH3 = .FALSE.
*
!         TEMP = Z( II ) / DELTA( II )
!<<<  TEMP = Z(II) / DELTA(II)
      T_q1 = Zh(II) / DELTAh(II)
      TEMPh = DELTAh(II) * T_q1
      T_u1 = 134217729.0 * DELTAh(II)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(II))
      T_p2 = DELTAh(II) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(II)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(II) - TEMPh
      T_r1 = T_s1 - Zh(II)
      T_s2 = (Zh(II) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(II)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(II)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(II) / DELTA(II)
!         DW = DPSI + DPHI + TEMP*TEMP
!<<<  tmp1 = TEMP * TEMP
      tmp1h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP * TEMP
!<<<  tmp2 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + DPSIl + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + DPHI
!<<<  DW = tmp2 + tmp1
      T_p1 = tmp2h + tmp1h
      T_p2 = T_p1 - tmp2h
      DWh = T_p1
      DWl = (tmp2h - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      DWl = DWl + tmp2l + tmp1l
      T_s1 = DWh + DWl
      DWl = DWl - (T_s1 - DWh)
      DWh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DW = tmp2 + tmp1
!         TEMP = Z( II )*TEMP
!<<<  TEMP = Z(II) * TEMP
      T_ch = Zh(II) * TEMPh
      T_u1 = 134217729.0 * Zh(II)
      T_p1 = T_u1
     +         - (T_u1 - Zh(II))
      T_p2 = Zh(II) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(II)*TEMPl
     +         +  Zl(II)*TEMPh)
      T_s1 = T_ch + T_cl
      TEMPl = T_cl - (T_s1 - T_ch)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP = Z(II) * TEMP
!         W = W + TEMP
!<<<  W = W + TEMP
      T_p1 = Wh + TEMPh
      T_p2 = T_p1 - Wh
      T_q1 = T_p1
      T_q2 = (Wh - (T_p1 - T_p2))
     +         + (TEMPh - T_p2)
      T_q2 = T_q2 + Wl + TEMPl
      Wh = T_q1 + T_q2
      Wl = T_q2 - (Wh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W = W + TEMP
!         ERRETM = EIGHT*( PHI-PSI ) + ERRETM + TWO*RHOINV +
!     $            THREE*ABS( TEMP ) + ABS( TAU )*DW
!<<<  tmp1 = PHI - PSI
      T_p1 = PHIh - PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +         - (PSIh + T_p2)
      tmp1l = tmp1l + PHIl - PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = PHI - PSI
!<<<  tmp2 = EIGHT * tmp1
      tmp2h = EIGHTh * tmp1h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EIGHTh*tmp1l
     +      + EIGHTl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EIGHT * tmp1
!<<<  tmp3 = TWO * RHOINV
      tmp3h = TWOh * RHOINVh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RHOINVh
      T_r1 = T_u1
     +         - (T_u1 - RHOINVh)
      T_r2 = RHOINVh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (TWOh*RHOINVl
     +      + TWOl*RHOINVh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = TWO * RHOINV
!<<<  tmp4 = abs(TEMP)
      if (TEMPh .lt. 0.0d+0) then
        tmp4h = -TEMPh
        tmp4l = -TEMPl
      elseif (
     +  (TEMPh .eq. 0.0d+0).and.
     +  (TEMPl .lt. 0.0d+0)) then
        tmp4h = TEMPh
        tmp4l = -TEMPl
      else
        tmp4h = TEMPh
        tmp4l = TEMPl
      endif
!>>>  tmp4 = abs(TEMP)
!<<<  tmp5 = THREE * tmp4
      tmp5h = THREEh * tmp4h
      T_u1 = 134217729.0 * THREEh
      T_p1 = T_u1
     +         - (T_u1 - THREEh)
      T_p2 = THREEh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (THREEh*tmp4l
     +      + THREEl*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = THREE * tmp4
!<<<  tmp6 = abs(TAU)
      if (TAUh .lt. 0.0d+0) then
        tmp6h = -TAUh
        tmp6l = -TAUl
      elseif (
     +  (TAUh .eq. 0.0d+0).and.
     +  (TAUl .lt. 0.0d+0)) then
        tmp6h = TAUh
        tmp6l = -TAUl
      else
        tmp6h = TAUh
        tmp6l = TAUl
      endif
!>>>  tmp6 = abs(TAU)
!<<<  tmp7 = tmp6 * DW
      tmp7h = tmp6h * DWh
      T_u1 = 134217729.0 * tmp6h
      T_p1 = T_u1
     +         - (T_u1 - tmp6h)
      T_p2 = tmp6h - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp7l = ((T_p1*T_r1
     +        - tmp7h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp7l = tmp7l
     +      + (tmp6h*DWl
     +      + tmp6l*DWh)
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp7 = tmp6 * DW
!<<<  tmp8 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp8h = T_p1
      tmp8l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp8l = tmp8l + tmp2l + tmp3l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp8 = tmp2 + tmp3
!<<<  tmp9 = tmp8 + tmp5
      T_p1 = tmp8h + tmp5h
      T_p2 = T_p1 - tmp8h
      tmp9h = T_p1
      tmp9l = (tmp8h - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      tmp9l = tmp9l + tmp8l + tmp5l
      T_s1 = tmp9h + tmp9l
      tmp9l = tmp9l - (T_s1 - tmp9h)
      tmp9h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp9 = tmp8 + tmp5
!<<<  tmp10 = tmp9 + ERRETM
      T_p1 = tmp9h + ERRETMh
      T_p2 = T_p1 - tmp9h
      tmp10h = T_p1
      tmp10l = (tmp9h - (T_p1 - T_p2))
     +      + (ERRETMh - T_p2)
      tmp10l = tmp10l + tmp9l + ERRETMl
      T_s1 = tmp10h + tmp10l
      tmp10l = tmp10l - (T_s1 - tmp10h)
      tmp10h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp10 = tmp9 + ERRETM
!<<<  ERRETM = tmp10 + tmp7
      T_p1 = tmp10h + tmp7h
      T_p2 = T_p1 - tmp10h
      ERRETMh = T_p1
      ERRETMl = (tmp10h - (T_p1 - T_p2))
     +      + (tmp7h - T_p2)
      ERRETMl = ERRETMl + tmp10l + tmp7l
      T_s1 = ERRETMh + ERRETMl
      ERRETMl = ERRETMl - (T_s1 - ERRETMh)
      ERRETMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  ERRETM = tmp10 + tmp7
*
*        Test for convergence
*
!<<<  tmp1 = abs(W)
      if (Wh .lt. 0.0d+0) then
        tmp1h = -Wh
        tmp1l = -Wl
      elseif (
     +  (Wh .eq. 0.0d+0).and.
     +  (Wl .lt. 0.0d+0)) then
        tmp1h = Wh
        tmp1l = -Wl
      else
        tmp1h = Wh
        tmp1l = Wl
      endif
!>>>  tmp1 = abs(W)
!<<<  tmp2 = EPS * ERRETM
      tmp2h = EPSh * ERRETMh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * ERRETMh
      T_r1 = T_u1
     +         - (T_u1 - ERRETMh)
      T_r2 = ERRETMh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EPSh*ERRETMl
     +      + EPSl*ERRETMh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EPS * ERRETM
         IF(
!            ABS( W ).LE.EPS*ERRETM
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $       ) THEN
            IF( ORGATI ) THEN
!               DLAM = D( I ) + TAU
!<<<  DLAM = D(I) + TAU
      T_p1 = Dh(I) + TAUh
      T_p2 = T_p1 - Dh(I)
      DLAMh = T_p1
      DLAMl = (Dh(I) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(I) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(I) + TAU
            ELSE
!               DLAM = D( IP1 ) + TAU
!<<<  DLAM = D(IP1) + TAU
      T_p1 = Dh(IP1) + TAUh
      T_p2 = T_p1 - Dh(IP1)
      DLAMh = T_p1
      DLAMl = (Dh(IP1) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(IP1) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(IP1) + TAU
            END IF
            GO TO 250
         END IF
*
         IF(
!            W.LE.ZERO
!       (W .le. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .le. ZEROl)))
     $      ) THEN
!            DLTLB = MAX( DLTLB, TAU )
             IF(
!       (DLTLB .ge. TAU)
     + ((DLTLBh .gt. TAUh) .or.
     +  ((DLTLBh .eq. TAUh) .and.
     +   (DLTLBl .ge. TAUl)))
     $       )THEN
      DLTLBh = DLTLBh
      DLTLBl = DLTLBl
             ELSE
      DLTLBh = TAUh
      DLTLBl = TAUl
             END IF
         ELSE
!            DLTUB = MIN( DLTUB, TAU )
             IF(
!       (DLTUB .le. TAU)
     + ((DLTUBh .lt. TAUh) .or.
     +  ((DLTUBh .eq. TAUh) .and.
     +   (DLTUBl .le. TAUl)))
     $       )THEN
      DLTUBh = DLTUBh
      DLTUBl = DLTUBl
             ELSE
      DLTUBh = TAUh
      DLTUBl = TAUl
             END IF

         END IF
*
*        Calculate the new step
*
         NITER = NITER + 1
         IF( .NOT.SWTCH3 ) THEN
            IF( ORGATI ) THEN
!               C = W - DELTA( IP1 )*DW - ( D( I )-D( IP1 ) )*
!     $             ( Z( I ) / DELTA( I ) )**2
!<<<  tmp1 = DELTA(IP1) * DW
      tmp1h = DELTAh(IP1) * DWh
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(IP1)*DWl
     +      + DELTAl(IP1)*DWh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(IP1) * DW
!<<<  tmp2 = D(I) - D(IP1)
      T_p1 = Dh(I) - Dh(IP1)
      T_p2 = T_p1 - Dh(I)
      tmp2h = T_p1
      tmp2l = (Dh(I) - (T_p1 - T_p2))
     +         - (Dh(IP1) + T_p2)
      tmp2l = tmp2l + Dl(I) - Dl(IP1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(I) - D(IP1)
!<<<  tmp3 = Z(I) / DELTA(I)
      T_q1 = Zh(I) / DELTAh(I)
      tmp3h = DELTAh(I) * T_q1
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (DELTAl(I)*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(I) - tmp3h
      T_r1 = T_s1 - Zh(I)
      T_s2 = (Zh(I) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(I)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(I)
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(I) / DELTA(I)
!<<<  tmp4 = tmp3 * tmp3
      tmp4h = tmp3h * tmp3h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*tmp3l
     +      + tmp3l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp3
!<<<  tmp5 = tmp2 * tmp4
      tmp5h = tmp2h * tmp4h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp2h*tmp4l
     +      + tmp2l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp2 * tmp4
!<<<  tmp6 = W - tmp1
      T_p1 = Wh - tmp1h
      T_p2 = T_p1 - Wh
      tmp6h = T_p1
      tmp6l = (Wh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      tmp6l = tmp6l + Wl - tmp1l
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp6 = W - tmp1
!<<<  C = tmp6 - tmp5
      T_p1 = tmp6h - tmp5h
      T_p2 = T_p1 - tmp6h
      Ch = T_p1
      Cl = (tmp6h - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Cl = Cl + tmp6l - tmp5l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = tmp6 - tmp5
            ELSE
!               C = W - DELTA( I )*DW - ( D( IP1 )-D( I ) )*
!     $             ( Z( IP1 ) / DELTA( IP1 ) )**2
!<<<  tmp1 = DELTA(I) * DW
      tmp1h = DELTAh(I) * DWh
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(I)*DWl
     +      + DELTAl(I)*DWh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(I) * DW
!<<<  tmp2 = D(IP1) - D(I)
      T_p1 = Dh(IP1) - Dh(I)
      T_p2 = T_p1 - Dh(IP1)
      tmp2h = T_p1
      tmp2l = (Dh(IP1) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      tmp2l = tmp2l + Dl(IP1) - Dl(I)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(IP1) - D(I)
!<<<  tmp3 = Z(IP1) / DELTA(IP1)
      T_q1 = Zh(IP1) / DELTAh(IP1)
      tmp3h = DELTAh(IP1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (DELTAl(IP1)*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(IP1) - tmp3h
      T_r1 = T_s1 - Zh(IP1)
      T_s2 = (Zh(IP1) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(IP1)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IP1)
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(IP1) / DELTA(IP1)
!<<<  tmp4 = tmp3 * tmp3
      tmp4h = tmp3h * tmp3h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*tmp3l
     +      + tmp3l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp3
!<<<  tmp5 = tmp2 * tmp4
      tmp5h = tmp2h * tmp4h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp2h*tmp4l
     +      + tmp2l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp2 * tmp4
!<<<  tmp6 = W - tmp1
      T_p1 = Wh - tmp1h
      T_p2 = T_p1 - Wh
      tmp6h = T_p1
      tmp6l = (Wh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      tmp6l = tmp6l + Wl - tmp1l
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp6 = W - tmp1
!<<<  C = tmp6 - tmp5
      T_p1 = tmp6h - tmp5h
      T_p2 = T_p1 - tmp6h
      Ch = T_p1
      Cl = (tmp6h - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Cl = Cl + tmp6l - tmp5l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = tmp6 - tmp5
            END IF
!            A = ( DELTA( I )+DELTA( IP1 ) )*W -
!     $          DELTA( I )*DELTA( IP1 )*DW
!<<<  tmp1 = DELTA(I) + DELTA(IP1)
      T_p1 = DELTAh(I) + DELTAh(IP1)
      T_p2 = T_p1 - DELTAh(I)
      tmp1h = T_p1
      tmp1l = (DELTAh(I) - (T_p1 - T_p2))
     +      + (DELTAh(IP1) - T_p2)
      tmp1l = tmp1l + DELTAl(I) + DELTAl(IP1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DELTA(I) + DELTA(IP1)
!<<<  tmp2 = tmp1 * W
      tmp2h = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * W
!<<<  tmp3 = DELTA(I) * DELTA(IP1)
      tmp3h = DELTAh(I) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(I)*DELTAl(IP1)
     +      + DELTAl(I)*DELTAh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(I) * DELTA(IP1)
!<<<  tmp4 = tmp3 * DW
      tmp4h = tmp3h * DWh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*DWl
     +      + tmp3l*DWh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * DW
!<<<  A = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      Al = Al + tmp2l - tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  A = tmp2 - tmp4
!            B = DELTA( I )*DELTA( IP1 )*W
!<<<  tmp1 = DELTA(I) * DELTA(IP1)
      tmp1h = DELTAh(I) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(I)*DELTAl(IP1)
     +      + DELTAl(I)*DELTAh(IP1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(I) * DELTA(IP1)
!<<<  B = tmp1 * W
      Bh = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * W
            IF( 
!               C.EQ.ZERO
!       (C .eq. ZERO)
     + ((Ch .eq. ZEROh) .and.
     +  (Cl .eq. ZEROl))
     $          ) THEN
               IF(
!                  A.EQ.ZERO
!       (A .eq. ZERO)
     + ((Ah .eq. ZEROh) .and.
     +  (Al .eq. ZEROl))
     $             ) THEN
                  IF( ORGATI ) THEN
!                     A = Z( I )*Z( I ) + DELTA( IP1 )*DELTA( IP1 )*
!     $                   ( DPSI+DPHI )
!<<<  tmp1 = Z(I) * Z(I)
      tmp1h = Zh(I) * Zh(I)
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * Zh(I)
      T_r1 = T_u1
     +         - (T_u1 - Zh(I))
      T_r2 = Zh(I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(I)*Zl(I)
     +      + Zl(I)*Zh(I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(I) * Z(I)
!<<<  tmp2 = DELTA(IP1) * DELTA(IP1)
      tmp2h = DELTAh(IP1) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IP1)*DELTAl(IP1)
     +      + DELTAl(IP1)*DELTAh(IP1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IP1) * DELTA(IP1)
!<<<  tmp3 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp3h = T_p1
      tmp3l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp3l = tmp3l + DPSIl + DPHIl
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = DPSI + DPHI
!<<<  tmp4 = tmp2 * tmp3
      tmp4h = tmp2h * tmp3h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp2h*tmp3l
     +      + tmp2l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp2 * tmp3
!<<<  A = tmp1 + tmp4
      T_p1 = tmp1h + tmp4h
      T_p2 = T_p1 - tmp1h
      Ah = T_p1
      Al = (tmp1h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Al = Al + tmp1l + tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp1 + tmp4
                  ELSE
!                     A = Z( IP1 )*Z( IP1 ) + DELTA( I )*DELTA( I )*
!     $                   ( DPSI+DPHI )
!<<<  tmp1 = Z(IP1) * Z(IP1)
      tmp1h = Zh(IP1) * Zh(IP1)
      T_u1 = 134217729.0 * Zh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_p2 = Zh(IP1) - T_p1
      T_u1 = 134217729.0 * Zh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_r2 = Zh(IP1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(IP1)*Zl(IP1)
     +      + Zl(IP1)*Zh(IP1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(IP1) * Z(IP1)
!<<<  tmp2 = DELTA(I) * DELTA(I)
      tmp2h = DELTAh(I) * DELTAh(I)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(I)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_r2 = DELTAh(I) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(I)*DELTAl(I)
     +      + DELTAl(I)*DELTAh(I))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(I) * DELTA(I)
!<<<  tmp3 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp3h = T_p1
      tmp3l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp3l = tmp3l + DPSIl + DPHIl
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = DPSI + DPHI
!<<<  tmp4 = tmp2 * tmp3
      tmp4h = tmp2h * tmp3h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp2h*tmp3l
     +      + tmp2l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp2 * tmp3
!<<<  A = tmp1 + tmp4
      T_p1 = tmp1h + tmp4h
      T_p2 = T_p1 - tmp1h
      Ah = T_p1
      Al = (tmp1h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Al = Al + tmp1l + tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp1 + tmp4

                  END IF
               END IF
!               ETA = B / A
!<<<  ETA = B / A
      T_q1 = Bh / Ah
      ETAh = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (Al*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = Bh - ETAh
      T_r1 = T_s1 - Bh
      T_s2 = (Bh 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = B / A
            ELSE IF(
!                    A.LE.ZERO
!       (A .le. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .le. ZEROl)))
     $              ) THEN
!               ETA = ( A-SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A - tmp6
      T_p1 = Ah - tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      tmp7l = tmp7l + Al - tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = A - tmp6
!<<<  ETA = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      ETAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp1l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp7h - ETAh
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp7 / tmp1

            ELSE
!               ETA = TWO*B / ( A+SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A + tmp6
      T_p1 = Ah + tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Al + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = A + tmp6
!<<<  ETA = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      ETAh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp7l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / tmp7

            END IF
         ELSE
*
*           Interpolation using THREE most relevant poles
*
!            TEMP = RHOINV + PSI + PHI
!<<<  TEMP = RHOINV + PSI
      T_p1 = RHOINVh + PSIh
      T_p2 = T_p1 - RHOINVh
      TEMPh = T_p1
      TEMPl = (RHOINVh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      TEMPl = TEMPl + RHOINVl + PSIl
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = RHOINV + PSI
!<<<  TEMP = TEMP + PHI
      T_p1 = TEMPh + PHIh
      T_p2 = T_p1 - TEMPh
      T_q1 = T_p1
      T_q2 = (TEMPh - (T_p1 - T_p2))
     +         + (PHIh - T_p2)
      T_q2 = T_q2 + TEMPl + PHIl
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2 - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TEMP = TEMP + PHI
            IF( ORGATI ) THEN
!               TEMP1 = Z( IIM1 ) / DELTA( IIM1 )
!<<<  TEMP1 = Z(IIM1) / DELTA(IIM1)
      T_q1 = Zh(IIM1) / DELTAh(IIM1)
      TEMP1h = DELTAh(IIM1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMP1l = ((T_p1*T_r1
     +       - TEMP1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMP1l = TEMP1l + (DELTAl(IIM1)*T_q1)
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
      T_s1 = Zh(IIM1) - TEMP1h
      T_r1 = T_s1 - Zh(IIM1)
      T_s2 = (Zh(IIM1) 
     +         - (T_s1 - T_r1))
     +         - (TEMP1h + T_r1)
      T_s2 = T_s2 - TEMP1l
      T_s2 = T_s2 + Zl(IIM1)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IIM1)
      TEMP1h = T_q1 + T_q2
      TEMP1l = T_q2
     +      - (TEMP1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP1 = Z(IIM1) / DELTA(IIM1)
!               TEMP1 = TEMP1*TEMP1
!<<<  TEMP1 = TEMP1 * TEMP1
      T_ch = TEMP1h * TEMP1h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TEMP1h*TEMP1l
     +         +  TEMP1l*TEMP1h)
      T_s1 = T_ch + T_cl
      TEMP1l = T_cl - (T_s1 - T_ch)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP1 = TEMP1 * TEMP1
!               C = TEMP - DELTA( IIP1 )*( DPSI+DPHI ) -
!     $             ( D( IIM1 )-D( IIP1 ) )*TEMP1
!<<<  tmp1 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp1l = tmp1l + DPSIl + DPHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DPSI + DPHI
!<<<  tmp2 = DELTA(IIP1) * tmp1
      tmp2h = DELTAh(IIP1) * tmp1h
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IIP1)*tmp1l
     +      + DELTAl(IIP1)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IIP1) * tmp1
!<<<  tmp3 = D(IIM1) - D(IIP1)
      T_p1 = Dh(IIM1) - Dh(IIP1)
      T_p2 = T_p1 - Dh(IIM1)
      tmp3h = T_p1
      tmp3l = (Dh(IIM1) - (T_p1 - T_p2))
     +         - (Dh(IIP1) + T_p2)
      tmp3l = tmp3l + Dl(IIM1) - Dl(IIP1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(IIM1) - D(IIP1)
!<<<  tmp4 = tmp3 * TEMP1
      tmp4h = tmp3h * TEMP1h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*TEMP1l
     +      + tmp3l*TEMP1h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * TEMP1
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  C = TEMP - tmp5
      T_p1 = TEMPh - tmp5h
      T_p2 = T_p1 - TEMPh
      Ch = T_p1
      Cl = (TEMPh - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Cl = Cl + TEMPl - tmp5l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = TEMP - tmp5
!               ZZ( 1 ) = Z( IIM1 )*Z( IIM1 )
!<<<  ZZ(1) = Z(IIM1) * Z(IIM1)
      ZZh(1) = Zh(IIM1) * Zh(IIM1)
      T_u1 = 134217729.0 * Zh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IIM1))
      T_p2 = Zh(IIM1) - T_p1
      T_u1 = 134217729.0 * Zh(IIM1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IIM1))
      T_r2 = Zh(IIM1) - T_r1
      ZZl(1) = ((T_p1*T_r1
     +        - ZZh(1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(1) = ZZl(1)
     +      + (Zh(IIM1)*Zl(IIM1)
     +      + Zl(IIM1)*Zh(IIM1))
      T_s1 = ZZh(1) + ZZl(1)
      ZZl(1) = ZZl(1) - (T_s1 - ZZh(1))
      ZZh(1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(1) = Z(IIM1) * Z(IIM1)
!               ZZ( 3 ) = DELTA( IIP1 )*DELTA( IIP1 )*
!     $                   ( ( DPSI-TEMP1 )+DPHI )
!<<<  tmp1 = DPSI - TEMP1
      T_p1 = DPSIh - TEMP1h
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +         - (TEMP1h + T_p2)
      tmp1l = tmp1l + DPSIl - TEMP1l
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DPSI - TEMP1
!<<<  tmp1 = tmp1 + DPHI
      T_p1 = tmp1h + DPHIh
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (DPHIh - T_p2)
      T_q2 = T_q2 + tmp1l + DPHIl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = tmp1 + DPHI
!<<<  tmp2 = DELTA(IIP1) * DELTA(IIP1)
      tmp2h = DELTAh(IIP1) * DELTAh(IIP1)
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_r2 = DELTAh(IIP1) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IIP1)*DELTAl(IIP1)
     +      + DELTAl(IIP1)*DELTAh(IIP1))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IIP1) * DELTA(IIP1)
!<<<  ZZ(3) = tmp1 * tmp2
      ZZh(3) = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      ZZl(3) = ((T_p1*T_r1
     +        - ZZh(3))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(3) = ZZl(3)
     +      + (tmp1h*tmp2l
     +      + tmp1l*tmp2h)
      T_s1 = ZZh(3) + ZZl(3)
      ZZl(3) = ZZl(3) - (T_s1 - ZZh(3))
      ZZh(3) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(3) = tmp1 * tmp2
            ELSE
!               TEMP1 = Z( IIP1 ) / DELTA( IIP1 )
!<<<  TEMP1 = Z(IIP1) / DELTA(IIP1)
      T_q1 = Zh(IIP1) / DELTAh(IIP1)
      TEMP1h = DELTAh(IIP1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMP1l = ((T_p1*T_r1
     +       - TEMP1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMP1l = TEMP1l + (DELTAl(IIP1)*T_q1)
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
      T_s1 = Zh(IIP1) - TEMP1h
      T_r1 = T_s1 - Zh(IIP1)
      T_s2 = (Zh(IIP1) 
     +         - (T_s1 - T_r1))
     +         - (TEMP1h + T_r1)
      T_s2 = T_s2 - TEMP1l
      T_s2 = T_s2 + Zl(IIP1)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IIP1)
      TEMP1h = T_q1 + T_q2
      TEMP1l = T_q2
     +      - (TEMP1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP1 = Z(IIP1) / DELTA(IIP1)
!               TEMP1 = TEMP1*TEMP1
!<<<  TEMP1 = TEMP1 * TEMP1
      T_ch = TEMP1h * TEMP1h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TEMP1h*TEMP1l
     +         +  TEMP1l*TEMP1h)
      T_s1 = T_ch + T_cl
      TEMP1l = T_cl - (T_s1 - T_ch)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP1 = TEMP1 * TEMP1
!               C = TEMP - DELTA( IIM1 )*( DPSI+DPHI ) -
!     $             ( D( IIP1 )-D( IIM1 ) )*TEMP1
!<<<  tmp1 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp1l = tmp1l + DPSIl + DPHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DPSI + DPHI
!<<<  tmp2 = DELTA(IIM1) * tmp1
      tmp2h = DELTAh(IIM1) * tmp1h
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IIM1)*tmp1l
     +      + DELTAl(IIM1)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IIM1) * tmp1
!<<<  tmp3 = D(IIP1) - D(IIM1)
      T_p1 = Dh(IIP1) - Dh(IIM1)
      T_p2 = T_p1 - Dh(IIP1)
      tmp3h = T_p1
      tmp3l = (Dh(IIP1) - (T_p1 - T_p2))
     +         - (Dh(IIM1) + T_p2)
      tmp3l = tmp3l + Dl(IIP1) - Dl(IIM1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(IIP1) - D(IIM1)
!<<<  tmp4 = tmp3 * TEMP1
      tmp4h = tmp3h * TEMP1h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*TEMP1l
     +      + tmp3l*TEMP1h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * TEMP1
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  C = TEMP - tmp5
      T_p1 = TEMPh - tmp5h
      T_p2 = T_p1 - TEMPh
      Ch = T_p1
      Cl = (TEMPh - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Cl = Cl + TEMPl - tmp5l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = TEMP - tmp5
!               ZZ( 1 ) = DELTA( IIM1 )*DELTA( IIM1 )*
!     $                   ( DPSI+( DPHI-TEMP1 ) )
!<<<  tmp1 = DPHI - TEMP1
      T_p1 = DPHIh - TEMP1h
      T_p2 = T_p1 - DPHIh
      tmp1h = T_p1
      tmp1l = (DPHIh - (T_p1 - T_p2))
     +         - (TEMP1h + T_p2)
      tmp1l = tmp1l + DPHIl - TEMP1l
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DPHI - TEMP1
!<<<  tmp2 = DPSI + tmp1
      T_p1 = DPSIh + tmp1h
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      tmp2l = tmp2l + DPSIl + tmp1l
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + tmp1
!<<<  tmp3 = DELTA(IIM1) * DELTA(IIM1)
      tmp3h = DELTAh(IIM1) * DELTAh(IIM1)
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_r2 = DELTAh(IIM1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(IIM1)*DELTAl(IIM1)
     +      + DELTAl(IIM1)*DELTAh(IIM1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(IIM1) * DELTA(IIM1)
!<<<  ZZ(1) = tmp3 * tmp2
      ZZh(1) = tmp3h * tmp2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      ZZl(1) = ((T_p1*T_r1
     +        - ZZh(1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(1) = ZZl(1)
     +      + (tmp3h*tmp2l
     +      + tmp3l*tmp2h)
      T_s1 = ZZh(1) + ZZl(1)
      ZZl(1) = ZZl(1) - (T_s1 - ZZh(1))
      ZZh(1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(1) = tmp3 * tmp2
!               ZZ( 3 ) = Z( IIP1 )*Z( IIP1 )
!<<<  ZZ(3) = Z(IIP1) * Z(IIP1)
      ZZh(3) = Zh(IIP1) * Zh(IIP1)
      T_u1 = 134217729.0 * Zh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IIP1))
      T_p2 = Zh(IIP1) - T_p1
      T_u1 = 134217729.0 * Zh(IIP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IIP1))
      T_r2 = Zh(IIP1) - T_r1
      ZZl(3) = ((T_p1*T_r1
     +        - ZZh(3))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(3) = ZZl(3)
     +      + (Zh(IIP1)*Zl(IIP1)
     +      + Zl(IIP1)*Zh(IIP1))
      T_s1 = ZZh(3) + ZZl(3)
      ZZl(3) = ZZl(3) - (T_s1 - ZZh(3))
      ZZh(3) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(3) = Z(IIP1) * Z(IIP1)
            END IF
!            ZZ( 2 ) = Z( II )*Z( II )
!<<<  ZZ(2) = Z(II) * Z(II)
      ZZh(2) = Zh(II) * Zh(II)
      T_u1 = 134217729.0 * Zh(II)
      T_p1 = T_u1
     +         - (T_u1 - Zh(II))
      T_p2 = Zh(II) - T_p1
      T_u1 = 134217729.0 * Zh(II)
      T_r1 = T_u1
     +         - (T_u1 - Zh(II))
      T_r2 = Zh(II) - T_r1
      ZZl(2) = ((T_p1*T_r1
     +        - ZZh(2))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(2) = ZZl(2)
     +      + (Zh(II)*Zl(II)
     +      + Zl(II)*Zh(II))
      T_s1 = ZZh(2) + ZZl(2)
      ZZl(2) = ZZl(2) - (T_s1 - ZZh(2))
      ZZh(2) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(2) = Z(II) * Z(II)
!            CALL DLAED6( NITER, ORGATI, C, DELTA( IIM1 ), ZZ, W, ETA,
!     $                   INFO )
            CALL DDLAED6( NITER, ORGATI, Ch, Cl, DELTAh( IIM1 ), 
     $                    DELTAl( IIM1 ), ZZh, ZZl, Wh, Wl, 
     $                   ETAh, ETAl, INFO )
            IF( INFO.NE.0 )
     $         GO TO 250
         END IF
*
*        Note, eta should be positive if w is negative, and
*        eta should be negative otherwise. However,
*        if for some reason caused by roundoff, eta*w > 0,
*        we simply use one Newton step instead. This way
*        will guarantee eta*w < 0.
*
!<<<  tmp1 = W * ETA
      tmp1h = Wh * ETAh
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * ETAh
      T_r1 = T_u1
     +         - (T_u1 - ETAh)
      T_r2 = ETAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Wh*ETAl
     +      + Wl*ETAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = W * ETA
         IF( 
!           W*ETA.GE.ZERO
!       (tmp1 .ge. ZERO)
     + ((tmp1h .gt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .ge. ZEROl)))
     $       ) THEN
!     $      ETA = -W / DW
!<<<  ETA = W / DW
      T_q1 = Wh / DWh
      ETAh = DWh * T_q1
      T_u1 = 134217729.0 * DWh
      T_p1 = T_u1
     +         - (T_u1 - DWh)
      T_p2 = DWh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (DWl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = Wh - ETAh
      T_r1 = T_s1 - Wh
      T_s2 = (Wh 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + Wl
      T_q2 = (T_s1 + T_s2)
     +         / DWh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = W / DW
!<<<  ETA = -ETA
      ETAh = -ETAh
      ETAl = -ETAl
!>>>  ETA = -ETA
         END IF
!         TEMP = TAU + ETA
!<<<  TEMP = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      TEMPh = T_p1
      TEMPl = (TAUh - (T_p1 - T_p2))
     +      + (ETAh - T_p2)
      TEMPl = TEMPl + TAUl + ETAl
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = TAU + ETA
         IF(
!            TEMP.GT.DLTUB 
!       (TEMP .gt. DLTUB)
     + ((TEMPh .gt. DLTUBh) .or.
     +  ((TEMPh .eq. DLTUBh) .and.
     +   (TEMPl .gt. DLTUBl)))
     $      .OR.
!            TEMP.LT.DLTLB
!       (TEMP .lt. DLTLB)
     + ((TEMPh .lt. DLTLBh) .or.
     +  ((TEMPh .eq. DLTLBh) .and.
     +   (TEMPl .lt. DLTLBl)))
     $      ) THEN
            IF(
!              W.LT.ZERO
!       (W .lt. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .lt. ZEROl)))
     $          ) THEN
!               ETA = ( DLTUB-TAU ) / TWO
!<<<  tmp1 = DLTUB - TAU
      T_p1 = DLTUBh - TAUh
      T_p2 = T_p1 - DLTUBh
      tmp1h = T_p1
      tmp1l = (DLTUBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTUBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTUB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
            ELSE
!               ETA = ( DLTLB-TAU ) / TWO
!<<<  tmp1 = DLTLB - TAU
      T_p1 = DLTLBh - TAUh
      T_p2 = T_p1 - DLTLBh
      tmp1h = T_p1
      tmp1l = (DLTLBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTLBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTLB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
            END IF
         END IF
*
!         PREW = W
      PREWh = Wh
      PREWl = Wl
*
         DO 180 J = 1, N
!            DELTA( J ) = DELTA( J ) - ETA
!<<<  DELTA(J) = DELTA(J) - ETA
      T_p1 = DELTAh(J) - ETAh
      T_p2 = T_p1 - DELTAh(J)
      T_ch = T_p1
      T_cl = (DELTAh(J) - (T_p1 - T_p2))
     +         - (ETAh + T_p2)
      T_cl = T_cl + DELTAl(J) - ETAl
      T_s1 = T_ch + T_cl
      DELTAl(J) = T_cl - (T_s1 - T_ch)
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DELTA(J) = DELTA(J) - ETA
  180    CONTINUE
*
*        Evaluate PSI and the derivative DPSI
*
!         DPSI = ZERO
!         PSI = ZERO
!         ERRETM = ZERO
      DPSIh = ZEROh
      DPSIl = ZEROl
      PSIh = ZEROh
      PSIl = ZEROl
      ERRETMh = ZEROh
      ERRETMl = ZEROl
         DO 190 J = 1, IIM1
!            TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!            PSI = PSI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PSI = PSI + tmp1
      T_p1 = PSIh + tmp1h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PSIl + tmp1l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp1
!            DPSI = DPSI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp2
      T_p1 = DPSIh + tmp2h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp2l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp2
!            ERRETM = ERRETM + PSI
!<<<  ERRETM = ERRETM + PSI
      T_p1 = ERRETMh + PSIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PSIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PSI
  190    CONTINUE
!         ERRETM = ABS( ERRETM )
!<<<  ERRETM = abs(ERRETM)
      if (ERRETMh .lt. 0.0d+0) then
        ERRETMh = -ERRETMh
        ERRETMl = -ERRETMl
      elseif (
     +  (ERRETMh .eq. 0.0d+0).and.
     +  (ERRETMl .lt. 0.0d+0)) then
        ERRETMh = ERRETMh
        ERRETMl = -ERRETMl
      else
        ERRETMh = ERRETMh
        ERRETMl = ERRETMl
      endif
!>>>  ERRETM = abs(ERRETM)
*
*        Evaluate PHI and the derivative DPHI
*
!         DPHI = ZERO
!         PHI = ZERO
      DPHIh = ZEROh
      DPHIl = ZEROl
      PHIh = ZEROh
      PHIl = ZEROl
         DO 200 J = N, IIP1, -1
!            TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!            PHI = PHI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PHI = PHI + tmp1
      T_p1 = PHIh + tmp1h
      T_p2 = T_p1 - PHIh
      T_q1 = T_p1
      T_q2 = (PHIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PHIl + tmp1l
      PHIh = T_q1 + T_q2
      PHIl = T_q2 - (PHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PHI = PHI + tmp1
!            DPHI = DPHI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPHI = DPHI + tmp2
      T_p1 = DPHIh + tmp2h
      T_p2 = T_p1 - DPHIh
      T_q1 = T_p1
      T_q2 = (DPHIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPHIl + tmp2l
      DPHIh = T_q1 + T_q2
      DPHIl = T_q2 - (DPHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPHI = DPHI + tmp2
!            ERRETM = ERRETM + PHI
!<<<  ERRETM = ERRETM + PHI
      T_p1 = ERRETMh + PHIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PHIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PHIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PHI
  200    CONTINUE
*
!         TEMP = Z( II ) / DELTA( II )
!<<<  TEMP = Z(II) / DELTA(II)
      T_q1 = Zh(II) / DELTAh(II)
      TEMPh = DELTAh(II) * T_q1
      T_u1 = 134217729.0 * DELTAh(II)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(II))
      T_p2 = DELTAh(II) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(II)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(II) - TEMPh
      T_r1 = T_s1 - Zh(II)
      T_s2 = (Zh(II) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(II)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(II)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(II) / DELTA(II)
!         DW = DPSI + DPHI + TEMP*TEMP
!<<<  tmp1 = TEMP * TEMP
      tmp1h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP * TEMP
!<<<  tmp2 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + DPSIl + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + DPHI
!<<<  DW = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      DWh = T_p1
      DWl = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      DWl = DWl + tmp1l + tmp2l
      T_s1 = DWh + DWl
      DWl = DWl - (T_s1 - DWh)
      DWh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DW = tmp1 + tmp2
!         TEMP = Z( II )*TEMP
!<<<  TEMP = Z(II) * TEMP
      T_ch = Zh(II) * TEMPh
      T_u1 = 134217729.0 * Zh(II)
      T_p1 = T_u1
     +         - (T_u1 - Zh(II))
      T_p2 = Zh(II) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(II)*TEMPl
     +         +  Zl(II)*TEMPh)
      T_s1 = T_ch + T_cl
      TEMPl = T_cl - (T_s1 - T_ch)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP = Z(II) * TEMP
!         W = RHOINV + PHI + PSI + TEMP
!<<<  W = RHOINV + PHI
      T_p1 = RHOINVh + PHIh
      T_p2 = T_p1 - RHOINVh
      Wh = T_p1
      Wl = (RHOINVh - (T_p1 - T_p2))
     +      + (PHIh - T_p2)
      Wl = Wl + RHOINVl + PHIl
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = RHOINV + PHI
!<<<  W = W + PSI
      T_p1 = Wh + PSIh
      T_p2 = T_p1 - Wh
      T_q1 = T_p1
      T_q2 = (Wh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + Wl + PSIl
      Wh = T_q1 + T_q2
      Wl = T_q2 - (Wh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W = W + PSI
!<<<  W = W + TEMP
      T_p1 = Wh + TEMPh
      T_p2 = T_p1 - Wh
      T_q1 = T_p1
      T_q2 = (Wh - (T_p1 - T_p2))
     +         + (TEMPh - T_p2)
      T_q2 = T_q2 + Wl + TEMPl
      Wh = T_q1 + T_q2
      Wl = T_q2 - (Wh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W = W + TEMP
!         ERRETM = EIGHT*( PHI-PSI ) + ERRETM + TWO*RHOINV +
!     $            THREE*ABS( TEMP ) + ABS( TAU+ETA )*DW
!<<<  tmp1 = PHI - PSI
      T_p1 = PHIh - PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +         - (PSIh + T_p2)
      tmp1l = tmp1l + PHIl - PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = PHI - PSI
!<<<  tmp2 = EIGHT * tmp1
      tmp2h = EIGHTh * tmp1h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EIGHTh*tmp1l
     +      + EIGHTl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EIGHT * tmp1
!<<<  tmp3 = TWO * RHOINV
      tmp3h = TWOh * RHOINVh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RHOINVh
      T_r1 = T_u1
     +         - (T_u1 - RHOINVh)
      T_r2 = RHOINVh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (TWOh*RHOINVl
     +      + TWOl*RHOINVh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = TWO * RHOINV
!<<<  tmp4 = abs(TEMP)
      if (TEMPh .lt. 0.0d+0) then
        tmp4h = -TEMPh
        tmp4l = -TEMPl
      elseif (
     +  (TEMPh .eq. 0.0d+0).and.
     +  (TEMPl .lt. 0.0d+0)) then
        tmp4h = TEMPh
        tmp4l = -TEMPl
      else
        tmp4h = TEMPh
        tmp4l = TEMPl
      endif
!>>>  tmp4 = abs(TEMP)
!<<<  tmp5 = THREE * tmp4
      tmp5h = THREEh * tmp4h
      T_u1 = 134217729.0 * THREEh
      T_p1 = T_u1
     +         - (T_u1 - THREEh)
      T_p2 = THREEh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (THREEh*tmp4l
     +      + THREEl*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = THREE * tmp4
!<<<  tmp6 = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      tmp6h = T_p1
      tmp6l = (TAUh - (T_p1 - T_p2))
     +      + (ETAh - T_p2)
      tmp6l = tmp6l + TAUl + ETAl
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = TAU + ETA
!<<<  tmp7 = abs(tmp6)
      if (tmp6h .lt. 0.0d+0) then
        tmp7h = -tmp6h
        tmp7l = -tmp6l
      elseif (
     +  (tmp6h .eq. 0.0d+0).and.
     +  (tmp6l .lt. 0.0d+0)) then
        tmp7h = tmp6h
        tmp7l = -tmp6l
      else
        tmp7h = tmp6h
        tmp7l = tmp6l
      endif
!>>>  tmp7 = abs(tmp6)
!<<<  tmp8 = tmp7 * DW
      tmp8h = tmp7h * DWh
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp8l = ((T_p1*T_r1
     +        - tmp8h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp8l = tmp8l
     +      + (tmp7h*DWl
     +      + tmp7l*DWh)
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp8 = tmp7 * DW
!<<<  ERRETM = tmp2 + ERRETM
      T_p1 = tmp2h + ERRETMh
      T_p2 = T_p1 - tmp2h
      T_q1 = T_p1
      T_q2 = (tmp2h - (T_p1 - T_p2))
     +         + (ERRETMh - T_p2)
      T_q2 = T_q2 + tmp2l + ERRETMl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = tmp2 + ERRETM
!<<<  ERRETM = ERRETM + tmp3
      T_p1 = ERRETMh + tmp3h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp3h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp3l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp3
!<<<  ERRETM = ERRETM + tmp5
      T_p1 = ERRETMh + tmp5h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp5h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp5l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp5
!<<<  ERRETM = ERRETM + tmp8
      T_p1 = ERRETMh + tmp8h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp8h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp8l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp8
*
         SWTCH = .FALSE.
         IF( ORGATI ) THEN
!<<<  tmp1 = abs(PREW)
      if (PREWh .lt. 0.0d+0) then
        tmp1h = -PREWh
        tmp1l = -PREWl
      elseif (
     +  (PREWh .eq. 0.0d+0).and.
     +  (PREWl .lt. 0.0d+0)) then
        tmp1h = PREWh
        tmp1l = -PREWl
      else
        tmp1h = PREWh
        tmp1l = PREWl
      endif
!>>>  tmp1 = abs(PREW)
!<<<  tmp2 = tmp1 / TEN
      T_q1 = tmp1h / TENh
      tmp2h = TENh * T_q1
      T_u1 = 134217729.0 * TENh
      T_p1 = T_u1
     +         - (T_u1 - TENh)
      T_p2 = TENh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp2l = ((T_p1*T_r1
     +       - tmp2h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp2l = tmp2l + (TENl*T_q1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
      T_s1 = tmp1h - tmp2h
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (tmp2h + T_r1)
      T_s2 = T_s2 - tmp2l
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TENh
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = tmp1 / TEN
!<<<  tmp3 = -W
      tmp3h = -Wh
      tmp3l = -Wl
!>>>  tmp3 = -W
            IF(
!               -W.GT.ABS( PREW ) / TEN
!       (tmp3 .gt. tmp2)
     + ((tmp3h .gt. tmp2h) .or.
     +  ((tmp3h .eq. tmp2h) .and.
     +   (tmp3l .gt. tmp2l)))
     $         )
     $         SWTCH = .TRUE.
         ELSE
            IF( 
!               W.GT.ABS( PREW ) / TEN
!       (W .gt. tmp2)
     + ((Wh .gt. tmp2h) .or.
     +  ((Wh .eq. tmp2h) .and.
     +   (Wl .gt. tmp2l)))
     $ )
     $         SWTCH = .TRUE.
         END IF
*
!         TAU = TAU + ETA
!<<<  TAU = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      T_q1 = T_p1
      T_q2 = (TAUh - (T_p1 - T_p2))
     +         + (ETAh - T_p2)
      T_q2 = T_q2 + TAUl + ETAl
      TAUh = T_q1 + T_q2
      TAUl = T_q2 - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TAU = TAU + ETA
*
*        Main loop to update the values of the array   DELTA
*
         ITER = NITER + 1
*
         DO 240 NITER = ITER, MAXIT
*
*           Test for convergence
*
!<<<  tmp1 = abs(W)
      if (Wh .lt. 0.0d+0) then
        tmp1h = -Wh
        tmp1l = -Wl
      elseif (
     +  (Wh .eq. 0.0d+0).and.
     +  (Wl .lt. 0.0d+0)) then
        tmp1h = Wh
        tmp1l = -Wl
      else
        tmp1h = Wh
        tmp1l = Wl
      endif
!>>>  tmp1 = abs(W)
!<<<  tmp2 = EPS * ERRETM
      tmp2h = EPSh * ERRETMh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * ERRETMh
      T_r1 = T_u1
     +         - (T_u1 - ERRETMh)
      T_r2 = ERRETMh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EPSh*ERRETMl
     +      + EPSl*ERRETMh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EPS * ERRETM
            IF(
!               ABS( W ).LE.EPS*ERRETM
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $          ) THEN
               IF( ORGATI ) THEN
!                  DLAM = D( I ) + TAU
!<<<  DLAM = D(I) + TAU
      T_p1 = Dh(I) + TAUh
      T_p2 = T_p1 - Dh(I)
      DLAMh = T_p1
      DLAMl = (Dh(I) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(I) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(I) + TAU
               ELSE
!                  DLAM = D( IP1 ) + TAU
!<<<  DLAM = D(IP1) + TAU
      T_p1 = Dh(IP1) + TAUh
      T_p2 = T_p1 - Dh(IP1)
      DLAMh = T_p1
      DLAMl = (Dh(IP1) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(IP1) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(IP1) + TAU
               END IF
               GO TO 250
            END IF
*
            IF(
!               W.LE.ZERO
!       (W .le. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .le. ZEROl)))
     $          ) THEN
!               DLTLB = MAX( DLTLB, TAU )
                IF(
!       (DLTLB .ge. TAU)
     + ((DLTLBh .gt. TAUh) .or.
     +  ((DLTLBh .eq. TAUh) .and.
     +   (DLTLBl .ge. TAUl)))
     $             ) THEN
      DLTLBh = DLTLBh
      DLTLBl = DLTLBl
                ELSE
      DLTLBh = TAUh
      DLTLBl = TAUl
                END IF
            ELSE
!               DLTUB = MIN( DLTUB, TAU )
                IF(
!       (DLTUB .le. TAU)
     + ((DLTUBh .lt. TAUh) .or.
     +  ((DLTUBh .eq. TAUh) .and.
     +   (DLTUBl .le. TAUl)))
     $             ) THEN
      DLTUBh = DLTUBh
      DLTUBl = DLTUBl
                ELSE
      DLTUBh = TAUh
      DLTUBl = TAUl
                END IF
            END IF
*
*           Calculate the new step
*
            IF( .NOT.SWTCH3 ) THEN
               IF( .NOT.SWTCH ) THEN
                  IF( ORGATI ) THEN
!                     C = W - DELTA( IP1 )*DW -
!     $                   ( D( I )-D( IP1 ) )*( Z( I ) / DELTA( I ) )**2
!<<<  tmp1 = DELTA(IP1) * DW
      tmp1h = DELTAh(IP1) * DWh
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(IP1)*DWl
     +      + DELTAl(IP1)*DWh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(IP1) * DW
!<<<  tmp2 = D(I) - D(IP1)
      T_p1 = Dh(I) - Dh(IP1)
      T_p2 = T_p1 - Dh(I)
      tmp2h = T_p1
      tmp2l = (Dh(I) - (T_p1 - T_p2))
     +         - (Dh(IP1) + T_p2)
      tmp2l = tmp2l + Dl(I) - Dl(IP1)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(I) - D(IP1)
!<<<  tmp3 = Z(I) / DELTA(I)
      T_q1 = Zh(I) / DELTAh(I)
      tmp3h = DELTAh(I) * T_q1
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (DELTAl(I)*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(I) - tmp3h
      T_r1 = T_s1 - Zh(I)
      T_s2 = (Zh(I) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(I)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(I)
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(I) / DELTA(I)
!<<<  tmp4 = tmp3 * tmp3
      tmp4h = tmp3h * tmp3h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*tmp3l
     +      + tmp3l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp3
!<<<  tmp5 = tmp2 * tmp4
      tmp5h = tmp2h * tmp4h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp2h*tmp4l
     +      + tmp2l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp2 * tmp4
!<<<  tmp6 = tmp1 + tmp5
      T_p1 = tmp1h + tmp5h
      T_p2 = T_p1 - tmp1h
      tmp6h = T_p1
      tmp6l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      tmp6l = tmp6l + tmp1l + tmp5l
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = tmp1 + tmp5
!<<<  C = W - tmp6
      T_p1 = Wh - tmp6h
      T_p2 = T_p1 - Wh
      Ch = T_p1
      Cl = (Wh - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      Cl = Cl + Wl - tmp6l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = W - tmp6
                  ELSE
!                     C = W - DELTA( I )*DW - ( D( IP1 )-D( I ) )*
!     $                   ( Z( IP1 ) / DELTA( IP1 ) )**2
!<<<  tmp1 = DELTA(I) * DW
      tmp1h = DELTAh(I) * DWh
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(I)*DWl
     +      + DELTAl(I)*DWh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(I) * DW
!<<<  tmp2 = D(IP1) - D(I)
      T_p1 = Dh(IP1) - Dh(I)
      T_p2 = T_p1 - Dh(IP1)
      tmp2h = T_p1
      tmp2l = (Dh(IP1) - (T_p1 - T_p2))
     +         - (Dh(I) + T_p2)
      tmp2l = tmp2l + Dl(IP1) - Dl(I)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = D(IP1) - D(I)
!<<<  tmp3 = Z(IP1) / DELTA(IP1)
      T_q1 = Zh(IP1) / DELTAh(IP1)
      tmp3h = DELTAh(IP1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (DELTAl(IP1)*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(IP1) - tmp3h
      T_r1 = T_s1 - Zh(IP1)
      T_s2 = (Zh(IP1) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(IP1)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IP1)
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(IP1) / DELTA(IP1)
!<<<  tmp4 = tmp3 * tmp3
      tmp4h = tmp3h * tmp3h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*tmp3l
     +      + tmp3l*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp3
!<<<  tmp5 = tmp2 * tmp4
      tmp5h = tmp2h * tmp4h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (tmp2h*tmp4l
     +      + tmp2l*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = tmp2 * tmp4
!<<<  tmp6 = tmp1 + tmp5
      T_p1 = tmp1h + tmp5h
      T_p2 = T_p1 - tmp1h
      tmp6h = T_p1
      tmp6l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp5h - T_p2)
      tmp6l = tmp6l + tmp1l + tmp5l
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp6 = tmp1 + tmp5
!<<<  C = W - tmp6
      T_p1 = Wh - tmp6h
      T_p2 = T_p1 - Wh
      Ch = T_p1
      Cl = (Wh - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      Cl = Cl + Wl - tmp6l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = W - tmp6
                  END IF
               ELSE
!                  TEMP = Z( II ) / DELTA( II )
!<<<  TEMP = Z(II) / DELTA(II)
      T_q1 = Zh(II) / DELTAh(II)
      TEMPh = DELTAh(II) * T_q1
      T_u1 = 134217729.0 * DELTAh(II)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(II))
      T_p2 = DELTAh(II) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(II)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(II) - TEMPh
      T_r1 = T_s1 - Zh(II)
      T_s2 = (Zh(II) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(II)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(II)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(II) / DELTA(II)
                  IF( ORGATI ) THEN
!                     DPSI = DPSI + TEMP*TEMP
!<<<  tmp1 = TEMP * TEMP
      tmp1h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp1
      T_p1 = DPSIh + tmp1h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp1l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp1
                  ELSE
!                     DPHI = DPHI + TEMP*TEMP
!<<<  tmp1 = TEMP * TEMP
      tmp1h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP * TEMP
!<<<  DPHI = DPHI + tmp1
      T_p1 = DPHIh + tmp1h
      T_p2 = T_p1 - DPHIh
      T_q1 = T_p1
      T_q2 = (DPHIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + DPHIl + tmp1l
      DPHIh = T_q1 + T_q2
      DPHIl = T_q2 - (DPHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPHI = DPHI + tmp1
                  END IF
!                  C = W - DELTA( I )*DPSI - DELTA( IP1 )*DPHI
!<<<  tmp1 = DELTA(I) * DPSI
      tmp1h = DELTAh(I) * DPSIh
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DPSIh
      T_r1 = T_u1
     +         - (T_u1 - DPSIh)
      T_r2 = DPSIh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(I)*DPSIl
     +      + DELTAl(I)*DPSIh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(I) * DPSI
!<<<  tmp2 = DELTA(IP1) * DPHI
      tmp2h = DELTAh(IP1) * DPHIh
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * DPHIh
      T_r1 = T_u1
     +         - (T_u1 - DPHIh)
      T_r2 = DPHIh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IP1)*DPHIl
     +      + DELTAl(IP1)*DPHIh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IP1) * DPHI
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  C = W - tmp3
      T_p1 = Wh - tmp3h
      T_p2 = T_p1 - Wh
      Ch = T_p1
      Cl = (Wh - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      Cl = Cl + Wl - tmp3l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = W - tmp3
               END IF
!               A = ( DELTA( I )+DELTA( IP1 ) )*W -
!     $             DELTA( I )*DELTA( IP1 )*DW
!<<<  tmp1 = DELTA(I) + DELTA(IP1)
      T_p1 = DELTAh(I) + DELTAh(IP1)
      T_p2 = T_p1 - DELTAh(I)
      tmp1h = T_p1
      tmp1l = (DELTAh(I) - (T_p1 - T_p2))
     +      + (DELTAh(IP1) - T_p2)
      tmp1l = tmp1l + DELTAl(I) + DELTAl(IP1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DELTA(I) + DELTA(IP1)
!<<<  tmp2 = tmp1 * W
      tmp2h = tmp1h * Wh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Wl
     +      + tmp1l*Wh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * W
!<<<  tmp3 = DELTA(I) * DELTA(IP1)
      tmp3h = DELTAh(I) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(I)*DELTAl(IP1)
     +      + DELTAl(I)*DELTAh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(I) * DELTA(IP1)
!<<<  tmp4 = tmp3 * DW
      tmp4h = tmp3h * DWh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*DWl
     +      + tmp3l*DWh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * DW
!<<<  A = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      Al = Al + tmp2l - tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  A = tmp2 - tmp4
!               B = DELTA( I )*DELTA( IP1 )*W
!<<<  tmp5 = DELTA(I) * DELTA(IP1)
      tmp5h = DELTAh(I) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (DELTAh(I)*DELTAl(IP1)
     +      + DELTAl(I)*DELTAh(IP1))
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = DELTA(I) * DELTA(IP1)
!<<<  B = tmp5 * W
      Bh = tmp5h * Wh
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * Wh
      T_r1 = T_u1
     +         - (T_u1 - Wh)
      T_r2 = Wh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp5h*Wl
     +      + tmp5l*Wh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp5 * W
               IF(
!                  C.EQ.ZERO
!       (C .eq. ZERO)
     + ((Ch .eq. ZEROh) .and.
     +  (Cl .eq. ZEROl))
     $             ) THEN
                  IF(
!                     A.EQ.ZERO
!       (A .eq. ZERO)
     + ((Ah .eq. ZEROh) .and.
     +  (Al .eq. ZEROl))
     $                ) THEN
                     IF( .NOT.SWTCH ) THEN
                        IF( ORGATI ) THEN
!                           A = Z( I )*Z( I ) + DELTA( IP1 )*
!     $                         DELTA( IP1 )*( DPSI+DPHI )
!<<<  tmp1 = Z(I) * Z(I)
      tmp1h = Zh(I) * Zh(I)
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * Zh(I)
      T_r1 = T_u1
     +         - (T_u1 - Zh(I))
      T_r2 = Zh(I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(I)*Zl(I)
     +      + Zl(I)*Zh(I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(I) * Z(I)
!<<<  tmp2 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + DPSIl + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + DPHI
!<<<  tmp3 = DELTA(IP1) * DELTA(IP1)
      tmp3h = DELTAh(IP1) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(IP1)*DELTAl(IP1)
     +      + DELTAl(IP1)*DELTAh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(IP1) * DELTA(IP1)
!<<<  tmp4 = tmp3 * tmp2
      tmp4h = tmp3h * tmp2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*tmp2l
     +      + tmp3l*tmp2h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp2
!<<<  A = tmp1 + tmp4
      T_p1 = tmp1h + tmp4h
      T_p2 = T_p1 - tmp1h
      Ah = T_p1
      Al = (tmp1h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Al = Al + tmp1l + tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp1 + tmp4
                        ELSE
!                           A = Z( IP1 )*Z( IP1 ) +
!     $                         DELTA( I )*DELTA( I )*( DPSI+DPHI )
!<<<  tmp1 = Z(IP1) * Z(IP1)
      tmp1h = Zh(IP1) * Zh(IP1)
      T_u1 = 134217729.0 * Zh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_p2 = Zh(IP1) - T_p1
      T_u1 = 134217729.0 * Zh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IP1))
      T_r2 = Zh(IP1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(IP1)*Zl(IP1)
     +      + Zl(IP1)*Zh(IP1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(IP1) * Z(IP1)
!<<<  tmp2 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + DPSIl + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + DPHI
!<<<  tmp3 = DELTA(I) * DELTA(I)
      tmp3h = DELTAh(I) * DELTAh(I)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(I)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_r2 = DELTAh(I) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(I)*DELTAl(I)
     +      + DELTAl(I)*DELTAh(I))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(I) * DELTA(I)
!<<<  tmp4 = tmp3 * tmp2
      tmp4h = tmp3h * tmp2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*tmp2l
     +      + tmp3l*tmp2h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp2
!<<<  A = tmp1 + tmp4
      T_p1 = tmp1h + tmp4h
      T_p2 = T_p1 - tmp1h
      Ah = T_p1
      Al = (tmp1h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Al = Al + tmp1l + tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp1 + tmp4

                        END IF
                     ELSE
!                        A = DELTA( I )*DELTA( I )*DPSI +
!     $                      DELTA( IP1 )*DELTA( IP1 )*DPHI
!<<<  tmp1 = DELTA(I) * DELTA(I)
      tmp1h = DELTAh(I) * DELTAh(I)
      T_u1 = 134217729.0 * DELTAh(I)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_p2 = DELTAh(I) - T_p1
      T_u1 = 134217729.0 * DELTAh(I)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(I))
      T_r2 = DELTAh(I) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(I)*DELTAl(I)
     +      + DELTAl(I)*DELTAh(I))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(I) * DELTA(I)
!<<<  tmp2 = tmp1 * DPSI
      tmp2h = tmp1h * DPSIh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DPSIh
      T_r1 = T_u1
     +         - (T_u1 - DPSIh)
      T_r2 = DPSIh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*DPSIl
     +      + tmp1l*DPSIh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * DPSI
!<<<  tmp3 = DELTA(IP1) * DELTA(IP1)
      tmp3h = DELTAh(IP1) * DELTAh(IP1)
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_p2 = DELTAh(IP1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IP1))
      T_r2 = DELTAh(IP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(IP1)*DELTAl(IP1)
     +      + DELTAl(IP1)*DELTAh(IP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(IP1) * DELTA(IP1)
!<<<  tmp4 = tmp3 * DPHI
      tmp4h = tmp3h * DPHIh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * DPHIh
      T_r1 = T_u1
     +         - (T_u1 - DPHIh)
      T_r2 = DPHIh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*DPHIl
     +      + tmp3l*DPHIh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * DPHI
!<<<  A = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Al = Al + tmp2l + tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp2 + tmp4
                     END IF
                  END IF
!                  ETA = B / A
!<<<  ETA = B / A
      T_q1 = Bh / Ah
      ETAh = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (Al*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = Bh - ETAh
      T_r1 = T_s1 - Bh
      T_s2 = (Bh 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = B / A
               ELSE IF(
!                       A.LE.ZERO
!       (A .le. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .le. ZEROl)))
     $                  ) THEN
!                  ETA = ( A-SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A - tmp6
      T_p1 = Ah - tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +         - (tmp6h + T_p2)
      tmp7l = tmp7l + Al - tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp7 = A - tmp6
!<<<  ETA = tmp7 / tmp1
      T_q1 = tmp7h / tmp1h
      ETAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp1l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp7h - ETAh
      T_r1 = T_s1 - tmp7h
      T_s2 = (tmp7h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp7l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp7 / tmp1
               ELSE
!                  ETA = TWO*B / ( A+SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp3 = tmp3 * C
      T_ch = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*Cl
     +         +  tmp3l*Ch)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp3 * C
!<<<  tmp4 = tmp2 - tmp3
      T_p1 = tmp2h - tmp3h
      T_p2 = T_p1 - tmp2h
      tmp4h = T_p1
      tmp4l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp4l = tmp4l + tmp2l - tmp3l
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp4 = tmp2 - tmp3
!<<<  tmp5 = abs(tmp4)
      if (tmp4h .lt. 0.0d+0) then
        tmp5h = -tmp4h
        tmp5l = -tmp4l
      elseif (
     +  (tmp4h .eq. 0.0d+0).and.
     +  (tmp4l .lt. 0.0d+0)) then
        tmp5h = tmp4h
        tmp5l = -tmp4l
      else
        tmp5h = tmp4h
        tmp5l = tmp4l
      endif
!>>>  tmp5 = abs(tmp4)
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      endif
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = A + tmp6
      T_p1 = Ah + tmp6h
      T_p2 = T_p1 - Ah
      tmp7h = T_p1
      tmp7l = (Ah - (T_p1 - T_p2))
     +      + (tmp6h - T_p2)
      tmp7l = tmp7l + Al + tmp6l
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp7 = A + tmp6
!<<<  ETA = tmp1 / tmp7
      T_q1 = tmp1h / tmp7h
      ETAh = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp7l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / tmp7
               END IF
            ELSE
*
*              Interpolation using THREE most relevant poles
*
!               TEMP = RHOINV + PSI + PHI
!<<<  TEMP = RHOINV + PSI
      T_p1 = RHOINVh + PSIh
      T_p2 = T_p1 - RHOINVh
      TEMPh = T_p1
      TEMPl = (RHOINVh - (T_p1 - T_p2))
     +      + (PSIh - T_p2)
      TEMPl = TEMPl + RHOINVl + PSIl
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = RHOINV + PSI
!<<<  TEMP = TEMP + PHI
      T_p1 = TEMPh + PHIh
      T_p2 = T_p1 - TEMPh
      T_q1 = T_p1
      T_q2 = (TEMPh - (T_p1 - T_p2))
     +         + (PHIh - T_p2)
      T_q2 = T_q2 + TEMPl + PHIl
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2 - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TEMP = TEMP + PHI
               IF( SWTCH ) THEN
!                  C = TEMP - DELTA( IIM1 )*DPSI - DELTA( IIP1 )*DPHI
!<<<  tmp1 = DELTA(IIM1) * DPSI
      tmp1h = DELTAh(IIM1) * DPSIh
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * DPSIh
      T_r1 = T_u1
     +         - (T_u1 - DPSIh)
      T_r2 = DPSIh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (DELTAh(IIM1)*DPSIl
     +      + DELTAl(IIM1)*DPSIh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = DELTA(IIM1) * DPSI
!<<<  tmp2 = DELTA(IIP1) * DPHI
      tmp2h = DELTAh(IIP1) * DPHIh
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * DPHIh
      T_r1 = T_u1
     +         - (T_u1 - DPHIh)
      T_r2 = DPHIh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IIP1)*DPHIl
     +      + DELTAl(IIP1)*DPHIh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IIP1) * DPHI
!<<<  tmp3 = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      tmp3h = T_p1
      tmp3l = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + tmp1l + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp1 + tmp2
!<<<  C = TEMP - tmp3
      T_p1 = TEMPh - tmp3h
      T_p2 = T_p1 - TEMPh
      Ch = T_p1
      Cl = (TEMPh - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      Cl = Cl + TEMPl - tmp3l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = TEMP - tmp3
!                  ZZ( 1 ) = DELTA( IIM1 )*DELTA( IIM1 )*DPSI
!<<<  ZZ(1) = DELTA(IIM1) * DELTA(IIM1)
      ZZh(1) = DELTAh(IIM1) * DELTAh(IIM1)
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_r2 = DELTAh(IIM1) - T_r1
      ZZl(1) = ((T_p1*T_r1
     +        - ZZh(1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(1) = ZZl(1)
     +      + (DELTAh(IIM1)*DELTAl(IIM1)
     +      + DELTAl(IIM1)*DELTAh(IIM1))
      T_s1 = ZZh(1) + ZZl(1)
      ZZl(1) = ZZl(1) - (T_s1 - ZZh(1))
      ZZh(1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(1) = DELTA(IIM1) * DELTA(IIM1)
!<<<  ZZ(1) = ZZ(1) * DPSI
      T_ch = ZZh(1) * DPSIh
      T_u1 = 134217729.0 * ZZh(1)
      T_p1 = T_u1
     +         - (T_u1 - ZZh(1))
      T_p2 = ZZh(1) - T_p1
      T_u1 = 134217729.0 * DPSIh
      T_r1 = T_u1
     +         - (T_u1 - DPSIh)
      T_r2 = DPSIh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ZZh(1)*DPSIl
     +         +  ZZl(1)*DPSIh)
      T_s1 = T_ch + T_cl
      ZZl(1) = T_cl - (T_s1 - T_ch)
      ZZh(1) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  ZZ(1) = ZZ(1) * DPSI
!                  ZZ( 3 ) = DELTA( IIP1 )*DELTA( IIP1 )*DPHI
!<<<  ZZ(3) = DELTA(IIP1) * DELTA(IIP1)
      ZZh(3) = DELTAh(IIP1) * DELTAh(IIP1)
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_r2 = DELTAh(IIP1) - T_r1
      ZZl(3) = ((T_p1*T_r1
     +        - ZZh(3))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(3) = ZZl(3)
     +      + (DELTAh(IIP1)*DELTAl(IIP1)
     +      + DELTAl(IIP1)*DELTAh(IIP1))
      T_s1 = ZZh(3) + ZZl(3)
      ZZl(3) = ZZl(3) - (T_s1 - ZZh(3))
      ZZh(3) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(3) = DELTA(IIP1) * DELTA(IIP1)
!<<<  ZZ(3) = ZZ(3) * DPHI
      T_ch = ZZh(3) * DPHIh
      T_u1 = 134217729.0 * ZZh(3)
      T_p1 = T_u1
     +         - (T_u1 - ZZh(3))
      T_p2 = ZZh(3) - T_p1
      T_u1 = 134217729.0 * DPHIh
      T_r1 = T_u1
     +         - (T_u1 - DPHIh)
      T_r2 = DPHIh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ZZh(3)*DPHIl
     +         +  ZZl(3)*DPHIh)
      T_s1 = T_ch + T_cl
      ZZl(3) = T_cl - (T_s1 - T_ch)
      ZZh(3) = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  ZZ(3) = ZZ(3) * DPHI
               ELSE
                  IF( ORGATI ) THEN
!                     TEMP1 = Z( IIM1 ) / DELTA( IIM1 )
!<<<  TEMP1 = Z(IIM1) / DELTA(IIM1)
      T_q1 = Zh(IIM1) / DELTAh(IIM1)
      TEMP1h = DELTAh(IIM1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMP1l = ((T_p1*T_r1
     +       - TEMP1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMP1l = TEMP1l + (DELTAl(IIM1)*T_q1)
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
      T_s1 = Zh(IIM1) - TEMP1h
      T_r1 = T_s1 - Zh(IIM1)
      T_s2 = (Zh(IIM1) 
     +         - (T_s1 - T_r1))
     +         - (TEMP1h + T_r1)
      T_s2 = T_s2 - TEMP1l
      T_s2 = T_s2 + Zl(IIM1)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IIM1)
      TEMP1h = T_q1 + T_q2
      TEMP1l = T_q2
     +      - (TEMP1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP1 = Z(IIM1) / DELTA(IIM1)
!                     TEMP1 = TEMP1*TEMP1
!<<<  TEMP1 = TEMP1 * TEMP1
      T_ch = TEMP1h * TEMP1h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TEMP1h*TEMP1l
     +         +  TEMP1l*TEMP1h)
      T_s1 = T_ch + T_cl
      TEMP1l = T_cl - (T_s1 - T_ch)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP1 = TEMP1 * TEMP1
!                     C = TEMP - DELTA( IIP1 )*( DPSI+DPHI ) -
!     $                   ( D( IIM1 )-D( IIP1 ) )*TEMP1
!<<<  tmp1 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp1l = tmp1l + DPSIl + DPHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DPSI + DPHI
!<<<  tmp2 = DELTA(IIP1) * tmp1
      tmp2h = DELTAh(IIP1) * tmp1h
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IIP1)*tmp1l
     +      + DELTAl(IIP1)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IIP1) * tmp1
!<<<  tmp3 = D(IIM1) - D(IIP1)
      T_p1 = Dh(IIM1) - Dh(IIP1)
      T_p2 = T_p1 - Dh(IIM1)
      tmp3h = T_p1
      tmp3l = (Dh(IIM1) - (T_p1 - T_p2))
     +         - (Dh(IIP1) + T_p2)
      tmp3l = tmp3l + Dl(IIM1) - Dl(IIP1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(IIM1) - D(IIP1)
!<<<  tmp4 = tmp3 * TEMP1
      tmp4h = tmp3h * TEMP1h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*TEMP1l
     +      + tmp3l*TEMP1h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * TEMP1
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  C = TEMP - tmp5
      T_p1 = TEMPh - tmp5h
      T_p2 = T_p1 - TEMPh
      Ch = T_p1
      Cl = (TEMPh - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Cl = Cl + TEMPl - tmp5l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = TEMP - tmp5
!                     ZZ( 1 ) = Z( IIM1 )*Z( IIM1 )
!<<<  ZZ(1) = Z(IIM1) * Z(IIM1)
      ZZh(1) = Zh(IIM1) * Zh(IIM1)
      T_u1 = 134217729.0 * Zh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IIM1))
      T_p2 = Zh(IIM1) - T_p1
      T_u1 = 134217729.0 * Zh(IIM1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IIM1))
      T_r2 = Zh(IIM1) - T_r1
      ZZl(1) = ((T_p1*T_r1
     +        - ZZh(1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(1) = ZZl(1)
     +      + (Zh(IIM1)*Zl(IIM1)
     +      + Zl(IIM1)*Zh(IIM1))
      T_s1 = ZZh(1) + ZZl(1)
      ZZl(1) = ZZl(1) - (T_s1 - ZZh(1))
      ZZh(1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(1) = Z(IIM1) * Z(IIM1)
!                     ZZ( 3 ) = DELTA( IIP1 )*DELTA( IIP1 )*
!     $                         ( ( DPSI-TEMP1 )+DPHI )
!<<<  tmp1 = DPSI - TEMP1
      T_p1 = DPSIh - TEMP1h
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +         - (TEMP1h + T_p2)
      tmp1l = tmp1l + DPSIl - TEMP1l
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DPSI - TEMP1
!<<<  tmp2 = tmp1 + DPHI
      T_p1 = tmp1h + DPHIh
      T_p2 = T_p1 - tmp1h
      tmp2h = T_p1
      tmp2l = (tmp1h - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + tmp1l + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = tmp1 + DPHI
!<<<  tmp3 = DELTA(IIP1) * DELTA(IIP1)
      tmp3h = DELTAh(IIP1) * DELTAh(IIP1)
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_r2 = DELTAh(IIP1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(IIP1)*DELTAl(IIP1)
     +      + DELTAl(IIP1)*DELTAh(IIP1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(IIP1) * DELTA(IIP1)
!<<<  ZZ(3) = tmp3 * tmp2
      ZZh(3) = tmp3h * tmp2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      ZZl(3) = ((T_p1*T_r1
     +        - ZZh(3))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(3) = ZZl(3)
     +      + (tmp3h*tmp2l
     +      + tmp3l*tmp2h)
      T_s1 = ZZh(3) + ZZl(3)
      ZZl(3) = ZZl(3) - (T_s1 - ZZh(3))
      ZZh(3) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(3) = tmp3 * tmp2
                  ELSE
!                     TEMP1 = Z( IIP1 ) / DELTA( IIP1 )
!<<<  TEMP1 = Z(IIP1) / DELTA(IIP1)
      T_q1 = Zh(IIP1) / DELTAh(IIP1)
      TEMP1h = DELTAh(IIP1) * T_q1
      T_u1 = 134217729.0 * DELTAh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIP1))
      T_p2 = DELTAh(IIP1) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMP1l = ((T_p1*T_r1
     +       - TEMP1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMP1l = TEMP1l + (DELTAl(IIP1)*T_q1)
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
      T_s1 = Zh(IIP1) - TEMP1h
      T_r1 = T_s1 - Zh(IIP1)
      T_s2 = (Zh(IIP1) 
     +         - (T_s1 - T_r1))
     +         - (TEMP1h + T_r1)
      T_s2 = T_s2 - TEMP1l
      T_s2 = T_s2 + Zl(IIP1)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(IIP1)
      TEMP1h = T_q1 + T_q2
      TEMP1l = T_q2
     +      - (TEMP1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP1 = Z(IIP1) / DELTA(IIP1)
!                     TEMP1 = TEMP1*TEMP1
!<<<  TEMP1 = TEMP1 * TEMP1
      T_ch = TEMP1h * TEMP1h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TEMP1h*TEMP1l
     +         +  TEMP1l*TEMP1h)
      T_s1 = T_ch + T_cl
      TEMP1l = T_cl - (T_s1 - T_ch)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP1 = TEMP1 * TEMP1
!                     C = TEMP - DELTA( IIM1 )*( DPSI+DPHI ) -
!     $                   ( D( IIP1 )-D( IIM1 ) )*TEMP1
!<<<  tmp1 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp1h = T_p1
      tmp1l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp1l = tmp1l + DPSIl + DPHIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = DPSI + DPHI
!<<<  tmp2 = DELTA(IIM1) * tmp1
      tmp2h = DELTAh(IIM1) * tmp1h
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (DELTAh(IIM1)*tmp1l
     +      + DELTAl(IIM1)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = DELTA(IIM1) * tmp1
!<<<  tmp3 = D(IIP1) - D(IIM1)
      T_p1 = Dh(IIP1) - Dh(IIM1)
      T_p2 = T_p1 - Dh(IIP1)
      tmp3h = T_p1
      tmp3l = (Dh(IIP1) - (T_p1 - T_p2))
     +         - (Dh(IIM1) + T_p2)
      tmp3l = tmp3l + Dl(IIP1) - Dl(IIM1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(IIP1) - D(IIM1)
!<<<  tmp4 = tmp3 * TEMP1
      tmp4h = tmp3h * TEMP1h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * TEMP1h
      T_r1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_r2 = TEMP1h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*TEMP1l
     +      + tmp3l*TEMP1h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * TEMP1
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  C = TEMP - tmp5
      T_p1 = TEMPh - tmp5h
      T_p2 = T_p1 - TEMPh
      Ch = T_p1
      Cl = (TEMPh - (T_p1 - T_p2))
     +         - (tmp5h + T_p2)
      Cl = Cl + TEMPl - tmp5l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  C = TEMP - tmp5
!                     ZZ( 1 ) = DELTA( IIM1 )*DELTA( IIM1 )*
!     $                         ( DPSI+( DPHI-TEMP1 ) )
!<<<  tmp1 = DPHI - TEMP1
      T_p1 = DPHIh - TEMP1h
      T_p2 = T_p1 - DPHIh
      tmp1h = T_p1
      tmp1l = (DPHIh - (T_p1 - T_p2))
     +         - (TEMP1h + T_p2)
      tmp1l = tmp1l + DPHIl - TEMP1l
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DPHI - TEMP1
!<<<  tmp2 = DPSI + tmp1
      T_p1 = DPSIh + tmp1h
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      tmp2l = tmp2l + DPSIl + tmp1l
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + tmp1
!<<<  tmp3 = DELTA(IIM1) * DELTA(IIM1)
      tmp3h = DELTAh(IIM1) * DELTAh(IIM1)
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_p2 = DELTAh(IIM1) - T_p1
      T_u1 = 134217729.0 * DELTAh(IIM1)
      T_r1 = T_u1
     +         - (T_u1 - DELTAh(IIM1))
      T_r2 = DELTAh(IIM1) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (DELTAh(IIM1)*DELTAl(IIM1)
     +      + DELTAl(IIM1)*DELTAh(IIM1))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = DELTA(IIM1) * DELTA(IIM1)
!<<<  ZZ(1) = tmp3 * tmp2
      ZZh(1) = tmp3h * tmp2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      ZZl(1) = ((T_p1*T_r1
     +        - ZZh(1))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(1) = ZZl(1)
     +      + (tmp3h*tmp2l
     +      + tmp3l*tmp2h)
      T_s1 = ZZh(1) + ZZl(1)
      ZZl(1) = ZZl(1) - (T_s1 - ZZh(1))
      ZZh(1) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(1) = tmp3 * tmp2
!                     ZZ( 3 ) = Z( IIP1 )*Z( IIP1 )
!<<<  ZZ(3) = Z(IIP1) * Z(IIP1)
      ZZh(3) = Zh(IIP1) * Zh(IIP1)
      T_u1 = 134217729.0 * Zh(IIP1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(IIP1))
      T_p2 = Zh(IIP1) - T_p1
      T_u1 = 134217729.0 * Zh(IIP1)
      T_r1 = T_u1
     +         - (T_u1 - Zh(IIP1))
      T_r2 = Zh(IIP1) - T_r1
      ZZl(3) = ((T_p1*T_r1
     +        - ZZh(3))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZZl(3) = ZZl(3)
     +      + (Zh(IIP1)*Zl(IIP1)
     +      + Zl(IIP1)*Zh(IIP1))
      T_s1 = ZZh(3) + ZZl(3)
      ZZl(3) = ZZl(3) - (T_s1 - ZZh(3))
      ZZh(3) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZZ(3) = Z(IIP1) * Z(IIP1)
                  END IF
               END IF
!               CALL DLAED6( NITER, ORGATI, C, DELTA( IIM1 ), ZZ, W, ETA,
!     $                      INFO )
               CALL DDLAED6( NITER, ORGATI, Ch, Cl, DELTAh( IIM1 ), 
     $                       DELTAl( IIM1 ), ZZh, ZZl, Wh, Wl, 
     $                      ETAh, ETAl, INFO )
               IF( INFO.NE.0 )
     $            GO TO 250
            END IF
*
*           Note, eta should be positive if w is negative, and
*           eta should be negative otherwise. However,
*           if for some reason caused by roundoff, eta*w > 0,
*           we simply use one Newton step instead. This way
*           will guarantee eta*w < 0.
*
!<<<  tmp1 = W * ETA
      tmp1h = Wh * ETAh
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * ETAh
      T_r1 = T_u1
     +         - (T_u1 - ETAh)
      T_r2 = ETAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Wh*ETAl
     +      + Wl*ETAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = W * ETA
            IF(
!               W*ETA.GE.ZERO
!       (tmp1 .ge. ZERO)
     + ((tmp1h .gt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .ge. ZEROl)))
     $         ) THEN
!     $         ETA = -W / DW
!<<<  ETA = W / DW
      T_q1 = Wh / DWh
      ETAh = DWh * T_q1
      T_u1 = 134217729.0 * DWh
      T_p1 = T_u1
     +         - (T_u1 - DWh)
      T_p2 = DWh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (DWl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = Wh - ETAh
      T_r1 = T_s1 - Wh
      T_s2 = (Wh 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + Wl
      T_q2 = (T_s1 + T_s2)
     +         / DWh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = W / DW
!<<<  ETA = -ETA
      ETAh = -ETAh
      ETAl = -ETAl
!>>>  ETA = -ETA
             END IF
!            TEMP = TAU + ETA
!<<<  TEMP = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      TEMPh = T_p1
      TEMPl = (TAUh - (T_p1 - T_p2))
     +      + (ETAh - T_p2)
      TEMPl = TEMPl + TAUl + ETAl
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = TAU + ETA
            IF(
!              TEMP.GT.DLTUB 
!       (TEMP .gt. DLTUB)
     + ((TEMPh .gt. DLTUBh) .or.
     +  ((TEMPh .eq. DLTUBh) .and.
     +   (TEMPl .gt. DLTUBl)))
     $         .OR. 
!              TEMP.LT.DLTLB
!       (TEMP .lt. DLTLB)
     + ((TEMPh .lt. DLTLBh) .or.
     +  ((TEMPh .eq. DLTLBh) .and.
     +   (TEMPl .lt. DLTLBl)))
     $          ) THEN
               IF(
!                 W.LT.ZERO
!       (W .lt. ZERO)
     + ((Wh .lt. ZEROh) .or.
     +  ((Wh .eq. ZEROh) .and.
     +   (Wl .lt. ZEROl)))
     $             ) THEN
!                  ETA = ( DLTUB-TAU ) / TWO
!<<<  tmp1 = DLTUB - TAU
      T_p1 = DLTUBh - TAUh
      T_p2 = T_p1 - DLTUBh
      tmp1h = T_p1
      tmp1l = (DLTUBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTUBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTUB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
               ELSE
!                  ETA = ( DLTLB-TAU ) / TWO
!<<<  tmp1 = DLTLB - TAU
      T_p1 = DLTLBh - TAUh
      T_p2 = T_p1 - DLTLBh
      tmp1h = T_p1
      tmp1l = (DLTLBh - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DLTLBl - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DLTLB - TAU
!<<<  ETA = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      ETAh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (TWOl*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / TWO
               END IF
            END IF
*
            DO 210 J = 1, N
!               DELTA( J ) = DELTA( J ) - ETA
!<<<  DELTA(J) = DELTA(J) - ETA
      T_p1 = DELTAh(J) - ETAh
      T_p2 = T_p1 - DELTAh(J)
      T_ch = T_p1
      T_cl = (DELTAh(J) - (T_p1 - T_p2))
     +         - (ETAh + T_p2)
      T_cl = T_cl + DELTAl(J) - ETAl
      T_s1 = T_ch + T_cl
      DELTAl(J) = T_cl - (T_s1 - T_ch)
      DELTAh(J) = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  DELTA(J) = DELTA(J) - ETA
  210       CONTINUE
*
!            TAU = TAU + ETA
!<<<  TAU = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      T_q1 = T_p1
      T_q2 = (TAUh - (T_p1 - T_p2))
     +         + (ETAh - T_p2)
      T_q2 = T_q2 + TAUl + ETAl
      TAUh = T_q1 + T_q2
      TAUl = T_q2 - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TAU = TAU + ETA
!            PREW = W
      PREWh = Wh
      PREWl = Wl
*
*           Evaluate PSI and the derivative DPSI
*
!            DPSI = ZERO
!            PSI = ZERO
!            ERRETM = ZERO
      DPSIh = ZEROh
      DPSIl = ZEROl
      PSIh = ZEROh
      PSIl = ZEROl
      ERRETMh = ZEROh
      ERRETMl = ZEROl
            DO 220 J = 1, IIM1
!               TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!               PSI = PSI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PSI = PSI + tmp1
      T_p1 = PSIh + tmp1h
      T_p2 = T_p1 - PSIh
      T_q1 = T_p1
      T_q2 = (PSIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PSIl + tmp1l
      PSIh = T_q1 + T_q2
      PSIl = T_q2 - (PSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PSI = PSI + tmp1
!               DPSI = DPSI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPSI = DPSI + tmp2
      T_p1 = DPSIh + tmp2h
      T_p2 = T_p1 - DPSIh
      T_q1 = T_p1
      T_q2 = (DPSIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPSIl + tmp2l
      DPSIh = T_q1 + T_q2
      DPSIl = T_q2 - (DPSIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPSI = DPSI + tmp2
!               ERRETM = ERRETM + PSI
!<<<  ERRETM = ERRETM + PSI
      T_p1 = ERRETMh + PSIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PSIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PSI
  220       CONTINUE
!            ERRETM = ABS( ERRETM )
!<<<  ERRETM = abs(ERRETM)
      if (ERRETMh .lt. 0.0d+0) then
        ERRETMh = -ERRETMh
        ERRETMl = -ERRETMl
      elseif (
     +  (ERRETMh .eq. 0.0d+0).and.
     +  (ERRETMl .lt. 0.0d+0)) then
        ERRETMh = ERRETMh
        ERRETMl = -ERRETMl
      else
        ERRETMh = ERRETMh
        ERRETMl = ERRETMl
      endif
!>>>  ERRETM = abs(ERRETM)
*
*           Evaluate PHI and the derivative DPHI
*
!            DPHI = ZERO
!            PHI = ZERO
      DPHIh = ZEROh
      DPHIl = ZEROl
      PHIh = ZEROh
      PHIl = ZEROl
            DO 230 J = N, IIP1, -1
!               TEMP = Z( J ) / DELTA( J )
!<<<  TEMP = Z(J) / DELTA(J)
      T_q1 = Zh(J) / DELTAh(J)
      TEMPh = DELTAh(J) * T_q1
      T_u1 = 134217729.0 * DELTAh(J)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(J))
      T_p2 = DELTAh(J) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(J)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(J) - TEMPh
      T_r1 = T_s1 - Zh(J)
      T_s2 = (Zh(J) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(J)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(J)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(J) / DELTA(J)
!               PHI = PHI + Z( J )*TEMP
!<<<  tmp1 = Z(J) * TEMP
      tmp1h = Zh(J) * TEMPh
      T_u1 = 134217729.0 * Zh(J)
      T_p1 = T_u1
     +         - (T_u1 - Zh(J))
      T_p2 = Zh(J) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Zh(J)*TEMPl
     +      + Zl(J)*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = Z(J) * TEMP
!<<<  PHI = PHI + tmp1
      T_p1 = PHIh + tmp1h
      T_p2 = T_p1 - PHIh
      T_q1 = T_p1
      T_q2 = (PHIh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + PHIl + tmp1l
      PHIh = T_q1 + T_q2
      PHIl = T_q2 - (PHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  PHI = PHI + tmp1
!               DPHI = DPHI + TEMP*TEMP
!<<<  tmp2 = TEMP * TEMP
      tmp2h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = TEMP * TEMP
!<<<  DPHI = DPHI + tmp2
      T_p1 = DPHIh + tmp2h
      T_p2 = T_p1 - DPHIh
      T_q1 = T_p1
      T_q2 = (DPHIh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + DPHIl + tmp2l
      DPHIh = T_q1 + T_q2
      DPHIl = T_q2 - (DPHIh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DPHI = DPHI + tmp2
!               ERRETM = ERRETM + PHI
!<<<  ERRETM = ERRETM + PHI
      T_p1 = ERRETMh + PHIh
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (PHIh - T_p2)
      T_q2 = T_q2 + ERRETMl + PHIl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + PHI
  230       CONTINUE
*
!            TEMP = Z( II ) / DELTA( II )
!<<<  TEMP = Z(II) / DELTA(II)
      T_q1 = Zh(II) / DELTAh(II)
      TEMPh = DELTAh(II) * T_q1
      T_u1 = 134217729.0 * DELTAh(II)
      T_p1 = T_u1
     +         - (T_u1 - DELTAh(II))
      T_p2 = DELTAh(II) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (DELTAl(II)*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = Zh(II) - TEMPh
      T_r1 = T_s1 - Zh(II)
      T_s2 = (Zh(II) 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + Zl(II)
      T_q2 = (T_s1 + T_s2)
     +         / DELTAh(II)
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = Z(II) / DELTA(II)
!            DW = DPSI + DPHI + TEMP*TEMP
!<<<  tmp1 = TEMP * TEMP
      tmp1h = TEMPh * TEMPh
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMPh*TEMPl
     +      + TEMPl*TEMPh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP * TEMP
!<<<  tmp2 = DPSI + DPHI
      T_p1 = DPSIh + DPHIh
      T_p2 = T_p1 - DPSIh
      tmp2h = T_p1
      tmp2l = (DPSIh - (T_p1 - T_p2))
     +      + (DPHIh - T_p2)
      tmp2l = tmp2l + DPSIl + DPHIl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp2 = DPSI + DPHI
!<<<  DW = tmp1 + tmp2
      T_p1 = tmp1h + tmp2h
      T_p2 = T_p1 - tmp1h
      DWh = T_p1
      DWl = (tmp1h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      DWl = DWl + tmp1l + tmp2l
      T_s1 = DWh + DWl
      DWl = DWl - (T_s1 - DWh)
      DWh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DW = tmp1 + tmp2
!            TEMP = Z( II )*TEMP
!<<<  TEMP = Z(II) * TEMP
      T_ch = Zh(II) * TEMPh
      T_u1 = 134217729.0 * Zh(II)
      T_p1 = T_u1
     +         - (T_u1 - Zh(II))
      T_p2 = Zh(II) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Zh(II)*TEMPl
     +         +  Zl(II)*TEMPh)
      T_s1 = T_ch + T_cl
      TEMPl = T_cl - (T_s1 - T_ch)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TEMP = Z(II) * TEMP
!            W = RHOINV + PHI + PSI + TEMP
!<<<  W = RHOINV + PHI
      T_p1 = RHOINVh + PHIh
      T_p2 = T_p1 - RHOINVh
      Wh = T_p1
      Wl = (RHOINVh - (T_p1 - T_p2))
     +      + (PHIh - T_p2)
      Wl = Wl + RHOINVl + PHIl
      T_s1 = Wh + Wl
      Wl = Wl - (T_s1 - Wh)
      Wh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  W = RHOINV + PHI
!<<<  W = W + PSI
      T_p1 = Wh + PSIh
      T_p2 = T_p1 - Wh
      T_q1 = T_p1
      T_q2 = (Wh - (T_p1 - T_p2))
     +         + (PSIh - T_p2)
      T_q2 = T_q2 + Wl + PSIl
      Wh = T_q1 + T_q2
      Wl = T_q2 - (Wh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W = W + PSI
!<<<  W = W + TEMP
      T_p1 = Wh + TEMPh
      T_p2 = T_p1 - Wh
      T_q1 = T_p1
      T_q2 = (Wh - (T_p1 - T_p2))
     +         + (TEMPh - T_p2)
      T_q2 = T_q2 + Wl + TEMPl
      Wh = T_q1 + T_q2
      Wl = T_q2 - (Wh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  W = W + TEMP
!            ERRETM = EIGHT*( PHI-PSI ) + ERRETM + TWO*RHOINV +
!     $               THREE*ABS( TEMP ) + ABS( TAU )*DW
!<<<  tmp1 = PHI - PSI
      T_p1 = PHIh - PSIh
      T_p2 = T_p1 - PHIh
      tmp1h = T_p1
      tmp1l = (PHIh - (T_p1 - T_p2))
     +         - (PSIh + T_p2)
      tmp1l = tmp1l + PHIl - PSIl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = PHI - PSI
!<<<  tmp2 = EIGHT * tmp1
      tmp2h = EIGHTh * tmp1h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EIGHTh*tmp1l
     +      + EIGHTl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EIGHT * tmp1
!<<<  tmp3 = TWO * RHOINV
      tmp3h = TWOh * RHOINVh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * RHOINVh
      T_r1 = T_u1
     +         - (T_u1 - RHOINVh)
      T_r2 = RHOINVh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (TWOh*RHOINVl
     +      + TWOl*RHOINVh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = TWO * RHOINV
!<<<  tmp4 = abs(TEMP)
      if (TEMPh .lt. 0.0d+0) then
        tmp4h = -TEMPh
        tmp4l = -TEMPl
      elseif (
     +  (TEMPh .eq. 0.0d+0).and.
     +  (TEMPl .lt. 0.0d+0)) then
        tmp4h = TEMPh
        tmp4l = -TEMPl
      else
        tmp4h = TEMPh
        tmp4l = TEMPl
      endif
!>>>  tmp4 = abs(TEMP)
!<<<  tmp5 = THREE * tmp4
      tmp5h = THREEh * tmp4h
      T_u1 = 134217729.0 * THREEh
      T_p1 = T_u1
     +         - (T_u1 - THREEh)
      T_p2 = THREEh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      tmp5l = ((T_p1*T_r1
     +        - tmp5h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp5l = tmp5l
     +      + (THREEh*tmp4l
     +      + THREEl*tmp4h)
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp5 = THREE * tmp4
!<<<  tmp6 = abs(TAU)
      if (TAUh .lt. 0.0d+0) then
        tmp6h = -TAUh
        tmp6l = -TAUl
      elseif (
     +  (TAUh .eq. 0.0d+0).and.
     +  (TAUl .lt. 0.0d+0)) then
        tmp6h = TAUh
        tmp6l = -TAUl
      else
        tmp6h = TAUh
        tmp6l = TAUl
      endif
!>>>  tmp6 = abs(TAU)
!<<<  tmp7 = tmp6 * DW
      tmp7h = tmp6h * DWh
      T_u1 = 134217729.0 * tmp6h
      T_p1 = T_u1
     +         - (T_u1 - tmp6h)
      T_p2 = tmp6h - T_p1
      T_u1 = 134217729.0 * DWh
      T_r1 = T_u1
     +         - (T_u1 - DWh)
      T_r2 = DWh - T_r1
      tmp7l = ((T_p1*T_r1
     +        - tmp7h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp7l = tmp7l
     +      + (tmp6h*DWl
     +      + tmp6l*DWh)
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp7 = tmp6 * DW
!<<<  ERRETM = tmp2 + ERRETM
      T_p1 = tmp2h + ERRETMh
      T_p2 = T_p1 - tmp2h
      T_q1 = T_p1
      T_q2 = (tmp2h - (T_p1 - T_p2))
     +         + (ERRETMh - T_p2)
      T_q2 = T_q2 + tmp2l + ERRETMl
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = tmp2 + ERRETM
!<<<  ERRETM = ERRETM + tmp3
      T_p1 = ERRETMh + tmp3h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp3h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp3l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp3
!<<<  ERRETM = ERRETM + tmp5
      T_p1 = ERRETMh + tmp5h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp5h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp5l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp5
!<<<  ERRETM = ERRETM + tmp7
      T_p1 = ERRETMh + tmp7h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp7h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp7l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp7
!<<<  tmp1 = W * PREW
      tmp1h = Wh * PREWh
      T_u1 = 134217729.0 * Wh
      T_p1 = T_u1
     +         - (T_u1 - Wh)
      T_p2 = Wh - T_p1
      T_u1 = 134217729.0 * PREWh
      T_r1 = T_u1
     +         - (T_u1 - PREWh)
      T_r2 = PREWh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Wh*PREWl
     +      + Wl*PREWh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = W * PREW
!<<<  tmp2 = abs(W)
      if (Wh .lt. 0.0d+0) then
        tmp2h = -Wh
        tmp2l = -Wl
      elseif (
     +  (Wh .eq. 0.0d+0).and.
     +  (Wl .lt. 0.0d+0)) then
        tmp2h = Wh
        tmp2l = -Wl
      else
        tmp2h = Wh
        tmp2l = Wl
      endif
!>>>  tmp2 = abs(W)
!<<<  tmp3 = abs(PREW)
      if (PREWh .lt. 0.0d+0) then
        tmp3h = -PREWh
        tmp3l = -PREWl
      elseif (
     +  (PREWh .eq. 0.0d+0).and.
     +  (PREWl .lt. 0.0d+0)) then
        tmp3h = PREWh
        tmp3l = -PREWl
      else
        tmp3h = PREWh
        tmp3l = PREWl
      endif
!>>>  tmp3 = abs(PREW)
!<<<  tmp4 = tmp3 / TEN
      T_q1 = tmp3h / TENh
      tmp4h = TENh * T_q1
      T_u1 = 134217729.0 * TENh
      T_p1 = T_u1
     +         - (T_u1 - TENh)
      T_p2 = TENh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp4l = ((T_p1*T_r1
     +       - tmp4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp4l = tmp4l + (TENl*T_q1)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
      T_s1 = tmp3h - tmp4h
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h 
     +         - (T_s1 - T_r1))
     +         - (tmp4h + T_r1)
      T_s2 = T_s2 - tmp4l
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / TENh
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = tmp3 / TEN
            IF(
!               W*PREW.GT.ZERO 
!       (tmp1 .gt. ZERO)
     + ((tmp1h .gt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .gt. ZEROl)))
     $         .AND. 
!               ABS( W ).GT.ABS( PREW ) / TEN
!       (tmp2 .gt. tmp4)
     + ((tmp2h .gt. tmp4h) .or.
     +  ((tmp2h .eq. tmp4h) .and.
     +   (tmp2l .gt. tmp4l)))
     $          )
     $         SWTCH = .NOT.SWTCH
*
  240    CONTINUE
*
*        Return with INFO = 1, NITER = MAXIT and not converged
*
         INFO = 1
         IF( ORGATI ) THEN
!            DLAM = D( I ) + TAU
!<<<  DLAM = D(I) + TAU
      T_p1 = Dh(I) + TAUh
      T_p2 = T_p1 - Dh(I)
      DLAMh = T_p1
      DLAMl = (Dh(I) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(I) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(I) + TAU
         ELSE
!            DLAM = D( IP1 ) + TAU
!<<<  DLAM = D(IP1) + TAU
      T_p1 = Dh(IP1) + TAUh
      T_p2 = T_p1 - Dh(IP1)
      DLAMh = T_p1
      DLAMl = (Dh(IP1) - (T_p1 - T_p2))
     +      + (TAUh - T_p2)
      DLAMl = DLAMl + Dl(IP1) + TAUl
      T_s1 = DLAMh + DLAMl
      DLAMl = DLAMl - (T_s1 - DLAMh)
      DLAMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  DLAM = D(IP1) + TAU
         END IF
*
      END IF
*
  250 CONTINUE
*
      RETURN
*
*     End of DLAED4
*
      END
