*> \brief \b DLASQ4
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASQ4 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasq4.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasq4.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasq4.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASQ4( I0, N0, Z, PP, N0IN, DMIN, DMIN1, DMIN2, DN,
*                          DN1, DN2, TAU, TTYPE, G )
* 
*       .. Scalar Arguments ..
*       INTEGER            I0, N0, N0IN, PP, TTYPE
*       DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DN1, DN2, G, TAU
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   Z( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASQ4 computes an approximation TAU to the smallest eigenvalue
*> using values of d from the previous transform.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] I0
*> \verbatim
*>          I0 is INTEGER
*>        First index.
*> \endverbatim
*>
*> \param[in] N0
*> \verbatim
*>          N0 is INTEGER
*>        Last index.
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension ( 4*N )
*>        Z holds the qd array.
*> \endverbatim
*>
*> \param[in] PP
*> \verbatim
*>          PP is INTEGER
*>        PP=0 for ping, PP=1 for pong.
*> \endverbatim
*>
*> \param[in] N0IN
*> \verbatim
*>          N0IN is INTEGER
*>        The value of N0 at start of EIGTEST.
*> \endverbatim
*>
*> \param[in] DMIN
*> \verbatim
*>          DMIN is DOUBLE PRECISION
*>        Minimum value of d.
*> \endverbatim
*>
*> \param[in] DMIN1
*> \verbatim
*>          DMIN1 is DOUBLE PRECISION
*>        Minimum value of d, excluding D( N0 ).
*> \endverbatim
*>
*> \param[in] DMIN2
*> \verbatim
*>          DMIN2 is DOUBLE PRECISION
*>        Minimum value of d, excluding D( N0 ) and D( N0-1 ).
*> \endverbatim
*>
*> \param[in] DN
*> \verbatim
*>          DN is DOUBLE PRECISION
*>        d(N)
*> \endverbatim
*>
*> \param[in] DN1
*> \verbatim
*>          DN1 is DOUBLE PRECISION
*>        d(N-1)
*> \endverbatim
*>
*> \param[in] DN2
*> \verbatim
*>          DN2 is DOUBLE PRECISION
*>        d(N-2)
*> \endverbatim
*>
*> \param[out] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION
*>        This is the shift.
*> \endverbatim
*>
*> \param[out] TTYPE
*> \verbatim
*>          TTYPE is INTEGER
*>        Shift type.
*> \endverbatim
*>
*> \param[in,out] G
*> \verbatim
*>          G is REAL
*>        G is passed as an argument in order to save its value between
*>        calls to DLASQ4.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  CNST1 = 9/16
*> \endverbatim
*>
*  =====================================================================
!!      SUBROUTINE DLASQ4( I0, N0, Z, PP, N0IN, DMIN, DMIN1, DMIN2, DN,
!!     $                   DN1, DN2, TAU, TTYPE, G )
      SUBROUTINE dDLASQ4( I0, N0, Zh, Zl, PP, N0IN, DMINh, DMINl,
     $                   DMIN1h, DMIN1l, DMIN2h, DMIN2l, DNh, DNl,
     $                   DN1h, DN1l, DN2h, DN2l, TAUh, TAUl, TTYPE,
     $                   Gh, Gl )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      INTEGER            I0, N0, N0IN, PP, TTYPE
!!      DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DN1, DN2, G, TAU
      double precision DMINh, DMINl
      double precision DMIN1h, DMIN1l
      double precision DMIN2h, DMIN2l
      double precision DNh, DNl
      double precision DN1h, DN1l
      double precision DN2h, DN2l
      double precision Gh, Gl
      double precision TAUh, TAUl
*     ..
*     .. Array Arguments ..
!!      DOUBLE PRECISION   Z( * )
      double precision Zh(*), Zl(*)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   CNST1, CNST2, CNST3
!!      PARAMETER          ( CNST1 = 0.5630D0, CNST2 = 1.010D0,
!!     $                   CNST3 = 1.050D0 )
      double precision CNST1h, CNST1l
      double precision CNST2h, CNST2l
      double precision CNST3h, CNST3l

!!      DOUBLE PRECISION   QURTR, THIRD, HALF, ZERO, ONE, TWO, HUNDRD
!!      PARAMETER          ( QURTR = 0.250D0, THIRD = 0.3330D0,
!!     $                   HALF = 0.50D0, ZERO = 0.0D0, ONE = 1.0D0,
!!     $                   TWO = 2.0D0, HUNDRD = 100.0D0 )
      double precision QURTRh, QURTRl
      double precision THIRDh, THIRDl
      double precision HALFh, HALFl
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision HUNDRDh, HUNDRDl
*     ..
*     .. Local Scalars ..
      INTEGER            I4, NN, NP
!!      DOUBLE PRECISION   A2, B1, B2, GAM, GAP1, GAP2, S
      double precision A2h, A2l
      double precision B1h, B1l
      double precision B2h, B2l
      double precision GAMh, GAMl
      double precision GAP1h, GAP1l
      double precision GAP2h, GAP2l
      double precision Sh, Sl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
*     ..
*     .. Executable Statements ..
      CNST1h = 0.5630D0
      CNST1l = 0.0d+0
      CNST2h = 1.010D0
      CNST2l = 0.0d+0
      CNST3h = 1.050D0
      CNST3l = 0.0d+0
      QURTRh = 0.250D0
      QURTRl = 0.0d+0
      THIRDh = 0.3330D0
      THIRDl = 0.0d+0
      HALFh = 0.50D0
      HALFl = 0.0d+0
      ZEROh = 0.0D0
      ZEROl = 0.0d+0
      ONEh = 1.0D0
      ONEl = 0.0d+0
      TWOh = 2.0D0
      TWOl = 0.0d+0
      HUNDRDh = 100.0D0
      HUNDRDl = 0.0d+0
*
*     A negative DMIN forces the shift to take that absolute value
*     TTYPE records the type of shift.
*
!!      IF( DMIN.LE.ZERO ) THEN
      IF(
!       (DMIN .le. ZERO)
     + ((DMINh .lt. ZEROh) .or.
     +  ((DMINh .eq. ZEROh) .and.
     +   (DMINl .le. ZEROl)))
     $  ) THEN
!!         TAU = -DMIN
!<<<  TAU = -DMIN
      TAUh = -DMINh
      TAUl = -DMINl
!>>>  TAU = -DMIN
         TTYPE = -1
         RETURN
      END IF
*       
      NN = 4*N0 + PP
      IF( N0IN.EQ.N0 ) THEN
*
*        No eigenvalues deflated.
*
!!         IF( DMIN.EQ.DN .OR. DMIN.EQ.DN1 ) THEN
         IF(
!       (DMIN .eq. DN)
     + ((DMINh .eq. DNh) .and.
     +  (DMINl .eq. DNl))
     $         .OR.
!       (DMIN .eq. DN1)
     + ((DMINh .eq. DN1h) .and.
     +  (DMINl .eq. DN1l))
     $     ) THEN
*
!!            B1 = SQRT( Z( NN-3 ) )*SQRT( Z( NN-5 ) )
!<<<  tmp1 = sqrt(Z(NN-3))
      if (Zh(NN-3).eq.0.0D+0.and.Zl(NN-3).eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (Zh(NN-3).lt.0.0d+0.or.
     +   (Zh(NN-3).eq.0.0d+0.and.Zl(NN-3).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Zh(NN-3))
      T_ax = Zh(NN-3) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Zh(NN-3),Zl(NN-3),T_z1,T_z2)
          T_p1 = Zh(NN-3) - T_z1
          T_p2 = T_p1 - Zh(NN-3)
          T_t2l = (Zh(NN-3)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + Zl(NN-3) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(Z(NN-3))
!<<<  tmp2 = sqrt(Z(NN-5))
      if (Zh(NN-5).eq.0.0D+0.and.Zl(NN-5).eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      else
      if (Zh(NN-5).lt.0.0d+0.or.
     +   (Zh(NN-5).eq.0.0d+0.and.Zl(NN-5).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Zh(NN-5))
      T_ax = Zh(NN-5) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Zh(NN-5),Zl(NN-5),T_z1,T_z2)
          T_p1 = Zh(NN-5) - T_z1
          T_p2 = T_p1 - Zh(NN-5)
          T_t2l = (Zh(NN-5)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + Zl(NN-5) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp2 = sqrt(Z(NN-5))
!<<<  B1 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      B1l = T_cl - (T_s1 - T_ch)
      B1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  B1 = tmp1 * tmp2
!!            B2 = SQRT( Z( NN-7 ) )*SQRT( Z( NN-9 ) )
!<<<  tmp1 = sqrt(Z(NN-7))
      if (Zh(NN-7).eq.0.0D+0.and.Zl(NN-7).eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (Zh(NN-7).lt.0.0d+0.or.
     +   (Zh(NN-7).eq.0.0d+0.and.Zl(NN-7).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Zh(NN-7))
      T_ax = Zh(NN-7) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Zh(NN-7),Zl(NN-7),T_z1,T_z2)
          T_p1 = Zh(NN-7) - T_z1
          T_p2 = T_p1 - Zh(NN-7)
          T_t2l = (Zh(NN-7)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + Zl(NN-7) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(Z(NN-7))
!<<<  tmp2 = sqrt(Z(NN-9))
      if (Zh(NN-9).eq.0.0D+0.and.Zl(NN-9).eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      else
      if (Zh(NN-9).lt.0.0d+0.or.
     +   (Zh(NN-9).eq.0.0d+0.and.Zl(NN-9).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Zh(NN-9))
      T_ax = Zh(NN-9) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Zh(NN-9),Zl(NN-9),T_z1,T_z2)
          T_p1 = Zh(NN-9) - T_z1
          T_p2 = T_p1 - Zh(NN-9)
          T_t2l = (Zh(NN-9)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + Zl(NN-9) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp2 = sqrt(Z(NN-9))
!<<<  B2 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      B2l = T_cl - (T_s1 - T_ch)
      B2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  B2 = tmp1 * tmp2

!!            A2 = Z( NN-7 ) + Z( NN-5 )
!<<<  A2 = Z(NN-7) + Z(NN-5)
      T_p1 = Zh(NN-7) + Zh(NN-5)
      T_p2 = T_p1 - Zh(NN-7)
      T_q1 = T_p1
      T_q2 = (Zh(NN-7) - (T_p1 - T_p2))
     +         + (Zh(NN-5) - T_p2)
      T_q2 = T_q2 + Zl(NN-7) + Zl(NN-5)
      A2h = T_q1 + T_q2
      A2l = T_q2 - (A2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  A2 = Z(NN-7) + Z(NN-5)
*
*           Cases 2 and 3.
*
!!            IF( DMIN.EQ.DN .AND. DMIN1.EQ.DN1 ) THEN
            IF(
!       (DMIN .eq. DN)
     + ((DMINh .eq. DNh) .and.
     +  (DMINl .eq. DNl))
     $            .AND.
!       (DMIN1 .eq. DN1)
     + ((DMIN1h .eq. DN1h) .and.
     +  (DMIN1l .eq. DN1l))
     $        ) THEN
!!               GAP2 = DMIN2 - A2 - DMIN2*QURTR
!<<<  tmp1 = DMIN2 - A2
      T_p1 = DMIN2h - A2h
      T_p2 = T_p1 - DMIN2h
      T_ch = T_p1
      T_cl = (DMIN2h - (T_p1 - T_p2))
     +         - (A2h + T_p2)
      T_cl = T_cl + DMIN2l - A2l
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = DMIN2 - A2
!<<<  tmp2 = DMIN2 * QURTR
      T_ch = DMIN2h * QURTRh
      T_u1 = 134217729.0 * DMIN2h
      T_p1 = T_u1
     +         - (T_u1 - DMIN2h)
      T_p2 = DMIN2h - T_p1
      T_u1 = 134217729.0 * QURTRh
      T_r1 = T_u1
     +         - (T_u1 - QURTRh)
      T_r2 = QURTRh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (DMIN2h*QURTRl
     +         +  DMIN2l*QURTRh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = DMIN2 * QURTR
!<<<  GAP2 = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + tmp1l - tmp2l
      T_s1 = T_ch + T_cl
      GAP2l = T_cl - (T_s1 - T_ch)
      GAP2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  GAP2 = tmp1 - tmp2
!!               IF( GAP2.GT.ZERO .AND. GAP2.GT.B2 ) THEN
               IF(
!       (GAP2 .gt. ZERO)
     + ((GAP2h .gt. ZEROh) .or.
     +  ((GAP2h .eq. ZEROh) .and.
     +   (GAP2l .gt. ZEROl)))
     $               .AND.
!       (GAP2 .gt. B2)
     + ((GAP2h .gt. B2h) .or.
     +  ((GAP2h .eq. B2h) .and.
     +   (GAP2l .gt. B2l)))
     $           ) THEN
!!                  GAP1 = A2 - DN - ( B2 / GAP2 )*B2
!<<<  tmp1 = A2 - DN
      T_p1 = A2h - DNh
      T_p2 = T_p1 - A2h
      T_ch = T_p1
      T_cl = (A2h - (T_p1 - T_p2))
     +         - (DNh + T_p2)
      T_cl = T_cl + A2l - DNl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = A2 - DN
!<<<  tmp2 = B2 / GAP2
      T_q1 = B2h / GAP2h
      T_ch = GAP2h * T_q1
      T_u1 = 134217729.0 * GAP2h
      T_p1 = T_u1
     +         - (T_u1 - GAP2h)
      T_p2 = GAP2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAP2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = B2h - T_ch
      T_r1 = T_s1 - B2h
      T_s2 = (B2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + B2l
      T_q2 = (T_s1 + T_s2)
     +         / GAP2h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = B2 / GAP2
!<<<  tmp3 = tmp2 * B2
      T_ch = tmp2h * B2h
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp2h*B2l
     +         +  tmp2l*B2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp2 * B2
!<<<  GAP1 = tmp1 - tmp3
      T_p1 = tmp1h - tmp3h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      T_cl = T_cl + tmp1l - tmp3l
      T_s1 = T_ch + T_cl
      GAP1l = T_cl - (T_s1 - T_ch)
      GAP1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  GAP1 = tmp1 - tmp3
               ELSE
!!                  GAP1 = A2 - DN - ( B1+B2 )
!<<<  tmp1 = A2 - DN
      T_p1 = A2h - DNh
      T_p2 = T_p1 - A2h
      T_ch = T_p1
      T_cl = (A2h - (T_p1 - T_p2))
     +         - (DNh + T_p2)
      T_cl = T_cl + A2l - DNl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = A2 - DN
!<<<  tmp2 = B1 + B2
      T_p1 = B1h + B2h
      T_p2 = T_p1 - B1h
      T_q1 = T_p1
      T_q2 = (B1h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + B1l + B2l
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = B1 + B2
!<<<  GAP1 = tmp1 - tmp2
      T_p1 = tmp1h - tmp2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + tmp1l - tmp2l
      T_s1 = T_ch + T_cl
      GAP1l = T_cl - (T_s1 - T_ch)
      GAP1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  GAP1 = tmp1 - tmp2
               END IF
!!               IF( GAP1.GT.ZERO .AND. GAP1.GT.B1 ) THEN
               IF(
!       (GAP1 .gt. ZERO)
     + ((GAP1h .gt. ZEROh) .or.
     +  ((GAP1h .eq. ZEROh) .and.
     +   (GAP1l .gt. ZEROl)))
     $               .AND.
!       (GAP1 .gt. B1)
     + ((GAP1h .gt. B1h) .or.
     +  ((GAP1h .eq. B1h) .and.
     +   (GAP1l .gt. B1l)))
     $           ) THEN
!!                  S = MAX( DN-( B1 / GAP1 )*B1, HALF*DMIN )
!<<<  tmp1 = B1 / GAP1
      T_q1 = B1h / GAP1h
      T_ch = GAP1h * T_q1
      T_u1 = 134217729.0 * GAP1h
      T_p1 = T_u1
     +         - (T_u1 - GAP1h)
      T_p2 = GAP1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAP1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = B1h - T_ch
      T_r1 = T_s1 - B1h
      T_s2 = (B1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + B1l
      T_q2 = (T_s1 + T_s2)
     +         / GAP1h
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = B1 / GAP1
!<<<  tmp2 = tmp1 * B1
      T_ch = tmp1h * B1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * B1h
      T_r1 = T_u1
     +         - (T_u1 - B1h)
      T_r2 = B1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*B1l
     +         +  tmp1l*B1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * B1
!<<<  tmp3 = DN - tmp2
      T_p1 = DNh - tmp2h
      T_p2 = T_p1 - DNh
      T_ch = T_p1
      T_cl = (DNh - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + DNl - tmp2l
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp3 = DN - tmp2
!<<<  tmp4 = HALF * DMIN
      T_ch = HALFh * DMINh
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * DMINh
      T_r1 = T_u1
     +         - (T_u1 - DMINh)
      T_r2 = DMINh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HALFh*DMINl
     +         +  HALFl*DMINh)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = HALF * DMIN
                  IF(
!       (tmp3 .gt. tmp4)
     + ((tmp3h .gt. tmp4h) .or.
     +  ((tmp3h .eq. tmp4h) .and.
     +   (tmp3l .gt. tmp4l)))
     $              ) THEN
      Sh = tmp3h
      Sl = tmp3l
                  ELSE
      Sh = tmp4h
      Sl = tmp4l
                  END IF
                  TTYPE = -2
               ELSE
!!                  S = ZERO
      Sh = ZEROh
      Sl = ZEROl
!!                  IF( DN.GT.B1 )
!!     $               S = DN - B1
                  IF(
!       (DN .gt. B1)
     + ((DNh .gt. B1h) .or.
     +  ((DNh .eq. B1h) .and.
     +   (DNl .gt. B1l)))
     $              ) THEN
!<<<  S = DN - B1
      T_p1 = DNh - B1h
      T_p2 = T_p1 - DNh
      T_ch = T_p1
      T_cl = (DNh - (T_p1 - T_p2))
     +         - (B1h + T_p2)
      T_cl = T_cl + DNl - B1l
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  S = DN - B1
                  END IF
!!                  IF( A2.GT.( B1+B2 ) )
!!     $               S = MIN( S, A2-( B1+B2 ) )
!<<<  tmp1 = B1 + B2
      T_p1 = B1h + B2h
      T_p2 = T_p1 - B1h
      T_q1 = T_p1
      T_q2 = (B1h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + B1l + B2l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = B1 + B2
                  IF(
!       (A2 .gt. tmp1)
     + ((A2h .gt. tmp1h) .or.
     +  ((A2h .eq. tmp1h) .and.
     +   (A2l .gt. tmp1l)))
     $              ) THEN
!<<<  tmp2 = B1 + B2
      T_p1 = B1h + B2h
      T_p2 = T_p1 - B1h
      T_q1 = T_p1
      T_q2 = (B1h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + B1l + B2l
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = B1 + B2
!<<<  tmp3 = A2 - tmp2
      T_p1 = A2h - tmp2h
      T_p2 = T_p1 - A2h
      T_ch = T_p1
      T_cl = (A2h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      T_cl = T_cl + A2l - tmp2l
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp3 = A2 - tmp2
                     IF(
!       (S .gt. tmp3)
     + ((Sh .gt. tmp3h) .or.
     +  ((Sh .eq. tmp3h) .and.
     +   (Sl .gt. tmp3l)))
     $                 ) THEN
      Sh = tmp3h
      Sl = tmp3l
                      END IF
                  END IF
!!                  S = MAX( S, THIRD*DMIN )
!<<<  tmp1 = THIRD * DMIN
      T_ch = THIRDh * DMINh
      T_u1 = 134217729.0 * THIRDh
      T_p1 = T_u1
     +         - (T_u1 - THIRDh)
      T_p2 = THIRDh - T_p1
      T_u1 = 134217729.0 * DMINh
      T_r1 = T_u1
     +         - (T_u1 - DMINh)
      T_r2 = DMINh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (THIRDh*DMINl
     +         +  THIRDl*DMINh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = THIRD * DMIN
                  IF(
!       (S .lt. tmp1)
     + ((Sh .lt. tmp1h) .or.
     +  ((Sh .eq. tmp1h) .and.
     +   (Sl .lt. tmp1l)))
     $              ) THEN
      Sh = tmp1h
      Sl = tmp1l
                  END IF
                  TTYPE = -3
               END IF
            ELSE
*
*              Case 4.
*
               TTYPE = -4
!!               S = QURTR*DMIN
!<<<  S = QURTR * DMIN
      T_ch = QURTRh * DMINh
      T_u1 = 134217729.0 * QURTRh
      T_p1 = T_u1
     +         - (T_u1 - QURTRh)
      T_p2 = QURTRh - T_p1
      T_u1 = 134217729.0 * DMINh
      T_r1 = T_u1
     +         - (T_u1 - DMINh)
      T_r2 = DMINh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (QURTRh*DMINl
     +         +  QURTRl*DMINh)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = QURTR * DMIN
!!               IF( DMIN.EQ.DN ) THEN
               IF(
!       (DMIN .eq. DN)
     + ((DMINh .eq. DNh) .and.
     +  (DMINl .eq. DNl))
     &           ) THEN
!!                  GAM = DN
      GAMh = DNh
      GAMl = DNl
!!                  A2 = ZERO
      A2h = ZEROh
      A2l = ZEROl
!!                  IF( Z( NN-5 ) .GT. Z( NN-7 ) )
!!     $               RETURN
                  IF(
!       (Z(NN-5) .gt. Z(NN-7))
     + ((Zh(NN-5) .gt. Zh(NN-7)) .or.
     +  ((Zh(NN-5) .eq. Zh(NN-7)) .and.
     +   (Zl(NN-5) .gt. Zl(NN-7))))
     $              ) THEN
                     RETURN
                  END IF
!!                  B2 = Z( NN-5 ) / Z( NN-7 )
!<<<  B2 = Z(NN-5) / Z(NN-7)
      T_q1 = Zh(NN-5) / Zh(NN-7)
      T_ch = Zh(NN-7) * T_q1
      T_u1 = 134217729.0 * Zh(NN-7)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-7))
      T_p2 = Zh(NN-7) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(NN-7) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-5) - T_ch
      T_r1 = T_s1 - Zh(NN-5)
      T_s2 = (Zh(NN-5)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-5)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(NN-7)
      B2h = T_q1 + T_q2
      B2l = T_q2
     +      - (B2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B2 = Z(NN-5) / Z(NN-7)
                  NP = NN - 9
               ELSE
                  NP = NN - 2*PP
!!                  B2 = Z( NP-2 )
      B2h = Zh(NP-2)
      B2l = Zl(NP-2)
!!                  GAM = DN1
      GAMh = DN1h
      GAMl = DN1l
!!                  IF( Z( NP-4 ) .GT. Z( NP-2 ) )
!!     $               RETURN
                  IF(
!       (Z(NP-4) .gt. Z(NP-2))
     + ((Zh(NP-4) .gt. Zh(NP-2)) .or.
     +  ((Zh(NP-4) .eq. Zh(NP-2)) .and.
     +   (Zl(NP-4) .gt. Zl(NP-2))))
     $              ) THEN
                     RETURN
                  END IF
!!                  A2 = Z( NP-4 ) / Z( NP-2 )
!<<<  A2 = Z(NP-4) / Z(NP-2)
      T_q1 = Zh(NP-4) / Zh(NP-2)
      T_ch = Zh(NP-2) * T_q1
      T_u1 = 134217729.0 * Zh(NP-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NP-2))
      T_p2 = Zh(NP-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(NP-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NP-4) - T_ch
      T_r1 = T_s1 - Zh(NP-4)
      T_s2 = (Zh(NP-4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NP-4)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(NP-2)
      A2h = T_q1 + T_q2
      A2l = T_q2
     +      - (A2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  A2 = Z(NP-4) / Z(NP-2)
!!                  IF( Z( NN-9 ) .GT. Z( NN-11 ) )
!!     $               RETURN
                  IF(
!       (Z(NN-9) .gt. Z(NN-11))
     + ((Zh(NN-9) .gt. Zh(NN-11)) .or.
     +  ((Zh(NN-9) .eq. Zh(NN-11)) .and.
     +   (Zl(NN-9) .gt. Zl(NN-11))))
     $              ) THEN
                     RETURN
                  END IF
!!                  B2 = Z( NN-9 ) / Z( NN-11 )
!<<<  B2 = Z(NN-9) / Z(NN-11)
      T_q1 = Zh(NN-9) / Zh(NN-11)
      T_ch = Zh(NN-11) * T_q1
      T_u1 = 134217729.0 * Zh(NN-11)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-11))
      T_p2 = Zh(NN-11) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(NN-11) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-9) - T_ch
      T_r1 = T_s1 - Zh(NN-9)
      T_s2 = (Zh(NN-9)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-9)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(NN-11)
      B2h = T_q1 + T_q2
      B2l = T_q2
     +      - (B2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B2 = Z(NN-9) / Z(NN-11)
                  NP = NN - 13
               END IF
*
*              Approximate contribution to norm squared from I < NN-1.
*
!!               A2 = A2 + B2
!<<<  tmp1 = A2 + B2
      T_p1 = A2h + B2h
      T_p2 = T_p1 - A2h
      T_q1 = T_p1
      T_q2 = (A2h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + A2l + B2l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = A2 + B2
      A2h = tmp1h
      A2l = tmp1l
               DO 10 I4 = NP, 4*I0 - 1 + PP, -4
!!                  IF( B2.EQ.ZERO )
!!     $               GO TO 20
                  IF(
!       (B2 .eq. ZERO)
     + ((B2h .eq. ZEROh) .and.
     +  (B2l .eq. ZEROl))
     $              ) THEN
                     GO TO 20
                  END IF
!!                  B1 = B2
      B1h = B2h
      B1l = B2l
!!                  IF( Z( I4 ) .GT. Z( I4-2 ) )
!!     $               RETURN
                  IF(
!       (Z(I4) .gt. Z(I4-2))
     + ((Zh(I4) .gt. Zh(I4-2)) .or.
     +  ((Zh(I4) .eq. Zh(I4-2)) .and.
     +   (Zl(I4) .gt. Zl(I4-2))))
     $              ) THEN
                     RETURN
                  END IF
!!                  B2 = B2*( Z( I4 ) / Z( I4-2 ) )
!<<<  tmp1 = Z(I4) / Z(I4-2)
      T_q1 = Zh(I4) / Zh(I4-2)
      T_ch = Zh(I4-2) * T_q1
      T_u1 = 134217729.0 * Zh(I4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I4-2))
      T_p2 = Zh(I4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(I4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(I4) - T_ch
      T_r1 = T_s1 - Zh(I4)
      T_s2 = (Zh(I4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(I4)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(I4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(I4) / Z(I4-2)
!<<<  tmp2 = B2 * tmp1
      T_ch = B2h * tmp1h
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B2h*tmp1l
     +         +  B2l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = B2 * tmp1
      B2h = tmp2h
      B2l = tmp2l
!!                  A2 = A2 + B2
!<<<  tmp1 = A2 + B2
      T_p1 = A2h + B2h
      T_p2 = T_p1 - A2h
      T_q1 = T_p1
      T_q2 = (A2h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + A2l + B2l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = A2 + B2
      A2h = tmp1h
      A2l = tmp1l
!!                  IF( HUNDRD*MAX( B2, B1 ).LT.A2 .OR. CNST1.LT.A2 ) 
!!     $               GO TO 20
                  IF(
!       (B2 .gt. B1)
     + ((B2h .gt. B1h) .or.
     +  ((B2h .eq. B1h) .and.
     +   (B2l .gt. B1l)))
     &              ) THEN
      tmp1h = B2h
      tmp1l = B2l
                  ELSE
      tmp1h = B1h
      tmp1l = B1l
                  END IF
!<<<  tmp2 = HUNDRD * tmp1
      T_ch = HUNDRDh * tmp1h
      T_u1 = 134217729.0 * HUNDRDh
      T_p1 = T_u1
     +         - (T_u1 - HUNDRDh)
      T_p2 = HUNDRDh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HUNDRDh*tmp1l
     +         +  HUNDRDl*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = HUNDRD * tmp1
                  IF(
!       (tmp2 .lt. A2)
     + ((tmp2h .lt. A2h) .or.
     +  ((tmp2h .eq. A2h) .and.
     +   (tmp2l .lt. A2l)))
     $                  .OR.
!       (CNST1 .lt. A2)
     + ((CNST1h .lt. A2h) .or.
     +  ((CNST1h .eq. A2h) .and.
     +   (CNST1l .lt. A2l)))
     $              ) THEN
                     GO TO 20
                  END IF
   10          CONTINUE
   20          CONTINUE
!!               A2 = CNST3*A2
!<<<  tmp1 = CNST3 * A2
      T_ch = CNST3h * A2h
      T_u1 = 134217729.0 * CNST3h
      T_p1 = T_u1
     +         - (T_u1 - CNST3h)
      T_p2 = CNST3h - T_p1
      T_u1 = 134217729.0 * A2h
      T_r1 = T_u1
     +         - (T_u1 - A2h)
      T_r2 = A2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST3h*A2l
     +         +  CNST3l*A2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST3 * A2
      A2h = tmp1h
      A2l = tmp1l
*
*              Rayleigh quotient residual bound.
*
!!               IF( A2.LT.CNST1 )
!!     $            S = GAM*( ONE-SQRT( A2 ) ) / ( ONE+A2 )
               IF(
!       (A2 .lt. CNST1)
     + ((A2h .lt. CNST1h) .or.
     +  ((A2h .eq. CNST1h) .and.
     +   (A2l .lt. CNST1l)))
     $           ) THEN
!<<<  tmp1 = sqrt(A2)
      if (A2h.eq.0.0D+0.and.A2l.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (A2h.lt.0.0d+0.or.
     +   (A2h.eq.0.0d+0.and.A2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(A2h)
      T_ax = A2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           A2h,A2l,T_z1,T_z2)
          T_p1 = A2h - T_z1
          T_p2 = T_p1 - A2h
          T_t2l = (A2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + A2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(A2)
!<<<  tmp2 = ONE - tmp1
      T_p1 = ONEh - tmp1h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + ONEl - tmp1l
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = ONE - tmp1
!<<<  tmp3 = GAM * tmp2
      T_ch = GAMh * tmp2h
      T_u1 = 134217729.0 * GAMh
      T_p1 = T_u1
     +         - (T_u1 - GAMh)
      T_p2 = GAMh - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (GAMh*tmp2l
     +         +  GAMl*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = GAM * tmp2
!<<<  tmp4 = ONE + A2
      T_p1 = ONEh + A2h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (A2h - T_p2)
      T_q2 = T_q2 + ONEl + A2l
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2 - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = ONE + A2
!<<<  S = tmp3 / tmp4
      T_q1 = tmp3h / tmp4h
      T_ch = tmp4h * T_q1
      T_u1 = 134217729.0 * tmp4h
      T_p1 = T_u1
     +         - (T_u1 - tmp4h)
      T_p2 = tmp4h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp4l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp3h - T_ch
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / tmp4h
      Sh = T_q1 + T_q2
      Sl = T_q2
     +      - (Sh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S = tmp3 / tmp4
               END IF
            END IF
!!         ELSE IF( DMIN.EQ.DN2 ) THEN
         ELSE IF(
!       (DMIN .eq. DN2)
     + ((DMINh .eq. DN2h) .and.
     +  (DMINl .eq. DN2l))
     $          ) THEN
*
*           Case 5.
*
            TTYPE = -5
!!            S = QURTR*DMIN
!<<<  S = QURTR * DMIN
      T_ch = QURTRh * DMINh
      T_u1 = 134217729.0 * QURTRh
      T_p1 = T_u1
     +         - (T_u1 - QURTRh)
      T_p2 = QURTRh - T_p1
      T_u1 = 134217729.0 * DMINh
      T_r1 = T_u1
     +         - (T_u1 - DMINh)
      T_r2 = DMINh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (QURTRh*DMINl
     +         +  QURTRl*DMINh)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = QURTR * DMIN
*
*           Compute contribution to norm squared from I > NN-2.
*
            NP = NN - 2*PP
!!            B1 = Z( NP-2 )
      B1h = Zh(NP-2)
      B1l = Zl(NP-2)
!!            B2 = Z( NP-6 )
      B2h = Zh(NP-6)
      B2l = Zl(NP-6)
!!            GAM = DN2
      GAMh = DN2h
      GAMl = DN2l
!!            IF( Z( NP-8 ).GT.B2 .OR. Z( NP-4 ).GT.B1 )
!!     $         RETURN
            IF(
!       (Z(NP-8) .gt. B2)
     + ((Zh(NP-8) .gt. B2h) .or.
     +  ((Zh(NP-8) .eq. B2h) .and.
     +   (Zl(NP-8) .gt. B2l)))
     $            .OR.
!       (Z(NP-4) .gt. B1)
     + ((Zh(NP-4) .gt. B1h) .or.
     +  ((Zh(NP-4) .eq. B1h) .and.
     +   (Zl(NP-4) .gt. B1l)))
     $        ) THEN
               RETURN
            END IF
!!            A2 = ( Z( NP-8 ) / B2 )*( ONE+Z( NP-4 ) / B1 )
!<<<  tmp1 = Z(NP-8) / B2
      T_q1 = Zh(NP-8) / B2h
      T_ch = B2h * T_q1
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (B2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NP-8) - T_ch
      T_r1 = T_s1 - Zh(NP-8)
      T_s2 = (Zh(NP-8)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NP-8)
      T_q2 = (T_s1 + T_s2)
     +         / B2h
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(NP-8) / B2
!<<<  tmp2 = Z(NP-4) / B1
      T_q1 = Zh(NP-4) / B1h
      T_ch = B1h * T_q1
      T_u1 = 134217729.0 * B1h
      T_p1 = T_u1
     +         - (T_u1 - B1h)
      T_p2 = B1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (B1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NP-4) - T_ch
      T_r1 = T_s1 - Zh(NP-4)
      T_s2 = (Zh(NP-4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NP-4)
      T_q2 = (T_s1 + T_s2)
     +         / B1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = Z(NP-4) / B1
!<<<  tmp3 = ONE + tmp2
      T_p1 = ONEh + tmp2h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + ONEl + tmp2l
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = ONE + tmp2
!<<<  A2 = tmp1 * tmp3
      T_ch = tmp1h * tmp3h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp3l
     +         +  tmp1l*tmp3h)
      T_s1 = T_ch + T_cl
      A2l = T_cl - (T_s1 - T_ch)
      A2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A2 = tmp1 * tmp3
*
*           Approximate contribution to norm squared from I < NN-2.
*
            IF( N0-I0.GT.2 ) THEN
!!               B2 = Z( NN-13 ) / Z( NN-15 )
!<<<  B2 = Z(NN-13) / Z(NN-15)
      T_q1 = Zh(NN-13) / Zh(NN-15)
      T_ch = Zh(NN-15) * T_q1
      T_u1 = 134217729.0 * Zh(NN-15)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-15))
      T_p2 = Zh(NN-15) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(NN-15) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-13) - T_ch
      T_r1 = T_s1 - Zh(NN-13)
      T_s2 = (Zh(NN-13)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-13)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(NN-15)
      B2h = T_q1 + T_q2
      B2l = T_q2
     +      - (B2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B2 = Z(NN-13) / Z(NN-15)
!!               A2 = A2 + B2
!<<<  tmp1 = A2 + B2
      T_p1 = A2h + B2h
      T_p2 = T_p1 - A2h
      T_q1 = T_p1
      T_q2 = (A2h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + A2l + B2l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = A2 + B2
      A2h = tmp1h
      A2l = tmp1l
               DO 30 I4 = NN - 17, 4*I0 - 1 + PP, -4
!!                  IF( B2.EQ.ZERO )
!!     $               GO TO 40
                  IF(
!       (B2 .eq. ZERO)
     + ((B2h .eq. ZEROh) .and.
     +  (B2l .eq. ZEROl))
     $              ) THEN
                     GO TO 40
                  END IF
!!                  B1 = B2
      B1h = B2h
      B1l = B2l
!!                  IF( Z( I4 ) .GT. Z( I4-2 ) )
!!     $               RETURN
                  IF(
!       (Z(I4) .gt. Z(I4-2))
     + ((Zh(I4) .gt. Zh(I4-2)) .or.
     +  ((Zh(I4) .eq. Zh(I4-2)) .and.
     +   (Zl(I4) .gt. Zl(I4-2))))
     $              ) THEN
                     RETURN
                  END IF
!!                  B2 = B2*( Z( I4 ) / Z( I4-2 ) )
!<<<  tmp1 = Z(I4) / Z(I4-2)
      T_q1 = Zh(I4) / Zh(I4-2)
      T_ch = Zh(I4-2) * T_q1
      T_u1 = 134217729.0 * Zh(I4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I4-2))
      T_p2 = Zh(I4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(I4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(I4) - T_ch
      T_r1 = T_s1 - Zh(I4)
      T_s2 = (Zh(I4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(I4)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(I4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(I4) / Z(I4-2)
!<<<  tmp2 = B2 * tmp1
      T_ch = B2h * tmp1h
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B2h*tmp1l
     +         +  B2l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = B2 * tmp1
      B2h = tmp2h
      B2l = tmp2l
!!                  A2 = A2 + B2
!<<<  tmp1 = A2 + B2
      T_p1 = A2h + B2h
      T_p2 = T_p1 - A2h
      T_q1 = T_p1
      T_q2 = (A2h - (T_p1 - T_p2))
     +         + (B2h - T_p2)
      T_q2 = T_q2 + A2l + B2l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = A2 + B2
      A2h = tmp1h
      A2l = tmp1l
!!                  IF( HUNDRD*MAX( B2, B1 ).LT.A2 .OR. CNST1.LT.A2 ) 
!!     $               GO TO 40
                  IF(
!       (B2 .gt. B1)
     + ((B2h .gt. B1h) .or.
     +  ((B2h .eq. B1h) .and.
     +   (B2l .gt. B1l)))
     $              ) THEN
      tmp1h = B2h
      tmp1l = B2l
                  ELSE
      tmp1h = B1h
      tmp1l = B1l
                  END IF
!<<<  tmp2 = HUNDRD * tmp1
      T_ch = HUNDRDh * tmp1h
      T_u1 = 134217729.0 * HUNDRDh
      T_p1 = T_u1
     +         - (T_u1 - HUNDRDh)
      T_p2 = HUNDRDh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HUNDRDh*tmp1l
     +         +  HUNDRDl*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = HUNDRD * tmp1
                  IF(
!       (tmp2 .lt. A2)
     + ((tmp2h .lt. A2h) .or.
     +  ((tmp2h .eq. A2h) .and.
     +   (tmp2l .lt. A2l)))
     $                  .OR.
!       (CNST1 .lt. A2)
     + ((CNST1h .lt. A2h) .or.
     +  ((CNST1h .eq. A2h) .and.
     +   (CNST1l .lt. A2l)))
     $              ) THEN
                     GO TO 40
                  END IF
   30          CONTINUE
   40          CONTINUE
!!               A2 = CNST3*A2
!<<<  tmp1 = CNST3 * A2
      T_ch = CNST3h * A2h
      T_u1 = 134217729.0 * CNST3h
      T_p1 = T_u1
     +         - (T_u1 - CNST3h)
      T_p2 = CNST3h - T_p1
      T_u1 = 134217729.0 * A2h
      T_r1 = T_u1
     +         - (T_u1 - A2h)
      T_r2 = A2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST3h*A2l
     +         +  CNST3l*A2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST3 * A2
      A2h = tmp1h
      A2l = tmp1l
            END IF
*
!!            IF( A2.LT.CNST1 )
!!     $         S = GAM*( ONE-SQRT( A2 ) ) / ( ONE+A2 )
            IF(
!       (A2 .lt. CNST1)
     + ((A2h .lt. CNST1h) .or.
     +  ((A2h .eq. CNST1h) .and.
     +   (A2l .lt. CNST1l)))
     $        ) THEN
!<<<  tmp1 = sqrt(A2)
      if (A2h.eq.0.0D+0.and.A2l.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (A2h.lt.0.0d+0.or.
     +   (A2h.eq.0.0d+0.and.A2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(A2h)
      T_ax = A2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           A2h,A2l,T_z1,T_z2)
          T_p1 = A2h - T_z1
          T_p2 = T_p1 - A2h
          T_t2l = (A2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + A2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(A2)
!<<<  tmp2 = ONE - tmp1
      T_p1 = ONEh - tmp1h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + ONEl - tmp1l
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = ONE - tmp1
!<<<  tmp3 = GAM * tmp2
      T_ch = GAMh * tmp2h
      T_u1 = 134217729.0 * GAMh
      T_p1 = T_u1
     +         - (T_u1 - GAMh)
      T_p2 = GAMh - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (GAMh*tmp2l
     +         +  GAMl*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = GAM * tmp2
!<<<  tmp4 = ONE + A2
      T_p1 = ONEh + A2h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (A2h - T_p2)
      T_q2 = T_q2 + ONEl + A2l
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2 - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = ONE + A2
!<<<  S = tmp3 / tmp4
      T_q1 = tmp3h / tmp4h
      T_ch = tmp4h * T_q1
      T_u1 = 134217729.0 * tmp4h
      T_p1 = T_u1
     +         - (T_u1 - tmp4h)
      T_p2 = tmp4h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp4l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp3h - T_ch
      T_r1 = T_s1 - tmp3h
      T_s2 = (tmp3h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp3l
      T_q2 = (T_s1 + T_s2)
     +         / tmp4h
      Sh = T_q1 + T_q2
      Sl = T_q2
     +      - (Sh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  S = tmp3 / tmp4
            END IF
         ELSE
*
*           Case 6, no information to guide us.
*
            IF( TTYPE.EQ.-6 ) THEN
!!               G = G + THIRD*( ONE-G )
!<<<  tmp1 = ONE - G
      T_p1 = ONEh - Gh
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (Gh + T_p2)
      T_cl = T_cl + ONEl - Gl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = ONE - G
!<<<  tmp2 = THIRD * tmp1
      T_ch = THIRDh * tmp1h
      T_u1 = 134217729.0 * THIRDh
      T_p1 = T_u1
     +         - (T_u1 - THIRDh)
      T_p2 = THIRDh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (THIRDh*tmp1l
     +         +  THIRDl*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = THIRD * tmp1
!<<<  tmp3 = G + tmp2
      T_p1 = Gh + tmp2h
      T_p2 = T_p1 - Gh
      T_q1 = T_p1
      T_q2 = (Gh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + Gl + tmp2l
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = G + tmp2
      Gh = tmp3h
      Gl = tmp3l
            ELSE IF( TTYPE.EQ.-18 ) THEN
!!               G = QURTR*THIRD
!<<<  G = QURTR * THIRD
      T_ch = QURTRh * THIRDh
      T_u1 = 134217729.0 * QURTRh
      T_p1 = T_u1
     +         - (T_u1 - QURTRh)
      T_p2 = QURTRh - T_p1
      T_u1 = 134217729.0 * THIRDh
      T_r1 = T_u1
     +         - (T_u1 - THIRDh)
      T_r2 = THIRDh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (QURTRh*THIRDl
     +         +  QURTRl*THIRDh)
      T_s1 = T_ch + T_cl
      Gl = T_cl - (T_s1 - T_ch)
      Gh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  G = QURTR * THIRD
            ELSE
!!               G = QURTR
      Gh = QURTRh
      Gl = QURTRl
            END IF
!!            S = G*DMIN
!<<<  S = G * DMIN
      T_ch = Gh * DMINh
      T_u1 = 134217729.0 * Gh
      T_p1 = T_u1
     +         - (T_u1 - Gh)
      T_p2 = Gh - T_p1
      T_u1 = 134217729.0 * DMINh
      T_r1 = T_u1
     +         - (T_u1 - DMINh)
      T_r2 = DMINh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Gh*DMINl
     +         +  Gl*DMINh)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = G * DMIN
            TTYPE = -6
         END IF
*
      ELSE IF( N0IN.EQ.( N0+1 ) ) THEN
*
*        One eigenvalue just deflated. Use DMIN1, DN1 for DMIN and DN.
*
!!         IF( DMIN1.EQ.DN1 .AND. DMIN2.EQ.DN2 ) THEN 
         IF(
!       (DMIN1 .eq. DN1)
     + ((DMIN1h .eq. DN1h) .and.
     +  (DMIN1l .eq. DN1l))
     $         .AND.
!       (DMIN2 .eq. DN2)
     + ((DMIN2h .eq. DN2h) .and.
     +  (DMIN2l .eq. DN2l))
     $     ) THEN
*
*           Cases 7 and 8.
*
            TTYPE = -7
!!            S = THIRD*DMIN1
!<<<  S = THIRD * DMIN1
      T_ch = THIRDh * DMIN1h
      T_u1 = 134217729.0 * THIRDh
      T_p1 = T_u1
     +         - (T_u1 - THIRDh)
      T_p2 = THIRDh - T_p1
      T_u1 = 134217729.0 * DMIN1h
      T_r1 = T_u1
     +         - (T_u1 - DMIN1h)
      T_r2 = DMIN1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (THIRDh*DMIN1l
     +         +  THIRDl*DMIN1h)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = THIRD * DMIN1
!!            IF( Z( NN-5 ).GT.Z( NN-7 ) )
!!     $         RETURN
            IF(
!       (Z(NN-5) .gt. Z(NN-7))
     + ((Zh(NN-5) .gt. Zh(NN-7)) .or.
     +  ((Zh(NN-5) .eq. Zh(NN-7)) .and.
     +   (Zl(NN-5) .gt. Zl(NN-7))))
     $        ) THEN
               RETURN
            END IF
!!            B1 = Z( NN-5 ) / Z( NN-7 )
!<<<  B1 = Z(NN-5) / Z(NN-7)
      T_q1 = Zh(NN-5) / Zh(NN-7)
      T_ch = Zh(NN-7) * T_q1
      T_u1 = 134217729.0 * Zh(NN-7)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-7))
      T_p2 = Zh(NN-7) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(NN-7) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-5) - T_ch
      T_r1 = T_s1 - Zh(NN-5)
      T_s2 = (Zh(NN-5)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-5)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(NN-7)
      B1h = T_q1 + T_q2
      B1l = T_q2
     +      - (B1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B1 = Z(NN-5) / Z(NN-7)
!!            B2 = B1
      B2h = B1h
      B2l = B1l
!!            IF( B2.EQ.ZERO )
!!     $         GO TO 60
            IF(
!       (B2 .eq. ZERO)
     + ((B2h .eq. ZEROh) .and.
     +  (B2l .eq. ZEROl))
     $        ) THEN
               GO TO 60
            END IF
            DO 50 I4 = 4*N0 - 9 + PP, 4*I0 - 1 + PP, -4
!!               A2 = B1
      A2h = B1h
      A2l = B1l
!!               IF( Z( I4 ).GT.Z( I4-2 ) )
!!     $            RETURN
               IF(
!       (Z(I4) .gt. Z(I4-2))
     + ((Zh(I4) .gt. Zh(I4-2)) .or.
     +  ((Zh(I4) .eq. Zh(I4-2)) .and.
     +   (Zl(I4) .gt. Zl(I4-2))))
     $           ) THEN
                  RETURN
               END IF
!!               B1 = B1*( Z( I4 ) / Z( I4-2 ) )
!<<<  tmp1 = Z(I4) / Z(I4-2)
      T_q1 = Zh(I4) / Zh(I4-2)
      T_ch = Zh(I4-2) * T_q1
      T_u1 = 134217729.0 * Zh(I4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I4-2))
      T_p2 = Zh(I4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(I4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(I4) - T_ch
      T_r1 = T_s1 - Zh(I4)
      T_s2 = (Zh(I4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(I4)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(I4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(I4) / Z(I4-2)
!<<<  tmp2 = B1 * tmp1
      T_ch = B1h * tmp1h
      T_u1 = 134217729.0 * B1h
      T_p1 = T_u1
     +         - (T_u1 - B1h)
      T_p2 = B1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B1h*tmp1l
     +         +  B1l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = B1 * tmp1
      B1h = tmp2h
      B1l = tmp2l
!!               B2 = B2 + B1
!<<<  tmp1 = B2 + B1
      T_p1 = B2h + B1h
      T_p2 = T_p1 - B2h
      T_q1 = T_p1
      T_q2 = (B2h - (T_p1 - T_p2))
     +         + (B1h - T_p2)
      T_q2 = T_q2 + B2l + B1l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = B2 + B1
      B2h = tmp1h
      B2l = tmp1l
!!               IF( HUNDRD*MAX( B1, A2 ).LT.B2 ) 
!!     $            GO TO 60
               IF(
!       (B1 .gt. A2)
     + ((B1h .gt. A2h) .or.
     +  ((B1h .eq. A2h) .and.
     +   (B1l .gt. A2l)))
     &           ) THEN
      tmp1h = B1h
      tmp1l = B1l
               ELSE
      tmp1h = A2h
      tmp1l = A2l
               END IF
!<<<  tmp2 = HUNDRD * tmp1
      T_ch = HUNDRDh * tmp1h
      T_u1 = 134217729.0 * HUNDRDh
      T_p1 = T_u1
     +         - (T_u1 - HUNDRDh)
      T_p2 = HUNDRDh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HUNDRDh*tmp1l
     +         +  HUNDRDl*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = HUNDRD * tmp1
               IF(
!       (tmp2 .lt. B2)
     + ((tmp2h .lt. B2h) .or.
     +  ((tmp2h .eq. B2h) .and.
     +   (tmp2l .lt. B2l)))
     $           ) THEN
                  GO TO 60
               END IF
   50       CONTINUE
   60       CONTINUE
!!            B2 = SQRT( CNST3*B2 )
!<<<  tmp1 = CNST3 * B2
      T_ch = CNST3h * B2h
      T_u1 = 134217729.0 * CNST3h
      T_p1 = T_u1
     +         - (T_u1 - CNST3h)
      T_p2 = CNST3h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST3h*B2l
     +         +  CNST3l*B2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST3 * B2
!<<<  tmp2 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp2 = sqrt(tmp1)
      B2h = tmp2h
      B2l = tmp2l
!!            A2 = DMIN1 / ( ONE+B2**2 )
!<<<  tmp1 = B2 * B2
      T_ch = B2h * B2h
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B2h*B2l
     +         +  B2l*B2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = B2 * B2
!<<<  tmp2 = ONE + tmp1
      T_p1 = ONEh + tmp1h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + ONEl + tmp1l
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = ONE + tmp1
!<<<  A2 = DMIN1 / tmp2
      T_q1 = DMIN1h / tmp2h
      T_ch = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DMIN1h - T_ch
      T_r1 = T_s1 - DMIN1h
      T_s2 = (DMIN1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DMIN1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      A2h = T_q1 + T_q2
      A2l = T_q2
     +      - (A2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  A2 = DMIN1 / tmp2
!!            GAP2 = HALF*DMIN2 - A2
!<<<  tmp1 = HALF * DMIN2
      T_ch = HALFh * DMIN2h
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * DMIN2h
      T_r1 = T_u1
     +         - (T_u1 - DMIN2h)
      T_r2 = DMIN2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HALFh*DMIN2l
     +         +  HALFl*DMIN2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = HALF * DMIN2
!<<<  GAP2 = tmp1 - A2
      T_p1 = tmp1h - A2h
      T_p2 = T_p1 - tmp1h
      T_ch = T_p1
      T_cl = (tmp1h - (T_p1 - T_p2))
     +         - (A2h + T_p2)
      T_cl = T_cl + tmp1l - A2l
      T_s1 = T_ch + T_cl
      GAP2l = T_cl - (T_s1 - T_ch)
      GAP2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  GAP2 = tmp1 - A2
!!            IF( GAP2.GT.ZERO .AND. GAP2.GT.B2*A2 ) THEN
!<<<  tmp1 = B2 * A2
      T_ch = B2h * A2h
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * A2h
      T_r1 = T_u1
     +         - (T_u1 - A2h)
      T_r2 = A2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B2h*A2l
     +         +  B2l*A2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = B2 * A2
            IF(
!       (GAP2 .gt. ZERO)
     + ((GAP2h .gt. ZEROh) .or.
     +  ((GAP2h .eq. ZEROh) .and.
     +   (GAP2l .gt. ZEROl)))
     $            .AND.
!       (GAP2 .gt. tmp1)
     + ((GAP2h .gt. tmp1h) .or.
     +  ((GAP2h .eq. tmp1h) .and.
     +   (GAP2l .gt. tmp1l)))
     &        ) THEN
!!               S = MAX( S, A2*( ONE-CNST2*A2*( B2 / GAP2 )*B2 ) )
!<<<  tmp1 = CNST2 * A2
      T_ch = CNST2h * A2h
      T_u1 = 134217729.0 * CNST2h
      T_p1 = T_u1
     +         - (T_u1 - CNST2h)
      T_p2 = CNST2h - T_p1
      T_u1 = 134217729.0 * A2h
      T_r1 = T_u1
     +         - (T_u1 - A2h)
      T_r2 = A2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST2h*A2l
     +         +  CNST2l*A2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST2 * A2
!<<<  tmp2 = B2 / GAP2
      T_q1 = B2h / GAP2h
      T_ch = GAP2h * T_q1
      T_u1 = 134217729.0 * GAP2h
      T_p1 = T_u1
     +         - (T_u1 - GAP2h)
      T_p2 = GAP2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAP2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = B2h - T_ch
      T_r1 = T_s1 - B2h
      T_s2 = (B2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + B2l
      T_q2 = (T_s1 + T_s2)
     +         / GAP2h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = B2 / GAP2
!<<<  tmp3 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
!<<<  tmp4 = tmp3 * B2
      T_ch = tmp3h * B2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*B2l
     +         +  tmp3l*B2h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp3 * B2
!<<<  tmp5 = ONE - tmp4
      T_p1 = ONEh - tmp4h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      T_cl = T_cl + ONEl - tmp4l
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp5 = ONE - tmp4
!<<<  tmp6 = A2 * tmp5
      T_ch = A2h * tmp5h
      T_u1 = 134217729.0 * A2h
      T_p1 = T_u1
     +         - (T_u1 - A2h)
      T_p2 = A2h - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (A2h*tmp5l
     +         +  A2l*tmp5h)
      T_s1 = T_ch + T_cl
      tmp6l = T_cl - (T_s1 - T_ch)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp6 = A2 * tmp5
               IF(
!       (S .lt. tmp6)
     + ((Sh .lt. tmp6h) .or.
     +  ((Sh .eq. tmp6h) .and.
     +   (Sl .lt. tmp6l)))
     $           ) THEN
      Sh = tmp6h
      Sl = tmp6l
               END IF
            ELSE 
!!               S = MAX( S, A2*( ONE-CNST2*B2 ) )
!<<<  tmp1 = CNST2 * B2
      T_ch = CNST2h * B2h
      T_u1 = 134217729.0 * CNST2h
      T_p1 = T_u1
     +         - (T_u1 - CNST2h)
      T_p2 = CNST2h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST2h*B2l
     +         +  CNST2l*B2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST2 * B2
!<<<  tmp2 = ONE - tmp1
      T_p1 = ONEh - tmp1h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + ONEl - tmp1l
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = ONE - tmp1
!<<<  tmp3 = A2 * tmp2
      T_ch = A2h * tmp2h
      T_u1 = 134217729.0 * A2h
      T_p1 = T_u1
     +         - (T_u1 - A2h)
      T_p2 = A2h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (A2h*tmp2l
     +         +  A2l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = A2 * tmp2
               IF(
!       (S .lt. tmp3)
     + ((Sh .lt. tmp3h) .or.
     +  ((Sh .eq. tmp3h) .and.
     +   (Sl .lt. tmp3l)))
     $           ) THEN
      Sh = tmp3h
      Sl = tmp3l
               END IF
               TTYPE = -8
            END IF
         ELSE
*
*           Case 9.
*
!!            S = QURTR*DMIN1
!<<<  S = QURTR * DMIN1
      T_ch = QURTRh * DMIN1h
      T_u1 = 134217729.0 * QURTRh
      T_p1 = T_u1
     +         - (T_u1 - QURTRh)
      T_p2 = QURTRh - T_p1
      T_u1 = 134217729.0 * DMIN1h
      T_r1 = T_u1
     +         - (T_u1 - DMIN1h)
      T_r2 = DMIN1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (QURTRh*DMIN1l
     +         +  QURTRl*DMIN1h)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = QURTR * DMIN1
!!            IF( DMIN1.EQ.DN1 )
!!     $         S = HALF*DMIN1
            IF(
!       (DMIN1 .eq. DN1)
     + ((DMIN1h .eq. DN1h) .and.
     +  (DMIN1l .eq. DN1l))
     $        ) THEN
!<<<  S = HALF * DMIN1
      T_ch = HALFh * DMIN1h
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * DMIN1h
      T_r1 = T_u1
     +         - (T_u1 - DMIN1h)
      T_r2 = DMIN1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HALFh*DMIN1l
     +         +  HALFl*DMIN1h)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = HALF * DMIN1
            END IF
            TTYPE = -9
         END IF
*
      ELSE IF( N0IN.EQ.( N0+2 ) ) THEN
*
*        Two eigenvalues deflated. Use DMIN2, DN2 for DMIN and DN.
*
*        Cases 10 and 11.
*
!!         IF( DMIN2.EQ.DN2 .AND. TWO*Z( NN-5 ).LT.Z( NN-7 ) ) THEN 
!<<<  tmp1 = TWO * Z(NN-5)
      T_ch = TWOh * Zh(NN-5)
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Zh(NN-5)
      T_r1 = T_u1
     +         - (T_u1 - Zh(NN-5))
      T_r2 = Zh(NN-5) - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TWOh*Zl(NN-5)
     +         +  TWOl*Zh(NN-5))
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = TWO * Z(NN-5)
         IF(
!       (DMIN2 .eq. DN2)
     + ((DMIN2h .eq. DN2h) .and.
     +  (DMIN2l .eq. DN2l))
     $         .AND.
!       (tmp1 .lt. Z(NN-7))
     + ((tmp1h .lt. Zh(NN-7)) .or.
     +  ((tmp1h .eq. Zh(NN-7)) .and.
     +   (tmp1l .lt. Zl(NN-7))))
     $     ) THEN
            TTYPE = -10
!!            S = THIRD*DMIN2
!<<<  S = THIRD * DMIN2
      T_ch = THIRDh * DMIN2h
      T_u1 = 134217729.0 * THIRDh
      T_p1 = T_u1
     +         - (T_u1 - THIRDh)
      T_p2 = THIRDh - T_p1
      T_u1 = 134217729.0 * DMIN2h
      T_r1 = T_u1
     +         - (T_u1 - DMIN2h)
      T_r2 = DMIN2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (THIRDh*DMIN2l
     +         +  THIRDl*DMIN2h)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = THIRD * DMIN2
!!            IF( Z( NN-5 ).GT.Z( NN-7 ) )
!!     $         RETURN
            IF(
!       (Z(NN-5) .gt. Z(NN-7))
     + ((Zh(NN-5) .gt. Zh(NN-7)) .or.
     +  ((Zh(NN-5) .eq. Zh(NN-7)) .and.
     +   (Zl(NN-5) .gt. Zl(NN-7))))
     $        ) THEN
               RETURN
            END IF
!!            B1 = Z( NN-5 ) / Z( NN-7 )
!<<<  B1 = Z(NN-5) / Z(NN-7)
      T_q1 = Zh(NN-5) / Zh(NN-7)
      T_ch = Zh(NN-7) * T_q1
      T_u1 = 134217729.0 * Zh(NN-7)
      T_p1 = T_u1
     +         - (T_u1 - Zh(NN-7))
      T_p2 = Zh(NN-7) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(NN-7) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(NN-5) - T_ch
      T_r1 = T_s1 - Zh(NN-5)
      T_s2 = (Zh(NN-5)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(NN-5)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(NN-7)
      B1h = T_q1 + T_q2
      B1l = T_q2
     +      - (B1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B1 = Z(NN-5) / Z(NN-7)
!!            B2 = B1
      B2h = B1h
      B2l = B1l
!!            IF( B2.EQ.ZERO )
!!     $         GO TO 80
            IF(
!       (B2 .eq. ZERO)
     + ((B2h .eq. ZEROh) .and.
     +  (B2l .eq. ZEROl))
     $        ) THEN
               GO TO 80
            END IF
            DO 70 I4 = 4*N0 - 9 + PP, 4*I0 - 1 + PP, -4
!!               IF( Z( I4 ).GT.Z( I4-2 ) )
!!     $            RETURN
               IF(
!       (Z(I4) .gt. Z(I4-2))
     + ((Zh(I4) .gt. Zh(I4-2)) .or.
     +  ((Zh(I4) .eq. Zh(I4-2)) .and.
     +   (Zl(I4) .gt. Zl(I4-2))))
     $           ) THEN
                  RETURN
               END IF
!!               B1 = B1*( Z( I4 ) / Z( I4-2 ) )
!<<<  tmp1 = Z(I4) / Z(I4-2)
      T_q1 = Zh(I4) / Zh(I4-2)
      T_ch = Zh(I4-2) * T_q1
      T_u1 = 134217729.0 * Zh(I4-2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I4-2))
      T_p2 = Zh(I4-2) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Zl(I4-2) * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Zh(I4) - T_ch
      T_r1 = T_s1 - Zh(I4)
      T_s2 = (Zh(I4)
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Zl(I4)
      T_q2 = (T_s1 + T_s2)
     +         / Zh(I4-2)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = Z(I4) / Z(I4-2)
!<<<  tmp2 = B1 * tmp1
      T_ch = B1h * tmp1h
      T_u1 = 134217729.0 * B1h
      T_p1 = T_u1
     +         - (T_u1 - B1h)
      T_p2 = B1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B1h*tmp1l
     +         +  B1l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = B1 * tmp1
      B1h = tmp2h
      B1l = tmp2l
!!               B2 = B2 + B1
!<<<  tmp1 = B2 + B1
      T_p1 = B2h + B1h
      T_p2 = T_p1 - B2h
      T_q1 = T_p1
      T_q2 = (B2h - (T_p1 - T_p2))
     +         + (B1h - T_p2)
      T_q2 = T_q2 + B2l + B1l
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = B2 + B1
      B2h = tmp1h
      B2l = tmp1l
!!               IF( HUNDRD*B1.LT.B2 )
!!     $            GO TO 80
!<<<  tmp1 = HUNDRD * B1
      T_ch = HUNDRDh * B1h
      T_u1 = 134217729.0 * HUNDRDh
      T_p1 = T_u1
     +         - (T_u1 - HUNDRDh)
      T_p2 = HUNDRDh - T_p1
      T_u1 = 134217729.0 * B1h
      T_r1 = T_u1
     +         - (T_u1 - B1h)
      T_r2 = B1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HUNDRDh*B1l
     +         +  HUNDRDl*B1h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = HUNDRD * B1
               IF(
!       (tmp1 .lt. B2)
     + ((tmp1h .lt. B2h) .or.
     +  ((tmp1h .eq. B2h) .and.
     +   (tmp1l .lt. B2l)))
     $           ) THEN
                  GO TO 80
               END IF
   70       CONTINUE
   80       CONTINUE
!!            B2 = SQRT( CNST3*B2 )
!<<<  tmp1 = CNST3 * B2
      T_ch = CNST3h * B2h
      T_u1 = 134217729.0 * CNST3h
      T_p1 = T_u1
     +         - (T_u1 - CNST3h)
      T_p2 = CNST3h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST3h*B2l
     +         +  CNST3l*B2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST3 * B2
!<<<  B2 = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        B2h = 0.0d+0
        B2l = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        B2h = 0.0d+0
        B2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          B2h = T_t1h
          B2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  B2 = sqrt(tmp1)
!!            A2 = DMIN2 / ( ONE+B2**2 )
!<<<  tmp1 = B2 * B2
      T_ch = B2h * B2h
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B2h*B2l
     +         +  B2l*B2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = B2 * B2
!<<<  tmp2 = ONE + tmp1
      T_p1 = ONEh + tmp1h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + ONEl + tmp1l
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = ONE + tmp1
!<<<  A2 = DMIN2 / tmp2
      T_q1 = DMIN2h / tmp2h
      T_ch = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = DMIN2h - T_ch
      T_r1 = T_s1 - DMIN2h
      T_s2 = (DMIN2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + DMIN2l
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      A2h = T_q1 + T_q2
      A2l = T_q2
     +      - (A2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  A2 = DMIN2 / tmp2
!!            GAP2 = Z( NN-7 ) + Z( NN-9 ) -
!!     $             SQRT( Z( NN-11 ) )*SQRT( Z( NN-9 ) ) - A2
!<<<  tmp1 = sqrt(Z(NN-11))
      if (Zh(NN-11).eq.0.0D+0.and.Zl(NN-11).eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      else
      if (Zh(NN-11).lt.0.0d+0.or.
     +   (Zh(NN-11).eq.0.0d+0.and.Zl(NN-11).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Zh(NN-11))
      T_ax = Zh(NN-11) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Zh(NN-11),Zl(NN-11),T_z1,T_z2)
          T_p1 = Zh(NN-11) - T_z1
          T_p2 = T_p1 - Zh(NN-11)
          T_t2l = (Zh(NN-11)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + Zl(NN-11) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp1 = sqrt(Z(NN-11))
!<<<  tmp2 = sqrt(Z(NN-9))
      if (Zh(NN-9).eq.0.0D+0.and.Zl(NN-9).eq.0.0D+0) then
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
      else
      if (Zh(NN-9).lt.0.0d+0.or.
     +   (Zh(NN-9).eq.0.0d+0.and.Zl(NN-9).lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp2h = 0.0d+0
        tmp2l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(Zh(NN-9))
      T_ax = Zh(NN-9) * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           Zh(NN-9),Zl(NN-9),T_z1,T_z2)
          T_p1 = Zh(NN-9) - T_z1
          T_p2 = T_p1 - Zh(NN-9)
          T_t2l = (Zh(NN-9)
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + Zl(NN-9) - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp2h = T_t1h
          tmp2l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp2 = sqrt(Z(NN-9))
!<<<  tmp3 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
!<<<  tmp4 = Z(NN-7) + Z(NN-9)
      T_p1 = Zh(NN-7) + Zh(NN-9)
      T_p2 = T_p1 - Zh(NN-7)
      T_q1 = T_p1
      T_q2 = (Zh(NN-7) - (T_p1 - T_p2))
     +         + (Zh(NN-9) - T_p2)
      T_q2 = T_q2 + Zl(NN-7) + Zl(NN-9)
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2 - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = Z(NN-7) + Z(NN-9)
!<<<  tmp5 = tmp4 - tmp3
      T_p1 = tmp4h - tmp3h
      T_p2 = T_p1 - tmp4h
      T_ch = T_p1
      T_cl = (tmp4h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      T_cl = T_cl + tmp4l - tmp3l
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp5 = tmp4 - tmp3
!<<<  GAP2 = tmp5 - A2
      T_p1 = tmp5h - A2h
      T_p2 = T_p1 - tmp5h
      T_ch = T_p1
      T_cl = (tmp5h - (T_p1 - T_p2))
     +         - (A2h + T_p2)
      T_cl = T_cl + tmp5l - A2l
      T_s1 = T_ch + T_cl
      GAP2l = T_cl - (T_s1 - T_ch)
      GAP2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  GAP2 = tmp5 - A2
!!            IF( GAP2.GT.ZERO .AND. GAP2.GT.B2*A2 ) THEN
!<<<  tmp1 = B2 * A2
      T_ch = B2h * A2h
      T_u1 = 134217729.0 * B2h
      T_p1 = T_u1
     +         - (T_u1 - B2h)
      T_p2 = B2h - T_p1
      T_u1 = 134217729.0 * A2h
      T_r1 = T_u1
     +         - (T_u1 - A2h)
      T_r2 = A2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (B2h*A2l
     +         +  B2l*A2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = B2 * A2
            IF(
!       (GAP2 .gt. ZERO)
     + ((GAP2h .gt. ZEROh) .or.
     +  ((GAP2h .eq. ZEROh) .and.
     +   (GAP2l .gt. ZEROl)))
     $            .AND.
!       (GAP2 .gt. tmp1)
     + ((GAP2h .gt. tmp1h) .or.
     +  ((GAP2h .eq. tmp1h) .and.
     +   (GAP2l .gt. tmp1l)))
     $        ) THEN
!!               S = MAX( S, A2*( ONE-CNST2*A2*( B2 / GAP2 )*B2 ) )
!<<<  tmp1 = CNST2 * A2
      T_ch = CNST2h * A2h
      T_u1 = 134217729.0 * CNST2h
      T_p1 = T_u1
     +         - (T_u1 - CNST2h)
      T_p2 = CNST2h - T_p1
      T_u1 = 134217729.0 * A2h
      T_r1 = T_u1
     +         - (T_u1 - A2h)
      T_r2 = A2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST2h*A2l
     +         +  CNST2l*A2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST2 * A2
!<<<  tmp2 = B2 / GAP2
      T_q1 = B2h / GAP2h
      T_ch = GAP2h * T_q1
      T_u1 = 134217729.0 * GAP2h
      T_p1 = T_u1
     +         - (T_u1 - GAP2h)
      T_p2 = GAP2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAP2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = B2h - T_ch
      T_r1 = T_s1 - B2h
      T_s2 = (B2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + B2l
      T_q2 = (T_s1 + T_s2)
     +         / GAP2h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = B2 / GAP2
!<<<  tmp3 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
!<<<  tmp4 = tmp3 * B2
      T_ch = tmp3h * B2h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*B2l
     +         +  tmp3l*B2h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp3 * B2
!<<<  tmp5 = ONE - tmp4
      T_p1 = ONEh - tmp4h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      T_cl = T_cl + ONEl - tmp4l
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp5 = ONE - tmp4
!<<<  tmp6 = A2 * tmp5
      T_ch = A2h * tmp5h
      T_u1 = 134217729.0 * A2h
      T_p1 = T_u1
     +         - (T_u1 - A2h)
      T_p2 = A2h - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (A2h*tmp5l
     +         +  A2l*tmp5h)
      T_s1 = T_ch + T_cl
      tmp6l = T_cl - (T_s1 - T_ch)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp6 = A2 * tmp5
               IF(
!       (S .lt. tmp6)
     + ((Sh .lt. tmp6h) .or.
     +  ((Sh .eq. tmp6h) .and.
     +   (Sl .lt. tmp6l)))
     $           ) THEN
      Sh = tmp6h
      Sl = tmp6l
               END IF
            ELSE 
!!               S = MAX( S, A2*( ONE-CNST2*B2 ) )
!<<<  tmp1 = CNST2 * B2
      T_ch = CNST2h * B2h
      T_u1 = 134217729.0 * CNST2h
      T_p1 = T_u1
     +         - (T_u1 - CNST2h)
      T_p2 = CNST2h - T_p1
      T_u1 = 134217729.0 * B2h
      T_r1 = T_u1
     +         - (T_u1 - B2h)
      T_r2 = B2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (CNST2h*B2l
     +         +  CNST2l*B2h)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = CNST2 * B2
!<<<  tmp2 = ONE - tmp1
      T_p1 = ONEh - tmp1h
      T_p2 = T_p1 - ONEh
      T_ch = T_p1
      T_cl = (ONEh - (T_p1 - T_p2))
     +         - (tmp1h + T_p2)
      T_cl = T_cl + ONEl - tmp1l
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp2 = ONE - tmp1
!<<<  tmp3 = A2 * tmp2
      T_ch = A2h * tmp2h
      T_u1 = 134217729.0 * A2h
      T_p1 = T_u1
     +         - (T_u1 - A2h)
      T_p2 = A2h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (A2h*tmp2l
     +         +  A2l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = A2 * tmp2
               IF(
!       (S .lt. tmp3)
     + ((Sh .lt. tmp3h) .or.
     +  ((Sh .eq. tmp3h) .and.
     +   (Sl .lt. tmp3l)))
     $           ) THEN
      Sh = tmp3h
      Sl = tmp3l
               END IF
            END IF
         ELSE
!!            S = QURTR*DMIN2
!<<<  S = QURTR * DMIN2
      T_ch = QURTRh * DMIN2h
      T_u1 = 134217729.0 * QURTRh
      T_p1 = T_u1
     +         - (T_u1 - QURTRh)
      T_p2 = QURTRh - T_p1
      T_u1 = 134217729.0 * DMIN2h
      T_r1 = T_u1
     +         - (T_u1 - DMIN2h)
      T_r2 = DMIN2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (QURTRh*DMIN2l
     +         +  QURTRl*DMIN2h)
      T_s1 = T_ch + T_cl
      Sl = T_cl - (T_s1 - T_ch)
      Sh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  S = QURTR * DMIN2
            TTYPE = -11
         END IF
      ELSE IF( N0IN.GT.( N0+2 ) ) THEN
*
*        Case 12, more than two eigenvalues deflated. No information.
*
!!         S = ZERO 
      Sh = ZEROh
      Sl = ZEROl
         TTYPE = -12
      END IF
*
!!      TAU = S
      TAUh = Sh
      TAUl = Sl
      RETURN
*
*     End of DLASQ4
*
      END
